#!/bin/sh
PATH=/usr/local/bin:/usr/local/sbin:/usr/bin:/usr/sbin:/bin:/sbin

C_FAL_EVENTLOG="/mnt/data/bin/fal_eventlog.py"
C_FAL_VOLCTRL="/mnt/data/bin/fal_volctrl.py"
C_FAL_BUZZER="/mnt/data/bin/fal_buzzer.py"
C_BUILD_MSGINFO="/mnt/data/bin/build_msginfo"
C_BUILD_VALIDINFO="/mnt/data/bin/build_validinfo"
C_MERGE_MSG="merge_msg.py"
C_SAVE_UPDATE_CHANGES="save_update_changes.py"
C_PKG_MANUAL_APPLY="pkg_manual_apply.py"

F_BETA="beta"
F_DEB_LIST="deb.list"
F_REMOVE_DEBPKG_LIST="remove_debpkg.list"
F_FW_UPDATE="update.tgz" # defined in "rc.local"
F_FAL_MSGINFO="/usr/local/lib/python2.7/dist-packages/fal_msginfo.py"
F_FAL_VALIDINFO="/usr/local/lib/python2.7/dist-packages/fal_validinfo.py"
F_DISK1_SAMBA_CONF_RO="samba/sataraid1/disk1.ro"
F_DISK1_SAMBA_CONF_RW="samba/sataraid1/disk1.rw"
F_CHANGES_BASE="changes.base.xml"
F_INSTALLED_FLAG_PATH="/mnt/data/conf/sysinfo/installed"
F_SYSINFO_CHANGES_PATH="/mnt/data/conf/sysinfo/changes.xml"
F_FLASH0_BIN="flash0.bin"
F_INITRAMFS_TOOLS_CONFIG="initramfs-tools-config.tar.gz"

UG_FILES="/etc/passwd /etc/shadow /etc/group /etc/gshadow"

D_KERNEL_MOD_DIR="/lib/modules/3.10.70"
D_UPDATE_TMP="/var/tmp/update"
D_OVERWRITE_DIR="/mnt/data/conf/overwrite"
D_PROFTPD_DIR="/etc/proftpd"

L_PROFTPD_DIR="../mnt/data/etc/proftpd"

MSG_FIRMWARE_UPDATE_START="37000001"
MSG_FIRMWARE_SUCCESS_FIRMWARE="37000003"
ERR_FIRMWARE_FAIL_FIRMWARE="37040001"

BG_PROCESS_END_MELODY="bgprocessend"
ERROR_MELODY="error"

INTERNAL_VOLUME="sataraid1"
NAME_FOR_INITRD=HDL-AA

REMOVE_RC_LIST="cups proftpd samba winbind ssh apcupsd nut"


request_to_fal()
{
        local fal_cmd="$1"
        local entry="$2"

        local request="<request><entrylist>$entry</entrylist></request>"

        echo "$request" | $fal_cmd
}


add_eventlog()
{
        local code="$1"
	local arg_name="$2"
	local arg_val="$3"
        local entry="<entry type=\"add\"><eventlog><log><code>${code}</code><arglist><arg name=\"${arg_name}\">${arg_val}</arg></arglist></log></eventlog></entry>"
        local response=$(
                request_to_fal "${C_FAL_EVENTLOG}" "${entry}" | \
                        grep "result=\"0\"" | \
                        wc -l
        )

        if [ "x${response}" != "x0" ]; then
                return 0
        fi

        return 1
}


__install_kernel_pkg()
{
	local kernel_pkg="$1"
	local kernel_dir="${D_KERNEL_MOD_DIR}/kernel"
	local modules_dep="${D_KERNEL_MOD_DIR}/modules.dep"

	mv $kernel_dir $kernel_dir.bkup
	rm -rf $modules_dep

	/usr/bin/dpkg -i $kernel_pkg

	mv -n $kernel_dir.bkup/* $kernel_dir
	if [ ${?} -ne 0 ]; then
		return 1
	fi
	rm -rf $kernel_dir.bkup
}


__save_config()
{
	local xmlconf_dir="/mnt/data/xmlconf"
	local conf_dir="/mnt/data/conf"

	mv "$xmlconf_dir" "$xmlconf_dir.bkup"
	mv "$conf_dir" "$conf_dir.bkup"

	mkdir -p ${D_UPDATE_TMP}
	cp -p ${UG_FILES} ${D_UPDATE_TMP}
}


__save_overwrite_files()
{
	mkdir -p ${D_UPDATE_TMP}/overwrite
	for file in ${D_OVERWRITE_DIR}/*.info; do
		cp -p `cat $file` ${D_UPDATE_TMP}/overwrite
	done
}


__force_update_saved_config()
{
	local xmlconf_dir="/mnt/data/xmlconf"
	local conf_dir="/mnt/data/conf"

	cp -rp "$xmlconf_dir/"* "$xmlconf_dir.bkup"
	cp -rp "$conf_dir/"* "$conf_dir.bkup"
	touch ${F_INSTALLED_FLAG_PATH}
}


__update_config()
{
	local script_dir=$1
	local xmlconf_dir="/mnt/data/xmlconf"
	local conf_dir="/mnt/data/conf"

	$script_dir/${C_MERGE_MSG} "$xmlconf_dir/messages.xml" \
		"$xmlconf_dir.bkup/messages.xml"

	${C_BUILD_MSGINFO} "$xmlconf_dir.bkup/messages.xml" ${F_FAL_MSGINFO}
	${C_BUILD_VALIDINFO} "$xmlconf_dir.bkup/validates.xml" ${F_FAL_VALIDINFO}
	/usr/bin/python -m `basename ${F_FAL_MSGINFO} .py` && \
		/usr/bin/python -m `basename ${F_FAL_VALIDINFO} .py`

	# Update file for a part of samba setting
        cp -p "$conf_dir/samba/?global.base" "$conf_dir.bkup/samba"

	# Fix a symbolic link for a proftpd configuration directory.
	if [ ! -L "${D_PROFTPD_DIR}" ]; then
		rm -rf /etc/proftpd
		ln -s "${L_PROFTPD_DIR}" "${D_PROFTPD_DIR}"
	fi
}


__restore_overwrite_files()
{
	for file in ${D_OVERWRITE_DIR}/*.info; do
		local target=`cat $file`

		cp -p ${D_UPDATE_TMP}/overwrite/`basename $target` $target
	done
	rm -rf ${D_UPDATE_TMP}/overwrite
}


__restore_config()
{
	local xmlconf_dir="/mnt/data/xmlconf"
	local conf_dir="/mnt/data/conf"

	mkdir -p ${xmlconf_dir}
	mkdir -p ${conf_dir}

	if [ ! -e "$conf_dir.bkup/${F_DISK1_SAMBA_CONF_RO}" -a \
		-e  "$conf_dir/${F_DISK1_SAMBA_CONF_RO}" ]; then

		rm -f "$conf_dir/${F_DISK1_SAMBA_CONF_RO}"
	fi
	if [ ! -e "$conf_dir.bkup/${F_DISK1_SAMBA_CONF_RW}" -a \
		-e  "$conf_dir/${F_DISK1_SAMBA_CONF_RW}" ]; then

		rm -f "$conf_dir/${F_DISK1_SAMBA_CONF_RW}"
	fi

	cp -rp "$xmlconf_dir.bkup/"* "$xmlconf_dir"
	cp -rp "$conf_dir.bkup/"* "$conf_dir"
	rm -rf "$xmlconf_dir.bkup" "$conf_dir.bkup"

	for file in ${UG_FILES}; do
		cp -p ${D_UPDATE_TMP}/`basename $file` "$file"
	done

	sync
}


__remove_rc()
{
	for target in ${REMOVE_RC_LIST}; do
		if [ -e /etc/init.d/$target ]; then
			/usr/sbin/update-rc.d $target remove
		fi
	done
}


__prepare_update_initramfs()
{
	local script_dir=$1

	tar -C / xvfz $script_dir/${F_INITRAMFS_TOOLS_CONFIG}
}


__mkimage_uInitrd()
{
	local file_initrd=$(find "/boot" -maxdepth 1 -name 'initrd.img-*')
        /usr/bin/mkimage -A arm -T ramdisk -C none -n ${NAME_FOR_INITRD} \
		-d "${file_initrd}" "/boot/uInitrd"
}


__update_initramfs()
{
	__prepare_update_initramfs $script_dir
	depmod
	/usr/sbin/update-initramfs -ut
	__mkimage_uInitrd
}


unmount_volume()
{
        local entry="<entry type=\"unmount\"><volctrl><volume><name type=\"volume\">${1}</name></volume></volctrl></entry>"
        local response=$(
                request_to_fal "${C_FAL_VOLCTRL}" "${entry}" | \
                        grep "result=\"0\"" | \
                        wc -l
        )

        if [ "x${response}" != "x0" ]; then
                return 0
        fi

        return 1
}


uninstall_pkg()
{
	local script_dir=$1
	locale result=0

	for debpkg in `cat $script_dir/$F_REMOVE_DEBPKG_LIST`; do
		/usr/bin/dpkg --purge "$debpkg"
		if [ ${?} -ne 0 ]; then
			result=1
			continue
		fi
	done

	return $result
}


install_pkg()
{
	local script_dir=$1
	local firm_file=$2
	local kernel_pkg=""
	local result=0
	local deb_list=""

	if [ -e "${script_dir}/debconf/config.dat" ]; then
		cp -f \
			"${script_dir}/debconf/config.dat" \
			"/var/cache/debconf/config.dat"
	fi

	for deb_file in `cat $script_dir/$F_DEB_LIST`; do
		if echo "$deb_file" | grep "linux-image-"; then
			kernel_pkg="$script_dir/deb/$deb_file"
		else
			deb_list="$deb_list $script_dir/deb/$deb_file"
		fi
	done

	if [ -n "$kernel_pkg" ]; then
		__install_kernel_pkg $kernel_pkg
		if [ ${?} -ne 0 ]; then
			return 1
		fi
	fi

	__save_config
	__save_overwrite_files
	for file in $deb_list; do
		/usr/bin/dpkg --force-confnew -i "$file"
		if [ ${?} -ne 0 ]; then
			result=1
			continue
		fi

		if echo "$file" | grep "hdlfind"; then
			update-rc.d hdlfind defaults
		elif echo "$file" | grep "fantempd"; then
			update-rc.d fantempd defaults
		elif echo "$file" | grep "spindownd"; then
			update-rc.d spindownd defaults
		elif echo "$file" | grep "hdl-aa"; then
			update-rc.d shutdown defaults
		fi
	done

	$script_dir/${C_PKG_MANUAL_APPLY}

	# This is for initializing
	if [ x"$firm_file" = x"${F_FW_UPDATE}" ]; then
		__force_update_saved_config
	fi
	__update_config $script_dir
	__restore_overwrite_files
	__restore_config
	__remove_rc
	__update_initramfs $script_dir
	return $result
}


save_file_to_part1()
{
	local src_file="$1"
	local disk_list="satad1 satad2 satad3 satad4 satad5 satad6 usb2 usb3"
	local mnt_point="/mnt/part1"

	mkdir -p ${mnt_point}
	for disk in ${disk_list}
	do
		if [ ! -e "/dev/${disk}p1" ]; then
			continue
		fi
		mount -t ext4 "/dev/${disk}p1" ${mnt_point} \
			1>/dev/null 2>/dev/null
		if [ ${?} -ne 0 ]; then
	                continue
	        fi
	
		if [ ! -d ${mnt_point}/landisk ]; then
			continue
		fi

		dir_path="${mnt_point}/landisk/update"
		rm -rf ${dir_path}
		mkdir ${dir_path}
		cp ${src_file} ${dir_path}/${F_FW_UPDATE}

		cp /boot/vmlinuz-3.10.70 /boot/zImage
		cp /boot/zImage ${mnt_point}
		cp /boot/vmlinuz-3.10.70 ${mnt_point}
		cp /boot/uInitrd ${mnt_point}
		cp /boot/initrd.img-3.10.70 ${mnt_point}
		cp /boot/landisk/initialize.sh ${mnt_point}/landisk/initialize.sh

		umount ${mnt_point}
	done
	rm -rf ${mnt_point}

	return 0
}


do_update()
{
	local script_dir="$1"
	local firm_file="$2"
	local version="$3"
	local beta_time="$4"

	unmount_volume ${INTERNAL_VOLUME}
	if [ ${?} -ne 0 ]; then
		return 1
	fi

	dd if=$script_dir/${F_FLASH0_BIN} of=/dev/mtdblock0
	if [ ${?} -ne 0 ]; then
		return 1
	fi

	uninstall_pkg $script_dir
	if [ ${?} -ne 0 ]; then
		return 1
	fi

	install_pkg $script_dir `basename $firm_file`
	if [ ${?} -ne 0 ]; then
		return 1
	fi

	save_file_to_part1 ${firm_file}
	if [ ${?} -ne 0 ]; then
		return 1
	fi

	$script_dir/${C_SAVE_UPDATE_CHANGES} $script_dir/${F_CHANGES_BASE} \
		"${F_SYSINFO_CHANGES_PATH}"

	echo $version > /var/lib/landisk/version
	echo $version > /etc/landisk_version
	if [ -n "$beta_time" ]; then
		echo $beta_time > /var/lib/landisk/beta
		echo $beta_time > /etc/landisk_beta
	fi

	return 0
}


play_melody()
{
	local type="${1}"
	local entry="<entry type=\"play\"><buzzer><melody type=\"${type}\"/></buzzer></entry>"
	local response=$(
                request_to_fal "${C_FAL_BUZZER}" "${entry}" | \
                        grep "result=\"0\"" | \
                        wc -l
        )

        if [ "x${response}" != "x0" ]; then
                return 0
        fi

        return 1
}


#####################################
script_dir=$(cd "$(dirname "$0")"; pwd)
firm_file=$1

ver_str=1.03
beta_file=$(find $script_dir -name ${F_BETA})
beta_time=""
if [ -n "$beta_file" ]; then
	beta_time=$(cat ${beta_file})
fi

add_eventlog "${MSG_FIRMWARE_UPDATE_START}"

/usr/local/bin/remount_root_write

# Do something
do_update $script_dir $firm_file $ver_str $beta_time
result=${?}

/usr/local/bin/remount_root_read_only

if [ ${result} -eq 0 ]; then
	play_melody "${BG_PROCESS_END_MELODY}"
	add_eventlog "${MSG_FIRMWARE_SUCCESS_FIRMWARE}" version "${ver_str}"
	/sbin/shutdown -r now
else
	play_melody "${ERROR_MELODY}"
	add_eventlog "${ERR_FIRMWARE_FAIL_FIRMWARE}"
fi
