#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for setting amazon s3"""

import os

import libfal.falcmd as falcmd
from ionas.request import Entry
from libfal.util import get_proxy, NoProxy
from fal_info import ReqTypes
from fal_info import Results
import pal_info
import fal_config as config

SHARENAME_NAME = "sharename"
RESULT_NAME = "result"
TARGET_SHARE_XPATH = "sharelist/share[name='%s']"
AMAZONS3_PATH = "shareservice/amazons3"
HTTPMETHOD_NAME = "httpmethod"
ENDPOINT_NAME = "endpoint"
S3_PORT_NAME = "port"
PROXY_NAME = "proxy"

MANUAL_NAME = "manual"
PACFILE_NAME = "pacfile"
URL_NAME = "url"
SERVER_LIST_NAME = "serverlist"
SERVER_NAME = "server"
HOST_NAME = "host"
PORT_NAME = "port"

ENDPOINT_URL_FMT = "%s://%s:%s/"

FAILED_RESULT = "Failed"

CHECK_FAILED_MSG = "Check Failed!!"

TEMP_PACFILE = os.path.join(config.TEMP_DIR, "temp.pac")


class FalAmazonS3(falcmd.FalCmd):
    """FAL AmazonS3 command implementation class."""

    def __get_endpoint_url(self, s3_info):

        protocol = s3_info.get_value(HTTPMETHOD_NAME)
        endpoint = s3_info.get_value(ENDPOINT_NAME)
        port = s3_info.get_value(S3_PORT_NAME)

        return ENDPOINT_URL_FMT % (protocol, endpoint, port)

    def __download_handler(self, entry):
        """Handle download request and set entry to result."""

        result_entry = self.send_entry(entry, pal_info.PAL_AMAZONS3_PATH)
        return result_entry

    def __get_proxy(self, s3_info):

        try:
            return get_proxy(self.__get_endpoint_url(s3_info))
        except NoProxy:
            pass

    def __check_handler(self, entry):
        """Handle check request and set entry to result."""

        try:
            s3_info = entry.get()
            proxy_info = self.__get_proxy(s3_info)

            if proxy_info:
                s3_info.append_child(proxy_info)

            result_entry = self.send_entry(
                             Entry(ReqTypes.CHECK, s3_info.get_clone()),
                             pal_info.PAL_AMAZONS3_PATH)

            entry.set_result(result_entry.get_result())
            entry.get().append_child(
                             RESULT_NAME,
                             result_entry.get().get_value(
                                     RESULT_NAME, FAILED_RESULT))

        except:
            self.exception(CHECK_FAILED_MSG)
            entry.set_result(Results.CHECK_ERROR)
            entry.get().append_child(RESULT_NAME, FAILED_RESULT)

        return entry

    def __init__(self, option=None):
        """Constructor."""

        falcmd.FalCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.DOWNLOAD, self.__download_handler)
        self.set_handler(ReqTypes.CHECK, self.__check_handler)

#
# Main
#

if __name__ == "__main__":
    FalAmazonS3(option=falcmd.FalCmdOption()).handle_request()
