#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for Apple Network Setting."""


import sys
import os

import ionas.request as request
import ionas.nascmd as nascmd
import libfal.falcmd as falcmd
import pal_info
from fal_info import (Results, ReqTypes)
import libfal.util as util
import fal_config as config

APPLENETWORK = "applenetwork"
APPLENETWORK_PACKAGE_NAME = "appleshare"


class FalAppleNetwork(falcmd.FalCmd):
    """FAL AppleNetwork command implementation class."""

    @falcmd.conf_change_handler_wrapper
    def __apply_handler(self, entry):

        if not util.is_installed_package(APPLENETWORK_PACKAGE_NAME):
            entry.set_result(Results.OK)
            return entry

        if entry.get().get_node("service").is_enabled():
            util.enable_service(APPLENETWORK, enable=True, do_start_stop=True)
        else:
            util.enable_service(APPLENETWORK, enable=False, do_start_stop=True)

        save_entry = request.Entry(ReqTypes.SAVE, str(entry.get()))
        save_result_entry = self.send_entry_with_file(
                            save_entry,
                            pal_info.PAL_CONFFILE_PATH,
                            config.APPLENETWORK_CONF_FILE)
        if save_result_entry.get_result() != Results.OK:
            return save_result_entry

        entry.set_result(Results.OK)
        return entry

    def __init__(self, option=None):
        """Constructor."""

        falcmd.FalCmd.__init__(self, option=option)

        save_and_load_handler = lambda entry: self.send_entry_with_file(entry,
                pal_info.PAL_CONFFILE_PATH, config.APPLENETWORK_CONF_FILE)

        update_handler = lambda entry: self.send_entry(entry,
                pal_info.PAL_NETATALK_PATH)

        self.set_handler(ReqTypes.SAVE, save_and_load_handler)
        self.set_handler(ReqTypes.LOAD, save_and_load_handler)
        self.set_handler(ReqTypes.APPLY, self.__apply_handler)
        self.set_handler(ReqTypes.UPDATE, update_handler)


#
# Main
#

if __name__ == "__main__":
    FalAppleNetwork(option=falcmd.FalCmdOption()).handle_request()
