#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for setting windows azure blob storage"""

import libfal.falcmd as falcmd
from ionas.request import Entry
from libfal.util import get_proxy, NoProxy
from fal_info import ReqTypes
from fal_info import Results
import pal_info

from azure import BLOB_SERVICE_HOST_BASE

RESULT_NAME = "result"
CONTAINER_NAME = "container"
HTTPMETHOD_NAME = "httpmethod"

ENDPOINT_URL_FMT = "%s://%s%s/"

FAILED_RESULT = "Failed"

CHECK_FAILED_MSG = "Check Failed!!"


class FalAzure(falcmd.FalCmd):
    """FAL Azure command implementation class."""

    def __get_endpoint_url(self, azure_info):

        container = azure_info.get_value(CONTAINER_NAME)
        protocol = azure_info.get_value(HTTPMETHOD_NAME)

        return ENDPOINT_URL_FMT % (protocol, container, BLOB_SERVICE_HOST_BASE)

    def __download_handler(self, entry):
        """Handle download request and set entry to result."""

        result_entry = self.send_entry(entry, pal_info.PAL_AZURE_PATH)
        return result_entry

    def __get_proxy(self, azure_info):

        try:
            return get_proxy(self.__get_endpoint_url(azure_info))
        except NoProxy:
            pass

    def __check_handler(self, entry):
        """Handle check request and set entry to result."""

        try:
            azure_info = entry.get()
            proxy_info = self.__get_proxy(azure_info)

            if proxy_info:
                azure_info.append_child(proxy_info)

            result_entry = self.send_entry(
                             Entry(ReqTypes.CHECK, azure_info.get_clone()),
                             pal_info.PAL_AZURE_PATH)

            entry.set_result(result_entry.get_result())
            entry.get().append_child(
                             RESULT_NAME,
                             result_entry.get().get_value(
                                     RESULT_NAME, FAILED_RESULT))

        except:
            self.exception(CHECK_FAILED_MSG)
            entry.set_result(Results.CHECK_ERROR)
            entry.get().append_child(RESULT_NAME, FAILED_RESULT)

        return entry

    def __init__(self, option=None):
        """Constructor."""

        falcmd.FalCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.DOWNLOAD, self.__download_handler)
        self.set_handler(ReqTypes.CHECK, self.__check_handler)

#
# Main
#

if __name__ == "__main__":
    FalAzure(option=falcmd.FalCmdOption()).handle_request()
