# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL configuration module for LAN DISK.

Note : DO NOT EDIT.  This is automatically generated.
"""

import os
import fal_info
from product_conf import *
from product_conf.devinfo import *
from fal_info import (ReqTypes)

VERSION = "2.19.20"
LOGGER_NAME = "FAL"
LOG_FILE = NASCMD_LOG

FIRMWARE_TEMP_DIR = TEMP_ALT_DIR + "/firmware"
PACKAGES_TEMP_DIR = TEMP_ALT_DIR + "/packages"

# Configuration Files
PROXY_CONF_FILE = "config-proxy.xml"
PACKAGES_CONF_FILE = "config-packages.xml"
INSTALLED_PACKAGES_FILE = "installed-packages.xml"
MESSAGES_XML_FILE = "messages.xml"
VALIDATES_XML_FILE = "validates.xml"
EVENTLOG_CONF_FILE = "config-eventlog.xml"
NOTICE_CONF_FILE = "config-notice.xml"
SHARES_CONF_FILE = "config-shares.xml"
FTP_CONF_FILE = "config-ftp.xml"
USERS_CONF_FILE = "config-users.xml"
GROUPS_CONF_FILE = "config-groups.xml"
FEATURES_CONF_FILE = "config-features.xml"
USED_NW_PORTS_FILE = "used_nw_ports.xml"
NETWORK_CONF_FILE = "config-network.xml"
UPS_CONF_FILE = "config-ups.xml"
POWER_CONF_FILE = "config-power.xml"
BACKUP_CONF_FILE = "config-backup.xml"
SYSINFO_CONF_FILE = "config-sysinfo.xml"
FIRMWARE_CONF_FILE = "config-firmware.xml"
MSNETWORK_CONF_FILE = "config-msnetwork.xml"
APPLENETWORK_CONF_FILE = "config-applenetwork.xml"
REPLICATION_CONF_FILE = "config-replication.xml"
QUOTA_CONF_FILE = "config-quota.xml"
DATETIME_CONF_FILE = "config-datetime.xml"
ECO_CONF_FILE = "config-eco.xml"
PORTS_CONF_FILE = "config-ports.xml"
SELFCHECK_CONF_FILE = "config-selfcheck.xml"
ANNOUNCE_FILE = SYS_LOG_DIR + "/announce.xml"
NARSUS_CONF_FILE = "config-narsus.xml"
SNMP_CONF_FILE = "config-snmp.xml"
SWITCH_CONF_FILE = "config-switch.xml"
VOLCTRL_CONF_FILE = "config-volctrl.xml"
RAIDCTRL_CONF_FILE = "config-raidctrl.xml"
SYSTEMLOG_CONF_FILE = "config-systemlog.xml"
LOCKKEY_CONF_FILE = "config-lockkey.xml"
LED_CONF_FILE = "config-led.xml"
SERVICECTRL_CONF_FILE = "config-servicectrl.xml"

# For getting the service name for function.
FUNC_TO_SERVICE_MAP = {
        "msnetwork": "samba",
        "applenetwork": "netatalk",
        "ftp": "proftpd",
        "narsus": "narsus",
        "dropbox": "nasdsync",
        "amazons3": "nasdsync",
        "firmware": "iosupdate",
        "packages": "iosupdate",
        "intervalalert": "intervalalert",
    }

# For fal_network
CONFIG_NETWORK_SUPPORTED = True

# Device/Volume Information
DEFAULT_VOLUME = "sataraid1"

# FAL Control Information
FAL_SAVE_ONLY = False
FAL_CTRL_SERVICE = True
FAL_DISPATCH_EVENT = True

LOCKKEY_DIR_PREFIX = "dont_delete_"
DECODED_LOCK_KEY_FILE_PATH = TEMP_DIR + "/lkdata"
KEY_SERVER_NO_LIST = ["1", "2", "3"]

# For fal_ports
IS_PORT_MODE_EXCLUSIVE = True

# For fal_raidctrl and fal_volctrl
INTERNAL_VOL = STOR_IF_TO_VOL["internal"][1]
MIRROR_DEV = VOL_INFO[INTERNAL_VOL]["dev"]

INTERNAL_DEV = None
if len(INTERNAL_RAID_DEV_LIST):
    for element in RAID_TO_ELEMENT[len(SATA_HDD_DEV_LIST)][MIRROR_DEV]:
        if element in INTERNAL_RAID_DEV_LIST:
            INTERNAL_DEV = element
            break

if MIRROR_DEV in RAID_TO_REAL_NAME:
    MIRROR_DEV = RAID_TO_REAL_NAME[MIRROR_DEV]

NEED_VFS_AUTO_REBUILD = "need_vfs_autorebuild"
VFS_ELEMENT = "vfs_element"
VFS_ADD_MKFS_ERR = "vfs_add_mkfs_error"
SINGLE_RAID_LEVEL = "single"

# For fal_volctrl
LANDISK_INFO_SECTOR_SIZE = 1  # 隠し領域

DISK_CACHE_FILE_TMPL = CONF_TEMP_DIR + "/disk/%s/%s"
MELODY_CACHE_FILE_TMPL = CONF_TEMP_DIR + "/buzzer/cur_melody"
VOLUME_CACHE_FILE_TMPL = \
    os.path.join(CONF_TEMP_DIR, "volume", "%s", "%s")

RAID_CACHE_FILE_TMPL = \
    os.path.join(CONF_TEMP_DIR, "raid", "%s", "%s")

CUR_ARRAY_NUM_TMPL = CONF_DIR + "/raid/%s/cur_array_num"

### Lock Name ###
EVENTLOG_LOCK_NAME = "eventlog_lock"

### For the priority of melody ###
# 再生しようとするメロディより現在演奏中のメロディの優先度が高い
# 場合、そのメロディは再生しない
MELODY_PRIORYTY_TABLE = \
    {fal_info.BUZZER_RAIDCRASH: 10,
     fal_info.BUZZER_FANERROR: 11,
     fal_info.BUZZER_RAIDFAIL: 11,
     fal_info.BUZZER_UPS: 12,
     fal_info.BUZZER_LOCKMODE: 12,
     fal_info.BUZZER_CAPACITY: 13,
     fal_info.BUZZER_DONGLE: 14,
     fal_info.BUZZER_BOOTUP: 100,
     fal_info.BUZZER_OK: 100,
     fal_info.BUZZER_ERROR: 100,
     fal_info.BUZZER_COPYEND: 100,
     fal_info.BUZZER_RAIDOK: 100,
     fal_info.BUZZER_LUMPREGISTEND: 100,
     fal_info.BUZZER_FORMATEND: 100,
     fal_info.BUZZER_BGPROCESSEND: 100,
     fal_info.BUZZER_MELODY: 10}

# disk label element name
DEFAULT_SWAP_VALUE = "default_swap_size"

# For fal_backups
MAX_JOB_NUM_DICT = {
        "backup": 7,
        "datacopy": 20,
    }


LABEL_MODEL_RE = "^HDL[0-9]*-AA.*"
