#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for setting time"""

import os
import socket

import ionas.request as request
from fal_info import ReqTypes
from fal_info import Results
import ionas.nascmd as nascmd
import libfal.falcmd as falcmd
from iobase.exception import (NotFoundError)

import pal_info
import fal_config as config

NTP_NAME = "ntp"
SERVERADDRESS_PATH = "ntp/serveraddress"
ADDRESSFAMILY_ATTR = "addressfamily"
IPV4_VALUE = "ipv4"
IPV6_VALUE = "ipv6"

APPLY_FAILED_MSG = "Apply failed!!"


class FalDatetime(falcmd.FalCmd):
    """FAL DateTime command implementation class."""

    def __execute_handler(self, entry):
        """Handle execute request and set entry to result."""

        return self.send_entry(entry, pal_info.PAL_DATETIME_PATH)

    def __query_handler(self, entry):
        """Handle query request and set entry to result."""

        return self.send_entry(entry, pal_info.PAL_DATETIME_PATH)

    @falcmd.conf_change_handler_wrapper
    def __apply_handler(self, entry):
        """Handle apply request and set entry to result."""

        apply_result_entry = self.send_entry(entry, pal_info.PAL_DATETIME_PATH)
        if apply_result_entry.get_result() != Results.OK:
            return apply_result_entry

        try:
            entry.get().delete_child("manual")
        except NotFoundError:
            pass
        save_entry = request.Entry(ReqTypes.SAVE, str(entry.get()))
        save_result_entry = self.send_entry_with_file(
                save_entry,
                pal_info.PAL_CONFFILE_PATH,
                config.DATETIME_CONF_FILE)
        if save_result_entry.get_result() != Results.OK:
            return save_result_entry

        return apply_result_entry

    def __init__(self, option=None):
        """Constructor."""

        falcmd.FalCmd.__init__(self, option=option)

        load_handler = lambda entry: self.send_entry_with_file(entry,
                        pal_info.PAL_CONFFILE_PATH,
                        config.DATETIME_CONF_FILE)

        self.set_handler(ReqTypes.LOAD, load_handler)
        self.set_handler(ReqTypes.QUERY, self.__query_handler)
        self.set_handler(ReqTypes.APPLY, self.__apply_handler)
        self.set_handler(ReqTypes.EXECUTE, self.__execute_handler)


#
# Main
#

if __name__ == "__main__":
    FalDatetime(option=falcmd.FalCmdOption()).handle_request()
