#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for setting dropbox"""

import os

import ionas.nascmd as nascmd
import libfal.falcmd as falcmd
from fal_info import ReqTypes
import pal_info
import fal_config as config
from libfal.util import servicectrl_suspend, servicectrl_resume


class FalDropbox(falcmd.FalCmd):
    """FAL Dropbox command implementation class."""

    def _restart_service(self):

        servicectrl_suspend(category_list=[], service_list=["nasdsync"])
        servicectrl_resume(category_list=[], service_list=["nasdsync"])

    def __auth_handler(self, entry):
        """Handle auth request and set entry to result."""

        result_entry = self.send_entry(entry, pal_info.PAL_DROPBOX_PATH)
        self._restart_service()
        return result_entry

    def __clear_handler(self, entry):
        """Handle clear request and set entry to result."""

        result_entry = self.send_entry(entry, pal_info.PAL_DROPBOX_PATH)
        self._restart_service()
        return result_entry

    def __init__(self, option=None):
        """Constructor."""

        falcmd.FalCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.AUTH, self.__auth_handler)
        self.set_handler(ReqTypes.CLEAR, self.__clear_handler)

#
# Main
#

if __name__ == "__main__":
    FalDropbox(option=falcmd.FalCmdOption()).handle_request()
