#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for setting eco"""

import os

import ionas.request as request
import ionas.nascmd as nascmd
import libfal.falcmd as falcmd
from fal_info import ReqTypes
from fal_info import Results
import pal_info
import fal_config as config


class FalEco(falcmd.FalCmd):
    """FAL Eco command implementation class."""

    def __load_eco_conf(self):

        load_entry = request.Entry(ReqTypes.LOAD)
        return self.send_entry_with_file(load_entry,
                                         pal_info.PAL_CONFFILE_PATH,
                                         config.ECO_CONF_FILE).get()

    @falcmd.conf_change_handler_wrapper
    def __apply_handler(self, entry):
        """Handle apply request and set entry to result."""

        apply_result_entry = self.send_entry(entry, pal_info.PAL_ECO_PATH)
        if apply_result_entry.get_result() != Results.OK:
            return apply_result_entry

        save_entry = request.Entry(ReqTypes.SAVE, str(entry.get()))
        save_result_entry = self.send_entry_with_file(
                save_entry,
                pal_info.PAL_CONFFILE_PATH,
                config.ECO_CONF_FILE)
        if save_result_entry.get_result() != Results.OK:
            return save_result_entry

        return apply_result_entry

    def __update_handler(self, entry):

        eco_node = self.__load_eco_conf()

        apply_entry = request.Entry(ReqTypes.APPLY, eco_node)
        result_entry = self.send_entry(apply_entry, pal_info.PAL_ECO_PATH)
        if result_entry.get_result() != Results.OK:
            entry.set_result(Results.APPLY_ERROR)
            return entry

        entry.set_result(Results.OK)
        return entry

    def __init__(self, option=None):
        """Constructor."""

        falcmd.FalCmd.__init__(self, option=option)

        load_handler = lambda entry: self.send_entry_with_file(entry,
                        pal_info.PAL_CONFFILE_PATH,
                        config.ECO_CONF_FILE)

        self.set_handler(ReqTypes.LOAD, load_handler)
        self.set_handler(ReqTypes.APPLY, self.__apply_handler)
        self.set_handler(ReqTypes.UPDATE, self.__update_handler)

#
# Main
#

if __name__ == "__main__":
    FalEco(option=falcmd.FalCmdOption()).handle_request()
