#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for Notice Settings."""


import os

import ionas.request as request
import ionas.nascmd as nascmd
import libfal.falcmd as falcmd
import libfal.util as util
import iobase.xmldata as xmldata
from fal_info import ReqTypes
from fal_info import Results
import pal_info
from iobase.exception import NotFoundError

import fal_config as config

REGULARNOTICE_NAME = "regularnotice"
INTERVAL_ALERT_SRV_NAME = "intervalalert"


class FalNotice(falcmd.FalCmd):
    """FAL Notice command implementation class."""

    def __get_notice_config(self):
        """Load maillist config and return XML data ."""

        return self.send_entry_with_file(request.Entry(ReqTypes.LOAD),
                                            pal_info.PAL_CONFFILE_PATH,
                                            config.NOTICE_CONF_FILE).get()

    def __start_stop_interval_alert(self, entry):

        try:
            regnotice_node = entry.get().get_node(REGULARNOTICE_NAME)
        except NotFoundError:
            return

        if regnotice_node.is_enabled():
            util.enable_service(INTERVAL_ALERT_SRV_NAME,
                                enable=True, do_start_stop=True)
        else:
            util.enable_service(INTERVAL_ALERT_SRV_NAME,
                                enable=False, do_start_stop=True)
        return

#    def __add_mail_handler(self, entry, mail, encode):
#        """Add mail address to mail list ."""
#
#        addmail = entry.get().get_node("mail")
#        mailaddr = addmail.get_text()
#        mailencode = addmail.get_attrs["encode"]
#
#        maillist = notice_config.get_node("notice").get_node("mailset")
#        maillist.append_child("mail", text=mailaddr,
#                                      attrs={"encode": mailencode})

    @falcmd.conf_change_handler_wrapper
    def __apply_handler(self, entry):
        """Handle apply request and set entry to result."""

        try:
            report_node = entry.get().get_node("report")
            pal_result = self.send_entry(entry, pal_info.PAL_NOTICE_PATH)
            if pal_result.get_result() != Results.OK:
                entry.set_result(pal_result.get_result())
                return entry
        except NotFoundError:
            pass

        # 定期通知デーモンの有効/無効
        self.__start_stop_interval_alert(entry)

        notice_config = self.__get_notice_config()
        for item in entry.get().get_node_list(xmldata.ANY_CHILD_NAME):
            try:
                notice_config.delete_child(item.get_node_name())
            except NotFoundError:
                pass
            notice_config.append_child(item)

        save_entry = request.Entry(ReqTypes.SAVE, notice_config)
        self.send_entry_with_file(save_entry, pal_info.PAL_CONFFILE_PATH,
                                              config.NOTICE_CONF_FILE)
        entry.set_result(Results.OK)
        return entry

    def __init__(self, option=None):
        """Constructor."""

        falcmd.FalCmd.__init__(self, option=option)

        save_and_load_handler = lambda entry: self.send_entry_with_file(entry,
                                            pal_info.PAL_CONFFILE_PATH,
                                            config.NOTICE_CONF_FILE)

        self.set_handler(ReqTypes.LOAD, save_and_load_handler)
        self.set_handler(ReqTypes.APPLY, self.__apply_handler)


#
# Main
#

if __name__ == "__main__":
    FalNotice(option=falcmd.FalCmdOption()).handle_request()
