#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for printer buffer clear"""


import os

import ionas.request as request
import ionas.nascmd as nascmd
import libfal.falcmd as falcmd
import pal_info

from fal_info import ReqTypes
import fal_config as config


class FalPrinter(falcmd.FalCmd):
    """FAL Printer command implementation class."""

    def __clear_handler(self, entry):
        """Handle clear request."""

        return self.send_entry(entry, pal_info.PAL_PRINTER_PATH)

    def __enable_handler(self, entry):
        """Handle enable request."""

        return self.send_entry(entry, pal_info.PAL_PRINTER_PATH)

    def __disable_handler(self, entry):
        """Handle disable request."""

        return self.send_entry(entry, pal_info.PAL_PRINTER_PATH)

    def __init__(self, option=None):
        """Constructor."""

        falcmd.FalCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.CLEAR, self.__clear_handler)
        self.set_handler(ReqTypes.ENABLE, self.__enable_handler)
        self.set_handler(ReqTypes.DISABLE, self.__disable_handler)

#
# Main
#

if __name__ == "__main__":
    FalPrinter(option=falcmd.FalCmdOption()).handle_request()
