#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for finding the Proxy Servers."""


import os

import ionas.request as request
import ionas.nascmd as nascmd

import libfal.falcmd as falcmd
import libfal.util as util
import pal_info
from fal_info import (ReqTypes, Results)
from iobase.exception import (InvalidArgError,)

import fal_config as config

TEMP_PACFILE = os.path.join(config.TEMP_DIR, "temp.pac")

PROXY_NAME = "proxy"
URL_NAME = "url"
SERVERLIST_NAME = "serverlist"


class FalProxy(falcmd.FalCmd):
    """FAL Proxy command implementation class."""

    def __check_body_node(self, node):
        """Check if entry's body node is valid."""

        if node.get_node_name() != PROXY_NAME:
            raise InvalidArgError(node.get_node_name())

        if not node.get_node(URL_NAME).get_text():
            raise InvalidArgError("URL empty")

    @falcmd.conf_change_handler_wrapper
    def __apply_handler(self, entry):
        """Handle "apply" request and set entry to result."""

        result_entry = self.send_entry_with_file(
                                request.Entry(ReqTypes.SAVE,
                                                entry.get().get_clone()),
                                pal_info.PAL_CONFFILE_PATH,
                                config.PROXY_CONF_FILE)
        entry.set_result(result_entry.get_result())
        return entry

    def __query_handler(self, entry):
        """Handle query request and set entry to result."""

        self.__check_body_node(entry.get())

        try:
            url = entry.get().get_node(URL_NAME).get_text()
            proxy = util.get_proxy(url)
            entry.get().append_child(proxy.get_node(SERVERLIST_NAME))
            entry.set_result(Results.OK)
        except:
            entry.set_result(Results.NO_PROXY)

        return entry

    def __init__(self, option=None):
        """Constructor."""

        falcmd.FalCmd.__init__(self, option=option)

        save_and_load_handler = lambda entry: self.send_entry_with_file(entry,
                                            pal_info.PAL_CONFFILE_PATH,
                                            config.PROXY_CONF_FILE)

        self.set_handler(ReqTypes.SAVE, save_and_load_handler)
        self.set_handler(ReqTypes.LOAD, save_and_load_handler)
        self.set_handler(ReqTypes.APPLY, self.__apply_handler)
        self.set_handler(ReqTypes.QUERY, self.__query_handler)


#
# Main
#

if __name__ == "__main__":
    FalProxy(option=falcmd.FalCmdOption()).handle_request()
