#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
import os
import logging

from iobase import xmldata
from ionas import request, nascmd
from libfal import falcmd
import fal_config as config
import pal_info
from fal_info import ReqTypes, Results
from iobase.exception import NotFoundError, InvalidArgError, ExecError


class FalQRcode(falcmd.FalCmd):
    def _pal_qrcode_encode(self, string, fpath):
        qrcode_node = xmldata.XmlNode('<qrcode />')
        qrcode_node.append_child('data', text=string)
        entry = request.Entry(ReqTypes.ENCODE, qrcode_node)
        entry.set_file(fpath)
        return self.send_entry(entry, pal_info.PAL_QRCODE_PATH)

    def __encode_handler(self, entry):
        """
        <entry type="encode" file="output_file_path">
            <qrcode>
                <data>encode_string</data>
            </qrcode>
        </entry>
        """
        try:
            qrcode_node = entry.get()

            data = qrcode_node.get_value('data', None)
            if data is None:
                entry.set_result(Results.ENCODE_ERROR)
                raise InvalidArgError('data is empty!!')
            try:
                fpath = entry.get_file()
            except:
                raise InvalidArgError('File path is empty!!')

            response_entry = self._pal_qrcode_encode(data, fpath)
            result = response_entry.get_result()
            if result != Results.OK:
                entry.set_result(result)
                raise ExecError(pal_info.PAL_QRCODE_PATH)

            entry.set_result(Results.OK)
        except:
            self.exception('Encode Error')
            try:
                result = entry.get_result()
            except NotFoundError:
                result = None
            if result is None:
                entry.set_result(Results.ENCODE_ERROR)

        return entry

    def __init__(self, option=None):
        """Constructor."""

        falcmd.FalCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.ENCODE, self.__encode_handler)


if __name__ == "__main__":
    FalQRcode(option=falcmd.FalCmdOption()).handle_request()
