#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for SlefCheck."""


import os

import ionas.request as request
import ionas.nascmd as nascmd
import libfal.falcmd as falcmd
import pal_info
from fal_info import ReqTypes
from fal_info import Results
from iobase.exception import ExecError

import fal_config as config


class FalSelfCheck(falcmd.FalCmd):
    """FAL SelfCheck command implementation class."""

    @falcmd.conf_change_handler_wrapper
    def __selfcheck_handler(self, entry):
        """Handle selfcheck request and set entry to result."""

        try:
            result_entry = self.send_entry(entry, pal_info.PAL_SELFCHECK_PATH)
        except:
            raise ExecError(pal_info.PAL_SELFCHECK_PATH)

        if result_entry.get_result() == Results.OK:
            save_entry = request.Entry(ReqTypes.SAVE, entry.get())
            self.send_entry_with_file(save_entry, pal_info.PAL_CONFFILE_PATH,
                                             config.SELFCHECK_CONF_FILE)
        return result_entry

    def __query_handler(self, entry):
        """Handle query request and set entry to result."""

        result_entry = self.send_entry(entry, pal_info.PAL_SELFCHECK_PATH)
        return result_entry

    def __init__(self, option=None):
        """Constructor."""

        falcmd.FalCmd.__init__(self, option=option)

        save_and_load_handler = lambda entry: self.send_entry_with_file(entry,
                                            pal_info.PAL_CONFFILE_PATH,
                                            config.SELFCHECK_CONF_FILE)

        self.set_handler(ReqTypes.SAVE, save_and_load_handler)
        self.set_handler(ReqTypes.LOAD, save_and_load_handler)
        self.set_handler(ReqTypes.SELFCHECK, self.__selfcheck_handler)
        self.set_handler(ReqTypes.QUERY, self.__query_handler)

#
# Main
#

if __name__ == "__main__":
    FalSelfCheck(option=falcmd.FalCmdOption()).handle_request()
