#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for setting snmp"""

import os

import ionas.request as request
import ionas.nascmd as nascmd
import libfal.falcmd as falcmd
from iobase.xmldata import (XmlNode)

import pal_info
from fal_info import ReqTypes
from fal_info import Results
import fal_config as config

TRAP_NAME = "trap"

MESSAGEINFO_NODE_FMT_STR = "<messageinfo><code>%s</code>" + \
    "<arglist><arg>%s</arg></arglist></messageinfo>"


CHECK_ERRCODE = "12345"
CHECK_ERRARGS = "abcde"


class FalSnmp(falcmd.FalCmd):
    """FAL Snmp command implementation class."""

    def __get_config(self):

        load_result_entry = self.send_entry_with_file(
                request.Entry(ReqTypes.LOAD),
                pal_info.PAL_CONFFILE_PATH,
                config.SNMP_CONF_FILE)

        return load_result_entry.get()

    def __check_handler(self, entry):
        """Handle check request and set entry to result."""

        snmp_node = self.__get_config()

        if not snmp_node.get_node(TRAP_NAME).is_enabled():
            # FIXME 診断しない場合resultcodeどうするか
            entry.set_result(Results.OK)
            return entry

        snmp_node.append_child(XmlNode(
            MESSAGEINFO_NODE_FMT_STR % (CHECK_ERRCODE, CHECK_ERRARGS)))

        check_result_entry = self.send_entry(
            request.Entry(ReqTypes.SEND, snmp_node),
            pal_info.PAL_SNMP_PATH)

        # FIXME 失敗時のログはpalにまかせるのか
        return check_result_entry

    def __query_handler(self, entry):
        """Handle query request and set entry to result."""

        query_result_entry = self.send_entry(
            entry,
            pal_info.PAL_SNMP_PATH)

        # FIXME 失敗時のログはpalにまかせるのか
        return query_result_entry

    @falcmd.conf_change_handler_wrapper
    def __apply_handler(self, entry):
        """Handle apply request and set entry to result."""

        conf = self.send_entry_with_file(
                            request.Entry(ReqTypes.LOAD),
                            pal_info.PAL_CONFFILE_PATH,
                            config.SNMP_CONF_FILE).get()

        conf.update(entry.get())

        res_entry = self.send_entry_with_file(
                            request.Entry(ReqTypes.SAVE, conf),
                            pal_info.PAL_CONFFILE_PATH,
                            config.SNMP_CONF_FILE)

        return res_entry

    def __init__(self, option=None):
        """Constructor."""

        falcmd.FalCmd.__init__(self, option=option)

        save_load_handler = lambda entry: self.send_entry_with_file(entry,
                             pal_info.PAL_CONFFILE_PATH,
                             config.SNMP_CONF_FILE)

        self.set_handler(ReqTypes.CHECK, self.__check_handler)
        self.set_handler(ReqTypes.QUERY, self.__query_handler)
        self.set_handler(ReqTypes.APPLY, self.__apply_handler)
        self.set_handler(ReqTypes.SAVE, save_load_handler)
        self.set_handler(ReqTypes.LOAD, save_load_handler)

#
# Main
#

if __name__ == "__main__":
    FalSnmp(option=falcmd.FalCmdOption()).handle_request()
