#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for handling switch function"""

import os
import ionas.nascmd as nascmd
import libfal.falcmd as falcmd
import fal_config as config
import pal_info
from fal_info import ReqTypes


class FalSwitch(falcmd.FalCmd):
    """FAL Switch command implementation class."""

    @falcmd.conf_change_handler_wrapper
    def __save_handler(self, entry):
        """Handle save request and set entry to result."""

        return self.send_entry_with_file(entry,
                             pal_info.PAL_CONFFILE_PATH,
                             config.SWITCH_CONF_FILE)

    def __init__(self, option=None):
        """Constructor."""

        falcmd.FalCmd.__init__(self, option=option)

        save_load_handler = lambda entry: self.send_entry_with_file(entry,
                             pal_info.PAL_CONFFILE_PATH,
                             config.SWITCH_CONF_FILE)

        self.set_handler(ReqTypes.SAVE, self.__save_handler)
        self.set_handler(ReqTypes.LOAD, save_load_handler)

#
# Main
#

if __name__ == "__main__":
    FalSwitch(option=falcmd.FalCmdOption()).handle_request()
