#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for sending data to or receiving data from the URL."""


import os

import ionas.nascmd as nascmd
from iobase.exception import (InvalidArgError,)

import libfal.falcmd as falcmd
import libfal.util as util
import pal_info
from fal_info import (ReqTypes,)

import fal_config as config


URLACCESS_NAME = "urlaccess"
URL_NAME = "url"
PROXY_NAME = "proxy"


class FalUrlAccess(falcmd.FalCmd):
    """FAL URL Access command implementation class."""

    def __check_body_node(self, node):
        """Check if entry's body node is valid."""

        if node.get_node_name() != URLACCESS_NAME:
            raise InvalidArgError(node.get_node_name())

        if not node.get_node(URL_NAME).get_text():
            raise InvalidArgError("URL empty")

    def __request_handler(self, entry):
        """Handle each request and return the result entry."""

        self.__check_body_node(entry.get())
        request_entry = entry.get_clone()

        try:
            request_entry.get_node(PROXY_NAME)
        except:
            try:
                url = request_entry.get().get_node(URL_NAME).get_text()
                request_entry.get().append_child(util.get_proxy(url))
            except:
                # Ignore error for getting proxy
                pass

        return self.send_entry(request_entry, pal_info.PAL_URLACCESS_PATH,
                                preserve_file=False)

    def __init__(self, option=None):
        """Constructor."""

        falcmd.FalCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.SEND, self.__request_handler)
        self.set_handler(ReqTypes.RECEIVE, self.__request_handler)
        self.set_handler(ReqTypes.GET, self.__request_handler)


#
# Main
#

if __name__ == "__main__":
    FalUrlAccess(option=falcmd.FalCmdOption()).handle_request()
