#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command modules.

Define classes to implement FAL commands.
"""


import os
import sys

import ionas.nascmd as nascmd
import fal_config as config

from libfal import util
from fal_info import Results


class FalCmdOption(nascmd.NasCmdOption):
    """FAL command option class."""

    def __init__(self, argv=sys.argv[1:], version=config.VERSION, ver_fmt=None,
            usage=None, description=None, option_list=[]):
        """Constructor."""

        nascmd.NasCmdOption.__init__(self, argv, version, ver_fmt, usage,
                            description, option_list)


class FalCmd(nascmd.NasCmd):
    """Base class for FAL command."""

    def __init__(self, option=None, logger_name=config.LOGGER_NAME,
            log=os.path.join(config.LOG_DIR, config.LOG_FILE),
            log_level=None):
        """Constructor."""

        nascmd.NasCmd.__init__(self, Results, option, logger_name,
                            log, log_level)


def conf_change_handler_wrapper(func):
    """Decolator for dispatching pre/post events when request handler
       is change the configuration.
    """

    def _dispatch_event(self, entry):
        """Dispatch pre/post events for request handler."""

        try:
            event_body = entry.get().get_clone()
            util.dispatch_event(event_body.get_node_name(),
                            entry.get_type(), event_body,
                            previous=True)

            result_entry = func(self, entry)

            if result_entry.get_result() == Results.OK:
                util.dispatch_event(event_body.get_node_name(),
                            entry.get_type(), event_body,
                            previous=False)
            return result_entry

        except util.EventError:
            entry.set_result(Results.EVENT_ERROR)
            return entry

    return _dispatch_event
