#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta


import re
import time

import iobase.xmldata as xmldata
import ionas.request as request
from fal_info import ReqTypes
from pal_info import (PAL_CONFFILE_PATH, PAL_NETWORK_PATH)
from fal_msginfo import Msg

import fal_config as config


RESULT_ATTR = "result"
NOT_NULL_ATTR = "not_null"
NOT_BEGIN_HYPHEN_ATTR = "not_begin_hyphen"
NOT_NUMBER_ONLY_ATTR = "not_number_only"
IN_LENGTH_RANGE_ATTR = "in_length_range"
IN_COUNT_RANGE_ATTR = "in_count_range"
FORMAT_TABLE_ATTR = "format_table"
IN_DOT_SPLIT_LENGTH_ATTR = "in_dot_split_length"
NOT_BEGIN_QUOTE_ATTR = "not_begin_quote"
NOT_END_QUOTE_ATTR = "not_end_quote"
NOT_END_DOLLER_ATTR = "not_end_doller"
NOT_BEGIN_SHARP_ATTR = "not_begin_sharp"
NOT_BEGIN_SPACE_ATTR = "not_begin_space"
NOT_BEGIN_DOT_ATTR = "not_begin_dot"
NOT_END_SPACE_ATTR = "not_end_space"
NOT_MULTI_SPACE_ATTR = "not_multi_space"
NOT_DUPLICATE_ATTR = "not_duplicate"
NOT_UPLOW_ATTR = "not_uplow"
FQDN_ATTR = "fqdn"
NOT_NETWORK_DUPLICATE_ATTR = "not_network_duplicate"
BEGIN_CHAR_ATTR = "begin_char"
NOT_RESERVED_ATTR = "not_reserved"
IN_ITEM_ATTR = "in_item"
NOT_END_HYPHEN_ATTR = "not_end_hyphen"
IP_ADDRESS_ATTR = "ip_address"
ISS_CODE_ATTR = "iss_code"
MAIL_ADDRESS_ATTR = "mail_address"
NUMERIC_ATTR = "numeric"
NOT_FORBIDDEN_PORT_ATTR = "not_forbidden_port"
NOT_USED_PORT_ATTR = "not_used_port"
IN_RANGE_ATTR = "in_range"
POINT_ATTR = "point"
SERIAL_NUMBER_ATTR = "serial_number"
NETBIOS_DOMAIN_NAME_ATTR = "netbios_domain_name"
DATE_TIME_ATTR = "date_time"
BOOLEAN_ATTR = "boolean"
UNC_SHARE_ATTR = "unc_share"

FAMILY_ATTR = "family"

MIN_OPTION = "min"
MAX_OPTION = "max"
MIN_LENGTH_OPTION = "min_length"
MAX_LENGTH_OPTION = "max_length"
MIN_COUNT_OPTION = "min_count"
MAX_COUNT_OPTION = "max_count"
TABLE_NUM_OPTION = "table_num"
REMOVABLE_OPTION = "removable"
HOURINTERVAL_OPTION = "hourinterval"
MINUTEINTERVAL_OPTION = "minuteinterval"
SECONDINTERVAL_OPTION = "secondinterval"

NETMASK_OPTION = "netmask"
FAMILY_OPTION = "family"
MULTI_OPTION = "multi"
SERVICE_OPTION = "service"
EXEMPT_NULL_OPTION = "exempt_null"
EXEMPT_DUPLICATE_OPTION = "exempt_duplicate"
EXEMPT_UPLOW_OPTION = "exempt_uplow"
EXEMPT_ID_OPTION = "exempt_id"
EXEMPT_NETWORK_DUPLICATE_OPTION = "exempt_network_duplicate"
REMOVABLE_ATTR = "removable"
CATEGORY_ATTR = "category"

USER_ATTR = "user"


BEGIN_HYPHEN_RE = r"^-"
NUMBER_ONLY_RE = r"^\d+$"
FORMAT_TABLE_1_RE = \
        r"^[\x20\x21\x23-\x25\x28-\x3b\x3d\x3f-\x5b\x5d-\x5f\x61-\x7e]+$"
FORMAT_TABLE_2_RE = r"^[a-zA-Z0-9_\-]+$"
FORMAT_TABLE_3_RE = r"^[a-zA-Z0-9_\-.]+$"
FORMAT_TABLE_4_RE = r"^[^\x00-\x1f\"&\'*+,/\x3a-\x3f\[\\\]`|\x7f]+$"
FORMAT_TABLE_5_RE = r"^[a-zA-Z0-9_\-.@,]+$"
# FORMAT_TABLE_5_REについて、
# マニュアルの表ではカンマは許していないので注意
# メールアドレスの禁則でのみ使用しており、
# カンマはis_mail_address()で
# 結果的に弾いている
FORMAT_TABLE_6_RE = r"^[a-zA-Z0-9\-]+$"  # TABLE_2の"_"なし版
BEGIN_QUOTE_RE = r'^"'
END_QUOTE_RE = r'"$'
END_DOLLER_RE = r"\$$"
BEGIN_SHARP_RE = r"^ *#"
BEGIN_SPACE_RE = r"^ "
END_SPACE_RE = r" $"
MULTI_SPACE_RE = r" {2,}"
IP_NUMBER_RE = r"^(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})$"
IP_V6_RE = '\\s*((([0-9A-Fa-f]{1,4}:){7}([0-9A-Fa-f]{1,4}|:))\
|(([0-9A-Fa-f]{1,4}:){6}(:[0-9A-Fa-f]{1,4}|\
((25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]?\\d)\
(.(25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]?\\d)){3})|:))|\
(([0-9A-Fa-f]{1,4}:){5}(((:[0-9A-Fa-f]{1,4}){1,2})|\
:((25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]?\\d)\
(.(25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]?\\d)){3})|:))|\
(([0-9A-Fa-f]{1,4}:){4}(((:[0-9A-Fa-f]{1,4}){1,3})|\
((:[0-9A-Fa-f]{1,4})?:((25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]?\\d)\
(.(25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]?\\d)){3}))|:))|\
(([0-9A-Fa-f]{1,4}:){3}(((:[0-9A-Fa-f]{1,4}){1,4})|\
((:[0-9A-Fa-f]{1,4}){0,2}:((25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]?\\d)\
(\\.(25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]?\\d)){3}))|:))|\
(([0-9A-Fa-f]{1,4}:){2}(((:[0-9A-Fa-f]{1,4}){1,5})|\
((:[0-9A-Fa-f]{1,4}){0,3}:((25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]?\\d)\
(\\.(25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]?\\d)){3}))|:))|\
(([0-9A-Fa-f]{1,4}:){1}(((:[0-9A-Fa-f]{1,4}){1,6})|\
((:[0-9A-Fa-f]{1,4}){0,4}:((25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]?\\d)\
(\\.(25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]?\\d)){3}))|:))|\
(:(((:[0-9A-Fa-f]{1,4}){1,7})|((:[0-9A-Fa-f]{1,4}){0,5}:\
((25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]?\\d)\
(\\.(25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]?\\d)){3}))|:)))(%.+)?\\s*$'
IP_V6_COLON_LESS_THAN_7_RE = ":[0-9a-fA-F]*:[0-9a-fA-F]*:\
[0-9a-fA-F]*:[0-9a-fA-F]*:[0-9a-fA-F]*:[0-9a-fA-F]*:[0-9a-fA-F]*:"
IP_V6_MULTI_DOUBLE_COLON_RE = r"^.*::.*::.*$"
IP_V6_BEGIN_FF_RE = r"^ff.*$"
IP_V6_BEGIN_DOUBLE_COLON_AFTER_1_RE = r"^::1$"
IP_V6_BEGIN_FE_C_TO_F_RE = r"^fe[c-f].*$"
IP_V6_ALL_ZERO_RE = r"^0*:0*:0*:0*:0*:0*:0*:0*$"
BEGIN_CHAR_RE = r"^[0-9a-zA-Z]"
END_HYPHEN_RE = r"-$"
ISS_CODE_RE = r"^([0-9A-Za-z]{6}\-){2}[0-9A-Za-z]{6}$"
COMMA_RE = r","
MAIL_ADDRESS_RE = r"^[^,\-.@_][\w\-.]*@[\w\-.]+\.[0-9a-zA-Z]+$"
BEGIN_ZERO_NUMERIC_RE = r"^[0]\d+$"
# fixme POINT系は小数点以下第一位までしか許さない？
POINT_RE = r"^\d+(\.\d)?$"
BEGIN_ZERO_POINT_RE = r"^[0]\d+(\.\d)?$"
# fixme SERIAL_NUMBERの禁則の"?"は必要なのか？
SERIAL_NUMBER_RE = r"^[0-9A-Z]{12}$"
BEGIN_DOT_RE = r"^\."
DOUBLE_DOT_RE = r"\.{2}"
UNC_SHARE_RE = r"""^\\\\[^*?"'<>|\r\n\\]+\\[^/:*?"'<>|\r\n\\]+$"""
IP_V4_RE = r"^(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})$"


NAME_NAME = "name"
OPTION_NAME = "option"
DECODE_NAME = "utf-8"
NUMBER_NAME = "number"
SHARE_LIST_CHILD_PATH = "shares/sharelist/*"
SHARELIST_CHILD_PATH = "sharelist/*"
GROUP_LIST_CHILD_PATH = "groups/grouplist/*"
GROUPLIST_CHILD_PATH = "grouplist/*"
USER_LIST_CHILD_PATH = "users/userlist/*"
USERLIST_CHILD_PATH = "userlist/*"
RESERVE_LIST_CHILD_PATH = "validates/reservelist/*"
PORT_LIST_CHILD_PATH = "ports/portlist/*"
HOSTNAME_PATH = "./hostdesc/name"
ADDR_LIST_PATH = "netiflist/netif/protocollist/protocol/addrlist/addr"
USED_PORT_PATH = "nwports/used/portlist/port[@service!='%s']"
FORBIDDEN_PORT_PATH = "nwports/forbidden/portlist/port[@service!='%s']"
DEFAULT_PORT_PATH = "nwports/forbidden/portlist/port[@service='%s']"

WELL_KNOWN_PORT_LIST = [str(port) for port in range(1, 1025)]

RESULT_OK = "0"
RESULT_NG = "1"
RESULT_NODE_TEXT = "<result>" + RESULT_OK + "</result>"

BROADCAST_ADDR = "255.255.255.255"
LOOPBACK_ADDR = "127.0.0.1"

# fixme 禁止ポート、除外ポートはどこで管理する？
FORBIDDEN_PORT_LIST = range(0, 1024) + [ \
                1024, \
                1900, \
                2049, \
                3260, \
                3689, \
                4700, \
                5353, \
                30000, \
                30001, \
                51055, \
                51065]
EXCEPT_PORT_LIST = [20, \
                21, \
                25, \
                161, \
                162, \
                587]

FTP_SERVICE = "ftp"
RA_SERVICE = "remoteaccess"

MESSAGEINFO_FMT_STR = "<messageinfo><code/><arglist/></messageinfo>"
ARG_FMT_STR = "<arg name='%s'>%s</arg>"

CODE_NAME = "code"
ARGLIST_NAME = "arglist"
ARG_DICT_KEY = "arg_dict"
CODE_KEY = "code"
MIN_ARG_KEY = "min"
MAX_ARG_KEY = "max"

# 形式は正しいが使用してはならないアドレス
INVALID_ADDR = "invalid_addr"

FETURES_NETIF_PORT2_XPATH = 'deviceinfo/netiflist/netif[@port="2"]'


class ValidateError(Exception):
    """Validate error."""

    def __init__(self, attr):
        self.attr = attr


class BaseValidator():
    """BaseValidator class."""

    def _is_lan_single(self):
        """Return if there is only a LAN port."""

        response = self.send_entry_with_file(
                request.Entry(ReqTypes.LOAD),
                PAL_CONFFILE_PATH,
                config.FEATURES_CONF_FILE)

        try:
            response.get().get_node(FETURES_NETIF_PORT2_XPATH)
            return False
        except:
            return True

    def __init__(self, target_node,
            validate_config_node=None):
        """Constructor."""

        self.validate_config_option_dict = {
            MIN_OPTION: None,
            MAX_OPTION: None,
            MIN_LENGTH_OPTION: None,
            MAX_LENGTH_OPTION: None,
            MIN_COUNT_OPTION: None,
            MAX_COUNT_OPTION: None,
            TABLE_NUM_OPTION: None,
            REMOVABLE_OPTION: None,
            HOURINTERVAL_OPTION: None,
            MINUTEINTERVAL_OPTION: None,
            SECONDINTERVAL_OPTION: None,
            NETMASK_OPTION: None,
            FAMILY_OPTION: None,
            MULTI_OPTION: None,
            SERVICE_OPTION: None,
            EXEMPT_NULL_OPTION: None,
            EXEMPT_DUPLICATE_OPTION: None,
            EXEMPT_UPLOW_OPTION: None,
            EXEMPT_ID_OPTION: None,
            EXEMPT_NETWORK_DUPLICATE_OPTION: None}

        self.target_node = target_node
        self.target = self.target_node.get_text(do_strip=False)

        result_node = xmldata.XmlConfNode(RESULT_NODE_TEXT)
        self.result_node = result_node

        if validate_config_node:
            self.validate_config_option_dict.update(
                    validate_config_node.get_node(OPTION_NAME).get_attrs())
            self.validate_config_option_dict.update(
                    self.target_node.get_attrs())

        self.msginfo_dict = {
            NOT_NULL_ATTR: {CODE_KEY: Msg.ERR_VALIDATE_NOT_NULL, ARG_DICT_KEY: {}},
            NOT_BEGIN_HYPHEN_ATTR: {CODE_KEY: Msg.ERR_VALIDATE_NOT_BEGIN_HYPHEN, ARG_DICT_KEY: {}},
            NOT_NUMBER_ONLY_ATTR: {CODE_KEY: Msg.ERR_VALIDATE_NOT_NUMBER_ONLY, ARG_DICT_KEY: {}},
            IN_LENGTH_RANGE_ATTR: {
                CODE_KEY: Msg.ERR_VALIDATE_IN_LENGTH_RANGE,
                ARG_DICT_KEY: {
                    MIN_ARG_KEY: self.validate_config_option_dict[MIN_LENGTH_OPTION],
                    MAX_ARG_KEY: self.validate_config_option_dict[MAX_LENGTH_OPTION]}},
            IN_COUNT_RANGE_ATTR: {
                CODE_KEY: Msg.ERR_VALIDATE_IN_COUNT_RANGE,
                ARG_DICT_KEY: {
                    MIN_ARG_KEY: self.validate_config_option_dict[MIN_COUNT_OPTION],
                    MAX_ARG_KEY: self.validate_config_option_dict[MAX_COUNT_OPTION]}},
            IN_RANGE_ATTR: {
                CODE_KEY: Msg.ERR_VALIDATE_IN_RANGE,
                ARG_DICT_KEY: {
                    MIN_ARG_KEY: self.validate_config_option_dict[MIN_OPTION],
                    MAX_ARG_KEY: self.validate_config_option_dict[MAX_OPTION]}},
            FORMAT_TABLE_ATTR: {CODE_KEY: Msg.ERR_VALIDATE_FORMAT, ARG_DICT_KEY: {}},
            IN_DOT_SPLIT_LENGTH_ATTR: {CODE_KEY: Msg.ERR_VALIDATE_FORMAT, ARG_DICT_KEY: {}},
            NOT_BEGIN_QUOTE_ATTR: {CODE_KEY: Msg.ERR_VALIDATE_NOT_BEGIN_QUOTE, ARG_DICT_KEY: {}},
            NOT_END_QUOTE_ATTR: {CODE_KEY: Msg.ERR_VALIDATE_NOT_END_QUOTE, ARG_DICT_KEY: {}},
            NOT_END_DOLLER_ATTR: {CODE_KEY: Msg.ERR_VALIDATE_NOT_END_DOLLER, ARG_DICT_KEY: {}},
            NOT_BEGIN_SHARP_ATTR: {CODE_KEY: Msg.ERR_VALIDATE_NOT_BEGIN_SHARP, ARG_DICT_KEY: {}},
            NOT_BEGIN_SPACE_ATTR: {CODE_KEY: Msg.ERR_VALIDATE_NOT_BEGIN_SPACE, ARG_DICT_KEY: {}},
            NOT_BEGIN_DOT_ATTR: {CODE_KEY: Msg.ERR_VALIDATE_NOT_BEGIN_DOT, ARG_DICT_KEY: {}},
            NOT_END_SPACE_ATTR: {CODE_KEY: Msg.ERR_VALIDATE_NOT_END_SPACE, ARG_DICT_KEY: {}},
            NOT_MULTI_SPACE_ATTR: {CODE_KEY: Msg.ERR_VALIDATE_NOT_MULTI_SPACE, ARG_DICT_KEY: {}},
            NOT_DUPLICATE_ATTR: {CODE_KEY: Msg.ERR_VALIDATE_NOT_DUPLICATE, ARG_DICT_KEY: {}},
            NOT_UPLOW_ATTR: {CODE_KEY: Msg.ERR_VALIDATE_NOT_UPLOW, ARG_DICT_KEY: {}},
            FQDN_ATTR: {CODE_KEY: Msg.ERR_VALIDATE_FORMAT, ARG_DICT_KEY: {}},
            NOT_NETWORK_DUPLICATE_ATTR: {CODE_KEY: Msg.ERR_VALIDATE_HOST_LAN1_LAN2_DUPLICATE, ARG_DICT_KEY: {}},
            BEGIN_CHAR_ATTR: {CODE_KEY: Msg.ERR_VALIDATE_BEGIN_CHAR, ARG_DICT_KEY: {}},
            NOT_RESERVED_ATTR: {CODE_KEY: Msg.ERR_VALIDATE_USABLE, ARG_DICT_KEY: {}},
            IN_ITEM_ATTR: {CODE_KEY: Msg.ERR_VALIDATE_IN_ITEM, ARG_DICT_KEY: {}},
            NOT_END_HYPHEN_ATTR: {CODE_KEY: Msg.ERR_VALIDATE_NOT_END_HYPHEN, ARG_DICT_KEY: {}},
            IP_ADDRESS_ATTR: {CODE_KEY: Msg.ERR_VALIDATE_FORMAT, ARG_DICT_KEY: {}},
            ISS_CODE_ATTR: {CODE_KEY: Msg.ERR_VALIDATE_FORMAT, ARG_DICT_KEY: {}},
            MAIL_ADDRESS_ATTR: {CODE_KEY: Msg.ERR_VALIDATE_FORMAT, ARG_DICT_KEY: {}},
            NUMERIC_ATTR: {CODE_KEY: Msg.ERR_VALIDATE_NUMERIC, ARG_DICT_KEY: {}},
            NOT_FORBIDDEN_PORT_ATTR: {CODE_KEY: Msg.ERR_VALIDATE_USABLE, ARG_DICT_KEY: {}},
            NOT_USED_PORT_ATTR: {CODE_KEY: Msg.ERR_VALIDATE_NOT_DUPLICATE, ARG_DICT_KEY: {}},
            POINT_ATTR: {CODE_KEY: Msg.ERR_VALIDATE_FORMAT, ARG_DICT_KEY: {}},
            SERIAL_NUMBER_ATTR: {CODE_KEY: Msg.ERR_VALIDATE_FORMAT, ARG_DICT_KEY: {}},
            NETBIOS_DOMAIN_NAME_ATTR: {CODE_KEY: Msg.ERR_VALIDATE_FORMAT, ARG_DICT_KEY: {}},
            DATE_TIME_ATTR: {CODE_KEY: Msg.ERR_VALIDATE_FORMAT, ARG_DICT_KEY: {}},
            BOOLEAN_ATTR: {CODE_KEY: Msg.ERR_VALIDATE_FORMAT, ARG_DICT_KEY: {}},
            UNC_SHARE_ATTR: {CODE_KEY: Msg.ERR_VALIDATE_FORMAT, ARG_DICT_KEY: {}}}

        if self._is_lan_single():
            self.msginfo_dict[NOT_NETWORK_DUPLICATE_ATTR][CODE_KEY] = \
                Msg.ERR_VALIDATE_HOST_LAN1_DUPLICATE

        self.VALIDATE_LIST = []

    def send_entry(self, entry, target):

        return request.Request(target).request(entry).get_entry()
        
    def send_entry_with_file(self, entry, target, file_name):

        entry.set_file(file_name)
        return self.send_entry(entry, target)

    def set_err_result(self, attr_name):
        """Set error result."""

        self.result_node.set_bool_attr(attr_name, true=False)
        self.result_node.set_text(RESULT_NG)

    def __is_ip_number(self):
        """Ip number check internal method."""

        if not re.search(IP_NUMBER_RE, self.target):
            return False

        sec = self.target.split(".")

        if (not((int(sec[0]) >= 0 and int(sec[0]) <= 255) and
            (int(sec[1]) >= 0 and int(sec[1]) <= 255) and
            (int(sec[2]) >= 0 and int(sec[2]) <= 255) and
            (int(sec[3]) >= 1 and int(sec[3]) <= 255))):
                return False

        return True

    def __is_not_nw_address(self, netmask):
        """Not network address check internal method."""

        if not re.search(IP_NUMBER_RE, self.target):
            return False

        sec_list = self.target.split(".")
        mask_list = netmask.split(".")

        not_mask_list = []
        for idx in range(0, 4):
            not_mask_list.append(str(2 ** 8 + ~int(mask_list[idx])))

        nwaddr_list = []
        for idx in range(0, 4):
            nwaddr_list.append(str(int(sec_list[idx]) & int(mask_list[idx])))

        bcaddr_list = []
        for idx in range(0, 4):
            bcaddr_list.append(
                    str(int(sec_list[idx]) | int(not_mask_list[idx])))

        if (self.target == ".".join(nwaddr_list) or
            self.target == ".".join(bcaddr_list)):
            return False

        return True

    def __is_not_mc_address(self):
        """Not multicast address check internal method."""

        if not re.search(IP_NUMBER_RE, self.target):
            return False

        sec_list = self.target.split(".")
        if int(sec_list[0]) in range(224, 240):
            return False

        return True

    def __is_ip_v4_address(self):
        """Ip v4 address check internal method."""

        netmask = self.validate_config_option_dict[NETMASK_OPTION]
        if not netmask:
            netmask = ""
        if not self.__is_ip_number():
            return False
        if netmask != "":
            if not self.__is_not_nw_address(netmask):
                return False
        if self.target == BROADCAST_ADDR:
            return False
        if self.target == LOOPBACK_ADDR:
            return False
        if not self.__is_not_mc_address():
            return False
        return True

    def __is_ip_v6_address(self):
        """Ip v6 address check internal method."""

        low_target = self.target.lower()

        if not re.search(IP_V6_RE, low_target):
            return False
        elif re.search(IP_V6_COLON_LESS_THAN_7_RE, low_target):
            return False
        elif re.search(IP_V6_MULTI_DOUBLE_COLON_RE, low_target):
            return False
        elif re.search(IP_V6_BEGIN_FF_RE, low_target):
            return False
        elif re.search(IP_V6_BEGIN_DOUBLE_COLON_AFTER_1_RE, low_target):
            return False
        elif re.search(IP_V6_BEGIN_FE_C_TO_F_RE, low_target):
            return False
        elif re.search(IP_V6_ALL_ZERO_RE, low_target):
            return False
        return True

    def __is_fqdn(self):
        """Fqdn check internal method."""

        # fixme XRに"暫定対策"とのコメントあり
        if not self.target:
            return True
        # fixme format_tableはreturn値考慮しておらず、
        # かつxmlにresult入るがどうする
        try:
            if self.is_begin_char() and self.is_format_table():
                return True
        except ValidateError:
            return False

    def __date_time_range_check(
        self, year, month, day, hour, minute, second):

        if (not re.search(r"^\d{4}$", year) or
                int(year) < 2013 or
                int(year) > 2037):
            raise ValidateError(DATE_TIME_ATTR)
        if (not re.search(r"^\d{1,2}$", month) or
                int(month) < 1 or
                int(month) > 12):
            raise ValidateError(DATE_TIME_ATTR)
        if (not re.search(r"^\d{1,2}$", day) or
                int(day) < 1 or
                int(day) > 31):
            raise ValidateError(DATE_TIME_ATTR)
        if (not re.search(r"^\d{1,2}$", hour) or
                int(hour) < 0 or
                int(hour) > 23):
            raise ValidateError(DATE_TIME_ATTR)
        if (not re.search(r"^\d{1,2}$", minute) or
                int(minute) < 0 or
                int(minute) > 59):
            raise ValidateError(DATE_TIME_ATTR)
        if (not re.search(r"^\d{1,2}$", minute) or
                int(minute) < 0 or
                int(minute) > 59):
            raise ValidateError(DATE_TIME_ATTR)

    def __date_time_interval_check(
        self, hour, minute, second):

        if self.validate_config_option_dict[HOURINTERVAL_OPTION]:
            hourinterval = int(
                self.validate_config_option_dict[HOURINTERVAL_OPTION])
            if hour % hourinterval:
                raise ValidateError(DATE_TIME_ATTR)

        if self.validate_config_option_dict[MINUTEINTERVAL_OPTION]:
            minuteinterval = int(
                self.validate_config_option_dict[MINUTEINTERVAL_OPTION])
            if minute % minuteinterval:
                raise ValidateError(DATE_TIME_ATTR)

        if self.validate_config_option_dict[SECONDINTERVAL_OPTION]:
            secondinterval = int(
                self.validate_config_option_dict[SECONDINTERVAL_OPTION])
            if second % secondinterval:
                raise ValidateError(DATE_TIME_ATTR)

    def __date_time_integrity_check(
        self, year, month, day, hour, minute, second):

        date_time = "%s/%s/%s %s:%s:%s" % (
            year, month, day, hour, minute, second)

        try:
            time.strptime(
                date_time, "%Y/%m/%d %H:%M:%S")
        except:
            raise ValidateError(DATE_TIME_ATTR)

    def get_user_list(self):
        """Get user list."""

        entry = self.send_entry_with_file(
                request.Entry(ReqTypes.LOAD),
                PAL_CONFFILE_PATH,
                config.USERS_CONF_FILE)
        return [
            node.get_node(NAME_NAME).get_text()
            for node
            in entry.get().get_node_list(USERLIST_CHILD_PATH)]

    def get_group_list(self):
        """Get group list."""

        entry = self.send_entry_with_file(
                request.Entry(ReqTypes.LOAD),
                PAL_CONFFILE_PATH,
                config.GROUPS_CONF_FILE)
        return [
            node.get_node(NAME_NAME).get_text()
            for node
            in entry.get().get_node_list(GROUPLIST_CHILD_PATH)]

    def get_share_list(self, removable=None):
        """Get share list."""

        entry = self.send_entry_with_file(
                request.Entry(ReqTypes.LOAD),
                PAL_CONFFILE_PATH,
                config.SHARES_CONF_FILE)

        share_list = []
        for node in entry.get().get_node_list(SHARELIST_CHILD_PATH):
            is_removable = node.is_set_to_true(REMOVABLE_ATTR)
            if is_removable and removable:
                share_list.append(node.get_node(NAME_NAME).get_text())
            elif (not is_removable and removable == False):
                share_list.append(node.get_node(NAME_NAME).get_text())
            else:
                share_list.append(node.get_node(NAME_NAME).get_text())

        return share_list
 
    def re_validator(self, reg, attr, match_flag=False):
        """Regular expression check base method."""
        if self.target:
            if not match_flag:
                if re.search(reg, self.target):
                    raise ValidateError(attr)
            else:
                if not re.search(reg, self.target):
                    raise ValidateError(attr)
        return attr

    def is_not_null(self):
        """Not null check."""

        if self.validate_config_option_dict[EXEMPT_NULL_OPTION] == "true":
            return NOT_NULL_ATTR

        if not self.target:
            raise ValidateError(NOT_NULL_ATTR)
        return NOT_NULL_ATTR

    def is_in_length_range(self):
        """In length range check."""

        if self.target:
            if (not int(self.validate_config_option_dict[MIN_LENGTH_OPTION]) <=
                    len(self.target) <=
                    int(self.validate_config_option_dict[MAX_LENGTH_OPTION])):
                raise ValidateError(IN_LENGTH_RANGE_ATTR)
        return IN_LENGTH_RANGE_ATTR

    def is_in_count_range(self):
        """In count range check."""
        if self.target:
            if (not int(self.validate_config_option_dict[MIN_COUNT_OPTION]) <=
                len(self.target.decode(DECODE_NAME)) <=
                int(self.validate_config_option_dict[MAX_COUNT_OPTION])):
                raise ValidateError(IN_COUNT_RANGE_ATTR)
        return IN_COUNT_RANGE_ATTR

    def is_format_table(self):
        """Format table check."""
        if self.target:
            format_table = {
                "1": FORMAT_TABLE_1_RE,
                "2": FORMAT_TABLE_2_RE,
                "3": FORMAT_TABLE_3_RE,
                "4": FORMAT_TABLE_4_RE,
                "5": FORMAT_TABLE_5_RE,
                "6": FORMAT_TABLE_6_RE}
            if not re.search(
                    format_table[
                        self.validate_config_option_dict[TABLE_NUM_OPTION]],
                    self.target):
                raise ValidateError(FORMAT_TABLE_ATTR)
        return FORMAT_TABLE_ATTR

    def is_in_dot_split_length(self):
        """In dot split length check."""
        if self.target:
            for dot_split in self.target.split('.'):
                if len(dot_split) > 63:
                    raise ValidateError(IN_DOT_SPLIT_LENGTH_ATTR)
        return IN_DOT_SPLIT_LENGTH_ATTR

    def is_not_duplicate(self, conffile, path):
        """Not duplicate check."""
        if self.validate_config_option_dict[EXEMPT_DUPLICATE_OPTION] == "true":
            return NOT_DUPLICATE_ATTR

        if self.target:
            entry = self.send_entry_with_file(
                    request.Entry(ReqTypes.LOAD),
                    PAL_CONFFILE_PATH,
                    conffile)
            if self.target in entry.get().get_value_list(path):
                raise ValidateError(NOT_DUPLICATE_ATTR)
        return NOT_DUPLICATE_ATTR

    def is_not_uplow(self, conffile, path):
        """Not uplow check."""
        if self.validate_config_option_dict[EXEMPT_UPLOW_OPTION] == "true":
            return NOT_UPLOW_ATTR

        if self.target:
            entry = self.send_entry_with_file(
                request.Entry(ReqTypes.LOAD),
                PAL_CONFFILE_PATH,
                conffile)
            for value in entry.get().get_value_list(path):
                if (self.target != value and
                    self.target.lower() == value.lower()):
                    raise ValidateError(NOT_UPLOW_ATTR)
        return NOT_UPLOW_ATTR

    def is_not_reserved(self, conffile, path, uplow_check=True):
        if self.target:
            entry = self.send_entry_with_file(
                    request.Entry(ReqTypes.LOAD),
                    PAL_CONFFILE_PATH,
                    conffile)
            value_list = entry.get().get_value_list(path)
            if self.target in value_list:
                raise ValidateError(NOT_RESERVED_ATTR)
            if uplow_check:
                if self.target.lower() in [
                    value.lower()
                    for value
                    in value_list]:
                    raise ValidateError(NOT_RESERVED_ATTR)
        return NOT_RESERVED_ATTR
 
    # fixme 重複系禁則 共通化できないか
    def is_not_network_duplicate(self):
        """Not network duplicate check."""
        if self.validate_config_option_dict[EXEMPT_NETWORK_DUPLICATE_OPTION] == "true":
            return NOT_NETWORK_DUPLICATE_ATTR

        if self.target:
            query_entry = request.Entry(ReqTypes.QUERY, xmldata.XmlConfNode("<network/>"))
            load_entry = request.Entry(ReqTypes.LOAD)

            nw_stat = self.send_entry(
                    query_entry,
                    PAL_NETWORK_PATH).get()
            sysinfo_conf = self.send_entry_with_file(
                    load_entry,
                    PAL_CONFFILE_PATH,
                    config.SYSINFO_CONF_FILE).get()

            hostname = sysinfo_conf.get_node(HOSTNAME_PATH).get_text().lower()
            addr_list = [
                node.get_text().lower()
                for node
                in nw_stat.get_node_list(ADDR_LIST_PATH)]
                    
            if (self.target.lower() == hostname or
                self.target.lower() in addr_list):
                raise ValidateError(NOT_NETWORK_DUPLICATE_ATTR)

        return NOT_NETWORK_DUPLICATE_ATTR

    def is_fqdn(self):
        """Fqdn check."""

        if self.target:

            family = self.validate_config_option_dict[FAMILY_OPTION]
            if not family:
                family = "10"

            if (family == "4" or family == "10"):
                if re.search(IP_V4_RE, self.target):
                    if self.__is_ip_v4_address():
                        return FQDN_ATTR
                    else:
                        raise ValidateError(FQDN_ATTR)

            if (family == "6" or family == "10"):
                if re.search(IP_V6_RE, self.target):
                    if self.__is_ip_v6_address():
                        return FQDN_ATTR
                    else:
                        raise ValidateError(FQDN_ATTR)

            if not self.__is_fqdn():
                raise ValidateError(FQDN_ATTR)

        return FQDN_ATTR
       
    def is_in_item(self, items):
        """In item check."""

        if self.target:
            if self.target in items:
                return IN_ITEM_ATTR
        raise ValidateError(IN_ITEM_ATTR)

    def is_ip_address(self):
        """IpAddress check."""

        if self.target:
            family = self.validate_config_option_dict[FAMILY_ATTR]
            if not family:
                family = "10"

            if family == "4" or family == "10":
                if self.__is_ip_v4_address() == True:
                    return IP_ADDRESS_ATTR
            if family == "6" or family == "10":
                if self.__is_ip_v6_address() == True:
                    return IP_ADDRESS_ATTR
        raise ValidateError(IP_ADDRESS_ATTR)

    def is_mail_address(self):
        """Mail address check."""

        if self.target:
            multi_flag = self.validate_config_option_dict[MULTI_OPTION]
            if not multi_flag:
                multi_flag = "false"

            if multi_flag == "false":
                if self.target != "" and re.search(COMMA_RE, self.target):
                    raise ValidateError(MAIL_ADDRESS_ATTR)
            mail_addr_list = self.target.split(",")
            for mail_addr in mail_addr_list:
                if (mail_addr != "" and 
                    not re.search(MAIL_ADDRESS_RE, mail_addr)):
                    raise ValidateError(MAIL_ADDRESS_ATTR)
        return MAIL_ADDRESS_ATTR

    def is_numeric(self):
        """Numeric check."""

        if self.target:
            if not re.search(NUMBER_ONLY_RE, self.target):
                raise ValidateError(NUMERIC_ATTR)
            if re.search(BEGIN_ZERO_NUMERIC_RE, self.target):
                raise ValidateError(NUMERIC_ATTR)
        return NUMERIC_ATTR

    def is_not_forbidden_port(self):
        """Not forbidden port check."""

        if self.target:
            if not re.search(NUMBER_ONLY_RE, self.target):
                return NOT_FORBIDDEN_PORT_ATTR

            service = self.validate_config_option_dict[SERVICE_OPTION]
            if not service:
                service = ""

            load_entry = request.Entry(ReqTypes.LOAD)
            load_response = self.send_entry_with_file(load_entry,
                                PAL_CONFFILE_PATH,
                                config.VALIDATES_XML_FILE)

            forbidden_port_list = load_response.get().\
                    get_value_list(FORBIDDEN_PORT_PATH % service)
            default_port = load_response.get().\
                    get_value(DEFAULT_PORT_PATH % service)
           
            forbidden_port_list += WELL_KNOWN_PORT_LIST

            # 重複削除
            forbidden_port_list = list(set(forbidden_port_list))

            try:
                forbidden_port_list.remove(default_port)
            except ValueError:
                pass
            
            if self.target in forbidden_port_list:
                raise ValidateError(NOT_FORBIDDEN_PORT_ATTR)

        return NOT_FORBIDDEN_PORT_ATTR

    def is_used_port(self):
        """Used port check."""

        if self.target:
            service = self.validate_config_option_dict[SERVICE_OPTION]
            if not service:
                service = ""

            load_entry = request.Entry(ReqTypes.LOAD)
            load_response = self.send_entry_with_file(load_entry,
                                PAL_CONFFILE_PATH,
                                config.VALIDATES_XML_FILE)

            used_port_list = load_response.get().\
                    get_value_list(USED_PORT_PATH % service)

            if self.target in used_port_list:
                raise ValidateError(NOT_USED_PORT_ATTR)

        return NOT_USED_PORT_ATTR

    def is_in_range(self):
        """In range check."""

        if self.target:
            try:
                float(self.target)
            except:
                raise ValidateError(IN_RANGE_ATTR)
                
            if not (float(self.validate_config_option_dict[MIN_OPTION]) <=
                float(self.target) <=
                float(self.validate_config_option_dict[MAX_OPTION])):
                raise ValidateError(IN_RANGE_ATTR)
        return IN_RANGE_ATTR

    def is_point(self):
        """Point check."""

        if self.target:
            if not re.search(POINT_RE, self.target):
                raise ValidateError(POINT_ATTR)
            if re.search(BEGIN_ZERO_POINT_RE, self.target):
                raise ValidateError(POINT_ATTR)
        return POINT_ATTR

    # fixme メソッド名微妙
    def is_netbios_domain_name(self):
        """netbios domain name check."""

        if self.target:
            if re.search(BEGIN_DOT_RE, self.target):
                raise ValidateError(NETBIOS_DOMAIN_NAME_ATTR)
            if re.search(DOUBLE_DOT_RE, self.target):
                raise ValidateError(NETBIOS_DOMAIN_NAME_ATTR)
        return NETBIOS_DOMAIN_NAME_ATTR


    def is_date_time(self):
        """Date time check."""

        if self.target:
            year = "2013"
            month = "1"
            day = "1"
            hour = "0"
            minute = "0"
            second = "0"

            item_list = self.target.split(",", 6)
            for item in item_list:
                if item.startswith("y:"):
                    year = item.split(":", 2)[1]
                if item.startswith("m:"):
                    month = item.split(":", 2)[1]
                if item.startswith("d:"):
                    day = item.split(":", 2)[1]
                if item.startswith("H:"):
                    hour = item.split(":", 2)[1]
                if item.startswith("M:"):
                    minute = item.split(":", 2)[1]
                if item.startswith("S:"):
                    second = item.split(":", 2)[1]

            self.__date_time_range_check(
                year, month, day, hour, minute, second)

            self.__date_time_interval_check(
                int(hour), int(minute), int(second))

            self.__date_time_integrity_check(
                year, month, day, hour, minute, second)

        return DATE_TIME_ATTR

    def is_boolean(self):
        """Boolean check."""

        if not (self.target == "1" or
            self.target == "0" or
            self.target == ""):
            raise ValidateError(BOOLEAN_ATTR)
        return BOOLEAN_ATTR

    is_not_begin_hyphen = lambda self: \
            self.re_validator(BEGIN_HYPHEN_RE, NOT_BEGIN_HYPHEN_ATTR)
    is_not_number_only = lambda self: \
            self.re_validator(NUMBER_ONLY_RE, NOT_NUMBER_ONLY_ATTR)
    is_not_begin_quote = lambda self: \
            self.re_validator(BEGIN_QUOTE_RE, NOT_BEGIN_QUOTE_ATTR)
    is_not_end_quote = lambda self: \
            self.re_validator(END_QUOTE_RE, NOT_END_QUOTE_ATTR)
    is_not_end_doller = lambda self: \
            self.re_validator(END_DOLLER_RE, NOT_END_DOLLER_ATTR)
    is_not_begin_sharp = lambda self: \
            self.re_validator(BEGIN_SHARP_RE, NOT_BEGIN_SHARP_ATTR)
    is_not_begin_space = lambda self: \
            self.re_validator(BEGIN_SPACE_RE, NOT_BEGIN_SPACE_ATTR)
    is_not_begin_dot = lambda self: \
            self.re_validator(BEGIN_DOT_RE, NOT_BEGIN_DOT_ATTR)
    is_not_end_space = lambda self: \
            self.re_validator(END_SPACE_RE, NOT_END_SPACE_ATTR)
    is_not_multi_space = lambda self: \
            self.re_validator(MULTI_SPACE_RE, NOT_MULTI_SPACE_ATTR)
    is_begin_char = lambda self: \
            self.re_validator(BEGIN_CHAR_RE, BEGIN_CHAR_ATTR, True)
    is_not_end_hyphen = lambda self: \
            self.re_validator(END_HYPHEN_RE, NOT_END_HYPHEN_ATTR)
    is_iss_code = lambda self: \
            self.re_validator(ISS_CODE_RE, ISS_CODE_ATTR, True)
    is_serial_number = lambda self: \
            self.re_validator(SERIAL_NUMBER_RE, SERIAL_NUMBER_ATTR, True)
    is_unc_share = lambda self: \
            self.re_validator(UNC_SHARE_RE, UNC_SHARE_ATTR, True)

    def validate_list_exec(self, validate_list):
        """Validate list exec method."""

        for valid in validate_list:
            try:
                self.result_node.set_bool_attr(valid(), true=True)
            except ValidateError, inst:
                self.set_err_result(inst.attr)

    def set_msginfo_node(self):

        msginfo_node = xmldata.XmlConfNode(MESSAGEINFO_FMT_STR)

        is_invalid = False

        for key, value in self.result_node.get_attrs().iteritems():
            if value == "false":

                is_invalid = True
                code = self.msginfo_dict[key][CODE_KEY]
                msginfo_node.get_node(CODE_NAME).set_text(code)
                arglist_node = msginfo_node.get_node(ARGLIST_NAME)
                for arg_key, arg_value in self.msginfo_dict[key][ARG_DICT_KEY].iteritems():
                    arglist_node.append_child(
                        xmldata.XmlConfNode(ARG_FMT_STR % (
                            arg_key,
                            arg_value)))

        if is_invalid:
            self.result_node.append_child(msginfo_node)

    def validate(self):
        self.validate_list_exec(self.VALIDATE_LIST)
        self.set_msginfo_node()
        return self.result_node
