#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
import math

import fal_config as config
from .basevalidator import (BaseValidator,
    REMOVABLE_OPTION, EXEMPT_ID_OPTION, MIN_LENGTH_OPTION, MAX_LENGTH_OPTION)
from fal_msginfo import Msg

from .extend import *


NAME_NAME = "name"
SHARE_NAME_LIST_CHILD_PATH = "sharelist/share/name"

EXTEND_CLASS_IDX = 1
NAME_IDX = 0
OBJ_IDX = 1
EXTEND_DIR = "extend"
EXTEND_MOD_SUFFIX = ".py"
MOD_SEP = "."


class LanDiskName(BaseValidator):
    """LanDiskName validate class."""

    def __init__(self, target_node,
            validate_config_node=None):

        BaseValidator.__init__(self, target_node,
            validate_config_node)

        self.VALIDATE_LIST = [
            self.is_not_null,
            self.is_not_begin_hyphen,
            self.is_not_number_only,
            self.is_in_length_range,
            self.is_format_table,
            ]


class AdUserName(BaseValidator):
    """AdUserName validate class."""

    def __init__(self, target_node,
            validate_config_node=None):

        BaseValidator.__init__(self, target_node,
            validate_config_node)

        self.VALIDATE_LIST = [
            self.is_not_null,
            self.is_in_length_range,
            self.is_format_table,
            ]


class Comment(BaseValidator):
    """Comment validate class."""

    def __init__(self, target_node,
            validate_config_node=None):

        BaseValidator.__init__(self, target_node,
            validate_config_node)

        self.VALIDATE_LIST = [
            self.is_not_null,
            self.is_in_count_range,
            self.is_format_table,
            ]


class DomainName(BaseValidator):
    """DomainName validate class."""

    def __init__(self, target_node,
            validate_config_node=None):

        BaseValidator.__init__(self, target_node,
            validate_config_node)

        self.VALIDATE_LIST = [
            self.is_not_null,
            self.is_in_length_range,
            self.is_format_table,
            self.is_in_dot_split_length,
            ]


class NetBIOSDomainName(BaseValidator):
    """NetBIOSDomainName validate class."""

    def __init__(self, target_node,
            validate_config_node=None):

        BaseValidator.__init__(self, target_node,
            validate_config_node)

        self.VALIDATE_LIST = [
            self.is_not_null,
            self.is_format_table,
            self.is_netbios_domain_name,
            self.is_not_network_duplicate,
            self.is_in_length_range,
            ]


class ShareComment(BaseValidator):
    """ShareComment validate class."""

    def __init__(self, target_node,
            validate_config_node=None):

        BaseValidator.__init__(self, target_node,
            validate_config_node)

        self.VALIDATE_LIST = [
            self.is_not_null,
            self.is_not_begin_quote,
            self.is_not_end_quote,
            self.is_in_length_range,
            ]


class ShareName(BaseValidator):
    """ShareName validate class."""

    def __init__(self, target_node,
            validate_config_node=None):

        BaseValidator.__init__(self, target_node,
            validate_config_node)

        self.is_share_name_not_duplicate = lambda: self.is_not_duplicate(
                conffile=config.SHARES_CONF_FILE,
                path=SHARE_NAME_LIST_CHILD_PATH)
        self.is_share_name_not_uplow = lambda: self.is_not_uplow(
                conffile=config.SHARES_CONF_FILE,
                path=SHARE_NAME_LIST_CHILD_PATH)
   
        self.msginfo_dict.update(
            {"in_length_range": {
                "code": Msg.ERR_VALIDATE_IN_SINGLE_MULTI_BYTE_LENGTH_RANGE,
                "arg_dict": {
                    "single_min": self.validate_config_option_dict[MIN_LENGTH_OPTION],
                    "single_max": self.validate_config_option_dict[MAX_LENGTH_OPTION],
                    "multi_min": str(int(math.ceil(float(self.validate_config_option_dict[MIN_LENGTH_OPTION]) / 3))),
                    "multi_max": str(int(self.validate_config_option_dict[MAX_LENGTH_OPTION]) / 3)}}})

        self.VALIDATE_LIST = [
            self.is_not_null,
            self.is_format_table,
            self.is_not_end_doller,
            self.is_not_begin_sharp,
            self.is_not_begin_space,
            self.is_not_end_space,
            self.is_not_multi_space,
            self.is_share_name_not_duplicate,
            self.is_share_name_not_uplow,
            self.is_in_length_range,
            ]

# fixme sharenameeditクラス名微妙、
# バックアップやレプリケーションの設定で
# 入力するフォルダー名の禁則である


class ShareNameEdit(BaseValidator):
    """ShareNameEdit validate class."""

    def __init__(self, target_node,
            validate_config_node=None):

        BaseValidator.__init__(self, target_node,
            validate_config_node)

        self.VALIDATE_LIST = [
            self.is_not_null,
            self.is_format_table,
            self.is_not_end_doller,
            self.is_in_length_range,
            ]


class Fqdn(BaseValidator):
    """Fqdn validate class."""

    def __init__(self, target_node,
            validate_config_node=None):

        BaseValidator.__init__(self, target_node,
            validate_config_node)

        self.VALIDATE_LIST = [
            self.is_not_null,
            self.is_fqdn,
            self.is_not_network_duplicate,
            self.is_not_number_only,
            self.is_in_length_range,
            ]


class GroupName(BaseValidator):
    """GroupName validate class."""

    def __init__(self, target_node,
            validate_config_node=None):

        BaseValidator.__init__(self, target_node,
            validate_config_node)

        self.is_group_name_not_reserved = lambda: self.is_not_reserved(
                conffile=config.GROUPS_CONF_FILE,
                path="grouplist/group[@reserved='true']/name")
        self.is_group_name_not_duplicate = lambda: self.is_not_duplicate(
                conffile=config.GROUPS_CONF_FILE,
                path="grouplist/group[not(@reserved) or @reserved!='true']/name")
        self.is_group_name_not_uplow = lambda: self.is_not_uplow(
                conffile=config.GROUPS_CONF_FILE,
                path="grouplist/group[not(@reserved) or @reserved!='true']/name")

        self.VALIDATE_LIST = [
            self.is_not_null,
            self.is_not_begin_hyphen,
            self.is_not_number_only,
            self.is_format_table,
            self.is_group_name_not_reserved,
            self.is_group_name_not_duplicate,
            self.is_group_name_not_uplow,
            self.is_in_length_range,
            ]


class InItem(BaseValidator):
    """InItem validate class."""

    def __init__(self, target_node,
            validate_config_node=None):

        BaseValidator.__init__(self, target_node,
            validate_config_node)

        self.VALIDATE_LIST = [
            self.is_not_null,
            self.is_in_item,
            ]


class IpAddress(BaseValidator):
    """IpAddress validate class."""

    def __init__(self, target_node,
            validate_config_node=None):

        BaseValidator.__init__(self, target_node,
            validate_config_node)

        self.VALIDATE_LIST = [
            self.is_not_null,
            self.is_ip_address,
            ]


class IssCode(BaseValidator):
    """IssCode validate class."""

    def __init__(self, target_node,
            validate_config_node=None):

        BaseValidator.__init__(self, target_node,
            validate_config_node)

        self.VALIDATE_LIST = [
            self.is_not_null,
            self.is_iss_code,
            self.is_in_length_range,
            ]


class LanDiskPass(BaseValidator):
    """LanDiskPass validate class."""

    def __init__(self, target_node,
            validate_config_node=None):

        BaseValidator.__init__(self, target_node,
            validate_config_node)

        self.VALIDATE_LIST = [
            self.is_not_null,
            self.is_format_table,
            self.is_in_length_range,
            ]


class MailUserName(BaseValidator):
    """MailUserName validate class."""

    def __init__(self, target_node,
            validate_config_node=None):

        BaseValidator.__init__(self, target_node,
            validate_config_node)

        self.VALIDATE_LIST = [
            self.is_not_null,
            self.is_format_table,
            self.is_in_length_range,
            ]


class MailAddress(BaseValidator):
    """MailAddress validate class."""

    def __init__(self, target_node,
            validate_config_node=None):

        BaseValidator.__init__(self, target_node,
            validate_config_node)

        self.VALIDATE_LIST = [
            self.is_not_null,
            self.is_format_table,
            self.is_mail_address,
            self.is_in_length_range,
            ]


class NoCheck(BaseValidator):
    """NoCheck validate class."""

    def __init__(self, target_node,
            validate_config_node=None):

        BaseValidator.__init__(self, target_node,
            validate_config_node)

        self.VALIDATE_LIST = [
            self.is_not_null,
            self.is_in_length_range,
            ]


class LocalPortNumber(BaseValidator):
    """LocalPortNumber validate class."""

    def __init__(self, target_node,
            validate_config_node=None):

        BaseValidator.__init__(self, target_node,
            validate_config_node)

        self.VALIDATE_LIST = [
            self.is_not_null,
            self.is_numeric,
            self.is_not_forbidden_port,
            self.is_used_port,
            self.is_in_range,
            ]


class RemotePortNumber(BaseValidator):
    """RemotePortNumber validate class."""

    def __init__(self, target_node,
            validate_config_node=None):

        BaseValidator.__init__(self, target_node,
            validate_config_node)

        self.VALIDATE_LIST = [
            self.is_not_null,
            self.is_numeric,
            self.is_in_range,
            ]


class Quota(BaseValidator):
    """Quota validate class."""

    def __init__(self, target_node,
            validate_config_node=None):

        BaseValidator.__init__(self, target_node,
            validate_config_node)

        self.VALIDATE_LIST = [
            self.is_not_null,
            self.is_point,
            self.is_in_range,
            ]


class SerialNumber(BaseValidator):
    """SerialNumber validate class."""

    def __init__(self, target_node,
            validate_config_node=None):

        BaseValidator.__init__(self, target_node,
            validate_config_node)

        self.VALIDATE_LIST = [
            self.is_not_null,
            self.is_serial_number,
            ]

# fixme XRはstrcmpは存在するが使用していないようだが

# fixme XRはtimestamp禁則があり、入力された日付を
# shのdate -dで変換できるかや、perlのtimelocal,localtimeで変換し
# 差異が出ないかなど確認している。6Hどうする


class DateTime(BaseValidator):
    """DateTime validate class."""

    def __init__(self, target_node,
            validate_config_node=None):

        BaseValidator.__init__(self, target_node,
            validate_config_node)

        self.VALIDATE_LIST = [
            self.is_not_null,
            self.is_date_time,
            ]


class Url(BaseValidator):
    """Url validate class."""

    def __init__(self, target_node,
            validate_config_node=None):

        BaseValidator.__init__(self, target_node,
            validate_config_node)

        self.VALIDATE_LIST = [
            self.is_not_null,
            self.is_in_length_range,
            ]


class UserName(BaseValidator):
    """UserName validate class."""

    def __init__(self, target_node,
            validate_config_node=None):

        BaseValidator.__init__(self, target_node,
            validate_config_node)

        self.is_user_name_not_reserved = lambda: self.is_not_reserved(
                conffile=config.USERS_CONF_FILE,
                path="userlist/user[@reserved='true']/name")
        self.is_user_name_not_duplicate = lambda: self.is_not_duplicate(
                conffile=config.USERS_CONF_FILE,
                path="userlist/user[not(@reserved) or @reserved!='true']/name")
        self.is_user_name_not_uplow = lambda: self.is_not_uplow(
                conffile=config.USERS_CONF_FILE,
                path="userlist/user[not(@reserved) or @reserved!='true']/name")

        self.VALIDATE_LIST = [
            self.is_not_null,
            self.is_not_begin_hyphen,
            self.is_not_number_only,
            self.is_format_table,
            self.is_user_name_not_reserved,
            self.is_user_name_not_duplicate,
            self.is_user_name_not_uplow,
            self.is_in_length_range,
            ]


class UserNameEdit(BaseValidator):
    """UserNameEdit validate class."""

    def __init__(self, target_node,
            validate_config_node=None):

        BaseValidator.__init__(self, target_node,
            validate_config_node)

        self.is_user_name_not_reserved = lambda: self.is_not_reserved(
                conffile=config.USERS_CONF_FILE,
                path="userlist/user[@reserved='true']")

        self.VALIDATE_LIST = [
            self.is_not_null,
            self.is_not_begin_hyphen,
            self.is_not_number_only,
            self.is_format_table,
            self.is_user_name_not_reserved,
            self.is_in_length_range,
            ]


class WorkGroup(BaseValidator):
    """WorkGroup validate class."""

    def __init__(self, target_node,
            validate_config_node=None):

        BaseValidator.__init__(self, target_node,
            validate_config_node)

        self.VALIDATE_LIST = [
            self.is_not_null,
            self.is_format_table,
            # fixme is_not_network_duplicate()で流用したが、
            # エラーメッセージは異なるものに
            # する必要ありそうだが、
            self.is_not_network_duplicate,
            self.is_in_length_range,
            ]


class Numeric(BaseValidator):
    """Numeric validate class."""

    def __init__(self, target_node,
            validate_config_node=None):

        BaseValidator.__init__(self, target_node,
            validate_config_node)

        self.VALIDATE_LIST = [
            self.is_not_null,
            self.is_numeric,
            self.is_in_range,
            ]


class UserInItem(BaseValidator):
    """User in item class."""

    def __init__(self, target_node,
            validate_config_node=None):

        BaseValidator.__init__(self, target_node,
            validate_config_node)

        self.is_user_in_item = lambda: self.is_in_item(
                self.get_user_list())

        self.VALIDATE_LIST = [
            self.is_not_null,
            self.is_user_in_item
            ]


class GroupInItem(BaseValidator):
    """Group in item class."""

    def __init__(self, target_node,
            validate_config_node=None):

        BaseValidator.__init__(self, target_node,
            validate_config_node)

        self.is_group_in_item = lambda: self.is_in_item(
                self.get_group_list())

        self.VALIDATE_LIST = [
            self.is_not_null,
            self.is_group_in_item
            ]


class ShareInItem(BaseValidator):
    """Share in item class."""

    def __init__(self, target_node,
            validate_config_node=None):

        BaseValidator.__init__(self, target_node,
            validate_config_node)

        removable = None
        try:
            if self.validate_config_option_dict[REMOVABLE_OPTION] == "true":
                removable = True
            elif self.validate_config_option_dict[REMOVABLE_OPTION] == "false":
                removable = False
            else:
                removable = None
        except KeyError:
            removable = None

        self.is_share_in_item = lambda: self.is_in_item(
                self.get_share_list(removable))

        self.VALIDATE_LIST = [
            self.is_not_null,
            self.is_share_in_item
            ]


class Boolean(BaseValidator):
    """Boolean class."""

    def __init__(self, target_node,
            validate_config_node=None):

        BaseValidator.__init__(self, target_node,
            validate_config_node)

        self.VALIDATE_LIST = [
            self.is_boolean
            ]


class Year(BaseValidator):
    """Year class."""

    def __init__(self, target_node,
            validate_config_node=None):

        BaseValidator.__init__(self, target_node,
            validate_config_node)


        self.is_year = lambda: self.is_in_item([
            str(num) for num in range(2012, 2038)])

        self.VALIDATE_LIST = [
            self.is_not_null,
            self.is_year
            ]


class Month(BaseValidator):
    """Month class."""

    def __init__(self, target_node,
            validate_config_node=None):

        BaseValidator.__init__(self, target_node,
            validate_config_node)


        self.is_month = lambda: self.is_in_item([
            str(num) for num in range(1, 13)])

        self.VALIDATE_LIST = [
            self.is_not_null,
            self.is_month
            ]


class Day(BaseValidator):
    """Day class."""

    def __init__(self, target_node,
            validate_config_node=None):

        BaseValidator.__init__(self, target_node,
            validate_config_node)


        self.is_day = lambda: self.is_in_item([
            str(num) for num in range(1, 32)])

        self.VALIDATE_LIST = [
            self.is_not_null,
            self.is_day
            ]


class Hour(BaseValidator):
    """Hour class."""

    def __init__(self, target_node,
            validate_config_node=None):

        BaseValidator.__init__(self, target_node,
            validate_config_node)

        self.is_hour = lambda: self.is_in_item([
            str(num) for num in range(0, 24)])

        self.VALIDATE_LIST = [
            self.is_not_null,
            self.is_hour
            ]


class Minute(BaseValidator):
    """Minute class."""

    def __init__(self, target_node,
            validate_config_node=None):

        BaseValidator.__init__(self, target_node,
            validate_config_node)

        self.is_minute = lambda: self.is_in_item([
            str(num) for num in range(0, 60)])

        self.VALIDATE_LIST = [
            self.is_not_null,
            self.is_minute
            ]


class Second(BaseValidator):
    """Second class."""

    def __init__(self, target_node,
            validate_config_node=None):

        BaseValidator.__init__(self, target_node,
            validate_config_node)

        self.is_second = lambda: self.is_in_item([
            str(num) for num in range(0, 60)])

        self.VALIDATE_LIST = [
            self.is_not_null,
            self.is_second
            ]


class MinuteInterval5(BaseValidator):
    """Minute interval 5 class."""

    def __init__(self, target_node,
            validate_config_node=None):

        BaseValidator.__init__(self, target_node,
            validate_config_node)

        self.is_minute_interval_5 = lambda: self.is_in_item([
            str(num) for num in range(0, 60, 5)])

        self.VALIDATE_LIST = [
            self.is_not_null,
            self.is_minute_interval_5
            ]


class MinuteInterval10(BaseValidator):
    """Minute interval 10."""

    def __init__(self, target_node,
            validate_config_node=None):

        BaseValidator.__init__(self, target_node,
            validate_config_node)

        self.is_minute_interval_10 = lambda: self.is_in_item([
            str(num) for num in range(0, 60, 10)])

        self.VALIDATE_LIST = [
            self.is_not_null,
            self.is_minute_interval_10
            ]


class String(BaseValidator):
    """String."""

    def __init__(self, target_node,
            validate_config_node=None):

        BaseValidator.__init__(self, target_node,
            validate_config_node)

        self.VALIDATE_LIST = [
            self.is_not_null,
            self.is_in_count_range
            ]


class BackupJobName(BaseValidator):
    """BackupJobName."""

    def __init__(self, target_node,
            validate_config_node=None):

        BaseValidator.__init__(self, target_node, validate_config_node)

        id = self.validate_config_option_dict.get(EXEMPT_ID_OPTION, "")

        if id:
            path = "backup/joblist/job[id!='%s']/name" % id
        else:
            path = "backup/joblist/job/name"

        self.is_job_name_not_duplicate = lambda: self.is_not_duplicate(
                conffile=config.BACKUP_CONF_FILE,
                path=path)
    
        self.VALIDATE_LIST = [
            self.is_not_null,
            self.is_not_begin_space,
            self.is_not_end_space,
            self.is_not_multi_space,
            self.is_not_begin_dot,
            self.is_in_count_range,
            self.is_format_table,
            self.is_job_name_not_duplicate
            ]


class DataCopyJobName(BaseValidator):
    """DataCopyJobName."""

    def __init__(self, target_node,
            validate_config_node=None):

        BaseValidator.__init__(self, target_node, validate_config_node)

        id = self.validate_config_option_dict.get(EXEMPT_ID_OPTION, "")

        if id:
            path = "datacopy/joblist/job[id!='%s']/name" % id
        else:
            path = "datacopy/joblist/job/name"

        self.is_job_name_not_duplicate = lambda: self.is_not_duplicate(
                conffile=config.BACKUP_CONF_FILE,
                path=path)
    
        self.VALIDATE_LIST = [
            self.is_not_null,
            self.is_not_begin_space,
            self.is_not_end_space,
            self.is_not_multi_space,
            self.is_not_begin_dot,
            self.is_in_count_range,
            self.is_format_table,
            self.is_job_name_not_duplicate
            ]


class UncShare(BaseValidator):
    """UncShare validate class."""

    def __init__(self, target_node,
            validate_config_node=None):

        BaseValidator.__init__(self, target_node,
            validate_config_node)

        self.VALIDATE_LIST = [
            self.is_not_null,
            self.is_unc_share,
            ]
