#!/bin/sh
# -*- mode: landisk-sh; coding: utf-8; -*-
#
# script_simple.sh: System initialization script for HDL-AA series.
# Copyright (C) 2013-2016 I-O DATA DEVICE, INC.  All rights reserved.
#

DEBUG=
#DEBUG=1

ZEROCLEAR=1

F_PROC_BTNLED="/proc/nas_ctrl/btnled"
F_PROC_MELODY="/proc/nas_ctrl/melody"


###############################################################################
# LED を設定する
###############################################################################
__set_led()
{
	local color="${1}"
	local action="${2}"

	if [ ! -e "${F_PROC_BTNLED}" ]; then
		return 1
	fi

	echo "${color} ${action}" >> "${F_PROC_BTNLED}"
	return 0
}


###############################################################################
# STATUS LED を設定する
# 入力:
#   arg1: 色(green, red, orange)
#   arg2: 動作(on, blink)
###############################################################################
__set_status_led()
{
	local color="${1}"
	local action="${2}"

	case "${color}" in
		green)
			__set_led "led_green" "${action}"
			__set_led "led_red" "off"
			__set_led "led_orange" "off"
			;;
		red)
			__set_led "led_green" "off"
			__set_led "led_red" "${action}"
			__set_led "led_orange" "off"
			;;
		orange)
			__set_led "led_green" "off"
			__set_led "led_red" "off"
			__set_led "led_orange" "${action}"
			;;
		*)
			;;
	esac

	return 0
}


###############################################################################
# エラー表示する
# 入力: 引数1=表示メッセージ
###############################################################################
__initialization_error()
{
	local msg="${1}"

	__set_status_led red blink
	echo "###### ERROR ######"
	echo "${msg}"

	# スクリプトを抜けて通常起動の流れへ
	exit 1
}


###############################################################################
# main
###############################################################################

echo "### system initialization start #########################################"

SRCDISK="${1}"

if [ ! -e "${SRCDISK}" ]; then
	# ディスクが存在しない(通常ここにはこない)
	__initialization_error "source disk not found."
fi

if [ ! -e "${SRCDISK}p1" ]; then
	# パーティションが存在しない(通常ここにはこない)
	__initialization_error "source partition not found."
fi

/bin/mount -t ext4 "${SRCDISK}p1" "${D_MP_INIT}" 1>/dev/null 2>/dev/null
if [ ${?} -ne 0 ]; then
	# マウントできない(通常ここにはこない)
	__initialization_error "failed to mount source."
fi

# 初期化処理
${D_INIT_FW}/initialize.sh "${SRCDISK}" "${ZEROCLEAR}"

# 初期化完了
if [ -e "${F_INITIALIZATION}" ]; then
	rm -f "${F_INITIALIZATION}" 1>/dev/null 2>/dev/null
fi

/bin/umount "${D_MP_INIT}" 1>/dev/null 2>/dev/null

if [ "x${DEBUG}" != "x" ]; then
	# デバッグ
	/bin/sh
fi

echo "### system initialization end ###########################################"
