#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""EVH command for handling UsbdPlug event"""

import ionas.request as request
import libevh.evhcmd as evhcmd
from fal_info import (FAL_ECO_PATH)
from iobase.exception import (InvalidArgError)
from product_conf.reqinfo import (Results, ReqTypes)


class EvhSpindownUpdate(evhcmd.EvhCmd):
    """UsbdPlug Event Hnadler command implementation class."""

    def __send_request(self, type, target_module, child_node=""):

        entry = request.Entry(type, child_node)
        return self.send_entry(entry, target_module)

    #
    # Request handlers
    #

    def __event_handler(self, entry):
        """Handle "event" request."""

        self.__send_request(ReqTypes.UPDATE, FAL_ECO_PATH)
        entry.set_result(Results.OK)
        return entry

    def __init__(self, option=None):
        """Constructor."""

        evhcmd.EvhCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.EVENT, self.__event_handler)

#
# Main
#

if __name__ == "__main__":
    EvhSpindownUpdate(option=evhcmd.EvhCmdOption()).handle_request()
