#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for handling event"""

import ionas.request as request
import libevh.evhcmd as evhcmd
import pal_info
from iobase.xmldata import XmlNode
from iobase.exception import ExecError
from product_conf.reqinfo import (Results, ReqTypes)

ABORT_TARGET_STATUS_LIST = ["PROCESSING"]

ABORT_STATUS = "ABORT"

SET_REQ_FMT = "<selfcheck><checklist/></selfcheck>"
CHECKLIST_NAME = "checklist"
CHECK_NODE_LIST_PATH = CHECKLIST_NAME + "/check"
NAME_NAME = "name"
STATUS_NAME = "status"


class EventHnadler(evhcmd.EvhCmd):
    """FAL Event Hnadler command implementation class."""

    def __send_request(self, target, type, body=""):
        """Send request entry to "target".""" 

        entry = request.Entry(type, body)
        return self.send_entry(entry, target)

    def __get_set_req(self, query_res):

        is_send_req = False
        set_req = XmlNode(SET_REQ_FMT)
        chk_list_node = set_req.get_node(CHECKLIST_NAME)
        
        for chk_node in query_res.get().get_node_list(CHECK_NODE_LIST_PATH):

            if not chk_node.get_value(STATUS_NAME) in ABORT_TARGET_STATUS_LIST:
                continue

            set_chk_node = XmlNode(
                "<check name='%s'><status>%s</status></check>" %
                (chk_node.get_attr_value(NAME_NAME), ABORT_STATUS))

            chk_list_node.append_child(set_chk_node)

            is_send_req = True

        return (set_req, is_send_req)

    def _system_boot_event(self, entry):

        category = entry.get().get_value('eventinfo/@category')
        action = entry.get().get_value('eventinfo/@action')

        if category != "system" or action != "boot":
            return

        query_res = self.__send_request(pal_info.PAL_SELFCHECK_PATH,
                                        ReqTypes.QUERY)

        set_req, is_send_req = self.__get_set_req(query_res)

        if is_send_req:
            self.__send_request(pal_info.PAL_SELFCHECK_PATH,
                                ReqTypes.SET,
                                set_req)

    def _selfcheck_set_event(self, entry):

        category = entry.get().get_value('eventinfo/@category')
        action = entry.get().get_value('eventinfo/@action')

        if category != "selfcheck" or action != "set":
            return

        body = entry.get().get_node('eventinfo/selfcheck')

        res_entry = self.send_entry(
                request.Entry(ReqTypes.SET, body),
                pal_info.PAL_SELFCHECK_PATH)

        if res_entry.get_result() != Results.OK:
            raise ExecError(pal_info.PAL_SELFCHECK_PATH)

    #
    # Request handlers
    #

    def __event_handler(self, entry):
        """Handle "event" request."""

        self._system_boot_event(entry)

        self._selfcheck_set_event(entry)

        entry.set_result(Results.OK)

        return entry

    def __init__(self, option=None):
        """Constructor."""

        evhcmd.EvhCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.EVENT, self.__event_handler)

#
# Main
#

if __name__ == "__main__":
    EventHnadler(option=evhcmd.EvhCmdOption()).handle_request()
