#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for handling LCM event"""

import ionas.request as request
import libevh.evhcmd as evhcmd
import fal_info
from iobase.exception import (NotFoundError, InvalidArgError, ExecError)
from product_conf.reqinfo import (Results, ReqTypes)


class EventHnadler(evhcmd.EvhCmd):
    """FAL LCM Event Hnadler command implementation class."""

    #
    # Request handlers
    #

    def __event_handler(self, entry):
        """Handle "event" request."""

        unode = entry.get().get_node('eventinfo/users/user')
        uname = unode.get_value('name', None)
        gid = unode.get_value('gid', None)
        if uname is None:
            raise InvalidArgError('User name is empty!!')
        if gid is None:
            raise InvalidArgError('User group is empty!!')

        groups_node = self.send_entry(
                request.Entry(ReqTypes.LOAD, '<groups />'),
                fal_info.FAL_GROUPS_PATH).get()
        gnode = groups_node.get_node('grouplist/group[id="%s"]' % gid)
        gname = gnode.get_value('name', None)
        ulist_node = gnode.get_node('userlist')
        ulist = [und.get_value('name') \
                for und in ulist_node.get_node_list('user') \
                        if und.get_value('name')]
        if gname is None:
            raise InvalidArgError('Group name is empty!!')

        if uname not in ulist:
            ulist.append(uname)
        req_entry = request.Entry(ReqTypes.MODIFY, '<groups><group /></groups>')
        req_gnode = req_entry.get().get_node('group')
        req_gnode.append_child('name', gname)
        req_gnode.append_child('userlist')
        req_ulnode = req_gnode.get_node('userlist')
        for member in ulist:
            req_unode = request.XmlNode('<user />')
            req_unode.append_child('name', text=str(member))
            req_ulnode.append_child(req_unode)
        res_entry = self.send_entry(req_entry, fal_info.FAL_GROUPS_PATH)
        if res_entry.get_result() != Results.OK:
            raise ExecError(fal_info.FAL_GROUPS_PATH)

        entry.set_result(Results.OK)
        return entry

    def __init__(self, option=None):
        """Constructor."""

        evhcmd.EvhCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.EVENT, self.__event_handler)

#
# Main
#

if __name__ == "__main__":
    EventHnadler(option=evhcmd.EvhCmdOption()).handle_request()
