#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for handling Users Ctrl event"""

from ionas.request import Entry
from iobase.exception import ExecError, NotFoundError
import libevh.evhcmd as evhcmd
import pal_info
from product_conf.reqinfo import Results, ReqTypes


USERS_PATH = "eventinfo/users"
USER_PATH = USERS_PATH + "/user"
ADMIN_ATTR = "admin"
TRUE_TEXT = "true"


class EvhUsersCtrl(evhcmd.EvhCmd):
    """Users Ctrl Event Handler command implementation class."""

    #
    # Utility method
    #

    def __is_admin(self, entry):

        try:
            if entry.get().get_node(USER_PATH).get_attr_value(
                ADMIN_ATTR) == TRUE_TEXT:
                return True
            return False
        except NotFoundError:
            return False

    def __send_request(self, type, entry):

        res_entry = self.send_entry(
            Entry(type, entry.get().get_node(USERS_PATH)),
            pal_info.PAL_REPLICATION_PATH)

        if res_entry.get_result() != Results.OK:
            raise ExecError(pal_info.PAL_REPLICATION_PATH)

    #
    # Request handlers
    #

    def __event_handler(self, entry):
        """Handle "event" request."""

        if self.__is_admin(entry):
            self.__send_request(ReqTypes.APPLY, entry)

        entry.set_result(Results.OK)
        return entry

    def __init__(self, option=None):
        """Constructor."""

        evhcmd.EvhCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.EVENT, self.__event_handler)

#
# Main
#

if __name__ == "__main__":
    EvhUsersCtrl(option=evhcmd.EvhCmdOption()).handle_request()
