#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for handling Samba Update event"""

import libevh.evhcmd as evhcmd
from ionas.request import Entry
from pal_info import (PAL_SAMBA_PATH, Results, ReqTypes)


class SambaUpdate(evhcmd.EvhCmd):
    """Samba Update Event Hnadler command implementation class."""

    def __event_handler(self, entry):
        """Handle "event" request."""

        response = self.send_entry(Entry(ReqTypes.UPDATE), PAL_SAMBA_PATH)
        if response.get_result() != Results.OK:
            entry.set_result(response.get_result())
            return entry

        entry.set_result(Results.OK)
        return entry

    def __init__(self, option=None):
        """Constructor."""

        evhcmd.EvhCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.EVENT, self.__event_handler)

#
# Main
#

if __name__ == "__main__":
    SambaUpdate(option=evhcmd.EvhCmdOption()).handle_request()
