#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta


import libevh.evhcmd as evhcmd
from ionas.request import Entry
from fal_info import FAL_SERVICECTRL_PATH
from product_conf.reqinfo import (Results, ReqTypes)
from product_conf import STOR_IF_TO_VOL
from iobase.exception import (InvalidArgError)

VOL_NAME_XPATH = "eventinfo/volume/name"
VOLUME_CATEGORY = "volume"
MOUNT_ACTION = "mount"
EVENTINFO_NAME = "eventinfo"
CATEGORY_ATTR = "category"
ACTION_ATTR = "action"
PREVIOUS_ATTR = "previous"

SERVICE_XML = "<servicectrl><servicelist><service><name>%s</name>" + \
            "</service></servicelist></servicectrl>"

SNMP_SERVICE_NAME = "snmpd"


class EventHnadler(evhcmd.EvhCmd):

    def __validate_event(self, entry):

        if VOLUME_CATEGORY != entry.get().get_node(EVENTINFO_NAME).\
                                    get_attr_value(CATEGORY_ATTR):
            raise InvalidArgError

        if entry.get().get_node(EVENTINFO_NAME)\
            .is_set_to_true(PREVIOUS_ATTR):
            raise InvalidArgError

        if MOUNT_ACTION != entry.get().get_node(EVENTINFO_NAME).\
                              get_attr_value(ACTION_ATTR):
            raise InvalidArgError

        return

    def __is_internal_vol(self, vol_name):

        if vol_name != STOR_IF_TO_VOL["internal"][1]:
            return False
        return True

    def __ctrl_service(self, type):
        entry = Entry(type, SERVICE_XML % SNMP_SERVICE_NAME)
        self.send_entry(entry, FAL_SERVICECTRL_PATH)

    #
    # Request handlers
    #

    def __event_handler(self, entry):
        """Handle "event" request."""

        self.__validate_event(entry)

        try:
            body = entry.get()
            vol_name = body.get_value(VOL_NAME_XPATH)

            if self.__is_internal_vol(vol_name):
                self.__ctrl_service(ReqTypes.STOP)
                self.__ctrl_service(ReqTypes.START)
        except:
            self.exception("fail to stop-start snmp service (do nothing)")
            pass

        entry.set_result(Results.OK)
        return entry

    def __init__(self, option=None):
        """Constructor."""

        evhcmd.EvhCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.EVENT, self.__event_handler)

#
# Main
#

if __name__ == "__main__":
    EventHnadler(option=evhcmd.EvhCmdOption()).handle_request()
