#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for handling LCM event"""

import ionas.request as request
import libevh.evhcmd as evhcmd
import pal_info
from iobase.exception import (NotFoundError, InvalidArgError, ExecError)
from product_conf.reqinfo import (Results, ReqTypes)


class EventHnadler(evhcmd.EvhCmd):
    """FAL LCM Event Hnadler command implementation class."""

    #
    # Request handlers
    #

    def __event_handler(self, entry):
        """Handle "event" request."""

        shares_node = entry.get().get_node('eventinfo/shares')
        try:
            enable = shares_node.get_node(
                    'share/shareservice/appleshare').is_enabled()
        except:
            enable = False

        if enable:
            res_entry = self.send_entry(
                    request.Entry(ReqTypes.ADD, shares_node),
                    pal_info.PAL_NETATALK_PATH)
            if res_entry.get_result() != Results.OK:
                raise ExecError(pal_info.PAL_NETATALK_PATH)
        else:
            res_entry = self.send_entry(
                    request.Entry(ReqTypes.DELETE, shares_node),
                    pal_info.PAL_NETATALK_PATH)
            if res_entry.get_result() != Results.OK:
                raise ExecError(pal_info.PAL_NETATALK_PATH)

        entry.set_result(Results.OK)
        return entry

    def __init__(self, option=None):
        """Constructor."""

        evhcmd.EvhCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.EVENT, self.__event_handler)

#
# Main
#

if __name__ == "__main__":
    EventHnadler(option=evhcmd.EvhCmdOption()).handle_request()
