#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

import ionas.request as request
import libevh.evhcmd as evhcmd
from fal_info import FAL_BACKUPS_PATH
from iobase.exception import NotFoundError
from iobase.xmldata import XmlNode, ANY_CHILD_PATH
from product_conf.reqinfo import (Results, ReqTypes)

ROOT_NODE_NAME = "backups"
JOBLIST_PATH = "joblist"
JOB_LIST_PATH = "joblist/job"
STATUS_PATH = "status"
ID_PATH = "id"

PROCESSING_STATUS = "PROCESSING"
ABORT_STATUS = "ABORT"

ABORT_TARGET_STATUS_LIST = ["PROCESSING", "QUEUE"]


def get_tip_node(node, fullpath):

    for path in fullpath.split("/"):
        try:
            node.get_node(path)
        except NotFoundError:
            node.append_child(path)
        finally:
            node = node.get_node(path)
    return node


class EvhBackupsApply(evhcmd.EvhCmd):
    """Event Handler to start or stop services for "disk" category."""

    def __send_request(self, target, type, body):
        """Send request entry to "target".""" 

        entry = request.Entry(type, body)
        return self.send_entry(entry, target)

    def __get_apply_req(self, query_res):

        is_send_req = False
        apply_req = XmlNode(ROOT_NODE_NAME)

        for category_node in query_res.get().get_node_list(ANY_CHILD_PATH):
            for job_node in category_node.get_node_list(JOB_LIST_PATH):

                if not job_node.get_value(STATUS_PATH) in ABORT_TARGET_STATUS_LIST:
                    continue

                apply_job_node = XmlNode(
                    "<job><id>%s</id><status>%s</status></job>" %
                    (job_node.get_value(ID_PATH), ABORT_STATUS))

                apply_joblist_node = get_tip_node(
                    apply_req,
                    category_node.get_node_name() + "/" + \
                    JOBLIST_PATH)

                apply_joblist_node.append_child(apply_job_node)

                is_send_req = True

        return (apply_req, is_send_req)

    def __event_handler(self, entry):
        """Handle "event" request."""

        query_res = self.__send_request(FAL_BACKUPS_PATH,
                                        ReqTypes.QUERY,
                                        ROOT_NODE_NAME)

        apply_req, is_send_req = self.__get_apply_req(query_res)

        if is_send_req:
            self.__send_request(FAL_BACKUPS_PATH,
                                ReqTypes.APPLY,
                                apply_req)

        entry.set_result(Results.OK)
        return entry

    def __init__(self, option=None):
        """Constructor."""

        evhcmd.EvhCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.EVENT, self.__event_handler)

#
# Main
#

if __name__ == "__main__":
    EvhBackupsApply(option=evhcmd.EvhCmdOption()).handle_request()
