#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for handling event"""

from ionas.request import Entry
from iobase.xmldata import XmlConfNode
from iobase.exception import ExecError, NotFoundError
import libevh.evhcmd as evhcmd
import pal_info
from product_conf.reqinfo import Results, ReqTypes


SYSINFO_PATH = "eventinfo/sysinfo"
DESC_PATH = "hostdesc/desc"


class EvhDpwsdRestart(evhcmd.EvhCmd):
    """Event Handler command implementation class."""

    #
    # Utility method
    #

    def __send_request(self, pal, type, node):

        res_entry = self.send_entry(Entry(type, node), pal)

        if res_entry.get_result() != Results.OK:
            raise ExecError(pal)

    def __restart_service(self):

        XML = """<servicectrl>
            <servicelist>
                <service><name>dpwsd</name></service>
            </servicelist>
        </servicectrl>"""

        self.__send_request(pal_info.PAL_SERVICECTRL_PATH,
            ReqTypes.STOP, XmlConfNode(XML))
        self.__send_request(pal_info.PAL_SERVICECTRL_PATH,
            ReqTypes.START, XmlConfNode(XML))

    #
    # Request handlers
    #

    def __event_handler(self, entry):
        """Handle "event" request."""

        self.__restart_service()

        entry.set_result(Results.OK)
        return entry

    def __init__(self, option=None):
        """Constructor."""

        evhcmd.EvhCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.EVENT, self.__event_handler)

#
# Main
#

if __name__ == "__main__":
    EvhDpwsdRestart(option=evhcmd.EvhCmdOption()).handle_request()
