#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for handling event"""

from iobase.xmldata import XmlConfNode
from iobase.exception import ExecError
from ionas.request import Entry
from product_conf.reqinfo import Results, ReqTypes

import libevh.evhcmd as evhcmd
import libevh.util as util
import evh_config as config
import pal_info


SYSINFO_PATH = "eventinfo/sysinfo"


class EvhFtpApply(evhcmd.EvhCmd):
    """Event Handler command implementation class."""

    #
    # Utility method
    #

    def __send_request(self, pal, type, node):

        res_entry = self.send_entry(Entry(type, node), pal)

        if res_entry.get_result() != Results.OK:
            raise ExecError(pal)

    def __restart_service(self):

        XML = """<servicectrl>
            <servicelist>
                <service><name>proftpd</name></service>
            </servicelist>
        </servicectrl>"""

        self.__send_request(pal_info.PAL_SERVICECTRL_PATH,
            ReqTypes.SUSPEND, XmlConfNode(XML))
        self.__send_request(pal_info.PAL_SERVICECTRL_PATH,
            ReqTypes.RESUME, XmlConfNode(XML))

    #
    # Request handlers
    #

    def __event_handler(self, entry):
        """Handle "event" request."""

        try:
            config_sysinfo = util.load_config(config.SYSINFO_CONF_FILE)
            entry_sysinfo_node = entry.get().get_node(SYSINFO_PATH)
            config_sysinfo.update(entry_sysinfo_node, override=True)
            self.__send_request(
                pal_info.PAL_PROFTPD_PATH, ReqTypes.APPLY, config_sysinfo)
            self.__restart_service()
        except:
            pass

        entry.set_result(Results.OK)
        return entry

    def __init__(self, option=None):
        """Constructor."""

        evhcmd.EvhCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.EVENT, self.__event_handler)

#
# Main
#

if __name__ == "__main__":
    EvhFtpApply(option=evhcmd.EvhCmdOption()).handle_request()
