#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for handling LCM event"""

import os
import ionas.nascmd as nascmd
import ionas.request as request
import libevh.evhcmd as evhcmd
from fal_info import (FAL_LCM_PATH, FAL_SYSINFO_PATH, FAL_VOLCTRL_PATH,
                      FAL_NETWORK_PATH)
import pal_info
from iobase.xmldata import (XmlNode)
from iobase.exception import (NotFoundError, InvalidArgError, ExecError)
from product_conf.reqinfo import (Results, ReqTypes)
from product_conf import *
#import evh_config


class SharesAddEventHnadler(evhcmd.EvhCmd):
    """FAL LCM Event Hnadler command implementation class."""

    #
    # Request handlers
    #

    def __event_handler(self, entry):
        """Handle "event" request."""

        ftp_node = XmlNode('ftp')
        for node in entry.get().get_node_list('eventinfo/shares/*'):
            ftp_node.append_child(node)

        res_entry = self.send_entry(
                request.Entry(
                        ReqTypes.DELETE,
                        ftp_node),
#                        entry.get().get_node('eventinfo/shares')),
                pal_info.PAL_PROFTPD_PATH)
        if res_entry.get_result() != Results.OK:
            raise ExecError(pal_info.PAL_PROFTPD_PATH)

        entry.set_result(Results.OK)
        return entry

    def __init__(self, option=None):
        """Constructor."""

        evhcmd.EvhCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.EVENT, self.__event_handler)

#
# Main
#

if __name__ == "__main__":
    SharesAddEventHnadler(option=evhcmd.EvhCmdOption()).handle_request()
