#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

import ionas.request as request
import libevh.evhcmd as evhcmd
from fal_info import FAL_LUMPREGIST_PATH
from iobase.xmldata import XmlNode
from product_conf.reqinfo import (Results, ReqTypes)

ROOT_NODE_NAME = "lumpregist"
STATUS_PATH = "status"

PROCESSING_STATUS = "PROCESSING"
ABORT_STATUS = "ABORT"

ABORT_TARGET_STATUS_LIST = [PROCESSING_STATUS]


class EvhLumpRegistApply(evhcmd.EvhCmd):

    def __send_request(self, target, type, body):
        """Send request entry to "target".""" 

        entry = request.Entry(type, body)
        return self.send_entry(entry, target)

    def __get_apply_node(self, query_res):

        apply_node = XmlNode(ROOT_NODE_NAME)

        if not query_res.get().get_value(STATUS_PATH) in ABORT_TARGET_STATUS_LIST:
            return

        apply_node.append_child(STATUS_PATH, ABORT_STATUS)

        return apply_node

    def __event_handler(self, entry):
        """Handle "event" request."""

        query_res = self.__send_request(FAL_LUMPREGIST_PATH,
                                        ReqTypes.QUERY,
                                        ROOT_NODE_NAME)

        apply_node = self.__get_apply_node(query_res)

        if apply_node:
            self.__send_request(FAL_LUMPREGIST_PATH,
                                ReqTypes.APPLY,
                                apply_node)

        entry.set_result(Results.OK)
        return entry


    def __init__(self, option=None):
        """Constructor."""

        evhcmd.EvhCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.EVENT, self.__event_handler)

#
# Main
#

if __name__ == "__main__":
    EvhLumpRegistApply(option=evhcmd.EvhCmdOption()).handle_request()
