#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for handling MKFS event"""

import os
import ionas.nascmd as nascmd
import ionas.request as request
import libevh.evhcmd as evhcmd
import libevh.util as util
import evh_config as config
from fal_info import (FAL_VOLCTRL_PATH)
from iobase.exception import (NotFoundError, InvalidArgError)
from product_conf.reqinfo import (Results, ReqTypes)
from product_conf import *
from product_conf.devinfo import *

PROGRESS_FILE_PATH = os.path.join(config.VOLUME_STATUS_DIR, "%s", "mkfs")

EVENTINFO_NAME = "eventinfo"
ACTION_ATTR = "action"

TARGET_XPATH = \
    "./eventinfo/progress/target"

# Action name
PROGRESS_ACT = "progress"


class EvhMkfs(evhcmd.EvhCmd):
    """MKFS Event Hnadler command implementation class."""

    #
    # Utility method
    #

    def __mkfs_progress_action(self, entry):

        dev_name = entry.get().get_value(TARGET_XPATH, None)
        vol_name = util.dev_name2vol_name(dev_name)
        if not vol_name:
            return entry

        util.save_progress(entry, PROGRESS_FILE_PATH % vol_name)
        return

    #
    # Request handlers
    #

    def __event_handler(self, entry):
        """Handle "event" request."""

        action_func = {PROGRESS_ACT: self.__mkfs_progress_action}

        action = entry.get().get_node(EVENTINFO_NAME).\
                     get_attr_value(ACTION_ATTR)

        action_func[action](entry)

        entry.set_result(Results.OK)
        return entry

    def __init__(self, option=None):
        """Constructor."""

        evhcmd.EvhCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.EVENT, self.__event_handler)

#
# Main
#

if __name__ == "__main__":
    EvhMkfs(option=evhcmd.EvhCmdOption()).handle_request()
