#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for handling LCM event"""

import libevh.evhcmd as evhcmd
import evh_config as config
import ionas.request as request
from product_conf.reqinfo import (Results, ReqTypes)
import pal_info

VOLUME_NAME_XPATH = "eventinfo/volume/name"
QUOTA_VOL_XPATH = "./volumelist/volume[text()='%s']"
EVENTINFO_NAME = "eventinfo"
QUOTA_CONF_FILE = "config-quota.xml"


class QuotaDisableEventHnadler(evhcmd.EvhCmd):

    class __RequestError(Exception):
        "Request Error"
        pass

    #
    # Request handlers
    #

    def __load_config(self):
        return self.send_entry_with_file(
                request.Entry(ReqTypes.LOAD),
                pal_info.PAL_CONFFILE_PATH,
                QUOTA_CONF_FILE).get()

    def __is_quota_enable(self, vol_name):

        mnt_point = config.VOL_INFO[vol_name]["mnt_point"]

        return self.__load_config().get_node(
            QUOTA_VOL_XPATH % mnt_point).is_enabled()

    def __disable_quota(self, vol_name):

        QUOTA_DIABLE_REQ = """<quota>
                                <volumelist>
                                  <volume enable="false">%s</volume>
                                </volumelist>
                              </quota>"""

        mnt_point = config.VOL_INFO[vol_name]["mnt_point"]

        req_entry = request.Entry(ReqTypes.APPLY, QUOTA_DIABLE_REQ % mnt_point)

        result_entry = self.send_entry(req_entry,
                                       pal_info.PAL_QUOTA_PATH)
        if result_entry.get_result() != Results.OK:
            raise self.__RequestError

    def __event_handler(self, entry):
        """Handle "event" request."""

        vol_name = entry.get().get_node(VOLUME_NAME_XPATH).get_text()

        try :
            if not self.__is_quota_enable(vol_name):
                self.__disable_quota(vol_name)
        except:
            pass

        entry.set_result(Results.OK)
        return entry

    def __init__(self, option=None):
        """Constructor."""

        evhcmd.EvhCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.EVENT, self.__event_handler)

#
# Main
#

if __name__ == "__main__":
    QuotaDisableEventHnadler(option=evhcmd.EvhCmdOption()).handle_request()
