#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""EVH command for handling UsbdPlug event"""

import libevh.evhcmd as evhcmd
import libevh.util as util
from iobase.exception import (InvalidArgError)
from product_conf.reqinfo import (Results, ReqTypes)

EVENTINFO_NAME = "eventinfo"
CATEGORY_ATTR = "category"
ACTION_ATTR = "action"
PREVIOUS_ATTR = "previous"
RAID_CATEGORY = "raid"
RAIDREBUILD_CATEGORY = "raidrebuild"
START_ACTION = "start"
PROGRESS_ACTION = "progress"


class RaidStart(evhcmd.EvhCmd):
    """Raid Start Event Hnadler command implementation class."""

    def __get_event_action(self, entry):

        if RAID_CATEGORY != entry.get().get_node(EVENTINFO_NAME).\
                                get_attr_value(CATEGORY_ATTR):
            raise InvalidArgError

        if entry.get().get_node(EVENTINFO_NAME).is_set_to_true(PREVIOUS_ATTR):
            raise InvalidArgError("Previous is True !!")

        action = entry.get().get_node(EVENTINFO_NAME).\
            get_attr_value(ACTION_ATTR)

        if action != START_ACTION:
            raise InvalidArgError("Invalid action !!" + action)

        return action

    def __action_start(self, entry):

        RAID_PROGRESS_XML_TMPL = \
            '<progress state="start"><name>%s</name></progress>'
        RAID_NAME_XPATH = "eventinfo/raid/name"
        PROGRESS_XPATH = "raid/progress"

        raid_name = entry.get().get_value(RAID_NAME_XPATH)

        try:
            raidctrl_node = util.fal_raidctrl_query(raid_name)
        except:
            raise self.__InternalError

        progress = raidctrl_node.get_value(PROGRESS_XPATH, "")
        if not progress:
            return

        util.dispatch_event(RAIDREBUILD_CATEGORY,
                            PROGRESS_ACTION,
                            RAID_PROGRESS_XML_TMPL % raid_name)
        return

    #
    # Request handlers
    #

    def __event_handler(self, entry):
        """Handle "event" request."""

        action = self.__get_event_action(entry)

        self.__action_func = \
            {START_ACTION: self.__action_start}

        self.__action_func[action](entry)

        entry.set_result(Results.OK)
        return entry

    def __init__(self, option=None):
        """Constructor."""

        evhcmd.EvhCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.EVENT, self.__event_handler)

#
# Main
#

if __name__ == "__main__":
    RaidStart(option=evhcmd.EvhCmdOption()).handle_request()
