#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for handling Shares Ctrl event"""

from ionas.request import Entry
from iobase.exception import ExecError, NotFoundError
import libevh.evhcmd as evhcmd
import pal_info
from product_conf.reqinfo import Results, ReqTypes


class EvhSharesCtrl(evhcmd.EvhCmd):
    """Shares Ctrl Event Handler command implementation class."""

    #
    # Utility method
    #

    def __send_request(self, type, entry):

        res_entry = self.send_entry(
            Entry(type),
            pal_info.PAL_REPLICATION_PATH)

        if res_entry.get_result() != Results.OK:
            raise ExecError(pal_info.PAL_REPLICATION_PATH)

    #
    # Request handlers
    #

    def __event_handler(self, entry):
        """Handle "event" request."""

        self.__send_request(ReqTypes.UPDATE, entry)

        entry.set_result(Results.OK)
        return entry

    def __init__(self, option=None):
        """Constructor."""

        evhcmd.EvhCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.EVENT, self.__event_handler)

#
# Main
#

if __name__ == "__main__":
    EvhSharesCtrl(option=evhcmd.EvhCmdOption()).handle_request()
