#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for handling smbpasswd modify event"""

import os
import ionas.nascmd as nascmd
import ionas.request as request
import libevh.evhcmd as evhcmd
from iobase.cmdexec import CmdExec, NULL, PIPE
from iobase.xmldata import (XmlNode)
from iobase.exception import (NotFoundError, InvalidArgError, ExecError)
from product_conf import SMBPASSWD_CMD
from product_conf.reqinfo import (Results, ReqTypes)

from pal_info import INSTALL_PATH
import sys
sys.path.append(INSTALL_PATH)
from libpal.util import RemountRoot  # FIXME 使っていい?


class EventHnadler(evhcmd.EvhCmd):
    """smbpasswd modify Event Hnadler command implementation class."""

    def __execute_smbpasswd_nopass(self, uname):
        """Execute smbpasswd command with no password."""

        with RemountRoot():
            try:
                cmd = [SMBPASSWD_CMD, '-an', uname]
                CmdExec(cmd, stdout=NULL, stderr=NULL).execute()
            except:
                pass

    def __execute_smbpasswd(self, uname, upasswd):
        """Execute smbpasswd command."""

        with RemountRoot():
            try:
                cmd = [SMBPASSWD_CMD, '-sa', uname]
                "%s\n%s" % (upasswd, upasswd) | CmdExec(
                    cmd, stdout=NULL, stderr=NULL)
            except:
                pass

    #
    # Request handlers
    #

    def __event_handler(self, entry):
        """Handle "event" request."""

        uname = None
        upasswd = None

        try:
            event = entry.get()
            uname = event.get_node('eventinfo/users/user/name').get_text()
            upasswd = event.get_node('eventinfo/users/user/passwd').get_text(
                do_strip=False)
        except:
            pass

        if uname is None or upasswd is None:
            entry.set_result(Results.OK)
            return entry

        if upasswd == "":
            self.__execute_smbpasswd_nopass(uname)
        else:
            self.__execute_smbpasswd(uname, upasswd)

        entry.set_result(Results.OK)
        return entry

    def __init__(self, option=None):
        """Constructor."""

        evhcmd.EvhCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.EVENT, self.__event_handler)

#
# Main
#

if __name__ == "__main__":
    EventHnadler(option=evhcmd.EvhCmdOption()).handle_request()
