#!/bin/sh
# -*- mode: landisk-sh; coding: utf-8; -*-
### BEGIN INIT INFO
# Provides:          landisk-model
# Required-Start:    mountall
# Required-Stop: 
# Default-Start:     S
# Default-Stop:
# Short-Description: Create LAN DISK information files.
# Description:
### END INIT INFO
#
# landisk-model.sh
# Copyright (C) 2013-2015 I-O DATA DEVICE, INC.  All rights reserved.
#

if [ ! -e /etc/product_conf.sh ]; then
	echo "ERROR: /etc/product_conf.sh not found."
	exit 1
fi
. /etc/product_conf.sh


D_RAM_LANDISK="/var/lib/landisk"
D_FAVICON="/usr/local/share/icons/landisk"

F_PROC_BOARDMODEL="/proc/nas_ctrl/boardmodel"
F_ETC_FIRMWARE="/etc/landisk_firmware"
F_ETC_VERSION="/etc/landisk_version"
F_ETC_BETA="/etc/landisk_beta"


# 有効なシステムディスクを取得する
get_bootdisk()
{
	if [ -e "/bootdisk" ]; then
		${CAT_CMD} "/bootdisk"
	else
		if [ -e /dev/satad1p2 ]; then
			echo "/dev/satad1"
		elif [ -e /dev/satad2p2 ]; then
			echo "/dev/satad2"
		elif [ -e /dev/usbd2p2 ]; then
			echo "/dev/usbd2"
		elif [ -e /dev/usbd3p2 ]; then
			echo "/dev/usbd3"
		else
			echo "/dev/satad1"
		fi
	fi
}


# ディスクラベル型番を取得
get_disklabel_model()
{
	local device="${1}"
	local mountpoint="${2}"

	local model=""

	if [ ! -e "${device}" ]; then
		return 1
	fi
	if [ ! -d "${mountpoint}" ]; then
		return 1
	fi

	${MOUNT_CMD} -t ext4 -o ro \
		"${device}" \
		"${mountpoint}" \
		1>/dev/null 2>/dev/null
	if [ ${?} != 0 ]; then
		return 1
	fi

	if [ ! -e "${mountpoint}/disklabel/model" ]; then
		${UMOUNT_CMD} "${mountpoint}" 1>/dev/null 2>/dev/null
		return 1
	fi

	model=$(
		${CAT_CMD} "${mountpoint}/disklabel/model" \
			2>/dev/null
	)

	${UMOUNT_CMD} "${mountpoint}" 1>/dev/null 2>/dev/null

	echo "${model}"
	return 0
}


# 基板モデル名を設定
set_landisk_pcb()
{
	local pcb=""
	local boardmodel=$(${CAT_CMD} "${F_PROC_BOARDMODEL}" 2>/dev/null)

	case "${boardmodel}" in
		"HDL-AA")
			pcb="HDL-AA"
			;;
		"HDL2-AA")
			pcb="HDL2-AA"
			;;
		*)
			pcb="HDL-AA"
			;;
	esac

	echo "${pcb}" > "${D_RAM_LANDISK}/pcb"
	return 0
}


# 製品モデル名を設定
set_landisk_product()
{
	local product=""
	local boardmodel=$(${CAT_CMD} "${F_PROC_BOARDMODEL}" 2>/dev/null)
	local firmware=$(${CAT_CMD} "${F_ETC_FIRMWARE}" 2>/dev/null)

	case "${boardmodel}" in
		"HDL-AA")
			product="HDL-AA"
			if [ x"${firmware}" = x"HDL-AAW" ]; then
				product="HDL-AAW"
			fi
			;;
		"HDL2-AA")
			product="HDL2-AA"
			if [ x"${firmware}" = x"HDL2-AAW" ]; then
				product="HDL2-AAW"
			fi
			;;
		*)
			product="HDL-AA"
			;;
	esac

	echo "${product}" > "${D_RAM_LANDISK}/product"
	return 0
}


# ファームウェアモデル名を設定
set_landisk_firmware()
{
	local firmware=$(${CAT_CMD} "${F_ETC_FIRMWARE}" 2>/dev/null)

	echo "${firmware}" > "${D_RAM_LANDISK}/firmware"

	(
		cd "${D_RAM_LANDISK}"; \
			${LN_CMD} -sf "${D_FAVICON}/${firmware}.ico" \
			product.ico
	)

	return 0
}


# リリースバージョンを設定
set_landisk_version()
{
	local version=$(${CAT_CMD} "${F_ETC_VERSION}" 2>/dev/null)

	echo "${version}" > "${D_RAM_LANDISK}/version"
	return 0
}


# ベータバージョンを設定
set_landisk_beta()
{
	local beta=""

	if [ -e "${F_ETC_BETA}" ]; then
		beta=$(${CAT_CMD} "${F_ETC_BETA}" 2>/dev/null)
	fi

	echo "${beta}" > "${D_RAM_LANDISK}/beta"
	return 0
}


# ディスクラベル型番を設定する
set_landisk_disk_label_model()
{
	local bootdisk=$(get_bootdisk)
	local disk_label_model=$(
		get_disklabel_model \
			"${bootdisk}p4" \
			"/mnt/disklabel_sata1"
	)

	echo "${disk_label_model}" > "${D_RAM_LANDISK}/disk_label_model"
	return 0
}


# 容量を含む製品型番を設定する
set_landisk_product_full()
{
	local product_full=""
	local disk_label_model=$(
		${CAT_CMD} "${D_RAM_LANDISK}/disk_label_model" \
			2>/dev/null
	)
	local model_amount=$(
		echo "${disk_label_model}" | \
			${SED_CMD} -e 's/HDL[0-9]*\-AA\([0-9]*\)W*/\1/' 2>/dev/null
	)

	case "${product}" in
		HDL-AA*W)
			product_full="HDL-AA${model_amount}W"
			;;
		HDL-AA*)
			product_full="HDL-AA${model_amount}"
			;;
		HDL2-AA*W)
			product_full="HDL2-AA${model_amount}W"
			;;
		HDL2-AA*)
			product_full="HDL2-AA${model_amount}"
			;;
		*)
			product_full="${disk_label_model}"
			;;
	esac

	echo "${product_full}" > "${D_RAM_LANDISK}/product_full"
	return 0
}


# 無保証フラグ値を設定する
set_landisk_nowarranty()
{
	local nowarranty=$(
		${EEDATA_CMD} \
			--read \
			--nowarranty \
			2>/dev/null
	)

	echo "${nowarranty}" > "${D_RAM_LANDISK}/nowarranty"
	return 0
}


do_start()
{
	set_landisk_pcb
	set_landisk_product
	set_landisk_firmware
	set_landisk_version
	set_landisk_beta
	set_landisk_disk_label_model
	set_landisk_product_full
	set_landisk_nowarranty
}


###############################################################################

case "$1" in
	start|"")
		do_start
		;;
	restart|reload|force-reload)
		echo "Error: argument '$1' not supported" >&2
		exit 3
		;;
	stop)
		# No-op
		;;
	*)
		echo "Usage: landisk-model.sh [start|stop]" >&2
		exit 3
		;;
esac
