#!/bin/bash
# -*- mode: landisk-sh; coding: utf-8; -*-
#
# sync_folders.sh
# Copyright (C) 2013-2015 I-O DATA DEVICE, INC.  All rights reserved.
#

. /etc/product_conf.sh


C_FAL_REPLICATION="${BIN_DIR}/fal_replication.py"

F_REPLICATION_EVENTQ="/proc/fsevsync/eventq"
D_REPLICATION_LOG="/mnt/sataraid1/replication"


###############################################################################

__sf_execute_fal_replication_report()
{
	local sharename="${1}"
	local timestamp="${2}"
	local syncresult="${3}"
	local request="<request><entrylist><entry type='report'><replication><sharelist><share><name>${sharename}</name><timestamp>${timestamp}</timestamp><result>${syncresult}</result></share></sharelist></replication></entry></entrylist></request>"

	echo "${request}" | "${C_FAL_REPLICATION}" 1>/dev/null 2>/dev/null
}


###############################################################################

TARGETS_IF="${PROC_DIR}/fsevsync/targets"

INFO_DIR="${CONF_DIR}/rsync_client/"
USER_NAME_FILE="user_name"
PASSWORD_FILE="password"
PATH_FILE="path"
DST_HOST_FILE="dst_host"
DST_SHARE_FILE="dst_share"

D_SRC="${1}"
TIMESTAMP_ORG="${2}"
TIMESTAMP_UPD="${3}"

SRC_SHARE_PATH=
USER_NAME=
PASSWORD_FILE_PATH=
DST_HOST=
DST_SHARE=

# 一時的に IFS を変更する
IFS_ORIG="${IFS}"
IFS=$'\n'

for FILE in $(${FIND_CMD} "${INFO_DIR}" -name "${PATH_FILE}"); do
	SRC_SHARE_PATH=$(${CAT_CMD} "${FILE}")

	echo "${D_SRC}" | \
		${GREP_CMD} "${SRC_SHARE_PATH}/" \
		> /dev/null 2>&1
	if [ ${?} -eq 0 ]; then
		SRC_SHARE=$(${BASENAME_CMD} "${SRC_SHARE_PATH}")
		SHARE_INFO_DIR=$(${DIRNAME_CMD} "${FILE}")
		USER_NAME=$(${CAT_CMD} "${SHARE_INFO_DIR}/${USER_NAME_FILE}")
		PASSWORD_FILE_PATH="${SHARE_INFO_DIR}/${PASSWORD_FILE}"
		DST_HOST=$(${CAT_CMD} "${SHARE_INFO_DIR}/${DST_HOST_FILE}")
		DST_SHARE=$(${CAT_CMD} "${SHARE_INFO_DIR}/${DST_SHARE_FILE}")
		break
	fi
done

# IFS を元に戻す
IFS="${IFS_ORIG}"

if [ "x${DST_HOST}" = "x" ]; then
	exit 0
fi
if [ "x${DST_SHARE}" = "x" ]; then
	exit 0
fi

if [ ! -e "${D_REPLICATION_LOG}" ]; then
	${MKDIR_CMD} \
		"${D_REPLICATION_LOG}" \
		1>/dev/null 2>/dev/null
fi

F_REPLICATION_LOG="${D_REPLICATION_LOG}/${SRC_SHARE}"

D_SRC_BASE="${SRC_SHARE_PATH}"
D_DST_BASE=rsync://${USER_NAME}@[${DST_HOST}]/${DST_SHARE}/

D_SRC_SUB=$(echo "${D_SRC}" | ${SED_CMD} -e "s|${D_SRC_BASE}||" -e "s|/$||")
D_DST="${D_DST_BASE}${D_SRC_SUB}"

# dir を rename すると、dir なのに末尾 "/" が無い path を
# ${D_SRC} に受け取る場合があることへの対処
if [ -d "${D_SRC}" ]; then
	D_SRC="${D_SRC}/"
fi

#echo "${RSYNC_CMD} -rlptgoD --inplace --delete --timeout=300 --password-file=\"${PASSWORD_FILE_PATH}\" --verbose \"${D_SRC}\" \"${D_DST}\"" >> /tmp/debug
${RSYNC_CMD} \
	-rlptgoD \
	--inplace \
	--delete \
	--timeout=300 \
	--password-file="${PASSWORD_FILE_PATH}" \
	--verbose \
	"${D_SRC}" \
	"${D_DST}" \
	< /dev/null \
	> "${F_REPLICATION_LOG}.new" 2>&1
RESULT=${?}

if [ ${RESULT} -eq 0 ]; then
	${RM_CMD} -f \
		"${F_REPLICATION_LOG}.new" \
		1>/dev/null 2>/dev/null
	if [ -e "${F_REPLICATION_LOG}" ]; then
		${RM_CMD} -f \
			"${F_REPLICATION_LOG}" \
			1>/dev/null 2>/dev/null
	fi

	# 成功時はエラー時のリトライ回数ファイルを削除
	RETRY_FILE="retry"
	RETRY_FILE_PATH="${INFO_DIR}/${SRC_SHARE}/${RETRY_FILE}"

	if [ -f "${RETRY_FILE_PATH}" ]; then
		${RM_CMD} "${RETRY_FILE_PATH}"
		${EVENTLOG_CMD} \
			--code=MSG_REPLICATION_RESTART \
			"name=${SRC_SHARE}"
	fi

	# お知らせクリア
	${PYTHON_CMD} \
		${CLEAR_ANNOUNCE_CMD} all REPLICATION \
		< /dev/null \
		> /dev/null 2>&1
else
	# 23, 24 は削除しようとしてすでに不在である場合などのエラーなので無視
	if [ ${RESULT} -eq 23 -o ${RESULT} -eq 24 ]; then
		${RM_CMD} -f \
			"${F_REPLICATION_LOG}.new" \
			1>/dev/null 2>/dev/null
		if [ -e "${F_REPLICATION_LOG}" ]; then
			${RM_CMD} -f \
				"${F_REPLICATION_LOG}" \
				1>/dev/null 2>/dev/null
		fi
		exit 0
	fi

	${MV_CMD} -f \
		"${F_REPLICATION_LOG}.new" \
		"${F_REPLICATION_LOG}" \
		1>/dev/null 2>/dev/null

	# ログ出力
	${PYTHON_CMD} \
		${LOG_RSYNC_CMD} ${RESULT} "${SRC_SHARE}" \
		< /dev/null \
		> /dev/null 2>&1

	# DELDIR で監視対象から除外し、background で sleep 後に
	# ADDDIR で初期同期
	echo "DELDIR:${D_SRC_BASE}/" > "${TARGETS_IF}"
	${SYNC_FOLDERS_RETRY_CMD} "${D_SRC_BASE}" & \
		< /dev/null \
		> /dev/null 2>&1
fi

# レプリケーション記録をつける
TIMESTAMP="${TIMESTAMP_ORG}"

if ${HEAD_CMD} --lines=1 "${F_REPLICATION_EVENTQ}" | \
	${GREP_CMD} --quiet 'QUEUE ITEMS: 0'; then
	TIMESTAMP="${TIMESTAMP_UPD}"
fi

__sf_execute_fal_replication_report \
	"${SRC_SHARE}" \
	"${TIMESTAMP}" \
	"${RESULT}"
