# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL commands information module.

Note : DO NOT EDIT.  This is automatically generated.
"""

import os
import glob
import sys

from product_conf import (PYTHON_LIB_DIR,)
from product_conf.reqinfo import (ReqTypes, Results)


# Import information module for package
def __import_addon():
    """Import addon modules installed by each package."""

    try:
        ADDON_DIR = "pkgaddon"
        package_info_list = glob.glob(os.path.join(PYTHON_LIB_DIR,
                                        ADDON_DIR, "fal_*_info.py"))

        if len(package_info_list) > 0:
            for package_info in package_info_list:
                mod_name = os.path.splitext(os.path.basename(package_info))[0]

                try:
                    module = __import__(ADDON_DIR + "." + mod_name, globals(),
                                            locals(), [""])
                
                    for attr_name in dir(module):
                        if not attr_name.startswith("_"):
                            setattr(sys.modules[__name__], attr_name, 
                                    getattr(module, attr_name))
                except:
                    # Ignore all errors
                    pass
    except:
        # Ignore all errors
        pass


__import_addon()


# Base XML string for each request to FAL commands.
FAL_PROXY_REQ_BASE_XML = {
        "load": "",
        "save": "",
        "query": "<proxy><url/></proxy>",
    }

FAL_REPLICATION_REQ_BASE_XML = {
        "load": "",
        "save": "",
    }

FAL_UPS_REQ_BASE_XML = {
        "load": "",
        "save": "",
        "apply": "",
        "query": "<ups />",
    }

FAL_SYSINFO_REQ_BASE_XML = {
        "load": "",
        "save": "",
        "query": "<sysinfo></sysinfo>",
    }

FAL_NOTICE_REQ_BASE_XML = {
        "load": "",
        "save": "",
        "apply": "<notice><mail/>" +
                     "<eventnotice/><report/><regularnotice/>" +
                     "<narsus/><snmp/></notice>",
    }

FAL_POWER_REQ_BASE_XML = {
        "load": "",
        "save": "",
        "apply": "",
    }

FAL_USER_REQ_BASE_XML = {
        "load": "",
        "save": "",
        "detail": "<user><name /></user>",
        "delete": "",
        "modify": "",
        "add": "",
    }

FAL_GROUP_REQ_BASE_XML = {
        "load": "",
        "save": "",
        "detail": "",
        "delete": "",
        "modify": "",
        "add": ""
    }

FAL_BACKUP_REQ_BASE_XML = {
        "load": "",
        "save": "",
        "execute": "<backup><job num='1' /></backup>",
        "detail": "<backup><job num='0' /></backup>",
        "delete": "<backup><job num='2' /></backup>",
        "modify": "<backup><job num='3'><name />" +
                  "<times /><differential /><schedule enable='true' />" +
                  "<trash /><shutdown /><sourcelist /><destination />" +
                  "</job></backup>",
    }

FAL_MSNETWORK_REQ_BASE_XML = {
        "load": "",
        "save": "",
        "apply": "<msnetwork><nt><domain /><dcname /></nt></msnetwork>",
    }

FAL_APPLENETWORK_REQ_BASE_XML = {
        "load": "",
        "save": "",
        "apply": "<applenetwork />",
    }

FAL_FIRM_REQ_BASE_XML = {
        "load": "",
        "save": "",
        "apply": "<firmware><notice/>" +
                     "<autoupdate><time><hour/><minute/></time></autoupdate>" +
                     "<update><filepath/></update></firmware>",
        "execute": "<firmware><notice/>" +
                     "<autoupdate><time><hour/><minute/></time></autoupdate>" +
                     "<update><filepath/></update></firmware>",
    }

FAL_NETWORK_REQ_BASE_XML = {
        "load": "",
        "save": "",
        "query": "<network/>",
        "apply": "<network/>",
    }

FAL_PACKAGES_REQ_BASE_XML = {
        "load": "",
        "save": "",
        "apply": "<packages/>",
        "list": "",
        "query": "<packages/>",
        "install": "<packages><packagelist><package><packageid/></package>" +
                    "</packagelist></packages>",

        "uninstall": "<packages><packagelist><package><packageid/></package>" +
                    "</packagelist></packages>",
    }

FAL_MESSAGES_REQ_BASE_XML = {
        "query": "<messages><messageinfo><code/></messageinfo></messages>",
    }

FAL_URLACCESS_REQ_BASE_XML = {
        "send": "<urlaccess><url/></urlaccess>",
        "receive": "<urlaccess><url/></urlaccess>",
        "get": "<urlaccess><url/></urlaccess>",
    }

FAL_EVENTS_REQ_BASE_XML = {
        "register": "<events><eventinfo/></events>",
        "unregister": "<events><eventinfo/></events>",
        "event": "<events><eventinfo/></events>",
    }

FAL_SERVICECTRL_REQ_BASE_XML = {
        "load": "",
        "save": "",
        "apply": "<servicectrl><servicelist/></servicectrl>",
        "query": "<servicectrl><servicelist/></servicectrl>",
        "start": "<servicectrl><servicelist/></servicectrl>",
        "stop": "<servicectrl><servicelist/></servicectrl>",
        "suspend": "<servicectrl><servicelist/></servicectrl>",
        "resume": "<servicectrl><servicelist/></servicectrl>",
    }

FAL_STATEMGR_REQ_BASE_XML = {
        "query": "<statemgr><statelist/></statemgr>",
        "set": "<statemgr><state/></statemgr>",
        "change": "<statemgr><state/></statemgr>",
    }


# buzzer names.
BUZZER_BOOTUP = "bootup"
BUZZER_OK = "ok"
BUZZER_ERROR = "error"
BUZZER_COPYEND = "copyend"
BUZZER_RAIDOK = "raidok"
BUZZER_RAIDFAIL = "raidfail"
BUZZER_RAIDCRASH = "raidcrash"
BUZZER_UPS = "ups"
BUZZER_CAPACITY = "capacity"
BUZZER_DONGLE = "dongle"
BUZZER_FANERROR = "fanerror"
BUZZER_MELODY = "melody"
BUZZER_UPDATE = "update"
BUZZER_LUMPREGISTEND = "lumpregistend"
BUZZER_FORMATEND = "formatend"
BUZZER_BGPROCESSEND = "bgprocessend"
BUZZER_LOCKMODE = "lockmode"


# Installed FAL commands path.
INSTALL_PATH = "/mnt/data/bin"

FAL_USERS_PATH = INSTALL_PATH + "/fal_users.py"
FAL_FTP_PATH = INSTALL_PATH + "/fal_ftp.py"
FAL_ECO_PATH = INSTALL_PATH + "/fal_eco.py"
FAL_DISKCTRL_PATH = INSTALL_PATH + "/fal_diskctrl.py"
FAL_QRCODE_PATH = INSTALL_PATH + "/fal_qrcode.py"
FAL_LUMPREGIST_PATH = INSTALL_PATH + "/fal_lumpregist.py"
FAL_SHARES_PATH = INSTALL_PATH + "/fal_shares.py"
FAL_MAILSEND_PATH = INSTALL_PATH + "/fal_mailsend.py"
FAL_NARSUS_PATH = INSTALL_PATH + "/fal_narsus.py"
FAL_GROUPS_PATH = INSTALL_PATH + "/fal_groups.py"
FAL_SYSTEMLOG_PATH = INSTALL_PATH + "/fal_systemlog.py"
FAL_ANNOUNCE_PATH = INSTALL_PATH + "/fal_announce.py"
FAL_SERVICECTRL_PATH = INSTALL_PATH + "/fal_servicectrl.py"
FAL_RAIDCTRL_PATH = INSTALL_PATH + "/fal_raidctrl.py"
FAL_EVENTS_PATH = INSTALL_PATH + "/fal_events.py"
FAL_URLACCESS_PATH = INSTALL_PATH + "/fal_urlaccess.py"
FAL_PORTS_PATH = INSTALL_PATH + "/fal_ports.py"
FAL_LOCKKEY_PATH = INSTALL_PATH + "/fal_lockkey.py"
FAL_UPS_PATH = INSTALL_PATH + "/fal_ups.py"
FAL_POWER_PATH = INSTALL_PATH + "/fal_power.py"
FAL_LED_PATH = INSTALL_PATH + "/fal_led.py"
FAL_FEATURES_PATH = INSTALL_PATH + "/fal_features.py"
FAL_MESSAGES_PATH = INSTALL_PATH + "/fal_messages.py"
FAL_NETWORK_PATH = INSTALL_PATH + "/fal_network.py"
FAL_VOLCTRL_PATH = INSTALL_PATH + "/fal_volctrl.py"
FAL_DEVCTRL_PATH = INSTALL_PATH + "/fal_devctrl.py"
FAL_SYSINFO_PATH = INSTALL_PATH + "/fal_sysinfo.py"
FAL_REPLICATION_PATH = INSTALL_PATH + "/fal_replication.py"
FAL_STATEMGR_PATH = INSTALL_PATH + "/fal_statemgr.py"
FAL_QUOTA_PATH = INSTALL_PATH + "/fal_quota.py"
FAL_MSNETWORK_PATH = INSTALL_PATH + "/fal_msnetwork.py"
FAL_LCM_PATH = INSTALL_PATH + "/fal_lcm.py"
FAL_SELFCHECK_PATH = INSTALL_PATH + "/fal_selfcheck.py"
FAL_DROPBOX_PATH = INSTALL_PATH + "/fal_dropbox.py"
FAL_SNMP_PATH = INSTALL_PATH + "/fal_snmp.py"
FAL_APPLENETWORK_PATH = INSTALL_PATH + "/fal_applenetwork.py"
FAL_SWITCH_PATH = INSTALL_PATH + "/fal_switch.py"
FAL_PACKAGES_PATH = INSTALL_PATH + "/fal_packages.py"
FAL_EVENTLOG_PATH = INSTALL_PATH + "/fal_eventlog.py"
FAL_INITIALIZE_PATH = INSTALL_PATH + "/fal_initialize.py"
FAL_AMAZONS3_PATH = INSTALL_PATH + "/fal_amazons3.py"
FAL_PROXY_PATH = INSTALL_PATH + "/fal_proxy.py"
FAL_BACKUPS_PATH = INSTALL_PATH + "/fal_backups.py"
FAL_DATETIME_PATH = INSTALL_PATH + "/fal_datetime.py"
FAL_NOTICE_PATH = INSTALL_PATH + "/fal_notice.py"
FAL_FIRMWARE_PATH = INSTALL_PATH + "/fal_firmware.py"
FAL_VALIDATE_PATH = INSTALL_PATH + "/fal_validate.py"
FAL_PRINTER_PATH = INSTALL_PATH + "/fal_printer.py"
FAL_AZURE_PATH = INSTALL_PATH + "/fal_azure.py"
FAL_BUZZER_PATH = INSTALL_PATH + "/fal_buzzer.py"
