# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""Variables for Product wide setting"""

from product_conf import (NET_IF_LIST,)

#
# Device/Volume Information
#

# For network
NET_DEV_LIST = ("netif1",)
NET_RAW_DEV_LIST = ("eth0",)
NET_IF_TO_DEV = dict(zip(NET_IF_LIST, NET_DEV_LIST))
NET_DEV_TO_IF = dict(zip(NET_IF_TO_DEV.values(), NET_IF_TO_DEV.keys()))
NET_DEV_TO_RAW_DEV = dict(zip(NET_DEV_LIST, NET_RAW_DEV_LIST))


# For LCM
LCM_SUPPORTED = False


# For storage
RAID_DEV_LIST = ("md2", "md5", "md13")
SYSTEM_RAID_DEV_LIST = ("md2", "md5")
INTERNAL_RAID_DEV_LIST = ()
SATA_HDD_DEV_LIST = ("satad1",)

ESATA_HDD_DEV_LIST = ()
USB_HDD_DEV_LIST = ("usbd1", "usbd2")
USB_UPS_DEV_LIST = ("usbups1", "usbups2")
SERIAL_UPS_DEV_LIST = ()

UPS_VENDOR_APC = "apc"
UPS_VENDOR_OMRON = "omron"
UPS_VENDOR_FUJI = "fuji"
UPS_VENDOR_SUFFIX_LIST = (UPS_VENDOR_APC, UPS_VENDOR_OMRON, UPS_VENDOR_FUJI)
UPS_SERIAL_TYPE_SUFFIX_LIST = ("smart",)
USB_LP_DEV_LIST = ("usblp1", "usblp2")
LOCKKEY_VOL_LIST = ("usb1",)
MIRRORING_VOL_LIST = ("usb2",)

REAL_USB_UPS_LIST = \
    tuple(".".join(name_suf) for name_suf in \
        zip(sorted(USB_UPS_DEV_LIST * len(UPS_VENDOR_SUFFIX_LIST)),
            UPS_VENDOR_SUFFIX_LIST * len(USB_UPS_DEV_LIST)))
REAL_SERIAL_UPS_LIST = \
    tuple(".".join(name_suf) for name_suf in \
        zip(SERIAL_UPS_DEV_LIST * len(UPS_SERIAL_TYPE_SUFFIX_LIST),
            tuple(".".join(suffix) for suffix in \
                zip(UPS_SERIAL_TYPE_SUFFIX_LIST,
                    [UPS_VENDOR_APC] * len(UPS_SERIAL_TYPE_SUFFIX_LIST)))))
REAL_UPS_DEV_LIST = REAL_USB_UPS_LIST + REAL_SERIAL_UPS_LIST

PORT_IF_LIST = ("sata", "usb",)
SATA_PORT_NO_LIST = ("1",)
ESATA_PORT_NO_LIST = ()
USB_PORT_NO_LIST = ("1", "2")
SERIAL_PORT_NO_LIST = ()

PORT_IF_TO_NO_LIST = dict(zip(PORT_IF_LIST,
                                (SATA_PORT_NO_LIST, USB_PORT_NO_LIST)))

FAN_NO_LIST = ()

HDD_DEV_LIST = SATA_HDD_DEV_LIST + USB_HDD_DEV_LIST + ESATA_HDD_DEV_LIST
UPS_DEV_LIST = USB_UPS_DEV_LIST + SERIAL_UPS_DEV_LIST
LP_DEV_LIST = USB_LP_DEV_LIST
ALL_DEV_LIST = HDD_DEV_LIST + ESATA_HDD_DEV_LIST + UPS_DEV_LIST + LP_DEV_LIST

DEV_TO_PORT_IF = dict(zip(SATA_HDD_DEV_LIST,
                            ("sata",) * len(SATA_HDD_DEV_LIST)) +
                    zip(USB_HDD_DEV_LIST,
                            ("usb",) * len(USB_HDD_DEV_LIST)) +
                    zip(UPS_DEV_LIST,
                            ("usb",) * len(USB_HDD_DEV_LIST) + ("serial",)) +
                    zip(USB_LP_DEV_LIST,
                            ("usb",) * len(USB_HDD_DEV_LIST)))

DEV_TO_PORT_NO = dict(zip(SATA_HDD_DEV_LIST, SATA_PORT_NO_LIST) +
                    zip(USB_HDD_DEV_LIST, USB_PORT_NO_LIST) +
                    zip(UPS_DEV_LIST, USB_PORT_NO_LIST + SERIAL_PORT_NO_LIST) +
                    zip(USB_LP_DEV_LIST, USB_PORT_NO_LIST))

PORT_IF_TO_DEVS = dict(zip(PORT_IF_LIST,
                        (dict(zip(SATA_PORT_NO_LIST, zip(SATA_HDD_DEV_LIST))),
                            dict(zip(USB_PORT_NO_LIST,
                                zip(USB_HDD_DEV_LIST, USB_UPS_DEV_LIST,
                                    USB_LP_DEV_LIST))),
                            dict(zip(SERIAL_PORT_NO_LIST,
                                zip(SERIAL_UPS_DEV_LIST))))))

DEV_TO_ENC_DEV = {
                "sataraidd1": "sataraidd1_enc",
                "usbd1": "usbd1_enc",
                "usbd2": "usbd2_enc",
                "satad1p3": "satad1p3_enc",
                "satad1p6": "satad1_vfs_enc",
                "usbd2p3": "usbd2p3_enc",
                "usbd2p6": "usbd2_enc",
    }

RAID_MODE_TO_REAL_RAID_MODE = {
                "vfsext"  : "vfsext",
                "vfsext_6": "vfsext",
                "vfsext_4": "vfsext",
                "vfsext_3": "vfsext_3",
                "vfsext_2": "vfsext_2",
                "raid0"   : "raid0",
                "raid0_6" : "raid0",
                "raid0_4" : "raid0",
                "raid0_3" : "raid0",
                "raid0_2" : "raid0",
                "raid1"   : "raid1",
                "raid1_4" : "raid1",
                "raid1_3" : "raid1",
                "raid1_2" : "raid1",
                "raid5"   : "raid5",
                "raid5_6" : "raid5",
                "raid5_4" : "raid5",
                "raid6"   : "raid6",
                "raid6_6" : "raid6",
                "raid6_4" : "raid6",
                "single"  : "single",
    }

EXTEND_RAID_MODE_MAP = {
                "vfsext"  : (),
                "raid0"   : (),
                "raid1"   : (),
                "single"  : (),
}

_RAID_MODE_LIST = {
        1: (),
    }

VOL_INFO = {
        "sataraid1": {
                "dev": "sataraidd1",
                "part_base": "",
                "mode_list": _RAID_MODE_LIST,
                "mnt_point": "/mnt/sataraid1",
                "alt_mnt_point": "/mnt/share/sataraid1",
                "swap_dev_list": {
                        1: tuple([dev + "p3" for dev in SATA_HDD_DEV_LIST]),
                },
                "share_root": "share",
            },
        "usb1": {
                "dev": "usbd1",
                "part_base": "usbd1p",
                "mode_list": (),
                "mnt_point": "/mnt/usb1",
                "alt_mnt_point": "/mnt/share/usb1",
                "share_root": "",
            },
        "usb2": {
                "dev": "usbd2",
                "part_base": "usbd2p",
                "mode_list": (),
                "mnt_point": "/mnt/usb2",
                "alt_mnt_point": "/mnt/share/usb2",
                "share_root": "",
            },
    }

DEV_TO_VOL = {
       "sataraidd1": "sataraid1",
       "usbd1": "usb1",
       "usbd2": "usb2",
    }

HDD_DEV_LIST_IN_VOL = {
        1:{
              "sataraid1": SATA_HDD_DEV_LIST + ("usbd2",),
              "usb1": ("usbd1",),
              "usb2": ("usbd2",),
        },
    }

RAID_MODE_INFO = {
        "vfsext": {
                "array_num": 1,
                "is_ext_raid": True,
        },
        "raid0": {
                "array_num": 1,
                "is_ext_raid": False,
        },
        "raid1": {
                "array_num": 1,
                "is_ext_raid": False,
        },
        "single": {
                "array_num": 1,
                "is_ext_raid": False,
        },
    }

RAID_TO_REAL_NAME = {
        "sataraidd1" : "md13",
    }

REAL_TO_RAID_NAME = \
    dict((real, symbl) for symbl, real in RAID_TO_REAL_NAME.iteritems())

RAID_TO_ELEMENT = {
        1: {
                "sataraidd1" : ("satad1p6", "usbd2p6", "missing"),
                "md2"  : tuple([dev + "p2" for dev in SATA_HDD_DEV_LIST] +
                                ["usbd2p2", "missing"]),
                "md5"  : tuple([dev + "p5" for dev in SATA_HDD_DEV_LIST] +
                                ["usbd2p5", "missing"]),
        },
    }

RAID_TO_LEVEL = {
        "sataraidd1": "raid1",
        "md2": "raid1",
        "md5": "raid1",
    }

EXT_RAID_TO_ELEMENT = {
        1: {},
    }

VFSEXT_INFO = {
              "filesystem": "xfs",
    }

EXT_RAID_DEV_INFO = {} 

MOUNT_OPT_INFO = {
        "xfs"  : "defaults,usrquota,grpquota,allocsize=64k",
        "vfat" : "codepage=932,utf8,shortname=winnt,umask=000,uid=nobody,gid=nogroup",
        "ext3" : "relatime",
        "ext4" : "relatime",
        "ntfs-fuse" : "ro,relatime,locale=ja_JP.UTF-8,umask=000",
        "vfsext": "usrquota,grpquota",
    }

SUPPORTED_FSCK_FSYS = ("ext3", "ext4", "xfs",)

SYS_VOL_INFO = {
        "rootfs": {
            "dev": "md2",
            "mnt_point": "/",
        },
        "boot": {
            "devlist": {
                1: tuple([dev + "p1" for dev in SATA_HDD_DEV_LIST])
            },
            "mnt_point": "/boot",
        }
    }

