// jquery & jquery-bottom & jquery-top & utils に依存
// *** 注意 ***
// これに渡された table はデータ構造が変わるので他の処理に影響を与える可能性有
// 詳しくは _scrollable() を参照
// ************
var Log = function(table, options) {
	var that = this;
	var table = typeof table == 'object' ? table : document.getElementById(table);
	this.tbody = getChildNodes(table, Array('tbody'))[0];

	this.options = $.extend(
		{
			max: 50,  // 保持するログの最大件数
			num: 10,  // 一度に読み込むログの件数
			init: 30,  // 最初に読み込むログの件数
			height: 500  // テーブルの高さ
		},
		options);

	this.head = 0;  // 保持しているログの先頭
	this.tail = 0;  // 保持しているログの末尾

	// tbody部分だけスクロール可能にする
	this._scrollable(table, this.options.height + 'px');

	// bottom&topイベントをバインド
	this._bind();

	// FIXME ここでスクロールバーがでる程度にログを読み込まないと
	// scrollイベントが発生しないので動かなくなる
	this.load(1, this.options.init, false);
}
Log.prototype = {
	up: function() {
		this.scroll.scrollTop -= this.options.height;
	},
	down: function() {
		this.scroll.scrollTop += this.options.height;
	},
	next: function() {
		this.load(this.tail+1, this.tail+this.options.num, false);
	},
	prev: function() {
		if (this.head <= 1) {  // this.head は0以下にならないが一応
			// これ以上前のログはないので何もしない
		} else if (this.head-this.options.num < 1) {
			this.load(1, this.head-1, true);
		} else {
			this.load(this.head-this.options.num, this.head-1, true);
		}
	},
	load: function(from, to, head) {
		var that = this;
		this._unbind();
		$.ajax({
			type:'GET',
			url:'./next/'+from+'-'+to,
			cache:false,
			dataType:'json',
			timeout:10000,
			success:function(data, dataType){
				if (head) {
					that.addHead(data);
				} else {
					that.addTail(data);
				}
			},
			error:function(XMLHttpRequest, textStatus, errorThrown){
//FIXME
			},
			complete:function(XMLHttpRequest, textStatus){
				that._bind();
			}
		});
	},
	addTail: function(data) {
		for (var i=0; i<data.length; i++) {
			var tr = document.createElement('tr');
			for (var j=0; j<data[i].length; j++) {
				var td = document.createElement('td');
				td.appendChild(document.createTextNode(data[i][j]));
				tr.appendChild(td);
			}
			var trs = getChildNodes(this.tbody, Array('tr'));
			if (trs.length >= this.options.max) {
				this.tbody.appendChild(tr);
				var _scrollHeight = this.scroll.scrollHeight;
				this.tbody.removeChild(trs[0])
				this.scroll.scrollTop = this.scroll.scrollTop - (_scrollHeight - this.scroll.scrollHeight);
				this.head++;
				this.tail++;
			} else {
				this.tbody.appendChild(tr);
				this.head = 1;
				this.tail++;
			}
		}
	},
	addHead: function(data) {
		for (var i=data.length-1; i>=0; i--){
			var tr = document.createElement('tr');
			for (var j=0; j<data[i].length; j++) {
				var td = document.createElement('td');
				td.appendChild(document.createTextNode(data[i][j]));
				tr.appendChild(td);
			}
			var trs = getChildNodes(this.tbody, Array('tr'));
			if (trs.length >= this.options.max) {
				var _scrollHeight = this.scroll.scrollHeight;
				this.tbody.insertBefore(tr, trs[0]);
				this.scroll.scrollTop = this.scroll.scrollTop + (this.scroll.scrollHeight - _scrollHeight);
				this.tbody.removeChild(trs[trs.length-1]);
				this.head--;
				this.tail--;
			} else {
				this.tbody.insertBefore(tr, trs[0]);
				this.head--;
				this.tail = this.options.max;
			}
		}
	},
	_unbind: function() {
		$(this.scroll).unbind('bottom');
		$(this.scroll).unbind('top');
		$(this.scroll).unbind('scroll');
	},
	_bind: function() {
		var that = this;
		$(this.scroll).bottom({proximity:0.01});
		$(this.scroll).bind('bottom', function(e){that.next();});
		$(this.scroll).top({proximity:0.01});
		$(this.scroll).bind('top', function(e){that.prev();});
	},
	_scrollable: function(table, height) {
		var CLASS_CONTAINER = 'scrollableContainer';
		var CLASS_CONTENT = 'scrollableContent';
		var CLASS_TABLE = 'scrollableTable';
		var THEAD_HEIGHT = '30px';
		//<div class="scrollableContainer">
		//  <div class="scrollableContent">
		//    <table class="scrollableTable ~~~">
		//      <thead>~~~</thead>
		//      <tbody>~~~</tbody>
		// 上記のようにHTML構造を変更
		var container = document.createElement('div');
		container.className = CLASS_CONTAINER;
		var content = document.createElement('div');
		content.className = CLASS_CONTENT;
		container.appendChild(content);
		table.parentNode.insertBefore(container, table);
		content.appendChild(table.parentNode.removeChild(table));
		this.scroll = content;
		table.className = CLASS_TABLE + ' ' + table.className;

		// tbodyだけをスクロールするようにcssルールを追加
		this._addCSS();
		stylesheet = document.styleSheets.item(0);
		stylesheet.addCSS('.'+CLASS_CONTAINER, 'position', 'relative');
		stylesheet.addCSS('.'+CLASS_CONTAINER, 'padding-top', THEAD_HEIGHT);
		stylesheet.addCSS('.'+CLASS_CONTENT, 'height', height);
		stylesheet.addCSS('.'+CLASS_CONTENT, 'overflow-y', 'scroll');
		stylesheet.addCSS('.'+CLASS_CONTENT, 'overflow-x', 'hidden');
		stylesheet.addCSS('.'+CLASS_TABLE+' thead tr', 'position', 'absolute');
		stylesheet.addCSS('.'+CLASS_TABLE+' thead tr', 'top', '0');
		stylesheet.addCSS('.'+CLASS_TABLE+' thead tr', 'left', '0');
		stylesheet.addCSS('.'+CLASS_TABLE+' thead tr', 'height', THEAD_HEIGHT);

		var thead = getChildNodes(table, Array('thead'))[0];
		var tr = getChildNodes(thead, Array('tr'))[0];
		var ths = getChildNodes(tr, Array('th'));
		for (var i=0; i<ths.length; i++) {
			stylesheet.addCSS('.'+CLASS_TABLE+' tbody td:nth-of-type('+(i+1)+')', 'width', ths[i].width);
		}
	},
	_addCSS: function() {
		var style = document.createElement('style');
		style.setAttribute('type', 'text/css');
		document.getElementsByTagName('head')[0].appendChild(style);
		var sheet = document.styleSheets[0];
		if (sheet.insertRule) {
			sheet.addCSS = function(selector, property, value) {
				sheet.insertRule(selector + '{' + property + ':' + value + ';}', sheet.cssRules.length);
			};
		} else {
			sheet.addCSS = function(selector, property, value) {
				sheet.addRule(selector, property + ':' + value);
			};
		}
	}
}
