# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta


def run(data):
    from root.admin.network.modules.network import Network
    from time import sleep
    from project.fal.statemgr import Statemgr
    from project.fal.command import logging

    while Statemgr().query()['filesystem'] != 'nooperation':
        sleep(1)

    nw = Network()

    nwok = True

    data1 = {
        'dhcp': data.lan1_dhcp,
        'ipaddr': data.lan1_ipaddr,
        'netmask': data.lan1_netmask,
        'framesize': data.lan1_framesize,
        'wol': data.lan1_wol,
        'common': data.nw_common,
        'gateway': data.nw_gateway,
        'dns': data.nw_dns,
    }
    if data.nw_mainif:
        data1['mainif'] = data.nw_mainif
    nwok = nwok and nw.lan1_ipv4_set(data=data1)
    while Statemgr().query()['system'] == 'busy':
        sleep(1)

    if data.lan2_dhcp:  # FIXME
        data2 = {
            'dhcp': data.lan2_dhcp,
            'ipaddr': data.lan2_ipaddr,
            'netmask': data.lan2_netmask,
            'framesize': data.lan2_framesize,
            'wol': data.lan2_wol,
            'common': data.nw_common,
            'gateway': data.nw_gateway,
            'dns': data.nw_dns,
        }
        if data.nw_mainif:
            data2['mainif'] = data.nw_mainif
        nwok = nwok and nw.lan2_ipv4_set(data=data2)
        while Statemgr().query()['system'] == 'busy':
            sleep(1)

    if nwok:
        logging("MSG_NET_CHANGE")
    else:
        logging("ERR_NET_CHANGE")

    sleep(100)

    from root.admin.usergroup.msusergroup.modules.msnetwork import MSnetwork
    msok = MSnetwork().msnetwork_set({
        'method': {'wg':'1','nt':'2','ad':'3'}.get(data.method),
        'wg_name': data.wg_name,
        'nt_name': data.nt_name,
        'nt_dcname': data.nt_dcname,
        'netbios': data.ad_netbios,
        'fulldns': data.ad_fulldns,
        'ad_dcname': data.ad_dcname,
        'adminname': data.ad_adminname,
        'passwd': data.ad_passwd,
        'winssrv': data.winssrv,
    })
    if msok:
        logging("MSG_MS_NET_CHANGE")
    else:
        logging("ERR_MS_NET_CHANGE")


if __name__ == '__main__':
    from optparse import OptionParser
    op = OptionParser()

    op.add_option('--lan1-dhcp',
                  action='store',
                  type='string',
                  dest='lan1_dhcp')
    op.add_option('--lan1-ipaddr',
                  action='store',
                  type='string',
                  dest='lan1_ipaddr')
    op.add_option('--lan1-netmask',
                  action='store',
                  type='string',
                  dest='lan1_netmask')
    op.add_option('--lan1-framesize',
                  action='store',
                  type='string',
                  dest='lan1_framesize')
    op.add_option('--lan1-wol-on',
                  action='store_true',
                  dest='lan1_wol',
                  default=False)
    op.add_option('--lan2-dhcp',
                  action='store',
                  type='string',
                  dest='lan2_dhcp')
    op.add_option('--lan2-ipaddr',
                  action='store',
                  type='string',
                  dest='lan2_ipaddr')
    op.add_option('--lan2-netmask',
                  action='store',
                  type='string',
                  dest='lan2_netmask')
    op.add_option('--lan2-framesize',
                  action='store',
                  type='string',
                  dest='lan2_framesize')
    op.add_option('--lan2-wol-on',
                  action='store_true',
                  dest='lan2_wol',
                  default=False)
    op.add_option('--nw-common',
                  action='store',
                  type='string',
                  dest='nw_common')
    op.add_option('--nw-mainif',
                  action='store',
                  type='string',
                  dest='nw_mainif')
    op.add_option('--nw-gateway',
                  action='store',
                  type='string',
                  dest='nw_gateway')
    op.add_option('--nw-dns',
                  action='store',
                  type='string',
                  dest='nw_dns')

    op.add_option('--method',
                  action='store',
                  type='string',
                  dest='method')
    op.add_option('--wg-name',
                  action='store',
                  type='string',
                  dest='wg_name')
    op.add_option('--nt-name',
                  action='store',
                  type='string',
                  dest='nt_name')
    op.add_option('--nt-dcname',
                  action='store',
                  type='string',
                  dest='nt_dcname')
    op.add_option('--ad-netbios',
                  action='store',
                  type='string',
                  dest='ad_netbios')
    op.add_option('--ad-fulldns',
                  action='store',
                  type='string',
                  dest='ad_fulldns')
    op.add_option('--ad-dcname',
                  action='store',
                  type='string',
                  dest='ad_dcname')
    op.add_option('--ad-adminname',
                  action='store',
                  type='string',
                  dest='ad_adminname')
    op.add_option('--ad-passwd',
                  action='store',
                  type='string',
                  dest='ad_passwd')
    op.add_option('--winssrv',
                  action='store',
                  type='string',
                  dest='winssrv')

    try:
        import os
        import sys

        pid = os.fork()
        if pid > 0:
            sys.exit(0)
        elif pid < 0:
            sys.exit(0)

        os.setsid()
        pid = os.fork()
        if pid > 0:
            sys.exit(0)

        from project.fal.led import LED
        from project.fal.buzzer import Buzzer
        LED().status(True, color='green', blink=True)
        try:
            (opts, args) = op.parse_args()
            run(opts)
        except:
            pass
        LED().status(True, color='green', blink=False)
        Buzzer().ok()

    except:
        pass
