# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from command import Command, escape, _
from ionas.request import Entry, XmlNode
import os
try:
    from fal_info import FAL_MESSAGES_PATH
except:
    FAL_MESSAGES_PATH = '/mnt/data/bin/fal_messages.py'


class Announce(Command):
    try:
        from fal_info import FAL_ANNOUNCE_PATH as COMMAND
    except:
        COMMAND_NAME = 'fal_announce.py'

    TYPE_QUERY = 'query'
    TYPE_CLEAR = 'clear'

    LEVEL = {
        0: _(u'Info'),
        1: _(u'Warning'),
        2: _(u'Error'),
    }
    level = lambda self, l: self.LEVEL.get(int(l), _(u'Unknown'))

    def query(self, to_list=True):
        entry = self.request(Entry(self.TYPE_QUERY, '<announce />'))
        return self._to_list(entry) if to_list else entry

    def _to_list(self, entry):
        announce_list = []
        try:
            entry_list = []
            for announce in entry.get().get_node_list('announce'):
                attrs = announce.get_attrs()
                level = attrs.get('level', None)  # FIXME None
                category = attrs.get('category', None)  # FIXME None
                code = announce.get_node('code').get_text()
                try:
                    arglist = dict([(n.get_attrs()['name'], n.get_text()) \
                            for n in announce.get_node_list('arglist/arg')])
                except:
                    arglist = []

                announce_list.append({
                    'code': code,
                    'arglist': arglist,
                    'level': self.level(level),
                    'category': category,
#                    'message': self._get_message(code, arglist=arglist),
                    })

                entry_list.append(Entry('query', """<messages>
                    <messageinfo>
                        <code>%(code)s</code>
                        <arglist>%(arglist)s</arglist>
                    </messageinfo>
                </messages>""" % {
                    'code': code,
                    'arglist': ''.join(['<arg name="%s">%s</arg>' % (
                            escape(name), escape(val))
                            for name, val in arglist.iteritems()]),
                }))

            messages = self._request(
                    entry_list,
                    command=FAL_MESSAGES_PATH)
#                    command=os.path.join(self.COMMAND_DIR, 'fal_messages'))
            for (e, a) in zip(
                    messages.get_entrylist().get_entry_list(),
                    announce_list):
                try:
                    a['message'] = e.get().get_value('messageinfo/message', '')
                except:
                    pass
        except:
            pass
        return announce_list

    def clear(self, level=1):  # level=1/2/all
        entry = self.request(Entry(self.TYPE_CLEAR, """<announce>
            <option level="%(level)s" /></announce>""" % {'level': level}))
        return entry

if __name__ == '__main__':
    i = Announce()
    announce_list = i.query()
    print "Query: %s" % i.get_result()
    print announce_list
    i.clear(level=2)
    print "Clear: %s" % i.get_result()
