# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
#from django.utils.translation import ugettext as _
from django.utils.translation import ugettext
from ionas.request import Request, Entry, EntryList
import re
import os
import traceback

try:
    from fal_info import INSTALL_PATH, FAL_MESSAGES_PATH, FAL_EVENTLOG_PATH
    from product_conf import SUDO_CMD
except ImportError:
    INSTALL_PATH = '/mnt/data/bin'
    FAL_MESSAGES_PATH = INSTALL_PATH + '/fal_messages.py'
    FAL_EVENTLOG_PATH = INSTALL_PATH + '/fal_eventlog.py'
    SUDO_CMD = '/usr/bin/sudo'


def _(u):
    try:
        return ugettext(u)
    except:
        return u


def escape(text):
    from xml.sax.saxutils import escape
    return escape(text, {}) if isinstance(text, (str, unicode)) else text


def _get_code(code, default=None):
    try:
        from fal_msginfo import Msg
        return getattr(Msg, code)
    except:
        return default


def logging(code, arglist={}):
    try:
        code = _get_code(code, default=code)
#        cmd = os.path.join(INSTALL_PATH, 'fal_eventlog.py')
        cmd = FAL_EVENTLOG_PATH
        xml = """<eventlog>
            <log>
                <code>%(code)s</code>
                <arglist>%(arglist)s</arglist>
            </log>
        </eventlog>""" % {
            'code': code,
            'arglist': ''.join(['<arg name="%(k)s">%(v)s</arg>' % {
                    'k':k, 'v':v} for k, v in arglist.iteritems()]),
        }
        request = Request(cmd, filter_cmd=SUDO_CMD)
        response = request.request(Entry('add', xml))
        entry = response.get_entrylist().get_entry()
        if int(entry.get_result()):
            return False
        else:
            return True
    except:
#        print traceback.format_exc()
        return False


def get_message(code, arglist={}, default=_(u'Unknown'), is_log=False):  # FIXME default
    try:
#        not is_log or logging(code, arglist=arglist)
        code = _get_code(code, default=code)
#        cmd = os.path.join(INSTALL_PATH, 'fal_messages')
        cmd = FAL_MESSAGES_PATH
        xml = """<messages>
            <messageinfo>
                <code>%(code)s</code>
                <arglist>%(arglist)s</arglist>
            </messageinfo>
        </messages>""" % {
            'code': code,
            'arglist': ''.join(['<arg name="%(k)s">%(v)s</arg>' % {
                    'k':k, 'v':v} for k, v in arglist.iteritems()]),
        }
        request = Request(cmd, filter_cmd=SUDO_CMD)
        response = request.request(Entry('query', xml))
        entry = response.get_entrylist().get_entry()
        message = entry.get().get_node('messageinfo/message').get_text()
        not is_log or logging(code, arglist=arglist)
        return message
    except:
        return default

def _dict_to_xml(_dict, table={True: "true", False: "false"}):
    def _dtx(_dict, ptag):
        ctags, attrs = {}, {}
        for k, v in _dict.iteritems():
            m = re.match("__([a-zA-Z0-9\_\-\.]+)__", k)
            if m:  # "__???__"形式は属性と扱う
                attrs[m.group(1)] = v
            else:
                ctags[k] = v

        xml = ""
        xml += '<%s%s>' % (
                ptag, "".join([' %s="%s"' % (str(k), table.get(v, str(v))) \
                        for k, v in attrs.iteritems()]))
        for tag, val in ctags.iteritems():
            if isinstance(val, dict):
                xml += _dtx(val, tag)
            else:
                xml += '<%s>%s</%s>' % (tag, val, tag)
        xml += '</%s>' % (ptag, )
        return xml

    xml = ""
    for k, v in _dict.iteritems():
        xml += _dtx(v, k)
    return xml

def encode_share_id(share_name, length=32):
    """Encode share ID from share name."""

    import hashlib

    md5 = hashlib.md5()
    md5.update(share_name)
    hex32 = md5.hexdigest()
    share_id = (("%%0%dd" % length) % int(hex32, 16))[:length]
    return share_id


class Command(object):
    TRUE_TEXT = 'true'
    FALSE_TEXT = 'false'

    BOOL_TO_TEXT = {
        True: 'true',
        False: 'false',
    }
    TEXT_TO_BOOL = {
        'true': True,
        'false': False,
    }
    # 上の text <-> bool 変換は過去互換のため一応残しておく
    bool_to_text = lambda self, b: str(b).lower()
    text_to_bool = lambda self, t: {'true': True,
                                    'false': False}.get(t.lower(), False)

    def dict_to_entry(self, _dict, _type):
        return Entry(_type, _dict_to_xml(_dict))

    COMMAND_DIR = INSTALL_PATH
    COMMAND_NAME = 'fal_dummy'
    COMMAND = None

    # リクエスト時に更新する
    _entry = None
    _message = None

    def __init__(self):
        self.COMMAND = self.COMMAND or \
                os.path.join(self.COMMAND_DIR, self.COMMAND_NAME)

    def _request(self, entrylist, command=None):
        try:
            #FIXME entrylist の処理 (EntryList / Entry / [Entry, ...])
            if isinstance(entrylist, (list, tuple)):
                el = EntryList()
                for e in entrylist:
                    el.append_entry(e)
                entrylist = el
            request = Request(command or self.COMMAND, filter_cmd=SUDO_CMD)
            response = request.request(entrylist)
            return response
        except:
#            import traceback
#            print traceback.print_exc()
            return None

    def request(self, entry, messages=None):
#        print entry.entry
        self._entry = self._message = None
        response = self._request(entry)
        if response is None:
            return None
        self._entry = response.get_entrylist().get_entry()
#        self._message = "message"  # 各リクエストタイプメソッド毎に設定
        return self._entry

    def get_result(self, default=-1, entry=None, boolean=False):
        try:
            from fal_info import Results
            entry = entry or self._entry
            result = int(entry.get_result())
            return result == int(Results.OK) if boolean else result
        except:
            return default

    def _get_message(self, code=None, arglist={}, default=_(u'Unknown'), log=False):
        if code is None:
            return default
        return get_message(code, arglist=arglist, default=default, is_log=log)

    def _set_message(self, code, arglist={}, log=False):
        self._message = self._get_message(code, arglist=arglist, log=log)

    def get_message(self, code=None, arglist={}, default=_(u'Unknown')):
        if code is None:
            return getattr(self, '_message', None) or default
        else:
            self._get_message(code=code, arglist=arglist, default=default)

    def get_value(self, xpath='/', default=None, entry=None):
        # 簡易実装なのでこれで無理なら諦めて xmldata/entry を直接いじる
        try:
            entry = entry or self._entry
            if re.match('^@', xpath):
                return entry.get().get_attrs()[xpath.split('@')[-1]]
            path = xpath.split('/@', 2)
            elem = entry.get().get_node(path[0])
            if len(path) > 1:
                return elem.get_attrs()[path[1]]
            else:
                return elem.get_text()
        except:
#            import traceback
#            traceback.print_exc()
            return default

    def is_true(self, value):
        if value == self.TRUE_TEXT:
            return True
        else:
            return False


if __name__ == '__main__':
    print _dict_to_xml({
        "group": {
            "name": "iodata",
            "member": {
                "taro": {
                    "__a_b__": "abc",
                },
                "jiro": {
                    "__a_b__": "abc",
                },
            },
            "perm": {
                "__r__": True,
                "__w__": False,
                "__x__": True,
            },
        },
    })
#    print "Message: %s" % get_message(4000023)
#    print "Message: %s" % get_message('MSG_SYSTEM_REBOOT')
#    print "Message: %s" % get_message(
#            'MSG_SYSTEM_WEB_LOGIN',
#            arglist={'name': 'abc'})
#    print "Logging: %s" % logging(4000023)
#
#    e = Entry('load', '<test>%s</test>' % escape('a<bc'))
#    print e.entry
#    print e.get().get_text()
