# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from command import Command, _, escape
from ionas.request import Entry, XmlNode
from fal_info import Results
from product_conf.devinfo import SATA_PORT_NO_LIST
SATA_PORT_NO_LIST = [int(no) for no in SATA_PORT_NO_LIST]


class Diskctrl(Command):
    try:
        from fal_info import FAL_DISKCTRL_PATH as COMMAND
    except:
        COMMAND_NAME = 'fal_diskctrl.py'

    TYPE_QUERY = 'query'
    TYPE_DELETE = 'delete'

    def _query(self, disk):
        if isinstance(disk, int):
            disk = "satad%s" % disk
        return self.request(Entry(self.TYPE_QUERY, """<diskctrl><disk><name>%s</name></disk></diskctrl>""" % disk))

    def _delete(self, disk):
        xml = """<diskctrl><disk><name>%s</name></disk></diskctrl>""" % disk
        return self.request(Entry(self.TYPE_DELETE, xml))

    def delete(self, disk, to_bool=True):
        entry = self._delete(disk)
        if to_bool:
            return entry.get_result() in [Results.OK]
        else:
            return entry

    def query(self, disk=SATA_PORT_NO_LIST[0], to_dict=True):
        def _to_dict(entry):
            dict_ = {}
            try:
                disk_node = entry.get().get_node('disk')
                dict_['status'] = disk_node.get_value('status', None)
                dict_['model'] = disk_node.get_value('model', None)
                dict_['size'] = disk_node.get_value('size', None)
            except:
                pass
            return dict_
        entry = self._query(disk)
        return _to_dict(entry) if to_dict else entry

    def status(self, disk=SATA_PORT_NO_LIST[0]):
        try:
            return self.query(disk=disk)['status']
        except:
            return None

    def is_connected(self, disk=SATA_PORT_NO_LIST[0]):
        try:
            if self.query(disk=disk, to_dict=False).get_result() \
                    == Results.DEVICE_NOT_FOUND:
                return False
            return True
        except:
            return True

    def is_capshortage(self, disk=SATA_PORT_NO_LIST[0]):
        try:
            if self.status(disk=disk) == 'capshortage':
                return True
            return False
        except:
            return False

    def is_fail(self, disk=SATA_PORT_NO_LIST[0]):
        try:
            if self.status(disk=disk) in ['fail', 'failpending', 'smarterror', 'lastsmarterror']:
                return True
            return False
        except:
            return False

if __name__ == '__main__':
    i = Diskctrl()
    print i.query()
    print "STATUS: %s" % i.status(disk=2)
    print "CONNECTED: %s" % i.is_connected(disk=2)
    print "CAPSHORTAGE: %s" % i.is_capshortage(disk=2)
    print "FAIL: %s" % i.is_fail(disk=2)
