# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from command import Command, _
from ionas.request import Entry

from fal_info import Results


class Eco(Command):
    try:
        from fal_info import FAL_ECO_PATH as COMMAND
    except:
        COMMAND = '/mnt/data/bin/fal_eco.py'

    def load(self):

        def __dict(entry):
            d = {}
            try:
                eco_node = entry.get()
                d['time'] = {}
                d['time']['minute'] = int(eco_node.get_value('time/minute'))
                d['volumelist'] = []
                for volume_node in eco_node.get_node_list('volumelist/volume'):
                    try:
                        name = volume_node.get_value('name')
                        enabled = volume_node.is_enabled()
                        d['volumelist'].append({
                            'name': name,
                            'enabled': enabled,
                        })
                    except:
                        pass
            except:
                pass
            return d

        entry = self._load()
        return __dict(entry)

    def apply(self, config={}, log=True, messages={}):
        """
        config = {
            'time': {
                'minute': MINUTE,
            },
            'volumelist': [
                {
                    'enabled': BOOL,
                    'name': VOLNAME,
                },
            ],
        }
        """
        __default_messages = {
            Results.OK: ['MSG_ECO_CHANGE', {}],
            None: ['ERR_ECO_CHANGE', {}]
        }
        __default_messages.update(messages)

        try:
            entry = self._apply(config)
            result = getattr(entry, 'get_result', lambda: None)()
        except:
            result = None

        message = __default_messages.get(result, __default_messages[None])
        self._set_message(message[0], arglist=message[1], log=log)

        return result in [Results.OK]

    def _load(self):
        entry = Entry('load', """<eco/>""")
        return self.request(entry)

    def _apply(self, config):
        entry = Entry('apply', """<eco/>""")

        eco_node = entry.get()

        if 'time' in config:
            eco_node.append_child('time')
            time_node = eco_node.get_node('time')
            if 'minute' in config['time']:
                time_node.append_child('minute', text=str(config['time']['minute']))
        if 'volumelist' in config:
            eco_node.append_child('volumelist')
            volumelist_node = eco_node.get_node('volumelist')
            for volume in config['volumelist']:
                if ('enabled' in volume) and ('name' in volume):
                    volumelist_node.append_child('volume')
                    volume_node = volumelist_node.get_node('volume[last()]')
                    volume_node.enable(volume['enabled'])
                    volume_node.append_child('name', text=str(volume['name']))

        return self.request(entry)


if __name__ == '__main__':
    eco = Eco()
    print eco.load()
