# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from command import Command, _, escape
from ionas.request import Entry, XmlNode
from datetime import datetime
import time
import os
import codecs
try:
    from django.conf import settings
    TMP = settings.TMP
except:
    TMP = '/tmp/'
try:
    from fal_info import FAL_MESSAGES_PATH
except:
    FAL_MESSAGES_PATH = '/mnt/data/bin/fal_messages.py'


class EventLog(Command):
    try:
        from fal_info import FAL_EVENTLOG_PATH as COMMAND
    except:
        COMMAND_NAME = 'fal_eventlog.py'

    TYPE_ROTATE = 'rotate'
    TYPE_SAVE = 'save'
    TYPE_LOAD = 'load'
    TYPE_ADD = 'add'
    TYPE_GET = 'get'

    LEVEL = {
        0: _(u'Info'),
        1: _(u'Warning'),
        2: _(u'Error'),
    }
    level = lambda self, l: self.LEVEL.get(int(l), _(u'Unknown'))

    epoch_to_datetime = lambda self, e: datetime(*time.localtime(e)[:6])
    datetime_to_epoch = lambda self, d: int(time.mktime(d.timetuple()))

    def get(self,
            index=0,
            maxnum=None,
            sort=None,
            reverse=False,
            filtering=False,
            to_list=True):
        _index = lambda i: 'index="%s"' % (i if isinstance(i, int) else 0)
        _maxnum = lambda n: 'maxnum="%s"' % n if isinstance(n, int) else ''
        _sort = lambda s: 'sort="%s"' % s if s in ('level', 'category', 'timestamp') else ''
        _reverse = lambda r: 'reverse="%s"' % ('true' if r else 'false')
        _filter = lambda f: 'filter="%s"' % ('true' if f else 'false')

        xml = """<eventlog>
            <option %(index)s %(maxnum)s %(sort)s %(reverse)s %(filter)s />
        </eventlog>""" % {
            'index': _index(index),
            'maxnum': _maxnum(maxnum),
            'sort': _sort(sort),
            'reverse': _reverse(reverse),
            'filter': _filter(filtering),
        }
        entry = self.request(Entry(self.TYPE_GET, xml))
        return self._to_list(entry) if to_list else entry

    def _to_xml(self, value):
        return escape(value)

    def _to_list(self, entry):
        try:
            from fal_msginfo import get_category_str, get_query_entry
            log_list_node = entry.get().get_node('loglist')
            total = int(log_list_node.get_attr_value('total'))
            log_list, entry_list = [], []
#            for node in entry.get().get_node_list('loglist/log'):
            for node in log_list_node.get_node_list('log'):
                try:
                    attrs = node.get_attrs()
                    stamp = datetime.fromtimestamp(int(attrs.get('timestamp', 0)))
                    level = attrs.get('level', None)
                    category = attrs.get('category', None)
                    code = node.get_node('code').get_text()
                    arglist = dict([(n.get_attrs()['name'], n.get_text()) \
                            for n in node.get_node_list('arglist/arg')])
                except:
                    continue
#                entry_list.append(get_query_entry(code, arg_list=arglist))
                entry_list.append(Entry('query', """<messages>
                    <messageinfo>
                        <code>%(code)s</code>
                        <arglist>%(arglist)s</arglist>
                    </messageinfo>
                </messages>""" % {
                    'code': code,
                    'arglist': ''.join(['<arg name="%s">%s</arg>' % (
                            self._to_xml(name), self._to_xml(val))
                            for name, val in arglist.iteritems()]),
                }))
                log_list.append({
                    'datetime': stamp,
                    'level': self.level(level),
                    'category': get_category_str(category),
                })
            response = self._request(
                    entry_list,
                    command=FAL_MESSAGES_PATH)
            for (e, l) in zip(
                    response.get_entrylist().get_entry_list(), log_list):
                try:
                    l['message'] = e.get().get_node(
                            'messageinfo/message').get_text()
                except:
                    pass
            return [log_list, total]
        except:
            return [[], 0]

    def file(self, path=None):
        def _str(text):
            if isinstance(text, unicode):
                return text.encode('utf-8')
            return text
        # FIXME 文字コード
        path = path or os.path.join(TMP, 'log.csv')  # FIXME 仮
        if not os.path.exists(os.path.dirname(path)):
            os.makedirs(os.path.dirname(path))
        log_list, total = self.get(sort='timestamp', reverse=True)
        with codecs.open(path, 'w', 'utf-8-sig') as f:
            for log in log_list:
                line = '%s,%s,%s,%s\r\n' % (
                        _str(log.get('datetime', u"")),
                        _str(log.get('level', u"")),
                        _str(log.get('category', u"")),
                        _str(log.get('message', u"")))
                f.write(line.decode('utf-8'))
        return path

    def load(self, to_dict=True):
        def _to_dict(entry):
            filt = entry.get().get_node('filter')
            try:
                from_to = filt.get_node('timestamplist/timestamp').get_attrs()
            except:
                from_to = {'from': None, 'to': None}
            frm, to = from_to.get('from', None), from_to.get('to', None)
            return {
                'level': [l.get_text() for l in filt.get_node_list('levellist/level')],
                'category': [c.get_text() for c in filt.get_node_list('categorylist/category')],
                'timestamp': {
                    'from': self.epoch_to_datetime(int(frm)) if frm else None,
                    'to': self.epoch_to_datetime(int(to)) if to else None,
                }
            }
        entry = self.request(Entry(self.TYPE_LOAD, '<eventlog />'))
        return _to_dict(entry) if to_dict else entry

    def save(self,
            timestamp={'from': None, 'to': None},  # datetime
            level=[],  # code list
            category=[],
            log=True):
        xml = """<eventlog>
            <filter>
                <levellist>%(llist)s</levellist>
                <categorylist>%(clist)s</categorylist>
                <timestamplist>
                    <timestamp from="%(from)s" to="%(to)s" />
                </timestamplist>
            </filter>
        </eventlog>""" % {
            'llist': ''.join(['<level>%s</level>' % l for l in level]),
            'clist': ''.join(['<category>%s</category>' % c for c in category]),
            'from': self.datetime_to_epoch(timestamp['from']) \
                    if timestamp['from'] else '',
            'to': self.datetime_to_epoch(timestamp['to']) \
                    if timestamp['to'] else '',
        }
        entry = self.request(Entry(self.TYPE_SAVE, xml))
        if self.get_result(entry=entry):
            self._message = self._get_message(
                    code='ERR_LOG_FILTER_CHANGE',
                    log=log)
        else:
            self._message = self._get_message(
                    code='MSG_LOG_FILTER_CHANGE',
                    log=log)
        return entry

    def get_categorys(self):
        try:
            from fal_msginfo import get_category_list, get_category_str
            return [(c, get_category_str(c)) for c in get_category_list()]
        except:
            return []

if __name__ == '__main__':
    i = EventLog()

#    print i.file()

#    print "Category"
#    for c in i.get_categorys():
#        print c
#    d = i.load()
#    print "Load"
#    print d

#    e = i.load()
#    print "Load: %s" % e.entry
#    logs = i.get()
#    print "Get: %s" % i.get_message()
#    for log in logs:
#        print log
#    i.file()
#    print "File: %s" % i.get_message()
#    i.save(
#            level=[0, 2],
#            category=[2, 3, 5],
#            timestamp={'from': datetime.now(), 'to': datetime.now()})
#    print "Save: %s(%s)" % (i.get_message(), i.get_result())

    from command import logging
    while True:
        logging('1000000')
