# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from command import Command, _
from ionas.request import Entry
from datetime import datetime, time

from fal_info import Results


class DateTime(Command):
    try:
        from fal_info import FAL_DATETIME_PATH as COMMAND
    except:
        COMMAND = '/mnt/data/bin/fal_datetime.py'

    TYPE_LOAD = 'load'
    TYPE_QUERY = 'query'
    TYPE_APPLY = 'apply'

    XML_FMT_APPLY_ENTRY = """<datetime>
        %(XML_FMT_NTP_TAG)s
        %(XML_FMT_MANUAL_TAG)s</datetime>"""
    XML_FMT_NTP_TAG = """<ntp>
        <serveraddress>%(ntp_srv)s</serveraddress>
        <sync>
            <boot enable="%(sync_boot)s" />
            <time enable="%(sync_daily)s">
                <hour>%(sync_hour)s</hour>
                <min>%(sync_min)s</min></time></sync></ntp>"""
    XML_FMT_MANUAL_TAG = """<manual>
        <date>
            <yy>%(YY)s</yy>
            <mm>%(MM)s</mm>
            <dd>%(DD)s</dd>
            <hour>%(hh)s</hour>
            <min>%(mm)s</min>
            <sec>%(ss)s</sec></date></manual>"""

    def load(self, to_dict=True):

        def __dict(entry):
            d = {}
            try:
                dt_node = entry.get()
                d['ntp'] = {}
                d['ntp']['enabled'] = dt_node.get_node('ntp').is_enabled()
                d['ntp']['srvaddr'] = dt_node.get_value('ntp/serveraddress', '')
                d['ntp']['sync'] = {}
                d['ntp']['sync']['boot'] = {}
                d['ntp']['sync']['boot']['enabled'] = dt_node.get_node('ntp/sync/boot').is_enabled()
                d['ntp']['sync']['time'] = {}
                d['ntp']['sync']['time']['enabled'] = dt_node.get_node('ntp/sync/time').is_enabled()
                d['ntp']['sync']['time']['time'] = time(int(dt_node.get_value('ntp/sync/time/hour')), int(dt_node.get_value('ntp/sync/time/minute')))
            except:
                pass
            return d

        entry = self.request(Entry(self.TYPE_LOAD, '<datetime />'))
        return __dict(entry) if to_dict else entry


    def query(self):
        return self.request(Entry(self.TYPE_QUERY, '<datetime />'))  # Entry

    def apply(self,
            manual=True,
            ntp_srv=None,
            sync_boot=False,
            sync_daily=False, sync_hour=None, sync_min=None,
            year=None, month=None, day=None,
            hour=None, min=None, sec=None):
        xml = '<datetime />'
        if manual:
            xml = self.XML_FMT_APPLY_ENTRY % {
                'XML_FMT_NTP_TAG': self.XML_FMT_NTP_TAG % {
                    'ntp_srv': '',
                    'sync_boot': self.BOOL_TO_TEXT[False],
                    'sync_daily': self.BOOL_TO_TEXT[False],
                    'sync_hour': '', 'sync_min': '',
                },
                'XML_FMT_MANUAL_TAG': self.XML_FMT_MANUAL_TAG % {
                    'YY': year or '2013', 'MM': month or '1', 'DD': day or '1',
                    'hh': hour or '0', 'mm': min or '0', 'ss': sec or '0',
                }
            }
        else:
            xml = self.XML_FMT_APPLY_ENTRY % {
                'XML_FMT_NTP_TAG': self.XML_FMT_NTP_TAG % {
                    'ntp_srv': ntp_srv or '',
                    'sync_boot': self.BOOL_TO_TEXT[sync_boot or False],
                    'sync_daily': self.BOOL_TO_TEXT[sync_daily or False],
                    'sync_hour': sync_hour or '', 'sync_min': sync_min or '',
                },
                'XML_FMT_MANUAL_TAG': '',
            }
        return self.request(Entry(self.TYPE_APPLY, xml))

    def apply_config(self, config={}, log=True, messages={}):
        """
        config = {
            'ntp': {
                'enabled': BOOL,
                'serveraddress': ADDR,
                'sync': {
                    'boot': {
                        'enabled': BOOL,
                    },
                    'time': {
                        'enabled': BOOL,
                        'time': TIME,
                    },
                },
            },
            'manual': {
                'datetime': DATETIME,
            },
        }
        """
        __default_messages = {
            Results.OK: ['MSG_DATE_CHANGE', {}],
            None: ['ERR_DATE_CHANGE', {}],
        }
        __default_messages.update(messages)

        entry = self._apply(config)
        result = getattr(entry, 'get_result', lambda: None)()

        message = __default_messages.get(result, __default_messages[None])
        self._set_message(message[0], arglist=message[1], log=log)

        return result in [Results.OK]

    def _apply(self, config):
        entry = Entry('apply', """<datetime/>""")
        datetime_node = entry.get()

        if config.get('ntp') is not None:
            datetime_node.append_child('ntp')
            ntp_node = datetime_node.get_node('ntp')
            if config['ntp'].get('enabled') is not None:
                ntp_node.enable(config['ntp']['enabled'])
            if config['ntp'].get('serveraddress') is not None:
                ntp_node.append_child('serveraddress', text=config['ntp']['serveraddress'])
            if config['ntp'].get('sync') is not None:
                ntp_node.append_child('sync')
                sync_node = ntp_node.get_node('sync')
                if config['ntp']['sync'].get('boot') is not None:
                    sync_node.append_child('boot')
                    boot_node = sync_node.get_node('boot')
                    if config['ntp']['sync']['boot'].get('enabled') is not None:
                        boot_node.enable(config['ntp']['sync']['boot']['enabled'])
                if config['ntp']['sync'].get('time') is not None:
                    sync_node.append_child('time')
                    time_node = sync_node.get_node('time')
                    if config['ntp']['sync']['time'].get('enabled') is not None:
                        time_node.enable(config['ntp']['sync']['time']['enabled'])
                    if config['ntp']['sync']['time'].get('time') is not None:
                        hour = config['ntp']['sync']['time']['time'].hour
                        minute = config['ntp']['sync']['time']['time'].minute
                        time_node.append_child('hour', text=str(hour))
                        time_node.append_child('minute', text=str(minute))
        if config.get('manual') is not None:
            datetime_node.append_child('manual')
            manual_node = datetime_node.get_node('manual')
            if config['manual'].get('datetime') is not None:
                manual_node.append_child('date')
                date_node = manual_node.get_node('date')
                year = config['manual']['datetime'].year
                month = config['manual']['datetime'].month
                day = config['manual']['datetime'].day
                hour = config['manual']['datetime'].hour
                minute = config['manual']['datetime'].minute
                second = config['manual']['datetime'].second
                date_node.append_child('year', text=str(year))
                date_node.append_child('month', text=str(month))
                date_node.append_child('day', text=str(day))
                date_node.append_child('hour', text=str(hour))
                date_node.append_child('min', text=str(minute))
                date_node.append_child('sec', text=str(second))

        return self.request(entry)



if __name__ == '__main__':
    dt = DateTime()
    print dt.load()
#    print dt.apply_config(config={
#        'ntp': {
#            'enabled': True,
#            'serveraddress': 'aaa.bb.cc',
#            'sync': {
#                'boot': {
#                    'enabled': True,
#                },
#                'time': {
#                    'enabled': True,
#                    'time': time(7, 30),
#                },
#            },
#        },
#    })
