# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from command import Command
from ionas.request import Entry


class Features(Command):
    try:
        from fal_info import FAL_FEATURES_PATH
        COMMAND = FAL_FEATURES_PATH
    except:
        COMMAND_NAME = "fal_features.py"

    TYPE_QUERY = 'query'

    FEATURES_REQUEST_NODE_BASE = "<features/>"

    def _query(self):
        entry = self.request(
            Entry(self.TYPE_QUERY, self.FEATURES_REQUEST_NODE_BASE))
        return entry

    #
    # 外部用
    #

    def query(self, to_dict=True):
        def _to_dict(entry):
            dict_ = {}
            try:
                cnd = entry.get()

                def _is_enabled(xpath, default=False):
                    try:
                        return cnd.get_node(xpath).is_enabled()
                    except:
                        return default

                dict_ = {
                    "functionlist": [
                        nd.get_text() for nd in cnd.get_node_list(
                            "functionlist/function")],
                    "raidvolumelist": [{
                        "name": raidvolume_node.get_value("@name"),
                        "modelist": [mode_node.get_text() for mode_node in raidvolume_node.get_node_list("modelist/mode")],
                    } for raidvolume_node in cnd.get_node_list("raidvolumelist/raidvolume")],
                    "deviceinfo": {
                        "satalist": [{
                            "port": sata_node.get_value("@port"),
                            "name": sata_node.get_value("@name"),
                            "coldswaponly": sata_node.is_set_to_true("coldswaponly"),
                        } for sata_node in cnd.get_node_list("deviceinfo/satalist/sata")],
                        "usblist": [{
                            "port": usb_node.get_value("@port"),
                            "name": usb_node.get_value("@name"),
                            "coldswaponly": usb_node.is_set_to_true("coldswaponly"),
                            "protocol": usb_node.get_value("@protocol"),
                            "modelist": [mode_node.get_text() for mode_node in usb_node.get_node_list("modelist/mode")],
                        } for usb_node in cnd.get_node_list("deviceinfo/usblist/usb")],
                        "netiflist": [{
                            "port": netif_node.get_value("@port"),
                            "name": netif_node.get_value("@name"),
                        } for netif_node in cnd.get_node_list("deviceinfo/netiflist/netif")],
                        "buttonlist": [{
                            "name": button_node.get_value("@name"),
                        } for button_node in cnd.get_node_list("deviceinfo/buttonlist/button")],
                    },
                }
            except:
                pass
            return dict_

        entry = self._query()
        return _to_dict(entry) if to_dict else entry


if __name__ == '__main__':
    features = Features()
    print features.query()
