# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from command import Command, _
from ionas.request import Entry
try:
    from fal_info import FAL_LED_PATH
except:
    FAL_LED_PATH = '/mnt/data/bin/fal_led.py'


class LED(Command):
    COMMAND = FAL_LED_PATH

    TYPE_CONTROL = 'control'

    BRIGHTNESS_LIST = ('low', 'middle', 'middle')

    def _brightness(self, index):
        try:
            return tuple([None] + list(self.BRIGHTNESS_LIST))[index]
        except:
            return None

    def _control(self,
            power=None,
            brightness=None):
        entry = Entry(self.TYPE_CONTROL, '<led />')
        led_node = entry.get()

        if isinstance(power, bool):
            led_node.append_child('power', text='on' if power else 'off')

        if isinstance(brightness, int):
            brightness = self._brightness(brightness)
        if brightness in self.BRIGHTNESS_LIST:
            led_node.append_child('brightness', text=brightness)

        entry = self.request(entry)
        print entry
        return entry

    def control(self, *args, **kwargs):
        return self._control(*args, **kwargs)

    def status(self, on, blink=False, color=None):
        ctrl = 'blink' if on and blink else 'on' if on else 'off'
        entry = Entry(
                self.TYPE_CONTROL,
                "<led><status>%s</status></led>" % ctrl)
        if color in ('green', 'red'):
            entry.get().get_node('status').set_attrs({'color': color})
        entry = self.request(entry)
        return entry


if __name__ == '__main__':
    import time
    i = LED()
    i.control(power=False)
    time.sleep(3)
    i.control(power=True)
    time.sleep(1)
    i.control(brightness=2)
    time.sleep(1)
    i.control(brightness=1)
    time.sleep(1)
    i.control(brightness=3)

    i.status(True, blink=True, color='red')
    time.sleep(3)
    i.status(True, blink=True, color='green')
    time.sleep(3)
    i.status(False)
    time.sleep(3)
    i.status(True)

