# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from command import Command, _, escape
from ionas.request import Entry, XmlNode, XmlConfNode
from product_conf import STOR_IF_TO_VOL
from fal_info import Results


class Lockkey(Command):
    try:
        from fal_info import FAL_LOCKKEY_PATH as COMMAND
    except:
        COMMAND_NAME = 'fal_lockkey.py'

    TYPE_QUERY = 'query'
    TYPE_SETLOCKKEY = 'set_lockkey'

    def query(self, to_dict=True):
        def _to_dict(entry):
            dict_ = {}
            try:
                lockkey_node = entry.get()
                dict_['key'] = lockkey_node.get_value('key', None)
                dict_['mac'] = lockkey_node.get_value('mac', None)
            except:
                pass
            return dict_
        entry = self.request(Entry(self.TYPE_QUERY, """<lockkey />"""))
        result = entry.get_result()
        if int(result) == int(Results.OK):
            pass
        elif int(result) == int(Results.DEVICE_NOT_FOUND):
            self._set_message('ERR_LOCKKEY_DISK_NOT_FOUND')
        return _to_dict(entry) if to_dict else entry

    def _set_lockkey(self,
                     operation=None,  # write/create/delete
                     mac=None,  # write only
                     key=None,  # write only
                     keyserverno=None,
                     password="",
                     *args, **kwargs):
        entry = Entry(self.TYPE_SETLOCKKEY, """<lockkey>
            <option operation="%(operation)s" />
        </lockkey>""" % {'operation': operation})
        if operation == "write":
            entry.get().append_child('mac', text=mac)
            entry.get().append_child('key', text=key)
        elif operation == "transform":
            pass
        elif operation == "download":
            lockkey_node = entry.get()
            lockkey_node.append_child("keyserver")
            keyserver_node = lockkey_node.get_node("keyserver")
            keyserver_node.append_child("registno", text=str(keyserverno))
            keyserver_node.append_child("passwd", text=str(password))
        return self.request(entry)

    def set_lockkey(self,
                    operation,
                    keyserverno=None,
                    log=True,
                    *args, **kwargs):
        keyservername = self.get_keyservername(keyserverno, default="")
        __all_messages = {
            "transform": {
                Results.OK: ["MSG_LOCKKEY_TRANSFORM", {}],
                Results.DEVICE_NOT_FOUND: ["ERR_LOCKKEY_LOCKKEY_NOT_FOUND", {}],
                Results.NO_LOCKKEY: ["ERR_LOCKKEY_LOCKKEY_NOT_FOUND", {}],
                None: ["ERR_LOCKKEY_TRANSFORM", {}],
            },
            "download": {
                Results.OK: ["MSG_LOCKKEY_DOWNLOAD", {}],
                Results.DEVICE_NOT_FOUND: ["ERR_LOCKKEY_LOCKKEY_NOT_FOUND", {}],
                Results.NO_LOCKKEY: ["ERR_LOCKKEY_LOCKKEY_NOT_FOUND", {}],
                Results.UNKNOWN_LOCKKEY: ["ERR_LOCKKEY_DISK_OTHER_LANDISK", {}],
                Results.OLD_VERSION_ERROR: ["ERR_LOCKKEY_OLD_KEY_SERVER", {"name": keyservername}],
                Results.AUTH_ERROR: ["ERR_LOCKKEY_KEY_SERVER_AUTH", {"name": keyservername}],
                Results.CONNECT_ERROR: ["ERR_LOCKKEY_KEY_SERVER_CONNECT", {"name": keyservername}],
                None: ["ERR_LOCKKEY_DOWNLOAD", {}],
            },
        }
        __default_messages = __all_messages.get(operation, None)
        if __default_messages is None:
            return False
        try:
            entry = self._set_lockkey(operation=operation, keyserverno=keyserverno, *args, **kwargs)
            result = getattr(entry, "get_result", lambda:None)()
        except:
            result = None
        message = __default_messages.get(result, __default_messages[None])
        self._set_message(message[0], arglist=message[1], log=log)
        return result in [Results.OK]

    def create(self, to_bool=True, log=False):
        entry = self._set_lockkey(operation='create')
        result = entry.get_result()
        if result == Results.OK:
            result = True
        else:
            self._set_message(
                {
                    Results.DEVICE_NOT_FOUND: 'ERR_LOCKKEY_DISK_NOT_FOUND',
                    Results.CAP_SHORTAGE: 'ERR_LOCKKEY_DISK_CAP_SHORTAGE',
                }.get(result, None),
                log=log)
            result = False
        return result if to_bool else entry

    def delete(self, to_bool=True, log=True):
        entry = self._set_lockkey(operation='delete')
        result = entry.get_result()
        if result == Results.OK:
            self._set_message('MSG_LOCKKEY_DELETE', log=log)
            result = True
        else:
            self._set_message('ERR_LOCKKEY_DELETE', log=log)
            result = False
        return result if to_bool else entry

    def load(self):

        def __dict(entry):
            d = {}
            try:
                lockkey_node = entry.get()
                d["keyserverlist"] = {}
                for keyserver_node in lockkey_node.get_node_list("keyserverlist/keyserver"):
                    try:
                        no = int(keyserver_node.get_value("registno"))
                        d["keyserverlist"][no] = {
                            "registno": no,
                            "enabled": keyserver_node.is_enabled(),
                            "hostname": keyserver_node.get_value("hostname"),
                            "port": keyserver_node.get_value("port"),
                        }
                    except TypeError, ValueError:
                        # NOTE: registno がNoneや空だった場合
                        pass
            except:
                pass
            return d

        try:
            entry = self._load()
            return __dict(entry)
        except:
            return {}

    def _load(self):
        entry = Entry("load", """<lockkey/>""")
        return self.request(entry)

    def list(self):

        def __dict(entry):
            d = {}
            try:
                lockkey_node = entry.get()
                for keyserver_node in lockkey_node.get_node_list("keyserverlist/keyserver"):
                    try:
                        no = int(keyserver_node.get_value("registno"))
                        d[no] = {
                            "registno": no,
                            "enabled": keyserver_node.is_enabled(),
                            "registered": {"true": True, "false": False}.get(keyserver_node.get_value("@registered", None), None),
                            "hostname": keyserver_node.get_value("hostname"),
                            "port": keyserver_node.get_value("port"),
                        }
                    except TypeError, ValueError:
                        # NOTE: registno がNoneや空だった場合
                        pass
            except:
                pass
            return d

        try:
            entry = self._list()
            return __dict(entry)
        except:
            return {}

    def _list(self):
        entry = Entry("list", """<lockkey/>""")
        return self.request(entry)

    def save(self, keyserverlist=[], log=True, messages={}):
        __default_messages = {
            Results.OK: ["MSG_LOCKKEY_CHANGE", {}],
            None: ["ERR_LOCKKEY_CHANGE", {}],
        }
        try:
            entry = self._save(keyserverlist)
            result = getattr(entry, "get_result", lambda: None)()
        except:
            result = None

        message = __default_messages.get(result, __default_messages[None])
        self._set_message(message[0], arglist=message[1], log=log)

        return result in [Results.OK]

    def _save(self, keyserverlist):
        entry = Entry("save", """<lockkey><keyserverlist/></lockkey>""")
        keyserverlist_node = entry.get().get_node("keyserverlist")
        for keyserver in keyserverlist:
            try:
                keyserver_node = XmlConfNode("""<keyserver/>""")
                keyserver_node.enable(keyserver["enabled"])
                keyserver_node.append_child("registno", text="%s" % keyserver["registno"])
                keyserver_node.append_child("hostname", text=keyserver["hostname"])
                keyserver_node.append_child("port", text="%s" % keyserver["port"])
                keyserverlist_node.append_child(keyserver_node)
            except:
                pass
        return self.request(entry)

    def get_keyservername(self, no, default=None):
        try:
            return self.load()["keyserverlist"][no]["hostname"]
        except:
            return default

    def register(self, no, password):
        keyservername = self.get_keyservername(no, default="")
        __default_messages = {
            Results.OK: ["MSG_LOCKKEY_REGISTER", {}],
            Results.DEVICE_NOT_FOUND: ["ERR_LOCKKEY_LOCKKEY_NOT_FOUND", {}],
            Results.NO_LOCKKEY: ["ERR_LOCKKEY_LOCKKEY_NOT_FOUND", {}],
            Results.OLD_VERSION_ERROR: ["ERR_LOCKKEY_OLD_KEY_SERVER", {"name": keyservername}],
            Results.AUTH_ERROR: ["ERR_LOCKKEY_KEY_SERVER_AUTH", {"name": keyservername}],
            Results.CONNECT_ERROR: ["ERR_LOCKKEY_KEY_SERVER_CONNECT", {"name": keyservername}],
            None: ["ERR_LOCKKEY_REGISTER", {}],
        }
        try:
            entry = self._register(no, password)
            result = getattr(entry, "get_result", lambda: None)()
        except:
            result = None
        message = __default_messages.get(result, __default_messages[None])
        self._set_message(message[0], arglist=message[1])
        return result in [Results.OK]

    def _register(self, no, password):
        entry = Entry("register", """<lockkey><keyserver/></lockkey>""")
        keyserver_node = entry.get().get_node("keyserver")
        keyserver_node.append_child("registno", text=str(no))
        keyserver_node.append_child("passwd", text=str(password))
        return self.request(entry)

    def modify(self, keyservers={}, log=True, messages={}):
        _keyservers = self.load().get("keyserverlist", {})
        _keyservers.update(keyservers)
        for no, keyserver in _keyservers.iteritems():
            try:
                keyserver["registno"] = int(no)
            except:
                pass
        return self.save(keyserverlist=_keyservers.values(), log=log, messages=messages)


if __name__ == '__main__':
    i = Lockkey()

    #print i.query()
    #print "QUERY: %s (%s)" % (i.get_message(), i.get_result())

    #print i.create()
    #print "CREATE: %s (%s)" % (i.get_message(), i.get_result())
    #print i.query()

    #print i.delete()
    #print "DELETE: %s (%s)" % (i.get_message(), i.get_result())
    #print i.query()

    #print i.load()
    #print i.modify(keyservers={
    #    1: {
    #        "enabled": True,
    #        #"registno": 1,
    #        "hostname": "keyservername",
    #        "port": 1234,
    #    },
    #})

    #print i.set_lockkey("download", keyserverno=1)
    print i.list()
