# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from command import Command
from ionas.request import Entry, XmlNode
from fal_info import Results
try:
    from fal_info import FAL_NARSUS_PATH
except:
    FAL_NARSUS_PATH = '/mnt/data/bin/fal_narsus.py'



class NarSuS(Command):
    COMMAND = FAL_NARSUS_PATH

    TYPE_QUERY = 'query'
    TYPE_APPLY = 'apply'
    TYPE_GET = 'get'

    def query(self):
        entry = self.request(Entry(self.TYPE_QUERY, '<narsus />'))
        return entry

    def apply(self, narsus=None, usecode=None, log=True, to_bool=True):
        xml = XmlNode('<narsus />')
        if isinstance(narsus, bool):
            xml.append_child(
                    'service',
                    attrs={'enable': self.bool_to_text(narsus)})
        if isinstance(usecode, (str, unicode)):
            xml.append_child(
                    'usecode',
                    text=usecode,
                    attrs={'enable': 'true'})
        entry = self.request(Entry(self.TYPE_APPLY, xml))
        result = entry.get_result()
        if result == Results.OK:
            self._set_message('MSG_NARSUS_CHANGE', log=log)
            result = True
        elif result == Results.INVALID_CODE:
            self._set_message('ERR_NARSUS_USECODE', log=log)
            result = False
        else:
            self._set_message('ERR_NARSUS_CHANGE', log=log)
            result = False
#        if self.get_result(entry=entry):
#            self._set_message('ERR_NARSUS_CHANGE', log=log)
#        else:
#            self._set_message('MSG_NARSUS_CHANGE', log=log)
        return result if to_bool else entry

    def is_registered(self):
        self.query()
        return self.TEXT_TO_BOOL[self.get_value('usecode/@enable', 'false')]

if __name__ == '__main__':
    i = NarSuS()
    i.query()
    print "Load: %s" % i.get_result()
    print "usecode: %s" % i.get_value('usecode')
    print "URL: %s" % i.get_value('url/regist')
    print "Is registered? %s" % i.is_registered()
    i.apply(narsus=True)
    print "Save: %s(%s)" %(i.get_message(), i.get_result())
