# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from command import Command
from ionas.request import Entry

class Network(Command):
    try:
        from fal_info import FAL_NETWORK_PATH as COMMAND
    except:
        COMMAND_NAME = 'fal_network.py'

    TYPE_LOAD = 'load'
    TYPE_QUERY = 'query'
    TYPE_APPLY = 'apply'

    XML_FMT_APPLY_ENTRY = """<network>
        <connection>FIXME</connection>
        <host>
            <name>%(hostname)s</name>
            <desc>%(hostdesc)s</desc>
        </host>
        <tcpip family="inet4" enable="%(ipv4)s">
            <common type="%(common)s">
                <gateway>%(gateway)s</gateway>
                <dnssrv>%(dnssrv)s</dnssrv>
            </common>
            <lan dhcp="%(ipv4_lan_dhcp)s">
                <ipsetting type="%(ipv4_lan_setting_type)s">
                    <ip>%(ipv4_lan_ip)s</ip>
                    <subnet>%(ipv4_lan_subnet)s</subnet>
                </ipsetting>
                <frame custom="%(lan_frame_custom)s">
                    <framesize>%(lan_framesize)s</framesize>
                </frame>
            </lan>
            <lan dhcp="%(ipv6_lan_dhcp)s">
                <ipsetting type="%(ipv6_lan_setting_type)s">
                    <ip>%(ipv6_lan_ip)s</ip>
                    <subnet>%(ipv6_lan_subnet)s</subnet>
                </ipsetting>
                <frame custom="%(lan_frame_custom)s">
                    <framesize>%(lan_framesize)s</framesize>
                </frame>
            </lan>
        </tcpip>
    </network>"""

    def load(self):
        return self.request(Entry(self.TYPE_LOAD, '<network />'))  # Entry

    def _query(self):
        return self.request(Entry(self.TYPE_QUERY, '<network />'))  # Entry

    def query(self, to_dict=True):
        def _to_dict(entry):
            try:
                nwnode = entry.get()
                dict_ = {}
                dict_['lan1'] = {}
                dict_['lan1']['ipv4'] = {}
                dict_['lan1']['ipv4']['addr'] = nwnode.get_value('netiflist/netif[@name="lan1"]/protocollist/protocol[@family="inet4"]/addrlist/addr', None)
                dict_['lan1']['macaddr'] = nwnode.get_value('netiflist/netif[@name="lan1"]/mac', None)
                dict_['lan2'] = {}
                dict_['lan2']['ipv4'] = {}
                dict_['lan2']['ipv4']['addr'] = nwnode.get_value('netiflist/netif[@name="lan2"]/protocollist/protocol[@family="inet4"]/addrlist/addr', None)
                dict_['lan2']['macaddr'] = nwnode.get_value('netiflist/netif[@name="lan2"]/mac', None)
                return dict_
            except:
                return {}
        entry = self._query()
        return _to_dict(entry) if to_dict else entry

    def apply(self):  #FIXME params
        #FIXME XML_FMT_APPLY_ENTRY
        return self.request(Entry(self.TYPE_APPLY, '<network />'))  # Entry


if __name__ == '__main__':
    i = Network()

    print i.query(to_dict=True)
