# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from command import Command, _, escape
from ionas.request import Entry, XmlNode
from product_conf import STOR_IF_TO_VOL, PORT_TO_MODE_LIST
from fal_info import Results
from project.utils import get_mirroring_device


class Port(Command):
    try:
        from fal_info import FAL_PORTS_PATH as COMMAND
    except:
        COMMAND_NAME = 'fal_ports.py'

    TYPE_SAVE = 'save'
    TYPE_LOAD = 'load'
    TYPE_APPLY = 'apply'

    def _load(self):
        entry = self.request(Entry(self.TYPE_LOAD, """<ports />"""))
        return entry

    def _apply(self,
            usblist=[],
            satalist=[]):
#            usblist=[
#                {
#                    'port': 1,
#                    'modelist': [
#                        {
#                            'mode': 'share',
#                            'enable': False,
#                        },
#                        {
#                            'mode': 'quickcopy',
#                            'enable': True,
#                            'share': 'disk1',
#                        }
#                    ]
#                }
#            ]
        entry = Entry(self.TYPE_APPLY, """<ports />""")
        ports_node = entry.get()

        if len(usblist):
            ports_node.append_child('usblist')
            usblist_node = ports_node.get_node('usblist')
            for usb in usblist:
                usb_node = XmlNode('usb')
                usb_node.set_attrs({'port': str(usb['port'])})
                usb_node.append_child('modelist')
                modelist_node = usb_node.get_node('modelist')
                for mode in usb['modelist']:
                    mode_node = XmlNode('mode')
                    mode_node.set_text(mode['mode'])
                    mode_node.set_attrs({'enable': str(mode['enable']).lower()})
                    if mode.get('share', None):
                        mode_node.set_attrs({'share': mode['share']})
                    
                    modelist_node.append_child(mode_node)
                usblist_node.append_child(usb_node)

        return self.request(entry)

    def load(self, to_dict=True):
        def _to_dict(entry):
            dict_ = {}
            for type_ in ('usb', 'sata'):
                dict_['%slist' % type_] = {}
                for type_node in entry.get().get_node_list(
                        '%(type)slist/%(type)s' % {'type': type_}):
                    port = int(type_node.get_value('@port'))
                    dict_['%slist' % type_][port] = {}
                    for mode_node in type_node.get_node_list('modelist/mode'):
                        func = mode_node.get_text()
                        enable = mode_node.is_enabled()
                        share = mode_node.get_value('@share', None)  # qcopy
                        if enable:
                            dict_['%slist' % type_][port]['mode'] = func
                            dict_['%slist' % type_][port]['share'] = share
            return dict_
        entry = self._load()
        return _to_dict(entry) if to_dict else entry

    def apply(self,
            devtype='usb',
            portnum=1,
            mode='share',
            share='disk1',
            to_bool=True, log=True):
        if devtype == 'usb':
            modelist = []
            for _mode in PORT_TO_MODE_LIST[devtype][portnum]:
                modelist.append({
                    'mode': _mode,
                    'enable': True if mode == _mode else False,
                    'share': share if _mode == 'quickcopy' else None})
            usblist = [{
                'port': portnum,
                'modelist': modelist,
            }]

        entry = self._apply(usblist=usblist)
        result = self.get_result(entry=entry)
        mirroring_device = get_mirroring_device(default=(u"usb", 3))
        _MSGCODE = {
            'mirroring': {
                str(Results.OK): 'MSG_VOLUME_MODE_CHANGE_MIRROR_START',
                str(Results.CAP_SHORTAGE): u"MSG_VOLUME_MODE_CHANGE_MIRROR_CAP_SHORTAGE_USB2" if mirroring_device == (u"usb", 2) else u"MSG_VOLUME_MODE_CHANGE_MIRROR_CAP_SHORTAGE_UBS3",
                str(Results.DEVICE_NOT_FOUND): u"MSG_VOLUME_MODE_CHANGE_MIRROR_NOT_CONNECTED_USB2" if mirroring_device == (u"usb", 2) else u"MSG_VOLUME_MODE_CHANGE_MIRROR_NOT_CONNECTED_USB3",
                str(Results.APPLY_ERROR): u"MSG_VOLUME_MODE_CHANGE_MIRROR_FAIL_USB2" if mirroring_device == (u"usb", 2) else u"MSG_VOLUME_MODE_CHANGE_MIRROR_FAIL_USB3",
                str(Results.WAITING): u"MSG_VOLUME_MODE_CHANGE_MIRROR_WAITING_USB2" if mirroring_device == (u"usb", 2) else u"MSG_VOLUME_MODE_CHANGE_MIRROR_WAITING_USB3",
                '_code': 'MSG_VOLUME_MODE_CHANGE_MIRROR_HOLD',
            },
            'vfsext': {
                str(Results.OK): 'MSG_VOLUME_MODE_CHANGE_VFSEXT_START',
                str(Results.CAP_SHORTAGE): 'MSG_VOLUME_MODE_CHANGE_VFSEXT_CAP_SHORTAGE',
                str(Results.DEVICE_NOT_FOUND): 'MSG_VOLUME_MODE_CHANGE_VFSEXT_NOT_CONNECTED',
                str(Results.APPLY_ERROR): 'MSG_VOLUME_MODE_CHANGE_VFSEXT_FAIL',
                str(Results.WAITING): 'MSG_VOLUME_MODE_CHANGE_VFSEXT_WAITING',
                '_code': 'MSG_VOLUME_MODE_CHANGE_VFSEXT_HOLD',
            },
            '_mode': {
                str(Results.OK): 'MSG_VOLUME_MODE_CHANGE',
                '_code': 'ERR_VOLUME_MODE_CHANGE',
            },
        }
        _mode = _MSGCODE.get(str(mode), _MSGCODE['_mode'])
        _code = _mode.get(str(result), _mode['_code'])
        self._set_message(_code, log=log)
        result = True if int(result) in [int(Results.OK)] else False

        return result if to_bool else entry

    def trans(self, text):
        return {
            'share': _(u'ShareMode'),
            'quickcopy': _(u'QuickCopyMode'),
            'mirroring': _(u'MirroringMode'),
            'vfsext': _(u'VFSMode'),
            }.get(text, text)



if __name__ == '__main__':
    i = Port()

    print i.load()

#    print i._apply(
#            usblist=[
#                {
#                    'port': 1,
#                    'modelist': [
#                        {
#                            'mode': 'quickcopy',
#                            'enable': True,
#                            'share': 'disk1'},
#                        {
#                            'mode': 'share',
#                            'enable': False},
#                    ]
#                },
#            ])
#    print i.load()

    print i.apply(devtype="usb", portnum=1, mode="quickcopy", share="disk1")
    print i.load()
    print i.apply(devtype="usb", portnum=1, mode="share")
    print i.load()

