# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from command import Command, _
from ionas.request import Entry, XmlNode
from fal_info import Results
from datetime import time


class Power(Command):
    try:
        from fal_info import FAL_POWER_PATH as COMMAND
    except:
        COMMAND_NAME = 'fal_power.py'

    TYPE_LOAD = 'load'
    TYPE_APPLY = 'apply'
    TYPE_SHUTDOWN = 'shutdown'
    TYPE_REBOOT = 'reboot'

    POWER_REQUEST_NODE_BASE = "<power/>"

    def load(self, to_dict=False):

        def __dict(entry):
            d = {}
            try:
                node = entry.get()
                node.get_node
                d['delay'] = node.get_node('powerschedule').is_set_to_true('delay')
                for day in ('sun', 'mon', 'tue', 'wed', 'thu', 'fri', 'sat'):
                    xpath_dayschedule = 'powerschedule/dayschedule[@day="%s"]' % day.capitalize()
                    d[day] = {}
                    d[day]['on'] = node.get_node('%s/ontime' % xpath_dayschedule).is_enabled()
                    d[day]['ontime'] = time(int(node.get_value('%s/ontime/hour' % xpath_dayschedule)),
                                            int(node.get_value('%s/ontime/minute' % xpath_dayschedule)))
                    d[day]['off'] = node.get_node('%s/offtime' % xpath_dayschedule).is_enabled()
                    d[day]['offtime'] = time(int(node.get_value('%s/offtime/hour' % xpath_dayschedule)),
                                             int(node.get_value('%s/offtime/minute' % xpath_dayschedule)))
            except:
                pass
            return d

        entry = self.request(Entry(self.TYPE_LOAD, '<powersetting />'))
        return __dict(entry) if to_dict else entry 

    def apply2(self, config=None, log=True, messages=None):
        #config={
        #    'autoresume': True,
        #    'schedule': {
        #        'delay': True,
        #        'daylist': {
        #            'Sun': {
        #                'on': {
        #                    'enable': True,
        #                    'time': time,
        #                },
        #                'off': {
        #                    'enable': True,
        #                    'time': time,
        #                },
        #            },
        #            'Mon': { ... },
        #            'Tue': { ... },
        #            'Wed': { ... },
        #            'Thu': { ... },
        #            'Fri': { ... },
        #            'Sat': { ... },
        #        },
        #    },
        #}
        pass
        _default_messages = {
            Results.OK: ['MSG_POWER_CHANGE', {}],
            None: ['ERR_POWER_CHANGE', {}],
        }
        _default_messages.update(messages or {})

        root_node = XmlNode('<powersetting />')
        if isinstance(config.get('autoresume', None), bool):
            autoresume = config.get('autoresume')
            root_node.append_child('autoresume')
            root_node.get_node('autoresume').enable(autoresume)
        if isinstance(config.get('schedule', None), dict):
            schedule = config.get('schedule')
            powerschedule_node = XmlNode('<powerschedule />')
            #powerschedule_node.enable(schedule.get('enable', False))
            powerschedule_node.set_attrs(
                    {'delay': str(schedule.get('delay', False)).lower()})
            for day, dayconf in schedule.get('daylist', {}).iteritems():
                dayschedule_node = XmlNode('<dayschedule />')
                dayschedule_node.set_attrs({'day': day[:3].capitalize()})
                for onoff, onoffconf in dayconf.iteritems():
                    onoffenable = onoffconf.get('enable', False)
                    onofftime = onoffconf.get('time', None)
                    if onoffenable and onofftime is None:
                        continue
                    onofftime_node = XmlNode("%stime" % onoff)
                    #onofftime_node.enable(onoffenable)
                    onofftime_node.set_attrs(
                            {'enable': str(onoffenable).lower()})
                    onofftime_node.append_child('hour', text=str(onofftime.hour))
                    onofftime_node.append_child('minute', text=str(onofftime.minute))
                    dayschedule_node.append_child(onofftime_node)
                powerschedule_node.append_child(dayschedule_node)
            root_node.append_child(powerschedule_node)
        
        entry = self.request(Entry(self.TYPE_APPLY, root_node))
        result = getattr(entry, 'get_result', lambda: None)()

        message = _default_messages.get(result, _default_messages[None])
        self._set_message(message[0], arglist=message[1], log=log)

        return (result in [Results.OK])

    def apply(self, autoresume=None, schedule=None, log=True):
# ex)                                schedule = {
#            'Sun': {'ontime': {'hour':0, 'min':0},
#                    'offtime':{'hour':0, 'min':0}},
#            'Mon': {'ontime': {'hour':0, 'min':0},
#                    'offtime':{'hour':0, 'min':0}},
#            'Tue': {'ontime': {'hour':0, 'min':0},
#                    'offtime':{'hour':0, 'min':0}},
#            'Wed': {'ontime': {'hour':0, 'min':0},
#                    'offtime':{'hour':0, 'min':0}},
#            'Thu': {'ontime': {'hour':0, 'min':0},
#                    'offtime':{'hour':0, 'min':0}},
#            'Fri': {'ontime': {'hour':0, 'min':0},
#                    'offtime':{'hour':0, 'min':0}},
#            'Sat': {'ontime': {'hour':0, 'min':0},
#                    'offtime':{'hour':0, 'min':0}}}
        root_node = XmlNode('<powersetting />')
        if isinstance(autoresume, bool):
            root_node.append_child(
                    'autoresume',
                    attrs={'enable': self.bool_to_text(autoresume)})
        if isinstance(schedule, dict):
            powerschedule_node = XmlNode('<powerschedule />')
            for day, time_dict in schedule.iteritems():
                dayschedule_node = XmlNode('<dayschedule />')
                dayschedule_node.set_attrs({'day': day[:3].capitalize()})
                for onofftime in ('ontime', 'offtime'):
                    try:
                        hour = int(time_dict[onofftime]['hour'])
                        min_ = int(time_dict[onofftime]['min'])
                    except (KeyError, ValueError):
                        continue
                    onofftime_node = XmlNode(onofftime)
                    onofftime_node.set_attrs({'enable': 'true'})
                    onofftime_node.append_child('hour', text=str(hour))
                    onofftime_node.append_child('minute', text=str(min_))
                    dayschedule_node.append_child(onofftime_node)
                powerschedule_node.append_child(dayschedule_node)
            root_node.append_child(powerschedule_node)
        entry = self.request(Entry(self.TYPE_APPLY, root_node))
        if self.get_result(entry=entry):
            self._message = self._get_message('ERR_POWER_CHANGE', log=log)
        else:
            self._message = self._get_message('MSG_POWER_CHANGE', log=log)
        return entry

    def shutdown(self, force=False):
        power_node = XmlNode(self.POWER_REQUEST_NODE_BASE)
        if force is True:
            power_node.append_child("option", attrs={"force": "true"})

        entry = self.request(Entry(self.TYPE_SHUTDOWN, str(power_node)))
        return entry

    def reboot(self, force=False):
        power_node = XmlNode(self.POWER_REQUEST_NODE_BASE)
        if force is True:
            power_node.append_child("option", attrs={"force": "true"})

        entry = self.request(Entry(self.TYPE_REBOOT, str(power_node)))
        return entry

if __name__ == '__main__':
    i = Power()
    print i.load(to_dict=True)
