# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from command import Command, _, escape
from ionas.request import Entry


class Proxy(Command):
    try:
        from fal_info import FAL_PROXY_PATH as COMMAND
    except:
        COMMAND_NAME = 'fal_proxy.py'

    TYPE_LOAD = 'load'
    TYPE_SAVE = 'save'
    TYPE_QUERY = 'query'

    XML_FMT_SAVE_ENTRY = u"""<proxy enable="%(enable)s">
        <manual enable="%(manual)s">
            <host>%(host)s</host>
            <port>%(port)s</port>
        </manual>
        <pacfile enable="%(pacfile)s">
            <url>%(url)s</url>
        </pacfile>
    </proxy>"""


    def load(self, to_dict=True):
        def _dict(self, entry):
            return {
                'enable': self.text_to_bool(
                        self.get_value('@enable', 'false')),
                'manual': {
                    'enable': self.text_to_bool(
                            self.get_value('manual/@enable', 'false')),
                    'host': self.get_value('manual/host', ''),
                    'port': self.get_value('manual/port', ''),
                },
                'pacfile': {
                    'enable': self.text_to_bool(
                            self.get_value('pacfile/@enable', 'false')),
                    'url': self.get_value('pacfile/url', ''),
                },
            }
        entry = self.request(Entry(self.TYPE_LOAD, '<proxy />'))
        return _dict(self, entry) if to_dict else entry

    def save(self,
            enable=False,
            manual=False,
            host='',
            port=None,
            pacfile=False,
            url=''):
        entry = self.request(Entry(self.TYPE_SAVE, self.XML_FMT_SAVE_ENTRY % {
            'enable': self.BOOL_TO_TEXT[enable],
            'manual': self.BOOL_TO_TEXT[manual],
            'host': host,
            'port': port if isinstance(port, int) else '',
            'pacfile': self.BOOL_TO_TEXT[pacfile],
            'url': escape(url),
        }))
        if self.get_result(entry=entry):
            self._message = self._get_message('ERR_PROXY_CHANGE')
        else:
            self._message = self._get_message('MSG_PROXY_CHANGE')
        return entry


if __name__ == '__main__':
    proxy = Proxy()
    proxy.load()
    print "proxy: %s" % proxy.get_value('@enable', 'NG')
    print "pacfile: %s" % proxy.get_value('pacfile/@enable', 'NG')
    print "url: %s" % proxy.get_value('pacfile/url', 'NG')
    print "manual: %s" % proxy.get_value('manual/@enable', 'NG')
    print "host: %s" % proxy.get_value('manual/host', 'NG')
    print "url: %s" % proxy.get_value('manual/port', 'NG')
#    print "MESSAGES: %s" % proxy._MESSAGES
#    proxy.load()
#    print proxy._entry.get()
#    print proxy.get_message()
#    print "Enable: %s" % proxy.get_value('@type')
#    print "Manual: %s" % proxy.get_value('manual/@enable')
#    print "Pacfile: %s" % proxy.get_value('pacfile/@enable')
#    print proxy.save()
#    print proxy.get_message()
