# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from command import Command, _, escape
from ionas.request import Entry, XmlNode
from product_conf import STOR_IF_TO_VOL
from fal_info import Results


class Quota(Command):
    try:
        from fal_info import FAL_QUOTA_PATH as COMMAND
    except:
        COMMAND_NAME = 'fal_quota.py'

    TYPE_LOAD = 'load'
    TYPE_QUERY = 'query'
    TYPE_APPLY = 'apply'

    def __init__(self, *args, **kwargs):
        super(Quota, self).__init__(*args, **kwargs)
        self.__query_entry_cache = None

    def _load(self):
        entry = Entry(self.TYPE_LOAD, '<quota />')
        return self.request(entry)

    def load(self, to_dict=True):
        def _to_dict(entry):
            try:
                dict_ = {}
                quota_node = entry.get()
                for volume_node in quota_node.get_node_list('volumelist/volume'):
                    dict_[volume_node.get_text()] = volume_node.is_enabled()
                return dict_
            except:
                return {}
        entry = self._load()
        return _to_dict(entry) if to_dict else entry

    def _query(self, volume="/mnt/sataraid1"):
        entry = Entry(self.TYPE_QUERY, """<quota>
            <volume>%(volume)s</volume>
        </quota>""" % {'volume': volume})
        return self.request(entry)

    def query(self, volume="/mnt/sataraid1", to_dict=True):
        def _to_dict(entry):
            try:
                dict_ = {}
                for ug in ('user', 'group'):
                    dict_["%slist" % ug] = {}
                    for ugnode in entry.get().get_node_list(
                            "%(ug)slist/%(ug)s" % {'ug': ug}):
                        name = ugnode.get_value('name', None)
                        if not name:
                            continue
                        used = ugnode.get_value('quota/used', None)
                        limit = ugnode.get_value('quota/limit', None)

                        dict_["%slist" % ug][name] = {}
                        dict_["%slist" %ug][name]['name'] = name
                        dict_["%slist" % ug][name]['used'] = used
                        dict_["%slist" % ug][name]['limit'] = limit
                return dict_
            except:
                return {}
        entry = self._query(volume=volume)
        return _to_dict(entry) if to_dict else entry

    def _apply(self, qdict, volume="/mnt/sataraid1"):
        """
        qdict = {
            'userlist': {
                'USER001': { 'limit': 1 },
                ...
            },
            'grouplist': {
                'GROUP001': { 'limit': 10 },
                ...
            },
        }
        """
        if not isinstance(qdict, dict):
            return None

        entry = Entry(self.TYPE_APPLY, '<quota />')
        quota_node = entry.get()
        for ug in ('user', 'group'):
            uglist = qdict.get('%slist' % ug, None)
            if not isinstance(uglist, dict):
                continue
            quota_node.append_child('%slist' % ug)
            uglist_node = quota_node.get_node('%slist' % ug)
            for name, param in uglist.iteritems():
                ugnode = XmlNode(ug)
                ugnode.append_child('name', text=name)
                ugnode.append_child('quota')
                ugnode.get_node('quota').set_attrs({'volume': volume})
                ugnode.get_node('quota').append_child(
                        'limit',
                        text=str(param.get('limit', 0.0)))
                uglist_node.append_child(ugnode)
        entry = self.request(entry)
        return entry

    def apply(self, qdict, volume="/mnt/sataraid1", log=True):
        entry = self._apply(qdict, volume=volume)
        if self.get_result(boolean=True):
            self._set_message('MSG_QUOTA_CHANGE', log=log)
            return True
        else:
            self._set_message('ERR_QUOTA_CHANGE', log=log)
            return False

    def is_enabled(self, volume='/mnt/sataraid1'):
        try:
            return self.load()[volume]
        except:
            return False


if __name__ == '__main__':
    i = Quota()
    print i.load()
    print i.is_enabled()
#    print i.query()
#    print i.apply({'userlist': {'nobody': {'limit': 10}}})
