# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from command import Command, _
from ionas.request import Entry
from datetime import datetime
import time


class Replication(Command):
    try:
        from fal_info import FAL_REPLICATION_PATH as COMMAND
    except:
        COMMAND_NAME = 'fal_replication.py'

    TYPE_FEATURE = 'feature'
    TYPE_READ = 'read'

    epoch_to_datetime = lambda self, e: datetime(*time.localtime(e)[:6])
    datetime_to_epoch = lambda self, d: int(time.mktime(d.timetuple()))

    def feature(self, to_dict=True):
        def _to_dict(entry):
            dic = dict()
            try:
                for node in entry.get().get_node_list('feature/*'):
                    dic[node.get_node_name()] = node.is_enabled()
            except:
                pass
            return dic
        entry = self._feature()
        return _to_dict(entry) if to_dict else entry

    def read(self, to_dict=True):
        def _to_dict(entry):
            dic = dict()
            try:
                replication_node = entry.get()
                for share_node in replication_node.get_node_list('sharelist/share'):
                    name = share_node.get_value('name')
                    timestamp = int(share_node.get_value('timestamp'))  # epoch
                    result = share_node.get_value('result')  # 0/1
                    dic[name] = dict()
                    dic[name]['name'] = name
                    dic[name]['timestamp'] = self.epoch_to_datetime(timestamp)
                    dic[name]['result'] = not bool(int(result))
            except:
                pass
            return dic
        entry = self._read()
        return _to_dict(entry) if to_dict else entry



    def _feature(self):
        entry = Entry(self.TYPE_FEATURE, '<replication />')
        return self.request(entry)

    def _read(self):
        entry = Entry(self.TYPE_READ, '<read />')
        return self.request(entry)


if __name__ == '__main__':
    rep = Replication()
    print rep.feature()
    print rep.read()
