# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from command import Command
from ionas.request import Entry


class Service(Command):
    try:
        from fal_info import FAL_SERVICECTRL_PATH as COMMAND
    except:
        COMMAND_NAME = 'fal_servicectrl.py'

    TYPE_QUERY = 'query'

    def _get_categorylist_node_str(self, category_list):
        return "<categorylist>%s</categorylist>" % "".join([
            "<category><name>%s</name></category>" % category
            for category
            in category_list])

    def _get_servicelist_node_str(self, service_list):
        return "<servicelist>%s</servicelist>" % "".join([
            "<service><name>%s</name></service>" % service
            for service
            in service_list])

    def query(self, category_list=[], service_list=[], to_dict=True):
        def _to_dict(entry):
            dict = {}
            for service_node in entry.get(
                ).get_node_list("servicelist/service"):
                service_name = service_node.get_value("name")
                dict[service_name] = {
                    "running": service_node.is_set_to_true("running"),
                    "suspend": service_node.is_set_to_true("suspend"),
                    "stop": service_node.is_set_to_true("stop"),
                    "enable": service_node.is_set_to_true("enable")}
            return dict
        servicectrl_node_str = "<servicectrl>%s%s</servicectrl>" % (
            self._get_categorylist_node_str(category_list),
            self._get_servicelist_node_str(service_list))
        entry = self.request(Entry(self.TYPE_QUERY, servicectrl_node_str))
        return _to_dict(entry) if to_dict else entry

if __name__ == '__main__':
    i = Service()
    print i.query(category_list=["disk"])
    print i.query(category_list=["disk", "ups"])
    print i.query(service_list=["samba"])
    print i.query(service_list=["samba", "rsync"])
    print i.query(service_list=["samba", "rsync"], to_dict=False)
