# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from command import Command, _, escape
from ionas.request import Entry, XmlNode

import os
import os.path as p

class Share(Command):
    try:
        from fal_info import FAL_SHARES_PATH as COMMAND
    except:
        COMMAND_NAME = 'fal_shares.py'

    TYPE_LOAD = 'load'
#    TYPE_ADD = 'add'
#    TYPE_MODIFY = 'modify'
#    TYPE_DELETE = 'delete'
    TYPE_QUERY = 'query'

    BASIC_SETTINGS = {
        'name': {
            'XPATH_FROM_SHARE': 'name',
            'TYPE': str},
        'quota_limit': {
            'XPATH_FROM_SHARE': 'shareservice/msshare/quota/limit',
            'TYPE': str},
        'undeletable': {
            'XPATH_FROM_SHARE': '@undeletable',
            'TYPE': lambda text: \
                    {'true': True, 'false': False}.get(text.lower(), False)},
    }

    EXTRA_SETTINGS_PATH = p.join(p.dirname(__file__), "extras", "share")

    def __init__(self, *args, **kwargs):
        super(Share, self).__init__(*args, **kwargs)
        self._settings = {}
        self._settings.update(self.BASIC_SETTINGS)
        self._settings.update(self.__get_extra_settings())

    def __get_extra_settings(self):
        extra_settings = {}
        for fname in os.listdir(self.EXTRA_SETTINGS_PATH):
            with open(p.join(self.EXTRA_SETTINGS_PATH, fname)) as f:
                extra_settings.update(eval(f.read()))
        return extra_settings

    def load(self, to_dict=True):
        def _to_dict(entry):
            dict_ = {}
            for share_node in entry.get().get_node_list('sharelist/share'):
                name = share_node.get_value('name', None)
                if name is None:
                    continue
                dict_[name] = {}
                for sname, sparams in self._settings.iteritems():
                    val = share_node.get_value(sparams['XPATH_FROM_SHARE'])
                    #dict_[name][sname] = val
                    dict_[name][sname] = sparams.get('TYPE', lambda v: v)(val)
            return dict_
        entry = self.request(Entry(self.TYPE_LOAD, '<shares />'))
        return _to_dict(entry) if to_dict else entry

    def query(self, to_dict=True):
        def _to_dict(entry):
            try:
                return dict([
                        (n.get_value('name'), n.get_value('quota/used')) \
                        for n in entry.get().get_node_list('sharelist/share')])
            except:
                return dict([])
        entry = self.request(Entry(self.TYPE_QUERY, '<shares />'))
        return _to_dict(entry) if to_dict else entry

if __name__ == '__main__':
    i = Share()
    print i.load()
    print i.query()
