# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from command import Command, _, escape
from ionas.request import Entry, XmlNode
from fal_info import Results


class Statemgr(Command):
    try:
        from fal_info import FAL_STATEMGR_PATH as COMMAND
    except:
        COMMAND_NAME = 'fal_statemgr.py'

    TYPE_QUERY = 'query'
    TYPE_CHANGE = 'change'

    HUMANIZE = {
        'raid': {
            'preparing': _(u'StateRaidPreparing'),
            'running': _(u'StateRaidRunning'),
            'stop': _(u'StateRaidStop'),
            'degrade': _(u'StateRaidDegrade'),
            'fail': _(u'StateRaidFail'),
            'crash': _(u'StateRaidCrash'),
        },
        'raidvol': {
            'preparing': _(u'StateRaidvolPreparing'),
            'keywaiting': _(u'StateRaidvolKeywaiting'),
            'running': _(u'StateRaidvolRunning'),
            'rebuilding': _(u'StateRaidvolRebuilding'),
            'fail': _(u'StateRaidvolFail'),
            'mountfail': _(u'StateRaidvolMountfail'),
        },
    }

    def query(self, to_dict=True):
        def _to_dict(entry):
            dict_ = {}
            try:
                for state_node in entry.get().get_node_list('statelist/state'):
                    cate = state_node.get_attr_value('category', None)
                    if cate is None:
                        continue
                    dict_[cate] = state_node.get_text()
            except:
                pass
            return dict_
        entry = self.request(Entry(self.TYPE_QUERY, """<statemgr />"""))
        return _to_dict(entry) if to_dict else entry

    def _change(self, category, state):
        entry = self.request(Entry(self.TYPE_CHANGE, """<statemgr>
            <state category="%(category)s">%(state)s</state>
        </statemgr>""" % {
            'category': category,
            'state': state,
        }))
        return entry

    def change(self, category, state, log=True):
        entry = self._change(category, state)
        if entry:
            if entry.get_result() == Results.OK:
                return True
            else:
                self._set_message('ERR_SYSTEM_STATE_CHANGE', log=log)
                return False
        else:
            self._set_message('ERR_SYSTEM_STATE_CHANGE', log=log)
            return False

    def is_keywaiting(self):
        state = self.query().get('raidvol', '')
        return True if state == 'keywaiting' else False

    def is_raidcrash(self):
        state = self.query().get('raid', '')
        return True if state == 'crash' else False

    def raid_status(self):
        info = self.query()
        raid = info.get('raid', None)
        raidvol = info.get('raidvol', None)

        output = self.HUMANIZE['raid'].get(raid, _(u'unknown'))
        _raidvol = self.HUMANIZE['raidvol'].get(raidvol, _(u'unknown'))
        if raid == 'running':
            output = _raidvol
        elif raid == 'degrade':
            if raidvol in ('keywaiting', 'rebuilding', 'fail', 'mountfail'):
                output = "%s/%s" % (output, _raidvol)
        return output


if __name__ == '__main__':
    i = Statemgr()

    print i.query()
    print "KEYWAITING: %s" % i.is_keywaiting()

    print "SYSTEM ERR: %s(False ok)" % i.change(category="system", state="err")
    print i.query()['system']
    print "SYSTEM BUSY: %s" % i.change(category="system", state="busy")
    print i.query()['system']
    print "SYSTEM RUNNING: %s" % i.change(category="system", state="running")
    print i.query()['system']

    print "RAID STATUS: %s" % i.raid_status()
