# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from command import Command, _
from ionas.request import Entry, XmlNode
from fal_info import Results


class SysInfo(Command):
    try:
        from fal_info import FAL_SYSINFO_PATH as COMMAND
    except:
        COMMAND = 'fal_sysinfo.py'

    TYPE_QUERY = 'query'
    TYPE_APPLY = 'apply'

    def query(self, to_dict=True):
        def _to_dict(entry):
            try:
                sysinfo_node = entry.get()
                hostdesc_node = sysinfo_node.get_node('hostdesc')
            except:
                class DummyXmlNode(object):
                    def get_value(self, xpath, default="", *args, **kwargs):
                        return default
                sysinfo_node = DummyXmlNode()
                hostdesc_node = DummyXmlNode()
            info = {}
            info['host'] = {}
            info['temperature'] = {}

            info['host']['name'] = hostdesc_node.get_value('name', None)
            info['host']['desc'] = hostdesc_node.get_value('desc', None)
            info['host']['location'] = hostdesc_node.get_value('location', None)
            info['series'] = sysinfo_node.get_value('series', None)
            info['model'] = sysinfo_node.get_value('model', None)
            info['fwversion'] = sysinfo_node.get_value('fwversion', None)
            info['temperature']['dimm'] = sysinfo_node.get_value(
                    'fwversion[@type="dimm"]', None)
            info['temperature']['mb'] = sysinfo_node.get_value(
                    'fwversion[@type="mb"]', None)
            info['temperature']['cpu'] = sysinfo_node.get_value(
                    'fwversion[@type="cpu"]', None)
            info['fanrpm'] = sysinfo_node.get_value('fanrpm', None)

            info['condition'] = sysinfo_node.get_value('condition', None)
            info['changelist'] = []
            for change_node in sysinfo_node.get_node_list('changelist/change'):
                version = change_node.get_value('@version')
                itemlist = []
                for item_node in change_node.get_node_list('itemlist/item'):
                    itemlist.append(item_node.get_text())
                info['changelist'].append({
                    'version': version,
                    'itemlist': itemlist,
                })

            return info
        entry = self.request(Entry(self.TYPE_QUERY, '<sysinfo />'))
        return _to_dict(entry) if to_dict else entry

    def _apply(self,
            hostname=None,
            hostlocation=None,
            hostdesc=None):
        entry = Entry(self.TYPE_APPLY, """<sysinfo><hostdesc /></sysinfo>""")
        host_node = entry.get().get_node('hostdesc')
        if hostname is not None:
            host_node.append_child('name', text=hostname)
        if hostlocation is not None:
            host_node.append_child('location', text=hostlocation)
        if hostdesc is not None:
            host_node.append_child('desc', text=hostdesc)
        return self.request(entry)

    def apply(self,
            hostname=None,
            hostlocation=None,
            hostdesc=None,
            to_bool=True, log=True):
        entry = self._apply(
                hostname=hostname,
                hostlocation=hostlocation,
                hostdesc=hostdesc)
        result = self.get_result(entry=entry)
        if int(result) == int(Results.OK):
            self._set_message('MSG_LANDISK_NAME_CHANGE', log=log)
            result = True
        else:
            self._set_message('ERR_LANDISK_NAME_CHANGE', log=log)
            result = False
        return result if to_bool else entry

    def clear(self):
        entry = self._clear()
        
        result = entry.get_result()

        return result in [Results.OK]

    def _clear(self):
        entry = Entry('clear', """<sysinfo><condition /></sysinfo>""")
        return self.request(entry)

if __name__ == '__main__':
    i = SysInfo()
    print i.query()
    print i.clear()
