# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from command import Command, _
from ionas.request import Entry, XmlNode


class UPS(Command):
    try:
        from fal_info import FAL_UPS_PATH as COMMAND
    except:
        COMMAND_NAME = 'fal_ups.py'
    MESSAGES = {
#        '257': _(u'TEST'),  #FIXME msg
    }

    TYPE_LOAD = 'load'
    TYPE_APPLY = 'apply'

    def load(self):
        return self.request(Entry(self.TYPE_LOAD, '<ups />'))

    def apply(self,
            warning=None,
            time=None,
            netshutdown=None):
        entry = Entry(self.TYPE_APPLY, '<ups />')
        ups = entry.get()
        if isinstance(warning, bool):
            ups.append_child('warning', attrs={
                'enable':self.BOOL_TO_TEXT[warning]})
        if isinstance(time, int):
            ups.append_child('time')
            ups.get_node('time').append_child('minute', text=str(time))
        if isinstance(netshutdown, list):
            node = XmlNode('<netshutdown><targetlandisklist></targetlandisklist></netshutdown>')
            targetlandisklist = node.get_node('targetlandisklist')
            for target in netshutdown:
                targetlandisklist.append_child(XmlNode(
                """<targetlandisk enable="%(enable)s" number="%(number)s">
                    <host>%(host)s</host>
                    <passwd>%(passwd)s</passwd>
                </targetlandisk>""" % {
                    'number': target.get('number', 0),
                    'enable': self.BOOL_TO_TEXT[target.get('enable', False)],
                    'host': target.get('host', ''),
                    'passwd': target.get('password', ''),
                }))
            ups.append_child(node)
        return self.request(entry)

if __name__ == '__main__':
    i = UPS()
    i.load()
    print "Load: %s" % i.get_message()
    print "Warning: %s" % i.get_value('warning', 'false')
    print "Time: %s" % i.get_value('time/minute', None)
    i.apply(warning=True, time=5)
    print "Apply: %s" % i.get_message()
    i.apply(netshutdown=[
            {
                'number': 1,
                'enable': True,
                'host': 'landisk-123456',
                'password': 'abc',
            },
            {
                'number': 3,
                'enable': False,
                'host': 'landisk-abcdef',
                'password': '123',
            },
    ])
    print "Apply: %s" % i.get_message()
