# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from subprocess import Popen, PIPE
from command import Command, _, escape
from ionas.request import Entry, XmlNode
from fal_info import Results, ReqTypes


class User(Command):
    try:
        from fal_info import FAL_USERS_PATH as COMMAND
    except:
        COMMAND_NAME = 'fal_users.py'

    TYPE_AUTH = 'auth'
    TYPE_LOAD = 'load'
    TYPE_ADD = 'add'
    TYPE_MODIFY = 'modify'
    TYPE_DELETE = 'delete'
    TYPE_QUERY = 'query'
    TYPE_DOMAIN_LIST = 'domain_list'

    def auth(self, user='admin', password='', to_bool=True):
        xml = """<users>
            <user>
                <name>%s</name>
                <passwd>%s</passwd>
            </user>
        </users>""" % (user, password)
        entry = self.request(Entry(self.TYPE_AUTH, xml))
        return not bool(self.get_result()) if to_bool else entry

    def load(self, to_dict=True, all=False):
        def to_dict(entry, all=False):
            dict_ = {}
            users = entry.get()
            for user in users.get_node_list('userlist/user'):
#                    'userlist/user[not(@hidden="true")]'):
                try:
                    name = user.get_value('name')
                    dict_[name] = {
                        'hidden': user.is_set_to_true('hidden'),

                        'name': name,
                        'uid': user.get_value('uid'),
                        'gid': user.get_value('gid'),
                        'fullname': user.get_value('fullname', ''),
                        'desc': user.get_value('desc', ''),
                        'passwd': user.get_value('passwd', do_strip=False),
                        'quota': user.get_value('quota/limit', None),
                        'mailaddr': user.get_value('mailaddr', None),
                        'timeout': user.get_value('timeout', None),
                        'help': bool(int(user.get_value('help', "0"))),
                    }
                except:
                    continue
            if all is False:
                for name, param in dict_.copy().iteritems():
                    if param['hidden'] is True:
                        del dict_[name]
            return dict_
        entry = self.request(Entry(self.TYPE_LOAD, '<users />'))
        return to_dict(entry, all=all) if to_dict else entry

    def _domain_list(self):
        entry = Entry(self.TYPE_DOMAIN_LIST, '<users />')
        return self.request(entry)

    def domain_list(self, to_dict=True):
        def _to_dict(entry):
            """<entry type="domain_list" result="0">
                <users>
                    <userlist>
                        <user>
                            <upn>DOMAIN@USER001</upn>
                            <domain>DOMAIN</domain>
                            <name>USER001</name>
                            <quota><limit>10</limit></quota>
                        </user>
                    </userlist>
                </users>
            </entry>"""
            dict_ = {}
            try:
                for user_node in entry.get().get_node_list('userlist/user'):
                    upn = user_node.get_value('upn', None)
                    domain = user_node.get_value('domain', None)
                    name = user_node.get_value('name', None)
                    qlimit = user_node.get_value('quota/limit', None)
                    if upn:
                        upn = upn if isinstance(upn, unicode) else upn.decode('utf-8')
                        dict_[upn] = {}
                        dict_[upn]["upn"] = upn
                        dict_[upn]["domain"] = domain
                        dict_[upn]["name"] = name
                        dict_[upn]["quota"] = qlimit
            except:
                pass
            return dict_
        entry = self._domain_list()
        return _to_dict(entry) if to_dict else entry

    def _req_add_or_mod(self,
            type_,  # TYPE_ADD / TYPE_MODIFY
            name,
            fullname=None,
            desc=None,
            uid=None,
            gid=None,
            passwd=None,
            quota=None,
            mailaddr=None,
            timeout=None,
            help=None):
        entry = Entry(type_, '<users><user /></users>')
        user = entry.get().get_node('user')
        if isinstance(name, (str, unicode)):
            user.append_child('name', text=name)
        if isinstance(fullname, (str, unicode)):
            user.append_child('fullname', text=fullname)
        if isinstance(desc, (str, unicode)):
            user.append_child('desc', text=desc)
        if isinstance(uid, int):
            user.append_child('uid', text=str(uid))
        if isinstance(gid, int):
            user.append_child('gid', text=str(gid))
        if gid == '':  # 空なら空でリクエストする(デフォルトgidになる)
            user.append_child('gid', text='')
        if isinstance(passwd, (str, unicode)):
            user.append_child('passwd', text=passwd)
        if isinstance(quota, (int, float)):
            user.append_child('quota')
            user.get_node('quota').append_child('limit', text=str(quota))
        if isinstance(mailaddr, (str, unicode)):
            user.append_child('mailaddr', text=mailaddr)  # FIXME mailaddr
        if isinstance(timeout, int):
            user.append_child('timeout', text=str(timeout))
        if isinstance(help, bool):
            user.append_child('help', text=str(int(help)))
        return self.request(entry)

    def add(self,
            name,
            fullname=None,
            desc=None,
            uid=None,
            gid=None,
            passwd=None,
            quota=None,
            mailaddr=None):
        entry = self._req_add_or_mod(
                self.TYPE_ADD,
                name,
                fullname=fullname,
                desc=desc,
                uid=uid,
                gid=gid,
                passwd=passwd,
                quota=quota,
                mailaddr=mailaddr)
        arglist = {'name': name}
        result = self.get_result()
        if result:
            if result == int(Results.MAX_EXCEED_ERROR):
                self._set_message('ERR_USER_NO_MORE', arglist=arglist, log=True)
            else:
                self._set_message('ERR_USER_ADD', arglist=arglist, log=True)
        else:
            self._set_message('MSG_USER_ADD', arglist=arglist, log=True)
        return entry

    def modify(self,
            name=None,
            fullname=None,
            desc=None,
            uid=None,
            gid=None,
            passwd=None,
            quota=None,
            mailaddr=None,
            log=True):
        entry = self._req_add_or_mod(
                self.TYPE_MODIFY,
                name,
                fullname=fullname,
                desc=desc,
                uid=uid,
                gid=gid,
                passwd=passwd,
                quota=quota,
                mailaddr=mailaddr)
        arglist = {'name': name}
        if self.get_result():
            self._set_message(
                    'ERR_USER_INFO_CHANGE',
                    arglist=arglist,
                    log=log)
        else:
            self._set_message(
                    'MSG_USER_INFO_CHANGE',
                    arglist=arglist,
                    log=log)
        return entry

    def delete(self, name):
        entry = Entry(self.TYPE_DELETE, '<users><user /></users>')
        user = entry.get().get_node('user')
        if isinstance(name, (str, unicode)):
            user.append_child('name', text=name)
        else:
            raise  # FIXME
        entry = self.request(entry)
        arglist = {'name': name}
        if self.get_result():
            self._set_message('ERR_USER_DELETE', arglist=arglist, log=True)
        else:
            self._set_message('MSG_USER_DELETE', arglist=arglist, log=True)
        return entry

    def query(self, to_dict=True, all=False):
        def to_dict(entry, all=False):
            try:
                return dict([
                        (n.get_value('name'), n.get_value('quota/used')) \
                        for n in entry.get().get_node_list('userlist/user') \
                            if all or not n.is_set_to_true('hidden')])
            except:
                return dict([])
        entry = self.request(Entry(self.TYPE_QUERY, '<users />'))
        return to_dict(entry, all=all) if to_dict else entry

    def adminmod(self,
                 passwd=None,
                 mailaddr=None,
                 timeout=None,
                 help=None,
                 log=True):
        entry = self._req_add_or_mod(
                self.TYPE_MODIFY,
                'admin',
                passwd=passwd,
                mailaddr=mailaddr,
                timeout=timeout,
                help=help)
        if self.get_result(entry=entry, boolean=True):
            self._set_message('MSG_ADMIN_CHANGE', log=log)
            return True
        else:
            self._set_message('ERR_ADMIN_CHANGE', log=log)
            return False
        return entry

    # ここから ===
    # 以下メソッドはwbinfoを直接実行している
    # 設計思想上は好ましくないがFALを使うと非常に遅いのでこうする
    def wbinfo_u(self):
        try:
            proc = Popen(['wbinfo', '-u'], stdout=PIPE)
            out, err = proc.communicate()
#            return sorted([upn for upn in out.splitlines()])

            def _get_workgroup():
                from fal_info import FAL_MSNETWORK_PATH
                response = self._request(
                        Entry(ReqTypes.LOAD),
                        command=FAL_MSNETWORK_PATH)
                ms_node = response.get_entrylist().get_entry().get()
                method = ms_node.get_value('assign/@method')
                return {
                    'wg': '',
                    'nt': ms_node.get_value('assign/nt/domain', ''),
                    'ad': ms_node.get_value('assign/ad/netbios', ''),
                }.get(method, '')
            workgroup = _get_workgroup()

            import product_conf
            return sorted([upn for upn in out.splitlines() \
                    if upn.split(getattr(product_conf, "WINBIND_SEPARATOR", "@"))[0].lower() == workgroup.lower()])
        except:
            return []

    def wbinfo_i(self, uname):
        info = dict()
        keys = ['name', 'passwd', 'uid', 'gid', 'fullname', 'homedir', 'shell']
        for k in keys:
            info[k] = None
        try:
            proc = Popen(['wbinfo', '--user-info', uname], stdout=PIPE)
            out, err = proc.communicate()
            for k, v in zip(keys, out.strip().split(':')):
                info[k] = v
            return info
        except:
            return info
    # === ここまで


if __name__ == '__main__':
    u = User()
    for name in u.wbinfo_u():
        print u.wbinfo_i(name)
