# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from command import Command, _, escape
from ionas.request import Entry, XmlNode
from product_conf import STOR_IF_TO_VOL
from fal_info import Results




class Volctrl(Command):
    try:
        from fal_info import FAL_VOLCTRL_PATH as COMMAND
    except:
        COMMAND_NAME = 'fal_volctrl.py'

    TYPE_QUERY = 'query'
    TYPE_MKFS = 'mkfs'
    TYPE_FSCK = 'fsck'
    TYPE_MOUNT = 'mount'
    TYPE_UNMOUNT = 'unmount'

    VOLUME = STOR_IF_TO_VOL
    #      = {
    #     "internal": {
    #         1: "sataraid1",
    #     },
    #     "usb": {
    #         1: "usb1",
    #         2: "usb2",
    #         3: "usb3",
    #         4: "usb4",
    #     },
    # }
    __get_vol_list = lambda: [vol for vols in VOLUME.values() for vol in vols.values()]
    #__get_vol_list = lambda: [vol for vols in STOR_IF_TO_VOL.values() for vol in vols.values()]

    ## Avoiding to KeyError for message.
    #for idx in range(1, 5):
    #    if idx not in VOLUME['usb']:
    #        VOLUME['usb'][idx] = ""

    FS = ['xfs', 'ext3', 'ext4', 'fat']

    def __init__(self, *args, **kwargs):
        super(Volctrl, self).__init__(*args, **kwargs)
        self.__query_cache = None

    def _is_vol(self, vol, vols=VOLUME):
        for val in vols.values():
            if isinstance(val, dict):
                if self._is_vol(vol, vols=val):
                    return True
            else:
                if vol == val:
                    return True
        return False

    def gb(self, byte):
        return byte / 1000 / 1000 / 1000

    def query(self, volume=VOLUME['internal'][1], to_dict=True):
        def _to_dict(entry):
            dict_ = {}
            try:
                vol = entry.get().get_node('volume')
                dict_['size'] = {}  # bytes
                dict_['size']['total'] = int(vol.get_value('capacity/total', 0))
                dict_['size']['use'] = int(vol.get_value('capacity/use', 0))
                dict_['mountdst'] = vol.get_value('mountdst', None)
                dict_['status'] = vol.get_value('status', None)
                dict_['filesystem'] = vol.get_value('filesystem', None)
                dict_['encrypt'] = vol.get_node('filesystem').is_set_to_true('encrypt')
            except:
                pass
            return dict_
        entry = self.request(Entry(self.TYPE_QUERY, """<volctrl>
            <volume>
                <name type="volume">%s</name>
            </volume>
        </volctrl>""" % volume))
        self.__query_cache = _to_dict(entry)
        return self.__query_cache if to_dict else entry

    def mkfs(self,
            # フォーマットはデータ喪失が伴うため対象は明示的に扱う
            #volume=VOLUME['internal'][1],
            volume=None,
            fs=FS[0],
            encrypt=False,
            to_bool=True, log=True):
        if not self._is_vol(volume):
            raise TypeError  # FIXME
        entry = Entry(self.TYPE_MKFS, """<volctrl>
            <volume encrypt="%(encrypt)s">
                <name type="volume">%(name)s</name>
                <filesystem>%(fs)s</filesystem>
            </volume>
        </volctrl>""" % {
            'name': volume,
            'fs': fs,
            'encrypt': self.bool_to_text(encrypt),
        })

        entry = self.request(entry)
        result = self.get_result()
        """
        _MSGCODE = {
            self.VOLUME['internal'][1]: {
                str(Results.OK): 'MSG_VOLUME_INTERNAL_FORMAT_START',
                str(Results.NOT_SUPPORTED): None,
                '_code': 'ERR_VOLUME_INTERNAL_FORMAT',
                },
            self.VOLUME['usb'][1]: {
                str(Results.OK): 'MSG_VOLUME_USB1_FORMAT_START',
                str(Results.NOT_SUPPORTED): None,
                '_code': 'ERR_VOLUME_USB1_FORMAT',
                },
            self.VOLUME['usb'][2]: {
                str(Results.OK): 'MSG_VOLUME_USB2_FORMAT_START',
                str(Results.NOT_SUPPORTED): None,
                '_code': 'ERR_VOLUME_USB2_FORMAT',
                },
            self.VOLUME['usb'][3]: {
                str(Results.OK): 'MSG_VOLUME_USB3_FORMAT_START',
                str(Results.NOT_SUPPORTED): None,
                '_code': 'ERR_VOLUME_USB3_FORMAT',
                },
            self.VOLUME['usb'][4]: {
                str(Results.OK): 'MSG_VOLUME_USB4_FORMAT_START',
                str(Results.NOT_SUPPORTED): None,
                '_code': 'ERR_VOLUME_USB4_FORMAT',
                },
            '_dev': {
                str(Results.OK): 'MSG_VOLUME_INTERNAL_FORMAT_START',  # FIXME
                str(Results.NOT_SUPPORTED): None,
                '_code': 'ERR_VOLUME_INTERNAL_FORMAT',  # FIXME
                },
            }
        _dev = _MSGCODE.get(volume, _MSGCODE['_dev'])
        _code = _dev.get(str(result), _dev['_code'])
        """
        _volume_to_messagevolume = {
            "sataraid1": "INTERNAL",
            "usb1": "USB1",
            "usb2": "USB2",
            "usb3": "USB3",
            "usb4": "USB4",
            "usb5": "USB5",
            "usb6": "USB6",
        }
        _code = {
            str(Results.OK): "MSG_VOLUME_%s_FORMAT_START" % _volume_to_messagevolume.get(volume, ""),
            str(Results.NOT_SUPPORTED): None,
        }.get(str(result), 'ERR_VOLUME_%s_FORMAT' % _volume_to_messagevolume.get(volume, ""))

        if _code:
            self._set_message(_code, log=log)
        result = True if int(result) in [int(Results.OK)] else False
        return result if to_bool else entry

    def fsck(self,
            volume=VOLUME['internal'][1],
            fs=FS[0],
            to_bool=True, log=True):
        entry = Entry(self.TYPE_FSCK, """<volctrl>
            <volume>
                <name type="volume">%(name)s</name>
                <filesystem>%(fs)s</filesystem>
            </volume>
        </volctrl>""" % {
            'name': volume,
            'fs': fs,
        })

        entry = self.request(entry)
        result = self.get_result()

        """
        if int(result) == int(Results.OK):
#            self._set_message('MSG_VOLUME_CHECKDISK_START', log=log)
            self._set_message(
                    {
                        self.VOLUME['internal'][1]: 'MSG_VOLUME_INTERNAL_CHECKDISK_START',
                        self.VOLUME['usb'][1]: 'MSG_VOLUME_USB1_CHECKDISK_START',
                        self.VOLUME['usb'][2]: 'MSG_VOLUME_USB2_CHECKDISK_START',
                        self.VOLUME['usb'][3]: 'MSG_VOLUME_USB3_CHECKDISK_START',
                        self.VOLUME['usb'][4]: 'MSG_VOLUME_USB4_CHECKDISK_START',
                    }.get(volume, None),
                    log=log)
            result = True
        elif int(result) == int(Results.NOT_SUPPORTED):
            self._set_message(
                    {
                        self.VOLUME['usb'][1]: 'ERR_VOLUME_USB1_CHECKDISK_NOT_SUPPORTED',
                        self.VOLUME['usb'][2]: 'ERR_VOLUME_USB2_CHECKDISK_NOT_SUPPORTED',
                        self.VOLUME['usb'][3]: 'ERR_VOLUME_USB3_CHECKDISK_NOT_SUPPORTED',
                        self.VOLUME['usb'][4]: 'ERR_VOLUME_USB4_CHECKDISK_NOT_SUPPORTED',
                    }.get(volume, None),
                    log=log)
            result = False
        else:
#            self._set_message('ERR_VOLUME_CHECKDISK', log=log)
            self._set_message(
                    {
                        self.VOLUME['internal'][1]: 'ERR_VOLUME_INTERNAL_CHECKDISK',
                        self.VOLUME['usb'][1]: 'ERR_VOLUME_USB1_CHECKDISK',
                        self.VOLUME['usb'][2]: 'ERR_VOLUME_USB2_CHECKDISK',
                        self.VOLUME['usb'][3]: 'ERR_VOLUME_USB3_CHECKDISK',
                        self.VOLUME['usb'][4]: 'ERR_VOLUME_USB4_CHECKDISK',
                    }.get(volume, None),
                    log=log)
            result = False
        """

        _volume_to_messagevolume = {
            "sataraid1": "INTERNAL",
            "usb1": "USB1",
            "usb2": "USB2",
            "usb3": "USB3",
            "usb4": "USB4",
            "usb5": "USB5",
            "usb6": "USB6",
        }
        _messagevolume = _volume_to_messagevolume.get(volume, "")
        _code = {
            str(Results.OK): "MSG_VOLUME_%s_CHECKDISK_START" % _messagevolume,
            str(Results.NOT_SUPPORTED): "ERR_VOLUME_%s_CHECKDISK_NOT_SUPPORTED" % _messagevolume,
        }.get(str(result), 'ERR_VOLUME_%s_CHECKDISK' % _messagevolume)
        if _code:
            self._set_message(_code, log=log)
        result = int(result) in [int(Results.OK)]

        return result if to_bool else entry

    def _mount_or_unmount(self, reqtype=TYPE_MOUNT, volume=VOLUME['usb'][1]):
        entry = Entry(reqtype, """<volctrl>
            <volume><name type="volume">%(volume)s</name></volume>
        </volctrl>""" % {'volume': volume})
        return self.request(entry)

    def _mount(self, volume=VOLUME['usb'][1]):
        return self._mount_or_unmount(reqtype=self.TYPE_MOUNT, volume=volume)

    def _unmount(self, volume=VOLUME['usb'][1]):
        return self._mount_or_unmount(reqtype=self.TYPE_UNMOUNT, volume=volume)

    def mount(self, volume=VOLUME['usb'][1], to_bool=True, log=True):
        entry = self._mount(volume=volume)
        result = self.get_result(entry=entry)
        arglist = {'name': volume}
        """
        if int(result) == int(Results.OK):
            self._set_message(
                    {
                        self.VOLUME['usb'][1]: 'MSG_VOLUME_USB1_MOUNT',
                        self.VOLUME['usb'][2]: 'MSG_VOLUME_USB2_MOUNT',
                        self.VOLUME['usb'][3]: 'MSG_VOLUME_USB3_MOUNT',
                        self.VOLUME['usb'][4]: 'MSG_VOLUME_USB4_MOUNT',
                    }.get(volume, None),
                    arglist=arglist,
                    log=log)
            result = True
        else:
            self._set_message(
                    {
                        self.VOLUME['usb'][1]: 'ERR_VOLUME_USB1_MOUNT',
                        self.VOLUME['usb'][2]: 'ERR_VOLUME_USB2_MOUNT',
                        self.VOLUME['usb'][3]: 'ERR_VOLUME_USB3_MOUNT',
                        self.VOLUME['usb'][4]: 'ERR_VOLUME_USB4_MOUNT',
                    }.get(volume, None),
                    arglist=arglist,
                    log=log)
            result = False
        """

        _volume_to_messagevolume = {
            "sataraid1": "INTERNAL",
            "usb1": "USB1",
            "usb2": "USB2",
            "usb3": "USB3",
            "usb4": "USB4",
            "usb5": "USB5",
            "usb6": "USB6",
        }
        _messagevolume = _volume_to_messagevolume.get(volume, "")
        _code = {
            str(Results.OK): "MSG_VOLUME_%s_MOUNT" % _messagevolume,
        }.get(str(result), "ERR_VOLUME_%s_MOUNT" % _messagevolume)
        if _code:
            self._set_message(_code, arglist=arglist, log=log)
        result = int(result) in [int(Results.OK)]

        return result if to_bool else entry

    def unmount(self, volume=VOLUME['usb'][1], to_bool=True, log=True):
        entry = self._unmount(volume=volume)
        result = self.get_result(entry=entry)
        arglist = {'name': volume}
        """
        if int(result) == int(Results.OK):
            self._set_message(
                    {
                        self.VOLUME['usb'][1]: 'MSG_VOLUME_USB1_UNMOUNT',
                        self.VOLUME['usb'][2]: 'MSG_VOLUME_USB2_UNMOUNT',
                        self.VOLUME['usb'][3]: 'MSG_VOLUME_USB3_UNMOUNT',
                        self.VOLUME['usb'][4]: 'MSG_VOLUME_USB4_UNMOUNT',
                    }.get(volume, None),
                    arglist=arglist,
                    log=log)
            result = True
        else:
            self._set_message(
                    {
                        self.VOLUME['usb'][1]: 'ERR_VOLUME_USB1_UNMOUNT',
                        self.VOLUME['usb'][2]: 'ERR_VOLUME_USB2_UNMOUNT',
                        self.VOLUME['usb'][3]: 'ERR_VOLUME_USB3_UNMOUNT',
                        self.VOLUME['usb'][4]: 'ERR_VOLUME_USB4_UNMOUNT',
                    }.get(volume, None),
                    arglist=arglist,
                    log=log)
            result = False
        """
        _volume_to_messagevolume = {
            "sataraid1": "INTERNAL",
            "usb1": "USB1",
            "usb2": "USB2",
            "usb3": "USB3",
            "usb4": "USB4",
            "usb5": "USB5",
            "usb6": "USB6",
        }
        _messagevolume = _volume_to_messagevolume.get(volume, "")
        _code = {
            str(Results.OK): "MSG_VOLUME_%s_UNMOUNT" % _messagevolume,
        }.get(str(result), "ERR_VOLUME_%s_UNMOUNT" % _messagevolume)
        if _code:
            self._set_message(_code, arglist=arglist, log=log)
        result = int(result) in [int(Results.OK)]

        return result if to_bool else entry

    def status(self,
            volume=VOLUME['internal'][1],
            humanize=False,
            cache=False):
        info = self.__query_cache if cache and self.__query_cache else \
                self.query(volume=volume)
        status = info.get('status', None)
        _humanize = {
            'mounted': _(u'VolumeStatusMounted'),
            'unmounted': _(u'VolumeStatusUnmounted'),
            'lockkey': _(u'VolumeStatusLockkey'),
            'keywaiting': _(u'VolumeStatusKeywaiting'),
            'mountfail': _(u'VolumeStatusMountfail'),
            'waiting': _(u'VolumeStatusWaiting'),
            'capshortage': _(u'VolumeStatusCapshortage'),
            'capshortage': _(u'VolumeStatusCapshortage'),
            'checking': _(u'VolumeStatusChecking'),
            'formatting': _(u'VolumeStatusFormatting'),
            'fail': _(u'VolumeStatusFail'),
            'unformated': _(u'VolumeStatusUnformated'),
            'mirroring': _(u'VolumeStatusMirroring'),
            'vfsext': _(u'VolumeStatusVFSEXT'),
            'othermirror': _(u"Mirror Storage (unrecoverable)"),
            'recoverablemirror': _(u"Mirror Storage (recoverable)"),
        }
        return _humanize.get(status, _(u'unknown')) if humanize else status

    def is_lockkey(self, num):
        try:
            volume = self.VOLUME['usb'][int(num)]
            if self.query(volume=volume)['status'] == 'lockkey':
                return True
            return False
        except:
            return False

if __name__ == '__main__':
    i = Volctrl()
    print i.query()
    print i.query(volume=i.VOLUME['usb'][1])

#    print i.status()
#    print i.status(humanize=True)

#    volume = i.VOLUME['usb'][1]  # 内臓を対象にすると面倒なのでusbでテスト
#    fs = 'ext3'
#
#    i.mkfs(volume=volume, fs=fs)
#    print "MKFS: %s (%s)" % (i.get_message(), i.get_result())
#    print i.query(volume=volume)
#
#    i.fsck(volume=volume, fs=fs)
#    print "FSCK: %s (%s)" % (i.get_message(), i.get_result())
#    print i.query(volume=volume)
#
#    i.mount(volume=volume)
#    print "MOUNT: %s (%s)" % (i.get_message(), i.get_result())
#    print i.query(volume=volume)
#    i.unmount(volume=volume)
#    print "UNMOUNT: %s (%s)" % (i.get_message(), i.get_result())
#    print i.query(volume=volume)
