# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from command import Command, _, escape
from ionas.request import Entry, XmlNode
from product_conf import STOR_IF_TO_VOL
from fal_info import Results
from raidctrl import Raidctrl
from volctrl import Volctrl
from devctrl import Devctrl


class Volinfo(object):
    def __init__(self, *args, **kwargs):
        self._raid = Raidctrl()
        self._vol = Volctrl()

    def status(self,
            volume='sataraid1',  # sataraid1/usb1/usb2/usb3/usb4
            humanize=True):
        status = _(u'unknown')
        if volume == 'sataraid1':
            rinfo1 = self._raid.query(volume='sataraid1', depth=1)
            rinfo2 = self._raid.query(volume='sataraid1', depth=2)
            vinfo = self._vol.query(volume='sataraid1')
            if rinfo2.get('status', None) in ('stop', 'degrade', 'activerepair'):
                status = self._raid.status(humanize=True, depth=2)
            elif rinfo2.get('status', None) == 'crash':
                status = {
                    'fail': _(u'RaidStatusFail'),
                    'crash': _(u'RaidStatusCrash'),
                }.get(rinfo1.get('status', None), _(u'unknown'))
            elif rinfo2.get('status', None) in ('running', 'rebuilding'):
                status = {
                    'mounted': _(u'RaidStatusRunning'),
                    'unmounted': _(u'RaidStatusRunning'),
                    'mountfail': _(u'VolumeStatusMountfail'),
                    'checking': _(u'VolumeStatusChecking'),
                    'formatting': _(u'VolumeStatusFormatting'),
                    'unformated': _(u'VolumeStatusUnformated'),
                    'keywaiting': _(u'VolumeStatusKeywaiting'),
                }.get(self._vol.status(cache=True), _(u'processing'))

            if rinfo2.get('status', None) in ('degrade', 'crash', 'activerepair') \
                    and vinfo.get('status', None) == 'keywaiting':
                status = '%s/%s' % (status, _(u'VolumeStatusKeywaiting'))

            if rinfo2.get('status', None) in ('degrade', 'rebuilding') \
                    and rinfo2.get('progress', None):
                status = '%s(%s)' % (
                        status,
                        "%s(%s%%)" % (
                                _(u'RaidStatusRebuilding'),
                                rinfo2.get('progress')))
        else:
            try:
                if self._vol.query(volume=volume, to_dict=False).get_result() == Results.DEVICE_NOT_FOUND:
                    return _(u'Unconnected')
            except:
                return _(u'unknown')

            status = self._vol.status(volume=volume, humanize=True)
            if self._vol.status(volume=volume, humanize=False) in ['mirroring']:
                if self._raid.query(volume='sataraid1', depth=1).get('status') in ('fail', 'rebuilding'):
                    status = "%s(%s)" % (status, self._raid.status(depth=1, humanize=True))

        return status

if __name__ == '__main__':
    i = Volinfo()

    print i.status()
    print i.status(humanize=True)

    print i.status(volume='usb1')
    print i.status(volume='usb1', humanize=True)
