# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
import os
import time
import datetime

from command import Command, _
from ionas.request import Entry, XmlNode


class Volume(Command):
    MESSAGES = {
#        '257': _(u'TEST'),  #FIXME msg
    }

    def request(self, entry, command):
        self._entry = None
        response = self._request(entry, command)
        if response is None:
            return None
        self._entry = response.get_entrylist().get_entry()
        return self._entry
    
    def get_status(self, path):
        execute_disabled = ""

        try:
            f = open(path, "r")
            status, content = f.read().split(":", 1)
            f.close()

            if status == "NOT_EXEC":
                status_str = _(content.decode("utf8"))
            elif status == "COMPLETE":
                # FIXME 時刻変換これでよいか
                mtime = time.asctime(time.localtime(os.stat(path).st_mtime))
                status_str = _(u'Last ' + content.decode("utf8") + u'(' + mtime.decode("utf8") + u')')
                #status_str = _(u'Last%(content)s') % {'content': 'abc'}
            elif status == "PROCESSING":
                status_str = _(content.decode("utf8"))
                execute_disabled = "disabled"
            else:
                status_str = _(u'Status get error')

            return status_str, execute_disabled
        except:
            return _(u'Status get error'), execute_disabled


class Format(Volume):
    # FIXME featureの設定ファイルから取得?
    RAID_MODE_LIST = ("RAID0", "RAID1", "RAID5", "RAID6")
    RAID_MODE_TUPPLE = zip(RAID_MODE_LIST, RAID_MODE_LIST)
    ENCRYPT_TUPPLE = (("enable", _(u"enable")), ("disable", _(u"disable")))

    # FIXME fal_infoからとる
    FAL_RAID_PATH = "/usr/local/bin/fal_raid"
    FAL_VOLUME_PATH = "/usr/local/bin/fal_volume"
    FAL_RAID_REQ_BASE_XML = {
        "load": ""}
    FAL_VOLUME_REQ_BASE_XML = {
        "mkfs": "<volume><targets><partition/></targets></volume>"}

    LOAD_TYPE = "load"
    MKFS_TYPE = "mkfs"
    QUERY_TYPE = "query"
    DEVNAME_ATTR = "devname"
    FSTYPE_ATTR = "fstype"
    ENCRYPT_ATTR = "encrypt"
    
    PARTITION_PATH = "targets/partition"
    FILESYSTEM_PATH = "target/filesystem"
    
    # FIXME 暫定パス
    CURRENTMODE_PATH = "currentmode"
    CURRENTENC_PATH = "currentmode/@encrypt"

    INTERNAL_DEV = "md6"
    INTERNAL_FS = "xfs"

    QUERY_BODY_FMT_STR = "<volume><target><name>%s</name></target></volume>"

    def __is_changed_mode(self, raid_mode, encrypt):
        current_raid_mode, current_encrypt = self.load()
        if (raid_mode == current_raid_mode and
            encrypt == current_encrypt):
            return False
        return True

    def __get_raid_request_entry(self, raid_mode, encrypt):
        # FIXME TBD
        entry = Entry(self.MKFS_TYPE, self.FAL_VOLUME_REQ_BASE_XML[self.MKFS_TYPE])
        entry.get().get_node(self.PARTITION_PATH).set_attrs(
            {self.DEVNAME_ATTR: "md6", self.FSTYPE_ATTR: "xfs"})
        return entry

    def __get_format_request_entry(self, target, file_system):
        entry = Entry(self.MKFS_TYPE, self.FAL_VOLUME_REQ_BASE_XML[self.MKFS_TYPE])
        entry.get().get_node(self.PARTITION_PATH).set_attrs(
            {self.DEVNAME_ATTR: target, self.FSTYPE_ATTR: file_system})
        return entry

    def __send_raid_request(self, raid_mode, encrypt):
        self.request(
            self.__get_raid_request_entry(raid_mode, encrypt),
            self.FAL_RAID_PATH)
 
    def __send_format_request(self, target, file_system):
        self.request(
            self.__get_format_request_entry(target, file_system),
            self.FAL_VOLUME_PATH)

    def load(self):
        self.request(Entry(self.LOAD_TYPE, self.FAL_RAID_REQ_BASE_XML[self.LOAD_TYPE]), self.FAL_RAID_PATH)
        return (
            self.get_value(self.CURRENTMODE_PATH),
            self.TEXT_TO_BOOL[self.get_value(self.CURRENTENC_PATH)])

    def query(self, target):
        self.request(Entry(self.QUERY_TYPE,
            self.QUERY_BODY_FMT_STR % target), self.FAL_VOLUME_PATH)
        return self.get_value(self.FILESYSTEM_PATH)

    def format_internal(self, raid_mode, encrypt):
        if self.__is_changed_mode(
                raid_mode,
                encrypt):
            self.__send_raid_request(raid_mode, encrypt)
            self.__send_format_request(self.INTERNAL_DEV, self.INTERNAL_FS)
        else:
            self.__send_format_request(self.INTERNAL_DEV, self.INTERNAL_FS)

    def format_external(self, target, file_system):
            self.__send_format_request(target, file_system)


class CheckDisk(Volume):
    # FIXME fal_infoからとる
    FAL_VOLUME_PATH = "/usr/local/bin/fal_volume"
    FSCK_TYPE = "fsck"
    FAL_VOLUME_REQ_BASE_XML = {
        "fsck": "<volume><targets><partition/></targets></volume>"}
    DEVNAME_ATTR = "devname"
    FSTYPE_ATTR = "fstype"
    
    PARTITION_PATH = "targets/partition"
    

    def __get_checkdisk_request_entry(self, target):
        entry = Entry(self.FSCK_TYPE, self.FAL_VOLUME_REQ_BASE_XML[self.FSCK_TYPE])
        entry.get().get_node(self.PARTITION_PATH).set_attrs(
            {self.DEVNAME_ATTR: target})
        return entry

    def checkdisk(self, target):
        self.request(
            self.__get_checkdisk_request_entry(target),
            self.FAL_VOLUME_PATH)


class ActiveRepair(Volume):
    # FIXME fal_infoからとる
    FAL_RAID_PATH = "/usr/local/bin/fal_raid"
    ACTIVEREPAIR_TYPE = "activerepair"
    LOAD_TYPE = "load"
    APPLY_TYPE = "apply"
    
    # FIXME 停止はどうする
    FAL_RAID_REQ_BASE_XML = {
        "activerepair": "",
        "load": "",
        "apply": "<raid><activerepair><schedulelist/></activerepair></raid>"}
    
    SCHEDULELIST_PATH = "activerepair/schedulelist"
    SCH_ENABLE_PATH = SCHEDULELIST_PATH + "/@enable"
    SCHEDULE_PATH = SCHEDULELIST_PATH + "/schedule"
    DAY_PATH = SCHEDULE_PATH + "/day"
    HOUR_PATH = SCHEDULE_PATH + "/hour"
    MINUTE_PATH = SCHEDULE_PATH + "/minute"

    SCHEDULE_NODE_FMT_STR = '<schedule><day>%s</day><hour>%s</hour><minute>%s</minute></schedule>'


    def __get_activerepair_request_entry(self):
        return Entry(self.ACTIVEREPAIR_TYPE, self.FAL_RAID_REQ_BASE_XML[self.ACTIVEREPAIR_TYPE])

    def __get_load_request_entry(self):
        return Entry(self.LOAD_TYPE, self.FAL_RAID_REQ_BASE_XML[self.LOAD_TYPE])

    def __get_apply_request_entry(
            self, enable, day, time):
        entry = Entry(self.APPLY_TYPE, self.FAL_RAID_REQ_BASE_XML[self.APPLY_TYPE])
        schlist_node = entry.get().get_node(self.SCHEDULELIST_PATH)
        schlist_node.enable(self.BOOL_TO_TEXT[enable])

        for d in day:
            schlist_node.append_child(XmlNode(self.SCHEDULE_NODE_FMT_STR % (
                d, time.hour, time.minute)))

        return entry
        
    def load(self):
        entry = self.request(self.__get_load_request_entry(),
            self.FAL_RAID_PATH)

        sch_enable = self.TEXT_TO_BOOL[self.get_value(self.SCH_ENABLE_PATH)]
        day = [
            day_node.get_text()
            for day_node
            in entry.get().get_node_list(self.DAY_PATH)]

        time = datetime.time(
            int(self.get_value(self.HOUR_PATH)), int(self.get_value(self.MINUTE_PATH)))

        return (
            sch_enable,
            day,
            time)

    def activerepair(self):
        self.request(self.__get_activerepair_request_entry(),
            self.FAL_RAID_PATH)

    def apply(self, enable, day, time):
        self.request(self.__get_apply_request_entry(
            enable, day, time), self.FAL_RAID_PATH)

class ForceRestore(Volume):
    # FIXME fal_infoからとる
    FAL_RAID_PATH = "/usr/local/bin/fal_raid"
    FORCERESTORE_TYPE = "forcerestore"
    FAL_RAID_REQ_BASE_XML = {
        "forcerestore": ""}

    def __get_raid_request_entry(self):
        entry = Entry(self.FORCERESTORE_TYPE, self.FAL_RAID_REQ_BASE_XML[self.FORCERESTORE_TYPE])
        return entry

    def forcerestore(self):
        self.request(self.__get_raid_request_entry(), self.FAL_RAID_PATH)
 

class PortMode(Volume):
    # FIXME fal_infoからとる
    FAL_PORTCONFIG_PATH = "/usr/local/bin/fal_portconfig"
    LOAD_TYPE = "load"
    APPLY_TYPE = "apply"
    FAL_PORTCONFIG_REQ_BASE_XML = {
        "load": "",
        "apply": '<portconfig><usblist><usb port="1"><modelist/></usb></usblist></portconfig>'}
    
    USB_PATH = "usblist/usb"
    MODE_PATH = "modelist/mode"
    MODELIST_PATH = "usblist/usb/modelist"
    
    PORT_MODE_INFO = "port_mode"
    QUICKCOPY_SHARE_INFO = "quickcopy_share"

    MODE_FMT_STR = '<mode enable="true">%s</mode>'
    QC_MODE_FMT_STR = '<mode enable="true" share="%s">quickcopy</mode>'


    def __get_load_request_entry(self):
        return Entry(self.LOAD_TYPE, self.FAL_PORTCONFIG_REQ_BASE_XML[self.LOAD_TYPE])

    def __get_apply_request_entry(self, port_mode, qc_share):
        # FIXME フォームデータが不正の場合どうする
        if port_mode == "quickcopy":
            mode_node = XmlNode(self.QC_MODE_FMT_STR % qc_share)
        else:
            mode_node = XmlNode(self.MODE_FMT_STR % port_mode)
            
        entry = Entry(self.APPLY_TYPE, self.FAL_PORTCONFIG_REQ_BASE_XML[self.APPLY_TYPE])
        entry.get().get_node(self.MODELIST_PATH).append_child(mode_node)

        return entry

    def load(self):
        entry = self.request(self.__get_load_request_entry(),
            self.FAL_PORTCONFIG_PATH)
        usb_node = entry.get().get_node(self.USB_PATH + '[@port="1"]')

        return (
            usb_node.get_node(
                self.MODE_PATH, {"enable": "true"}).get_text(),
            usb_node.get_node(
                self.MODE_PATH + '[text()="quickcopy"]').get_attr_value("share"))

    def apply(self, port_mode, qc_share):
        self.request(self.__get_apply_request_entry(port_mode, qc_share),
            self.FAL_PORTCONFIG_PATH)
   
class Remove(Volume):
    # FIXME fal_infoからとる
    FAL_VOLUME_PATH = "/usr/local/bin/fal_volume"
    EJECT_TYPE = "eject"
    FAL_VOLUME_REQ_BASE_XML = {
        "eject": "<volume><target/></volume>"}
    
    TARGET_NAME = "target"

    def __get_eject_request_entry(self, target):
        entry = Entry(self.EJECT_TYPE, self.FAL_VOLUME_REQ_BASE_XML[self.EJECT_TYPE])
        entry.get().get_node(self.TARGET_NAME).set_text(target)
        return entry

    def eject(self, target):
        self.request(self.__get_eject_request_entry(target),
            self.FAL_VOLUME_PATH)


if __name__ == '__main__':
    print "### Format"
    fi = Format()
    fi.format_internal(raid_mode="RAID0", encrypt=True)
    print "format_internal: %s, %s" % (fi.get_result(), fi.get_message())
    fi.format_external(target="USB1", file_system="ext3")
    print "format_external: %s, %s" % (fi.get_result(), fi.get_message())
    load_result = fi.load()
    print "load: %s, %s" % (load_result[0], load_result[1])
    print "query: %s" % fi.query("USB1")
    print ""

    print "### CheckDisk"
    ci = CheckDisk()
    ci.checkdisk("md6")
    print "checkdisk: %s, %s" % (ci.get_result(), ci.get_message())
    print ""

    print "### ActiveRepair"
    ai = ActiveRepair()
    ai.activerepair()
    print "activerepair: %s, %s" % (ai.get_result(), ai.get_message())
    load_result = ai.load()
    print "load: %s, %s, %s, %s" % (
        load_result[0], load_result[1], str(load_result[2].hour), str(load_result[2].minute))
    ai.apply(enable=True, day="sun", time=datetime.time(12, 45))
    print "apply: %s, %s" % (ai.get_result(), ai.get_message())
    print ""

    print "### ForceRestore"
    fri = ForceRestore()
    fri.forcerestore()
    print "forcerestore: %s, %s" % (fri.get_result(), fri.get_message())
    print ""

    print "### PortMode"
    pi = PortMode()
    print "load: %s, %s" % pi.load()
    pi.apply("share", "share1")
    print "apply: %s, %s" % (pi.get_result(), pi.get_message())
    print ""

    print "### Remove"
    ri = Remove()
    ri.eject("usbd1")
    print "eject: %s, %s" % (ri.get_result(), ri.get_message())
    print ""

    print "### Status"
    print fi.get_status("/tmp/status/internal_format")
    print ci.get_status("/tmp/status/internal_checkdisk")
    print ai.get_status("/tmp/status/activerepair")
    print fri.get_status("/tmp/status/forcerestore")
