# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django.utils.translation import ugettext as _
from ..select import SelectField


class SelectRaidDiskField(SelectField):
    #widget = forms.widgets.Select
    default_error_message = {
    #    'required': _(u'This field is required.'),
    #    'invalid': _(u'Enter a valid value.'),
    #    'invalid_choice': _(u'Select a valid choice. %(value)s is not one of the available choices.'),
    }

    _disknames = dict(
        [('satad%s' % no, _(u'internal disk%(no)s') % {'no': no}) for no in range(1, 7)]
        + [('usbd%s' % no, _(u'usb disk%(no)s') % {'no': no}) for no in range(1, 5)])

    def _choice(self, value):
        return (value, self._disknames.get(value, value))

    def _get_coldswaponly_devices(self):
        try:
            from project.fal.features import Features
            deviceinfo = Features().query()["deviceinfo"]
            devices = []
            devices += ["satad%s" % d["port"] for d in deviceinfo["satalist"] if d["coldswaponly"]]
            devices += ["usbd%s" % d["port"] for d in deviceinfo["usblist"] if d["coldswaponly"]]
            return devices
        except:
            return []

    def _members(self):
        try:
            from project.fal.raidctrl import Raidctrl
            from product_conf import DEV_DIR
            from project.fal.devctrl import Devctrl
            from ionas.device import DevMgr
            devmgr = DevMgr(DEV_DIR, hdd_dev_list=Devctrl().list().keys())
            members = [devmgr.get_base_dev_name(dev) \
                    for dev in sorted(Raidctrl().query()['elementlist'].keys())]
            coldswaponly_devices = self._get_coldswaponly_devices()
            members = [m for m in members if m not in coldswaponly_devices]
            return members
            #return [devmgr.get_base_dev_name(dev) \
            #        for dev in sorted(Raidctrl().query()['elementlist'].keys())]
        except:
            return ['satad%s' % no for no in range(1, 7)]

    def __init__(self, *args, **kwargs):
        kwargs['choices'] = [('', _(u'Please select.'))] + \
                [self._choice(m) for m in self._members()]
        super(SelectRaidDiskField, self).__init__(*args, **kwargs)
