# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django.utils.translation import ugettext as _
from django import forms
from django.core import validators
from ..text import TextField
from ..password import PasswordField
from validators.fal import FALValidator
from fal_validinfo import (WorkGroup, DomainName,
    NetBIOSDomainName, LanDiskPass, AdUserName, Fqdn)


class WorkGroupValidator(FALValidator):
    validator = WorkGroup.VALIDATOR
    options = {}


class DomainNameValidator(FALValidator):
    validator = DomainName.VALIDATOR
    options = {}


class NetBIOSDomainValidator(FALValidator):
    validator = NetBIOSDomainName.VALIDATOR
    options = {}


class AdAdminPassValidator(FALValidator):
    validator = LanDiskPass.VALIDATOR
    options = {"exempt_null": "true"}


class AdUserValidator(FALValidator):
    validator = AdUserName.VALIDATOR
    options = {}


class WinsSrvValidator(FALValidator):
    validator = Fqdn.VALIDATOR
    options = {"exempt_null": "true"}


class WorkGroupField(TextField):
    default_error_messages = {}
    default_validators = [WorkGroupValidator()]

    def __init__(self, max_length=WorkGroup.MAX_LENGTH, *args, **kwargs):
        super(WorkGroupField, self).__init__(
                max_length=max_length,
                *args, **kwargs)


class NtControllerField(TextField):
    default_error_messages = {}
    default_validators = [WorkGroupValidator()]

    def __init__(self, max_length=WorkGroup.MAX_LENGTH, *args, **kwargs):
        super(NtControllerField, self).__init__(
                max_length=max_length,
                *args, **kwargs)


class DomainNameField(TextField):
    default_error_messages = {}
    default_validators = [DomainNameValidator()]

    def __init__(self, max_length=DomainName.MAX_LENGTH, *args, **kwargs):
        super(DomainNameField, self).__init__(
                max_length=max_length,
                *args, **kwargs)


class AdControllerField(TextField):
    default_error_messages = {}
    default_validators = [DomainNameValidator()]

    def __init__(self, max_length=DomainName.MAX_LENGTH, *args, **kwargs):
        super(AdControllerField, self).__init__(
                max_length=max_length,
                *args, **kwargs)


class NetBIOSDomainField(TextField):
    default_error_messages = {}
    default_validators = [NetBIOSDomainValidator()]

    def __init__(self, max_length=NetBIOSDomainName.MAX_LENGTH, *args, **kwargs):
        super(NetBIOSDomainField, self).__init__(
                max_length=max_length,
                *args, **kwargs)


class AdAdminPassField(PasswordField):
    default_error_messages = {}
    default_validators = [AdAdminPassValidator()]

    def __init__(self, max_length=LanDiskPass.MAX_LENGTH, *args, **kwargs):
        super(AdAdminPassField, self).__init__(
                max_length=max_length,
                *args, **kwargs)


class AdUserField(TextField):
    default_error_messages = {}
    default_validators = [AdUserValidator()]

    def __init__(self, max_length=AdUserName.MAX_LENGTH, *args, **kwargs):
        super(AdUserField, self).__init__(
                max_length=max_length,
                *args, **kwargs)


class WinsSrvField(TextField):
    default_error_messages = {}
    default_validators = [WinsSrvValidator()]

    def __init__(self, max_length=Fqdn.MAX_LENGTH, *args, **kwargs):
        super(WinsSrvField, self).__init__(
                max_length=max_length,
                *args, **kwargs)
