# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django.utils.translation import ugettext as _
from django import forms
from django.core import validators
from itertools import chain
from django.utils.encoding import force_unicode
from django.utils.html import escape
from ..text import TextField
from ..select import SelectField, MultiSelectField
from validators.fal import FALValidator
from project.fal.group import Group
from fal_validinfo import GroupName


class GroupNameValidator(FALValidator):
    validator = GroupName.VALIDATOR
    options = {}


class GroupNameField(TextField):
#    widget = forms.widgets.TextInput
    default_error_messages = {
#        'required': _(u'This field is required.'),
#        'invalid': _(u'Enter a valid value.'),
#        'max_length': _(u'Ensure this value has at most %(limit_value)d characters (it has %(show_value)d).'),
#        'min_length': _(u'Ensure this value has at least %(limit_value)d characters (it has %(show_value)d).'),
    }
    default_validators = [GroupNameValidator()]

    def __init__(self,
            max_length=GroupName.MAX_LENGTH,
            *args, **kwargs):
        super(GroupNameField, self).__init__(
                max_length=max_length,
                *args, **kwargs)


class SelectGroupField(SelectField):
    #widget = forms.widgets.Select
    default_error_message = {
    #    'required': _(u'This field is required.'),
    #    'invalid': _(u'Enter a valid value.'),
    #    'invalid_choice': _(u'Select a valid choice. %(value)s is not one of the available choices.'),
    }

    def __init__(self, key='name', *args, **kwargs):
        choices = [(grp[key], grp['name']) for grp in sorted(
                Group().load().values(),
                key=lambda grp: grp['name'])]
        choices.insert(0, ('', _(u'Please select.')))
        super(SelectGroupField, self).__init__(
                choices=choices,
                *args, **kwargs)


class SelectRenderSelectedOnly(forms.widgets.SelectMultiple):
    def render_options(self, choices, selected_choices):
        # Normalize to strings.
        selected_choices = set(force_unicode(v) for v in selected_choices)
        output = []
        for option_value, option_label in chain(self.choices, choices):
            if isinstance(option_label, (list, tuple)):
                output.append(u'<optgroup label="%s">' % escape(force_unicode(option_value)))
                for option in option_label:
                    output.append(self.render_option(selected_choices, *option))
                output.append(u'</optgroup>')
            else:
                if force_unicode(option_value) not in selected_choices:
                    continue
                output.append(self.render_option(
                        selected_choices,
                        option_value,
                        option_label))
        return u'\n'.join(output)


class MultiSelectGroupField(MultiSelectField):
    #widget = forms.widgets.SelectMultiple
    #default_validators = []
    default_error_message = {
    #    'required': _(u'This field is required.'),
    #    'invalid': _(u'Enter a valid value.'),
    #    'invalid_choice': _(u'Select a valid choice. %(value)s is not one of the available choices.'),
    #    'invalid_list': _(u'Enter a list of values.'),
    }

    def groups(self):
        try:
            return Group().load().keys()
        except:
            return []

    def __init__(self,
            domain=False,
            exclude=[],
            selected_only=False,
            *args, **kwargs):
        exclude = [force_unicode(choice) for choice in exclude]
        gobj = Group()
        choices = [(name, name) \
                for name in sorted(gobj.load().keys()) \
                    if force_unicode(name) not in exclude]
        if domain:
            choices = choices + [(upn, upn) \
                    for upn in sorted(gobj.wbinfo_g()) \
                        if force_unicode(upn) not in exclude]
        if selected_only:
            # selectedなものだけoptionsの要素としてレンダリングする
            self.widget = SelectRenderSelectedOnly
        super(MultiSelectField, self).__init__(choices=choices, *args, **kwargs)
