# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django.utils.translation import ugettext as _
from django import forms
from django.core import validators
from ..text import TextField
from ..select import SelectField, MultiSelectField
from validators.fal import FALValidator
from fal_validinfo import ShareName
from project.fal.share import Share


class ShareNameValidator(FALValidator):
    validator = ShareName.VALIDATOR
    options = {}


class ShareNameField(TextField):
#    widget = forms.widgets.TextInput
    default_error_messages = {
#        'required': _(u'This field is required.'),
#        'invalid': _(u'Enter a valid value.'),
#        'max_length': _(u'Ensure this value has at most %(limit_value)d characters (it has %(show_value)d).'),
#        'min_length': _(u'Ensure this value has at least %(limit_value)d characters (it has %(show_value)d).'),
    }
    default_validators = [ShareNameValidator()]

    def __init__(self,
            max_length=ShareName.MAX_LENGTH,
            *args, **kwargs):
        super(ShareNameField, self).__init__(
                max_length=max_length,
                *args, **kwargs)


class SelectShareField(SelectField):
    #widget = forms.widgets.Select
    default_error_message = {
    #    'required': _(u'This field is required.'),
    #    'invalid': _(u'Enter a valid value.'),
    #    'invalid_choice': _(u'Select a valid choice. %(value)s is not one of the available choices.'),
    }

    def shares(self):
        try:
            return sorted(Share().load().keys())
        except:
            return []

    def __init__(self, exclude=[], *args, **kwargs):
        choices = [(name, name) for name in self.shares() if name not in exclude]
        choices.insert(0, ('', _(u'Please select.')))
        super(SelectShareField, self).__init__(
                choices=choices,
                *args, **kwargs)


class MultiSelectShareField(MultiSelectField):
    #widget = forms.widgets.SelectMultiple
    #default_validators = []
    default_error_message = {
    #    'required': _(u'This field is required.'),
    #    'invalid': _(u'Enter a valid value.'),
    #    'invalid_choice': _(u'Select a valid choice. %(value)s is not one of the available choices.'),
    #    'invalid_list': _(u'Enter a list of values.'),
    }

    def shares(self, deletable=True):
        try:
            shares = Share().load()
            if deletable:
                shares = dict([(k, v) for k, v in shares.iteritems() \
                        if v.get('undeletable', False) is False])
            return sorted(shares.keys())
        except:
            return []

    def __init__(self, deletable=True, choices=None, *args, **kwargs):
        if choices == None:
            choices = [(name, name) for name in self.shares(deletable=deletable)]
        super(MultiSelectShareField, self).__init__(choices=choices, *args, **kwargs)
