# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django.utils.translation import ugettext as _
from ..select import SelectField


class USBModeField(SelectField):

    def __get_modelist(self, port):
        try:
            from project.fal.features import Features
            for usb in Features().query()["deviceinfo"]["usblist"]:
                if str(usb["port"]) == str(port):
                    return usb["modelist"]
        except:
            return []

    def __mode_humanize(self, mode):
        return {
            "share": _(u"USBMODE: Share"),
            "quickcopy": _(u"USBMODE: Quickcopy"),
            "mirroring": _(u"USBMODE: Mirroring"),
            "vfsext": _(u"USBMODE: VFSEXT"),
        }.get(mode, mode)

    def __init__(self, port=1, *args, **kwargs):
        choices = [(mode, self.__mode_humanize(mode))
                   for mode in self.__get_modelist(port)]
        super(USBModeField, self).__init__(choices=choices,
                                           *args, **kwargs)
