# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django.utils.translation import ugettext as _
from django.http import HttpResponseRedirect, HttpResponse
import os
from datetime import datetime
from project.view import FuncView
from root.modules.form import BaseForm
import settings
from ionas.request import (Request, Entry)
from product_conf import SUDO_CMD
from fal_info import (FAL_SYSTEMLOG_PATH, ReqTypes, Results)


LEN = 3
MAX = 100
TIMESTAMP = "timestamp"
GET_ENTRY = '<systemlog><accesslog><option index="%(index)s" ' + \
            'maxnum="%(maxnum)s" sort="%(sort)s" reverse="%(reverse)s"/>' + \
            '</accesslog></systemlog>'
LOGLIST_XPATH = "./accesslog/loglist"
LOG_XPATH = "./accesslog/loglist/log"
USERNAME_XPATH = "./accessinfo/username"
CLIENT_XPATH = "./accessinfo/client"
OPERATION_XPATH = "./accessinfo/operation"
TARGETSHARE_XPATH = "./accessinfo/targetshare"
TARGETFILE_XPATH = "./accessinfo/targetfile"
#LOG_BODY = '%(user)s(%(client)s): %(operation)s(%(result)s): ' + \
#           '%(targetshare)s    %(targetfile)s'
LOG_BODY = '%(user)s(%(clientname)s, %(client)s): %(operation)s(%(result)s): %(targetshare)s    %(targetfile)s'


def get_log_list(page=1, sort="timestamp", reverse="true"):
    index = (page - 1) * MAX
    entry_body = GET_ENTRY % {
            "index": str(index),
            "maxnum": str(MAX),
            "sort": sort,
            "reverse": reverse}
    entry = Entry(ReqTypes.GET, entry_body)
    result = Request(FAL_SYSTEMLOG_PATH, filter_cmd=SUDO_CMD).request(
                entry).get_entrylist().get_entry().get()
    total = int(result.get_node(LOGLIST_XPATH).get_attr_value("total"))
    log_list = []
    for log in result.get_node_list(LOG_XPATH):
        log_dic = {}
        log_dic["date"] = log.get_attr_value("date")
        log_dic["username"] = log.get_value("accessinfo/username")
        log_dic["client"] = log.get_value("accessinfo/client")
        log_dic["clientname"] = log.get_value("accessinfo/clientname")
        log_dic["operation"] = log.get_value("accessinfo/operation")
        log_dic["result"] = log.get_node(OPERATION_XPATH).get_attr_value("result")
        log_dic["targetshare"] = log.get_value("accessinfo/targetshare")
        log_dic["targetfile"] = log.get_value("accessinfo/targetfile")
        log_dic["log"] = LOG_BODY % {
            "user": log.get_node(USERNAME_XPATH).get_text(),
            "client": log.get_node(CLIENT_XPATH).get_text(),
            "clientname": log.get_value("accessinfo/clientname"),
            "operation": log.get_node(OPERATION_XPATH).get_text(),
            "result": log.get_node(OPERATION_XPATH).get_attr_value("result"),
            "targetshare": log.get_node(TARGETSHARE_XPATH).get_text(),
            "targetfile": log.get_node(TARGETFILE_XPATH).get_text()}
        log_list.append(log_dic)
    return [log_list, total]


class LogView(FuncView):
    views = []

    def view(self, request, page=1, sort=None, reverse=None):
        page = int(page) if page else 1
        sort = "none" if sort == "none" else TIMESTAMP
        reverse = reverse if reverse == "false" else "true"
        logs, total = get_log_list(
            page=page,
            sort=sort,
            reverse=reverse)

        if total % MAX:
            total_pages = range(1, total / MAX + 2)
        else:
            total_pages = range(1, total / MAX + 1)

        if page in total_pages:
            start = page - LEN - 1
            start = start if start >= 0 else 0
            pages = total_pages[start:page + LEN]
        else:
            pages = total_pages[0:LEN]

        return {
            self.KEY_TEMPLATE: os.path.join(self.TEMPLATES, 'accesslog.html'),
            self.KEY_TEMPLATE_DIC: {
                'pages': pages,
                'page_now': page,
                'page_prev': page - 1 if page - 1 in pages else None,
                'page_next': page + 1 if page + 1 in pages else None,
                'logs': logs,
                'sort': sort,
                'reverse': reverse,
                'URL': '/'.join([
                    settings.MENU_URL,
                    'accesslog;sort:%s;reverse:%s' % (TIMESTAMP, reverse)]),
                'URL_st': '/'.join([
                    settings.MENU_URL,
                    'accesslog;sort:%s;reverse:%s' % (
                        TIMESTAMP, "false" if reverse == "true" else "true")])
            }
        }


class DownloadView(FuncView):
    IN_FRAME = False
    views = []

    def view(self, request):
        import zipfile
        try:
            from django.conf import settings
            TMP = settings.TMP
        except:
            TMP = '/tmp/'

        name = 'accesslog_' + datetime.now().strftime('%Y%m%d%H%M%S')
        lpath = os.path.join(TMP, 'accesslog', 'accesslog.csv')
        if not os.path.exists(os.path.dirname(lpath)):
            os.makedirs(os.path.dirname(lpath))
        entry = Entry(ReqTypes.CSV_TRANSFORM, "<systemlog/>")
        entry.set_file(lpath)
        result = Request(FAL_SYSTEMLOG_PATH, filter_cmd=SUDO_CMD).request(
                entry).get_entrylist().get_entry()

        zpath = os.path.join(TMP, 'accesslog', 'accesslog.zip')
        z = zipfile.ZipFile(zpath, 'w')
        z.write(lpath, '%s.csv' % name)
        z.close()

        response = HttpResponse(
            open(zpath, 'rb').read(),
            mimetype="application/zip")
        response['Content-Disposition'] = 'attachment; filename=%s.zip' % name
        return {self.KEY_RESPONSE: response}
