# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django.utils.translation import ugettext as _
from django.http import HttpResponseRedirect, HttpResponse
import os
from datetime import datetime
from project.view import FuncView, RootView
from root.modules.form import BaseForm
import settings

from django.core.exceptions import ValidationError
from project.fal.eventlog import EventLog
from project.forms.fields.extras.datetime import SelectDateTimeField
from project.forms.fields.checkbox import MultiCheckboxField
from project.forms.fields.extras.eventlog import LevelField, CategoryField


class ListView(FuncView):
    LEN = 3
    MAX = 100
    SORTKEYS = ('timestamp', 'level', 'category')

    views = []

    class Form(BaseForm):
        from project.forms.fields.extras.enable import EnableField
        from project.forms.fields.select import SelectField
        sort = SelectField(
            label=_(u'sort'),
            choices=(
                ('timestamp:DESC', "%s:%s" % (_(u'DateTime'), _(u'DESC'))),
                ('timestamp:ASC', "%s:%s" % (_(u'DateTime'), _(u'ASC'))),
                ('level:DESC', "%s:%s" % (_(u'Level'), _(u'DESC'))),
                ('level:ASC', "%s:%s" % (_(u'Level'), _(u'ASC'))),
                ('category:DESC', "%s:%s" % (_(u'Category'), _(u'DESC'))),
                ('category:ASC', "%s:%s" % (_(u'Category'), _(u'ASC'))),
            )
        )
        filtering = EnableField(label=_(u'filtering'), required=False)

    def _get_log_list(self,
            page=1,
            sortkey='timestamp',
            reverse=False,
            filtering=False):
        return EventLog().get(
#                index=page * self.MAX * -1,
                index=(page - 1) * self.MAX,
                maxnum=self.MAX,
                sort=sortkey,
                reverse=reverse,
                filtering=filtering)

    def view(self, request, page=None):
        page = int(page) if page is not None else 1
        form = self.Form(data=request.POST)
        if form.is_valid():
#            page = int(page) if page is not None else 1
            filtering = form.cleaned_data['filtering']
            (sortkey, reverse) = form.cleaned_data['sort'].split(':')[:2]
            reverse = {'DESC': True, 'ASC': False}[reverse]
        else:
#            page = 1
            filtering = False
            (sortkey, reverse) = (self.SORTKEYS[0], True)
            form = self.Form()

        logs, total = self._get_log_list(
                page=page,
                sortkey=sortkey,
                reverse=reverse,
                filtering=filtering)
        if total % self.MAX:
            total_pages = range(1, total / self.MAX + 2)
        else:
            total_pages = range(1, total / self.MAX + 1)
        if page in total_pages:
            start = page - self.LEN - 1
            start = start if start >= 0 else 0
            pages = total_pages[start:page + self.LEN]
        else:
            pages = total_pages[0:self.LEN]

        return {
            self.KEY_TEMPLATE: os.path.join(self.TEMPLATES, 'log.html'),
            self.KEY_TEMPLATE_DIC: {
                'URL': '/'.join([settings.MENU_URL, 'list']),
                'pages': pages,
                'page_now': page,
                'page_prev': page - 1 if page - 1 in pages else None,
                'page_next': page + 1 if page + 1 in pages else None,
                'logs': logs,
                'form': form}}


class DownloadView(RootView):
#    IN_FRAME = False
    NOT_LOGINED_URL = '/'.join([settings.MENU_URL, 'list'])
    views = []

#    def view(self, request):
#        from django.conf import settings
#        fname = 'log.csv'
#        fpath = os.path.join(settings.TMP, 'eventlog', fname)  # FIXME path
#        eventlog = EventLog()
#        fpath = eventlog.file(path=fpath)
#        import codecs
#        response = HttpResponse(
#                codecs.open(fpath, 'rb').read(),
#                mimetype="text/csv")
#        response['Content-Disposition'] = 'attachment; filename=%s' % fname
#        return {self.KEY_RESPONSE: response}

    def view(self, request):
        from django.conf import settings
        import zipfile

        name = datetime.now().strftime('%Y%m%d%H%M%S')

        lpath = os.path.join(settings.TMP, 'eventlog', 'log.csv')
        lpath = EventLog().file(path=lpath)

#        zpath = os.path.join(settings.TMP, 'eventlog', 'log.zip')
#        z = zipfile.ZipFile(zpath, 'w')
#        z.write(lpath, '%s.csv' % name)
#        z.close()
#
#        response = HttpResponse(
#                open(zpath, 'rb').read(),
#                mimetype='application/zip')
#        response['Content-Disposition'] = 'attachment; filename=%s.zip' % name
        response = HttpResponse(
                open(lpath, 'rb').read(),
                mimetype='text/csv')
        response['Content-Disposition'] = 'attachment; filename=%s.csv' % name
        return {self.KEY_RESPONSE: response}


class FilterView(FuncView):
    TITLE = _(u'Filter Setting')

    class Form(BaseForm):
        datetime_from = SelectDateTimeField(
                label="%s %s" % (_(u'DateTime'), _(u'Timestamp(from)')),
                required=False)
        datetime_to = SelectDateTimeField(
                label="%s %s" % (_(u'DateTime'), _(u'Timestamp(to)')),
                required=False)
        level = LevelField()
        category = CategoryField()

        def clean(self):
            data = self.cleaned_data
            f = self.cleaned_data.get('datetime_from', None)
            t = self.cleaned_data.get('datetime_to', None)
            if isinstance(f, datetime) and isinstance(t, datetime) and f > t:
                raise ValidationError(_(u'"To DateTime" before "From DateTime"'))
            return data

    views = ['apply']

    def __init__(self):
        super(FilterView, self).__init__()
        self.TEMPLATE = os.path.join(self.TEMPLATES, 'filter.html')

    def view(self, request):
        eventlog = EventLog()
        config = eventlog.load()
        initial = {
            'level': config['level'],
            'category': config['category'],
            'datetime_from': config['timestamp']['from'],
            'datetime_to': config['timestamp']['to'],
        }
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {'form': self.Form(initial=initial)}}

    def view_apply(self, request):
        with self.processing(constname='ERR_LOG_STATE_CHANGE') as p:
            form = self.Form(data=request.POST)
            p.break_if_state_change()

            if form.is_valid():
                eventlog = EventLog()
                eventlog.save(
                    level=form.cleaned_data['level'],
                    category=form.cleaned_data['category'],
                    timestamp={
                        'from':form.cleaned_data['datetime_from'],
                        'to':form.cleaned_data['datetime_to'],
                    }
                )
                self.set_message(eventlog.get_message())
                p.result(eventlog.get_result(boolean=True))
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {'form': form}}
