# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django.conf import settings
from django.utils.translation import ugettext as _
import re
import os
from iobase.exception import NotFoundError
from ionas.request import (Request, Entry)
from product_conf import (SUDO_CMD, STOR_IF_TO_VOL, DEV_DIR, devinfo)
from project.fal.volctrl import Volctrl
from project.fal.volinfo import Volinfo
from project.fal.raidctrl import Raidctrl
from project.fal.command import logging
from fal_info import (FAL_ANNOUNCE_PATH, FAL_MESSAGES_PATH, FAL_SYSINFO_PATH,
    FAL_DATETIME_PATH, FAL_BACKUPS_PATH, FAL_DISKCTRL_PATH,
    FAL_ECO_PATH, FAL_POWER_PATH, FAL_UPS_PATH, FAL_LED_PATH, FAL_PORTS_PATH,
    FAL_NETWORK_PATH, FAL_MSNETWORK_PATH, FAL_PROXY_PATH, ReqTypes, Results)


ANNOUNCE_ENTRY = "<announce/>"
MESSAGE_ENTRY = "<messages><messageinfo><code/></messageinfo></messages>"
TIME_ENTRY = "<datetime/>"
UPS_ENTRY = "<ups/>"
NETWORK_ENTRY = "<network/>"
DISK_ENTRY = "<diskctrl><disk><name>%s</name></disk></diskctrl>"

ERR_XPATH = "./announce[@level='2']"    # FIXME
ALERT_XPATH = "./announce[@level='1']"    # FIXME
MODEL_XPATH = "./model"
FIRM_XPATH = "./fwversion"
SYNC_XPATH = "./ntp/sync"
SRV_XPATH = "./ntp/serveraddress"
JOB_XPATH = "./joblist/job"
STATUS_XPATH = "./status"
BATTERY_XPATH = "./battery"
TEMPERATURE_XPATH = "./temperature[@type='mb']"
FANRPM_XPATH = "./fanrpm"
HOSTNAME_XPATH = "./hostdesc/name"
HOSTDESC_XPATH = "./hostdesc/desc"
LOCATION_XPATH = "./hostdesc/location"
INET_XPATH = "./familyconfig/%s"
NETIF_XPATH = "./netiflist/netif[@name='%s']"

INET6 = "inet6"
INET4 = "inet4"
GB_UNIT = 1000000000
WARNING = "warning"
ERROR = "error"


NETIF_NAMES = ["lan%s" % netif["port"] for netif in settings.NETWORK_INTERFACES if netif.get("port")]


class Information(object):

    def send_request(self, cmd, req_entry):
        try:
            response = Request(cmd, filter_cmd=SUDO_CMD).request(req_entry)
            if response.get_entrylist().get_entry().get_result() == Results.OK:
                return True
            else:
                return False
        except:
            return False

    def load_request(self, cmd):
        result = Request(cmd, filter_cmd=SUDO_CMD).request(
                Entry(ReqTypes.LOAD)).get_entrylist().get_entry().get()
        return result

    def get_announces(self):
        try:
            from project.fal.announce import Announce
            return [
            {
                'mesg': announce['message'],
                'level': announce['level'],
            } for announce in Announce().query() \
            if announce.get('message', None) and announce.get('level', None)]
        except:
            return []
#
#        messages = []
#        entry = Entry(ReqTypes.QUERY, ANNOUNCE_ENTRY)
#        result = Request(FAL_ANNOUNCE_PATH, filter_cmd=SUDO_CMD).request(
#                        entry).get_entrylist().get_entry().get()
#
#        for item in result.get_node_list("announce"):
#            mesg_dic = {}
#            if item.get_attr_value("level") == "1":
#                mesg_dic["level"] = WARNING
#            else:
#                mesg_dic["level"] = ERROR
#
#            code = item.get_node("code").get_text()
#            mesg_entry = Entry(ReqTypes.QUERY, MESSAGE_ENTRY)
#            mesg_entry.get().get_node("messageinfo/code").set_text(code)
#            try:
#                arglist = item.get_node("arglist")
#                mesg_entry.get().get_node("messageinfo").append_child(arglist)
#            except NotFoundError:
#                pass
#            try:
#                mesg_request = Request(
#                    FAL_MESSAGES_PATH,
#                    filter_cmd=SUDO_CMD).request(mesg_entry)
#                mesg_entrylist = mesg_request.get_entrylist()
#                mesg_entry = mesg_entrylist.get_entry()
#                mesg_result = mesg_entry.get()
#                mesg_dic["mesg"] = mesg_result.get_node(
#                            "messageinfo/message").get_text()
#                messages.append(mesg_dic)
#            except:
#                pass
#        return messages

    def announces_clear(self, level="1"):
        entry = Entry(ReqTypes.CLEAR, ANNOUNCE_ENTRY)
        entry.get().append_child("option", attrs={"level": level})
        return self.send_request(FAL_ANNOUNCE_PATH, entry)

    def get_basic_info(self, query_info):
        infos = {}
        infos["service"] = ["Microsoft", "AppleShare", "FTP", "Network Backup",
                            "Quota Limit"]      # FIXME    FAL_SERVICE
        if not query_info:
            return infos
        try:
            infos["model"] = query_info.get_node(MODEL_XPATH).get_text()
            infos["firm"] = query_info.get_node(FIRM_XPATH).get_text()
        except:
            pass
        return infos

    def get_time_settings(self):
        time_info = {"time": "", "sync": "Invalid", "srv": ""}
        try:
            time_entry = Entry(ReqTypes.QUERY, TIME_ENTRY)
            time_result = Request(
                    FAL_DATETIME_PATH, filter_cmd=SUDO_CMD).request(
                    time_entry).get_entrylist().get_entry().get()
            year = time_result.get_node("date/year").get_text()
            month = time_result.get_node("date/month").get_text()
            day = time_result.get_node("date/day").get_text()
            hour = time_result.get_node("time/hour").get_text()
            if len(hour) == 1:
                hour = "0" + hour
            minute = time_result.get_node("time/min").get_text()
            if len(minute) == 1:
                minute = "0" + minute
            sec = time_result.get_node("time/sec").get_text()
            if len(sec) == 1:
                sec = "0" + sec
            time_info["time"] = "%(Y)s/%(M)s/%(D)s %(h)s:%(m)s:%(s)s" % {
                "Y": year, "M": month, "D": day,
                "h": hour, "m": minute, "s": sec}
            time_ntp = self.load_request(FAL_DATETIME_PATH)
            if time_ntp.get_node("ntp").is_enabled():
                time_info["sync"] = "Valid"
                time_info["srv"] = time_ntp.get_node(SRV_XPATH).get_text()
        except:
            pass

        return time_info

    def get_raid_info(self):
        raidctrl = Raidctrl()
        raid_info = raidctrl.query()
        if raid_info.get("level"):
            return raid_info["level"]
        else:
            return "-"

    def get_quickcopy_set(self):
        ports = self.load_request(FAL_PORTS_PATH)
        try:
            usb_port1 = ports.get_node("./usblist/usb[@port='1']")
            quickcp = usb_port1.get_node("./modelist/mode[text()='quickcopy']")
            if quickcp.is_enabled():
                return 'Valid'
            else:
                return 'Invalid'
        except NotFoundError:
            return 'Invalid'

    def get_backup_info(self, type="backup"):
        try:
            backup = self.load_request(FAL_BACKUPS_PATH)
            for job in backup.get_node(type).get_node_list(JOB_XPATH):
                if job.get_node("schedule").get_attr_value("enable") == "true":
                    return 'Valid'
            return 'Invalid'
        except:
            return ""

    def get_eco_info(self):
        time = "0"
        try:
            eco = self.load_request(FAL_ECO_PATH)
            if eco.get_node_list("volumelist/volume[@enable='true']"):
                time = eco.get_node("time/minute").get_text()
        except:
            pass

        return time

    def get_led_info(self):
        try:
            led = self.load_request(FAL_LED_PATH)
            brightness = led.get_node("./brightness").get_text()
        except:
            brightness = ""
        return brightness

    def get_poweron_schedule(self):
        try:
            x_ontime = "powerschedule/dayschedule/ontime[@enable='true']"
            x_offtime = "powerschedule/dayschedule/offtime[@enable='true']"
            power = self.load_request(FAL_POWER_PATH)
            ontime_list = power.get_node_list(x_ontime)
            offtime_list = power.get_node_list(x_offtime)
            if len(ontime_list) > 0:
                return 'Valid'
            elif len(offtime_list) > 0:
                return 'Valid'
            else:
                return 'Invalid'
        except:
            return 'Invalid'

    def get_printer_info(self):
        for dev in devinfo.USB_LP_DEV_LIST:
            dev_path = os.path.join(DEV_DIR, dev)
            if os.path.exists(dev_path):
                return True
        return False

    def get_ups_info(self):
        ups_info = {"status": "DISCONNECT",
                    "warning": "Invalid",
                    "battery": "-"}
        entry = Entry(ReqTypes.QUERY, UPS_ENTRY)
        response = Request(FAL_UPS_PATH, filter_cmd=SUDO_CMD).request(
                    entry).get_entrylist().get_entry()
        if response.get_result() != Results.OK:
            return ups_info
        try:
            ups_status = response.get().get_node(STATUS_XPATH).get_text()
            ups_info["status"] = ups_status
            if ups_status == "UNKNOWN":
                from fal_msginfo import Msg
                logging(Msg.ERR_UPS_GET_STATUS)
        except NotFoundError:
            from fal_msginfo import Msg
            logging(Msg.ERR_UPS_GET_STATUS)
        battery = response.get().get_node(BATTERY_XPATH).get_text()
        if battery:
            ups_info["battery"] = battery + "%"

        ups_config = self.load_request(FAL_UPS_PATH)
        if ups_config.get_node("warning").is_enabled():
            ups_info["warning"] = 'Valid'
        return ups_info

    def query_sysinfo(self):
        entry = Entry(ReqTypes.QUERY)
        response = Request(FAL_SYSINFO_PATH, filter_cmd=SUDO_CMD).request(
                                     entry).get_entrylist().get_entry()
        return response.get()

    def get_fan_info(self, query_info):
        fan_info = {"fanrpm": "", "temperature": ""}
        if not query_info:
            return fan_info
        try:
            fan_info["temperature"] = query_info.get_node(
                                TEMPERATURE_XPATH).get_text()
            fan_info["fanrpm"] = query_info.get_node(FANRPM_XPATH).get_text()
        except:
            pass
        return fan_info

    def get_host_info(self):
        infos = {"hostname": "", "hostdesc": ""}
        config = self.load_request(FAL_SYSINFO_PATH)
        infos["hostname"] = config.get_node(HOSTNAME_XPATH).get_text()
        infos["hostdesc"] = config.get_node(HOSTDESC_XPATH).get_text()
        infos["location"] = config.get_node(LOCATION_XPATH).get_text()
        return infos

    def query_network_info(self):
        entry = Entry(ReqTypes.QUERY, NETWORK_ENTRY)
        request_base = Request(FAL_NETWORK_PATH, filter_cmd=SUDO_CMD)
        request = request_base.request(entry)
        response_entrylist = request.get_entrylist()
        response = response_entrylist.get_entry()
        return response.get()

    def get_tcpip_info(self, query_info, aftype=INET4):
        """Get TCP/IP dic data for aftype."""

        infos = {}

        # common
        load_info = self.load_request(FAL_NETWORK_PATH)
        inet = load_info.get_node(INET_XPATH % aftype)
        mainif = inet.get_node("dns").get_attr_value("source")
        infos["common"] = mainif.upper()

        infos.update(self.get_network_netif_info(load_info, aftype))
        infos.update(self.get_network_query_info(query_info, aftype))

        for ifname in NETIF_NAMES:
            if infos[ifname + "_method"] == 'Invalid':
                del_key_list = [
                    "_ip", "_netmask", "_prefixlen",
                    "_state", "_framesize", "_mac", "_wol"]
                for key in del_key_list:
                    try:
                        del infos[ifname + key]
                    except KeyError:
                        pass

        return infos

    def get_network_netif_info(self, load_info, aftype):
        """Convert FAL network load result nodes to dic data."""

        infos = {}

        for ifname in NETIF_NAMES:
            # lan?_method
            xpath = NETIF_XPATH % ifname
            netif = load_info.get_node(xpath)
            if not netif.is_enabled():
                infos[ifname + "_method"] = 'Invalid'
                continue

            xpath = "protocollist/protocol[@family='%s']" % aftype
            protocol = netif.get_node(xpath)
            if not protocol.is_enabled():
                infos[ifname + "_method"] = 'Invalid'
                continue

            xpath = "dhcp"
            dhcp = protocol.get_node(xpath)
            if dhcp.is_enabled():
                infos[ifname + "_method"] = 'dhcp'
            else:
                infos[ifname + "_method"] = 'manual'

            # lan?_wol
            attr = "wol"
            if netif.get_attr_value(attr) == "true":
                infos[ifname + "_wol"] = 'Valid'
            else:
                infos[ifname + "_wol"] = 'Invalid'

        return infos

    def get_network_query_info(self, query_info, aftype):
        """Convert FAL network query result nodes to dic data."""

        infos = {}

        # gateway
        try:
            infos["gateway"] = query_info.get_node(
                INET_XPATH % aftype).get_node("gateway/addr").get_text()
        except NotFoundError:
            pass

        # dnssrv
        try:
            infos["dnssrv"] = query_info.get_node(
                INET_XPATH % aftype).get_node("dns/addrlist/addr").get_text()
        except NotFoundError:
            pass

        for ifname in NETIF_NAMES:
            netif = query_info.get_node(NETIF_XPATH % ifname)

            # lan?_state
            try:
                infos[ifname + "_state"] = netif.get_attr_value("speed")
            except NotFoundError:
                pass

            # lan?_framesize
            infos[ifname + "_framesize"] = netif.get_attr_value("mtu")

            # lan?_mac
            infos[ifname + "_mac"] = netif.get_node("mac").get_text()

            try:
                # lan?_ip
                xpath = "protocollist/protocol[@family='%s']" % aftype
                protocol = netif.get_node(xpath)
                addr = protocol.get_node("addrlist/addr")
                infos[ifname + "_ip"] = addr.get_text()

                # lan?_netmask/lan?_prefixlen
                if aftype == INET4:
                    infos[ifname + "_netmask"] = \
                        addr.get_attr_value("netmask")
                elif aftype == INET6:
                    infos[ifname + "_prefixlen"] = \
                        addr.get_attr_value("prefixlen")
            except NotFoundError:
                pass

        return infos

    def get_msnetwork_info(self):
        infos = {}
        config = self.load_request(FAL_MSNETWORK_PATH)
        assign = config.get_node("assign")
        method = assign.get_attr_value("method")
        infos["method"] = method
        if method == "wg":
            infos["wg"] = assign.get_node("wg/workgroup").get_text()
        elif method == "nt":
            infos["domain"] = assign.get_node("nt/domain").get_text()
            infos["dcname"] = assign.get_node("nt/dcname").get_text()
        elif method == "ad":
            infos["netbios"] = assign.get_node("ad/netbios").get_text()
            infos["fulldns"] = assign.get_node("ad/fulldns").get_text()
            infos["dcname"] = assign.get_node("ad/dcname").get_text()
        infos["winssrv"] = config.get_node("winssrv").get_text()
        return infos

    def get_volume_info(self, name):
        infos = {"status": "-", "mode": "-"}
        volctrl = Volctrl()
        volinfo = Volinfo()
        if name == "internal":
            data_dict = volctrl.query(volume=STOR_IF_TO_VOL[name][1])
            infos["status"] = volinfo.status()
            infos["disks"] = self.get_disk_info()
            infos["mode"] = self.get_raid_info()
            infos.update(self.__get_vol_infos(data_dict))
        else:
            from project.fal.port import Port
            _mode = self.get_usb_port_mode(name)
            infos["mode"] = Port().trans(_mode)
            infos["status"] = volinfo.status(name)
            #infos["no_vol"] = "false"
            infos["no_vol"] = False
            if (infos["status"] == _(u"Unconnected")) \
                    or (_mode in ["mirroring", "vfsext"]) \
                    or (volinfo.status(name, humanize=False) == "lockkey"):
                #infos["no_vol"] = "true"
                infos["no_vol"] = True
            #if infos["no_vol"] == "false":
            if not infos["no_vol"]:
                try:
                    data_dict = volctrl.query(volume=name)
                    if data_dict.get("status") in ["unmounted"]:
                        #infos["no_vol"] = "true"
                        infos["no_vol"] = True
                    infos.update(self.__get_vol_infos(data_dict))
                except:
                    #infos["no_vol"] = "true"
                    infos["no_vol"] = True
        return infos

    def __get_vol_infos(self, data_dict):
        infos = {}
        if data_dict.get("filesystem"):
            infos["filesystem"] = data_dict["filesystem"]
        else:
            infos["filesystem"] = "-"
        infos["encrypt"] = data_dict.get("encrypt")
        size = data_dict.get("size")
        if size:
            infos.update(self.__get_disk_size(
                size.get("total"), size.get("use")))
        else:
            infos.update(self.__get_disk_size("0.0", "0.0"))
        return infos

    def __get_disk_size(self, total, used):
        infos = {"size": "-", "usedsize": "-"}
        if not total or float(total) == 0:
            return infos
        total_g = float(total) / GB_UNIT
        infos["size"] = "%.2fGB (%sbyte)" % (total_g, total)
        if used == None or used == "":
            used == 0
        used_g = float(used) / GB_UNIT
        per = float(used) / float(total) * 100
        infos["usedsize"] = \
                "%.2fGB (%sbyte)  %.2f%%" % (used_g, used, per)
        return infos

    def get_usb_port_mode(self, name):
        port_config = self.load_request(FAL_PORTS_PATH)
        match = re.match("usb(\d+)", name)
        port_num = match.group(1)
        try:
            target = port_config.get_node(
                    "usblist/usb[@port='%s']" % port_num)
            mode = target.get_node(
                    "./modelist/mode[@enable='true']").get_text()
            return mode
        except NotFoundError:
            return "-"

    def get_disk_info(self):
        disks = {}
        for disk in devinfo.SATA_HDD_DEV_LIST:
            data = {"info": "", "status": ""}
            entry = Entry(ReqTypes.QUERY, DISK_ENTRY % disk)
            response = Request(FAL_DISKCTRL_PATH, filter_cmd=SUDO_CMD).request(
                entry).get_entrylist().get_entry()
            if response.get_result() == Results.DEVICE_NOT_FOUND:
                data["status"] = "unconnected"
            elif response.get_result() == Results.OK:
                try:
                    status = response.get().get_node("disk/status").get_text()
                    if status == "":
                        data["status"] = "unconnected"
                    else:
                        data["status"] = status
                    model = response.get().get_node("disk/model").get_text()
                    size = response.get().get_node("disk/size").get_text()
                    size_gb = float(size) * 512 / GB_UNIT
                    data["info"] = model + "   %.2f GB" % size_gb
                except:
                    pass
            disk_re = re.match("satad(\d)", disk)
#            disks["disk" + disk_re.group(1)] = data
            disks[int(disk_re.group(1))] = data
        return disks

    def get_proxy_info(self):
        info = {}
        config = self.load_request(FAL_PROXY_PATH)
        if config.is_enabled():
            manual = config.get_node("manual")
            pacfile = config.get_node("pacfile")
            if manual.is_enabled():
                info["proxy"] = "manual"
                info["host"] = manual.get_node("host").get_text()
                info["port"] = manual.get_node("port").get_text()
            elif pacfile.is_enabled():
                info["proxy"] = "pacfile"
                info["url"] = pacfile.get_node("url").get_text()
        else:
            info["proxy"] = "disable"

        return info
