# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django.conf import settings
import os
import sys
from django.http import HttpResponseRedirect
from project.view import FuncView
from project.fal.raidctrl import Raidctrl
from project.forms.fields.extras.spindown import SpindownTimeField
from root.admin.system.service.modules.services import Services

IMPORT_PREFIX = "from methods."
IMPORT_SUFFIX = " import Method"


def root(request):
    return HttpResponseRedirect('./sysinfo')


class View(FuncView):
    from ..modules.info import Information
    info = Information()

    def __get_enabled_fix_services(self):
        service = Services()
        fix_services_conf = service.get_fix_service_init_config()
        service_tuple = (
            "quota", "ups", "snmp", "narsus", "accesslog",
            "firm_auto", "firm_notice", "pkg_auto", "pkg_notice", "msnetwork")
        service_list = []
        for name in service_tuple:
            if fix_services_conf.get(name) == "true":
                service_list.append(name)
        return tuple(service_list)

    def __get_ext_services(self):
        ext_services = []
        current = os.path.dirname(sys.modules[self.__module__].__file__)
        dpath = os.path.join(current, "methods")
        for fname in sorted(os.listdir(dpath)):
            if (fname != "__init__.py" and
                fname.endswith(".py")):
                ext_services.append(fname.replace(".py", ""))
        return ext_services

    def __get_enabled_ext_services(self):
        service_list = []
        for name in self.__get_ext_services():
            if name in self.__get_enabled_fix_services():
                continue
            exec(IMPORT_PREFIX + name + IMPORT_SUFFIX)
            method = Method()
            if method.get():
                service_list.append(name)
        return tuple(service_list)

    def __get_enabled_services(self):
        fix_services = self.__get_enabled_fix_services()
        ext_services = self.__get_enabled_ext_services()
        return fix_services + ext_services

    def view(self, request):
        try:
            query_info = self.info.query_sysinfo()
        except:
            query_info = None
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                "INTERNAL_VOLUME_RAID_IS_SUPPORTED": settings.INTERNAL_VOLUME_RAID_IS_SUPPORTED,
                'basic': self.info.get_basic_info(query_info),
                'services': self.__get_enabled_services(),
                'time': self.info.get_time_settings(),
                'mode': Raidctrl().trans(self.info.get_raid_info()),
                'quickcp': self.info.get_quickcopy_set(),
                'backup': self.info.get_backup_info(),
                'datacopy': self.info.get_backup_info(type="datacopy"),
                'eco': SpindownTimeField().text(self.info.get_eco_info()),
                'led': self.info.get_led_info(),
                'poweron': self.info.get_poweron_schedule(),
                'printer': self.info.get_printer_info(),
                'ups': self.info.get_ups_info(),
                'fan': self.info.get_fan_info(query_info),
            },
        }

sysinfo = View.as_view()
