# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

import os

from django.utils.translation import ugettext as _
from django.http import HttpResponseRedirect
from django import forms
from project.view import FuncView
from django.core.exceptions import ValidationError
from root.modules.form import BaseForm
from project.fal.command import (get_message, logging)
import settings
from root.modules.form.fields.radio import RadioField
from root.modules.form.fields.checkbox import CheckboxField
from root.modules.form.fields.select import SelectField
from project.forms.fields.extras.datetime import SelectDateTimeField
from modules.initsetting import InitSetting
from project.forms.fields.extras.landiskname import LandiskNameField
from project.forms.fields.extras.comment import CommentField
from project.forms.fields.extras.ipv4address import IPv4AddressField
from project.forms.fields.extras.ipv4subnetmask import IPv4SubnetMaskField
from project.forms.fields.extras.fqdn import FQDNField

from django.conf import settings as project_settings


NAME_SET = {
    'name': '',
    'desc': '',
    'location': '',
}
LAN_SET = {
    'dhcp': '',
    'ipaddr': '',
    'subnet': '',
    'gateway': '',
    'dnssrv': '',
}
TIME_SET = {
    'sync': None,
    'datetime': None,
    'ntpsrv': '',
    'boot': False,
    'daily': False,
    'hour': 0,
    'minute': 0,
}
SHARES_SET = {
    'shares': None,
}

ACCESS_SUFFIX = '_access'

SHARE_ACCESS_NOCHANGE = 'nochange'
SHARE_ACCESS_EVERYONE = 'everyone'
SHARE_ACCESS_ADMIN = 'admin'

initsetting = InitSetting()


def root(request):
    # 初期化
    NAME_SET['name'] = ''
    NAME_SET['desc'] = ''
    NAME_SET['location'] = ''
    LAN_SET['dhcp'] = ''
    LAN_SET['ipaddr'] = ''
    LAN_SET['subnet'] = ''
    LAN_SET['gateway'] = ''
    LAN_SET['dnssrv'] = ''
    TIME_SET['sync'] = None
    TIME_SET['datetime'] = None
    TIME_SET['ntpsrv'] = ''
    TIME_SET['boot'] = False
    TIME_SET['daily'] = False
    TIME_SET['hour'] = 0
    TIME_SET['minute'] = 0
    SHARES_SET['shares'] = None
    return HttpResponseRedirect('./initset')


class NameView(FuncView):
    class Form(BaseForm):
        name = LandiskNameField(
                label=_(u'LAN DISK Name'),
                required=True)
        desc = CommentField(
                label=_(u'LAN DISK Description'),
                required=False)
        location = CommentField(
                label=_(u'LAN DISK Location'),
                required=False)

    views = ['next']

    def view(self, request):
        if NAME_SET["name"] != "":
            init = NAME_SET
        else:
            init = initsetting.get_landisk_name()
        return {
            self.KEY_TEMPLATE: os.path.join(self.TEMPLATES, "nasname.html"),
            self.KEY_TEMPLATE_DIC: {'form': self.Form(initial=init)}}

    def view_next(self, request):
        form = self.Form(data=request.POST)
        if form.is_valid():
            clean_data = form.cleaned_data
            NAME_SET["name"] = clean_data.get("name")
            NAME_SET["desc"] = clean_data.get("desc")
            NAME_SET["location"] = clean_data.get("location")
            return {
                self.KEY_REDIRECT: "/".join([settings.MENU_URL, "lan1ipv4"])}

        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {"form": form}}


class LanView(FuncView):
    class Form(BaseForm):
        dhcp = RadioField(
            label=_(u'IP Address Setting Method'),
            choices=(('auto', _(u'Obtain an IP address automatically(DHCP)')),
                     ('custom', _(u'Use the following IP address'))),
            initial='auto',
            required=True)
        ipaddr = IPv4AddressField(
                label=_(u'IP Address'),
                required=False,
                widget=forms.TextInput(attrs={"class": "lan"}))
        subnet = IPv4SubnetMaskField(
                label=_(u'Subnet Mask'),
                required=False)
        gateway = IPv4AddressField(
                label=_(u'Default Gateway'),
                required=False,
                widget=forms.TextInput(attrs={"class": "lan"}))
        dnssrv = IPv4AddressField(
                label=_(u'DNS Server'),
                required=False,
                widget=forms.TextInput(attrs={"class": "lan"}))

        def clean(self):
            if self.cleaned_data.get("dhcp") == "custom":
                if self.cleaned_data.get("ipaddr") == "":
                    raise ValidationError(_(u'"IP Address" is required.'))
                if self.cleaned_data.get("subnet") == "":
                    raise ValidationError(_(u'"Subnet Mask" is required.'))
            return self.cleaned_data

    views = ['next', 'previous']

    def view(self, request):
        if LAN_SET["dhcp"] != "":
            init = LAN_SET
        else:
            init = initsetting.get_lan1ipv4_info()
        return {
            self.KEY_TEMPLATE: os.path.join(self.TEMPLATES, "lan1ipv4.html"),
            self.KEY_TEMPLATE_DIC: {
                'form': self.Form(initial=init),
                "interfaces": project_settings.NETWORK_INTERFACES,
            },
        }

    def view_next(self, request):
        form = self.Form(data=request.POST)
        if form.is_valid():
            clean_data = form.cleaned_data
            LAN_SET["dhcp"] = clean_data.get("dhcp")
            LAN_SET["ipaddr"] = clean_data.get("ipaddr")
            LAN_SET["subnet"] = clean_data.get("subnet")
            LAN_SET["gateway"] = clean_data.get("gateway")
            LAN_SET["dnssrv"] = clean_data.get("dnssrv")
            return {
                self.KEY_REDIRECT: "/".join([settings.MENU_URL, "time"])}

        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                "form": form,
                "interfaces": project_settings.NETWORK_INTERFACES,
            },
        }

    def view_previous(self, request):
        return {
            self.KEY_REDIRECT: "/".join([settings.MENU_URL, "nasname"])}


class TimeView(FuncView):
    class Form(BaseForm):
        sync = RadioField(
                choices=((0, _(u'manual')), (1, _(u'synchronous'))),
                label=_(u'Datetime Setting'),
                initial=0,
                required=True,
                validators=[])
        datetime = SelectDateTimeField(
                label=_(u'Datetime'),
                initial=None,
                required=False,
                show_seconds=False,
                validators=[])
        ntpsrv = FQDNField(
                label=_(u'NTP Server'),
                required=False,
                widget=forms.TextInput(attrs={"class": "ntp"}))
        boot = CheckboxField(
                label=_(u'Boot'),
                initial=False,
                required=False,
                widget=forms.CheckboxInput(attrs={"class": "ntp"}))
        daily = CheckboxField(
                label=_(u'Daily'),
                initial=False,
                required=False,
                widget=forms.CheckboxInput(attrs={"class": "ntp"}))
        hour = SelectField(
                choices=((h, h) for h in range(0, 24)),
                label=_(u'hour'),
                initial=0,
                required=False,
                widget=forms.Select(attrs={"class": "ntp"}))
        minute = SelectField(
                choices=((h, h) for h in range(0, 60, 5)),
                label=_(u'minute'),
                initial=0,
                required=False,
                widget=forms.Select(attrs={"class": "ntp"}))

        def clean(self):
            if self.cleaned_data.get("sync") == "1":
                if self.cleaned_data.get("ntpsrv") == "":
                    raise ValidationError(_(u'"NTP Server" is required.'))
                if not self.cleaned_data.get("boot"):
                    if not self.cleaned_data.get("daily"):
                        raise ValidationError(
                            _(u'Syncronous Timing is required.'))
            else:
                if not self.cleaned_data.get("datetime", True):
                    raise ValidationError(_(u'This field is required.'))
            return self.cleaned_data

    views = ['next', 'previous']

    def view(self, request):
        if TIME_SET["sync"] != None:
            init = TIME_SET
        else:
            init = initsetting.get_time_info()
        return {
            self.KEY_TEMPLATE: os.path.join(self.TEMPLATES, "time.html"),
            self.KEY_TEMPLATE_DIC: {'form': self.Form(initial=init)}}

    def view_next(self, request):
        form = self.Form(data=request.POST)
        if form.is_valid():
            clean_data = form.cleaned_data
            TIME_SET["sync"] = clean_data.get("sync")
            TIME_SET["ntpsrv"] = clean_data.get("ntpsrv")
            TIME_SET["datetime"] = clean_data.get("datetime")
            TIME_SET["boot"] = clean_data.get("boot")
            TIME_SET["daily"] = clean_data.get("daily")
            hour = clean_data.get("hour")
            if hour and len(hour) == 1:
                hour = "0" + hour
            TIME_SET["hour"] = hour
            minute = clean_data.get("minute")
            if minute and len(minute) == 1:
                minute = "0" + minute
            TIME_SET["minute"] = minute
            return {
                self.KEY_REDIRECT: "/".join([settings.MENU_URL, "shares"])}

        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {"form": form}}

    def view_previous(self, request):
        return {
            self.KEY_REDIRECT: "/".join([settings.MENU_URL, "lan1ipv4"])}


class SharesForm(BaseForm):
    def __init__(self, *args, **kwargs):
        super(SharesForm, self).__init__(*args, **kwargs)

        if SHARES_SET['shares'] != None:
            init = SHARES_SET['shares']
        else:
            init = initsetting.get_shares_info()

        DEFAULT_SHARE_CHOICES = (
            (SHARE_ACCESS_NOCHANGE, _(u'No Change')),
            (SHARE_ACCESS_EVERYONE, _(u'Everyone Share')),
            (SHARE_ACCESS_ADMIN, _(u'Admin Share')))

        for share in init:
            self.fields[share['id'] + ACCESS_SUFFIX] = SelectField(
                choices=DEFAULT_SHARE_CHOICES,
                label=_(u'Access Config'),
                initial=share['access'],
                required=False,
                validators=[])


class SharesView(FuncView):
    views = ['confirm', 'previous']

    def __get_datalist(self, form, datalist):
        dst_datalist = []
        for item in datalist:
            try:
                dst_item = {}
                dst_item['id'] = item['id']
                dst_item['name'] = item['name']
                dst_item['access'] = form[item['id'] + ACCESS_SUFFIX]
                dst_datalist.append(dst_item)
            except:
                continue

        return dst_datalist

    def view(self, request):
        if SHARES_SET['shares'] != None:
            init = SHARES_SET['shares']
        else:
            init = initsetting.get_shares_info()

        form = SharesForm()

        return {
            self.KEY_TEMPLATE: os.path.join(self.TEMPLATES, "shares.html"),
            self.KEY_TEMPLATE_DIC: {
                'item_list': self.__get_datalist(form, init),
            },
        }

    def view_confirm(self, request):
        form = SharesForm(data=request.POST)
        if form.is_valid():
            clean_data = form.cleaned_data
            item_list = []
            for share in initsetting.get_shares_info():
                item = {
                    'id': share['id'],
                    'name': share['name'],
                    'access': clean_data[share['id'] + ACCESS_SUFFIX],
                }
                item_list.append(item)

            SHARES_SET['shares'] = item_list
            return {
                self.KEY_REDIRECT: "/".join([settings.MENU_URL, "confirm"]),
            }

        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {"form": form},
        }

    def view_previous(self, request):
        return {self.KEY_REDIRECT: "/".join([settings.MENU_URL, "time"])}


class ConfirmView(FuncView):
    views = ['apply', 'previous']

    def __get_data(self):
        data = {}
        for dict in [NAME_SET, LAN_SET, TIME_SET, SHARES_SET]:
            data.update(dict)
        return data

    def view(self, request):
        if not LAN_SET.get("dhcp"):
            return {self.KEY_REDIRECT: '/admin/menu'}
        return {
            self.KEY_TEMPLATE: os.path.join(self.TEMPLATES, "set.html"),
            self.KEY_TEMPLATE_DIC: {'config': self.__get_data()}}

    def __get_code(self, msg):
        return initsetting.get_code(msg)

    def __apply(self, data):
        # sysinfo
        if not initsetting.sysinfo_set(data):
            logging(self.__get_code("ERR_LANDISK_NAME_CHANGE"))
            return False
        logging(self.__get_code("MSG_LANDISK_NAME_CHANGE"))

        # datetime
        if not initsetting.time_set(data):
            logging(self.__get_code("ERR_DATE_CHANGE"))
            return False
        logging(self.__get_code("MSG_DATE_CHANGE"))

        # shares
        if not initsetting.shares_set(data):
            return False

        # network
        if not initsetting.network_set(data):
            logging(self.__get_code("ERR_NET_CHANGE"))
            return False
        logging(self.__get_code("MSG_NET_CHANGE"))

        return True

    def view_apply(self, request):
        with self.processing(constname="ERR_ADMIN_STATE_CHANGE") as p:
            p.break_if_state_change()

            try:
                conf = self.__get_data()
                if self.__apply(conf):
                    self.set_message(
                        _(u'Setting up the network. ') + \
                        _(u'Please wait until STATUS lamp blinking stops.'))
                    p.result(True)
                else:
                    self.set_message(get_message('ERR_DATE_CHANGE'))
            except:
                self.set_message(get_message('ERR_DATE_CHANGE'))

        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {}}

    def view_previous(self, request):
        return {self.KEY_REDIRECT: "/".join([settings.MENU_URL, "shares"])}


nasname = NameView.as_view()
lan1ipv4 = LanView.as_view()
time = TimeView.as_view()
shares = SharesView.as_view()
confirm = ConfirmView.as_view()
