# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

import csv

from ionas.request import (Request, Entry)
from fal_info import FAL_LUMPREGIST_PATH, ReqTypes, Results
from product_conf import SUDO_CMD

from django.utils.translation import ugettext as _
from django.http import HttpResponseRedirect
from project.view import FuncView
from root.modules.form import BaseForm

from project.fal.command import get_message
from project.fal.buzzer import Buzzer
from project.fal.led import LED

from project.forms.fields.extras.file import FileField

from .. modules.status_view import StatusView


FAL_LUMPREGIST_REQ_BASE_XML = {ReqTypes.LUMP_REGIST: "<lumpregist/>"}

DEST_FILE_PATH = "/tmp/csv.temp"

MAX_CSV_FILE_SIZE = 20 * 1024 * 1024


def root(request):
    return HttpResponseRedirect('./csv_lumpregist')


class View(FuncView):
    class Form(BaseForm):
        csv_file = FileField(label=_(u'csv file'), required=True)

    views = ['apply']

    def __start(self):
        self.led.status(True, blink=True, color='green')

    def __end(self, result):
        self.led.status(True, blink=False, color='green')
        if result:
            self.buz.ok()
        else:
            self.buz.error()

    def __encoder(self, csv_file):
        for line in csv_file.readlines():
            try:
                yield line.decode("cp932").encode("utf-8")
            except UnicodeDecodeError:
                line.decode("utf-8")
                yield line

    def __is_csv(self, csv_file):
        try:
            reader = csv.reader(self.__encoder(csv_file))
            # COMMENT 不正ならば下記forでcsv.Error例外発生する
            for record in reader:
                pass
            return True
        except (csv.Error, UnicodeDecodeError):
            return False

    def __check_file_size(self, csv_file):
        if csv_file.size > MAX_CSV_FILE_SIZE:
            return False
        return True
 
    def __handle_uploaded_file(self, csv_file):
        dest = open(DEST_FILE_PATH, "wb")

        for chunk in csv_file.chunks():
            dest.write(chunk)
        dest.close()

    def __get_request_entry(self):
        entry = Entry(ReqTypes.LUMP_REGIST, FAL_LUMPREGIST_REQ_BASE_XML[ReqTypes.LUMP_REGIST])
        entry.set_file(DEST_FILE_PATH)
        return entry

    def __send_request(self, form):
        return Request(FAL_LUMPREGIST_PATH, filter_cmd=SUDO_CMD).request(
            self.__get_request_entry()).get_entry().get_result()

    def view(self, request):
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                'status': StatusView()._status(),
                'form': self.Form()}}

    def view_apply(self, request):
        form = self.Form(data=request.POST, files=request.FILES)

        result = False

        self.led = LED()
        self.buz = Buzzer()

        self.__start()

        try:
            if form.is_valid():
                if not self.__check_file_size(form.cleaned_data["csv_file"]):
                    self.set_message(get_message('ERR_LUMP_REG_FILE_SIZE_OVER'))
                elif not self.__is_csv(form.cleaned_data["csv_file"]):
                    self.set_message(get_message('ERR_LUMP_REG_INVALID_FILE'))
                else:
                    self.__handle_uploaded_file(form.cleaned_data["csv_file"])
                    req_result = self.__send_request(form)

                    if req_result == Results.BUSY_ERROR:
                        self.set_message(get_message('ERR_LUMP_REG_STATE_CHANGE', is_log=True))
                    else:
                        self.set_message(get_message('MSG_LUMP_REG_START', is_log=True))
                        result = True
        finally:
            self.__end(result)

        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                'status': StatusView()._status(),
                'form': form}}

csv_lumpregist = View.as_view()
status = StatusView.as_view()
