# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django.http import HttpResponseRedirect
from project.view import FuncView
from project.fal.command import get_message
import settings
from .. import settings as ps
from ...modules.unc import LocalListView, NetworkListView

MAX_JOB_NUM = 7


def root(request):
    return HttpResponseRedirect('/'.join([settings.MENU_URL, 'add']))


class View(FuncView):
    from ..modules.form import Form
    from ..modules.backup import Backup
    views = ['add', 'list']

    def is_diff(self, x):
        try:
            from project.fal.volctrl import Volctrl
            obj = Volctrl()
            return obj.query(volume=obj.VOLUME['usb'][x])['filesystem'].lower() == 'ext4'
        except:
            return False

    def view(self, request):
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                'usbx_is_diff_backup': dict([(x, self.is_diff(x)) for x in range(1, 5)]),
                'form': self.Form(),
            },
        }

    def view_add(self, request):
        with self.processing(constname="ERR_BACKUP_STATE_CHANGE") as p:
            form = self.Form(data=request.POST)
            rst = False
            p.break_if_state_change()

            if form.is_valid():
                clean_data = form.cleaned_data
                backup = self.Backup()
                joblist = backup.get_job_list()
                if len(joblist) < MAX_JOB_NUM:
                    if backup.add_job(clean_data):
                        self.set_message(get_message('MSG_BACKUP_ADD',
                            arglist={"name": clean_data.get("name")},
                            is_log=True))
                        rst = True
                    else:
                        self.set_message(get_message('ERR_BACKUP_ADD',
                            arglist={"name": clean_data.get("name")},
                            is_log=True))
                else:
                    self.set_message(get_message('ERR_BACKUP_JOB_NO_MORE',
                        arglist={"name": clean_data.get("name")}, is_log=True))
                p.result(rst)

        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                'usbx_is_diff_backup': dict([(x, self.is_diff(x)) for x in range(1, 5)]),
                'form': form,
            },
        }

    def view_list(self, request):
        return {
            self.KEY_REDIRECT: "/".join(["..", ps.LIST_APPLI, ps.LIST_VIEW])}


add = View.as_view()
locallist = LocalListView.as_view()
networklist = NetworkListView.as_view()
