# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django.http import HttpResponseRedirect
from django import forms
from project.view import FuncView
from project.fal.command import get_message
from project.fal.buzzer import Buzzer
from project.fal.led import LED
import settings
from .. import settings as ps
from ... modules.status_view import StatusView
from fal_info import Results


def root(request):
    return HttpResponseRedirect('/'.join([settings.MENU_URL, 'execute']))


class Form(forms.Form):
    pass

class View(FuncView):
    from ..modules.backup import Backup
    views = ['execute', 'stop', 'list']
    backup = Backup()

    def __start(self):
        self.led.status(True, blink=True, color='green')

    def __end(self, result):
        self.led.status(True, blink=False, color='green')
        if result:
            self.buz.ok()
        else:
            self.buz.error()

    def view(self, request, id):
        form = Form()
        info = self.backup.delete_or_execute_init_info(id)
        if not info:
            return {self.KEY_REDIRECT:
                    "/".join([ps.MENU_URL, ps.LIST_APPLI, ps.LIST_VIEW])}

        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                'form': form,
                'status': StatusView()._status(id=id),
                'info': info,
            },
        }

    def view_execute(self, request, id):
        form = Form()

        result = False

        self.led = LED()
        self.buz = Buzzer()

        self.__start()

        try:
            req_result = self.backup.execute_job(id)
            if req_result == Results.OK:
                self.set_message(get_message(
                    'MSG_BACKUP_START', is_log=True))
                result = True
            elif req_result == Results.QUEUE:
                job_name = self.backup.id_to_jobinfo(id)["name"]
                self.set_message(get_message(
                    'MSG_BACKUP_QUEUE', is_log=True,
                    arglist={"name": job_name}))
                result = True
            elif req_result == Results.BUSY_ERROR:
                self.set_message(get_message(
                    'ERR_BACKUP_STATE_CHANGE', is_log=True))
            else:
                self.set_message(get_message(
                    'ERR_BACKUP_OPERATION', is_log=True))
        finally:
            self.__end(result)
 
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                'form': form,
                'status': StatusView()._status(id=id)}}

    def view_stop(self, request, id):
        from project.fal.led import LED
        from project.fal.buzzer import Buzzer
        form = Form(data=request.POST)
        result = False
        if form.is_valid():
            from project.fal.backups import Backups as FalBackup
            backup = FalBackup()
            result = backup.stop(id)
            self.set_message(backup.get_message())
        if result:
            Buzzer().ok()
        else:
            Buzzer().error()
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                'form': form,
                'status': StatusView()._status(id=id),
            },
        }

    def view_list(self, request, id):
        return {self.KEY_REDIRECT:
                "/".join([ps.MENU_URL, ps.LIST_APPLI, ps.LIST_VIEW])}


class StatusView(StatusView):
    CATEGORY = "backup"


execute = View.as_view()
status = StatusView.as_view()
