# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
import re
import os
from ionas.request import Request
from django.utils.translation import ugettext as _
from django.http import HttpResponseRedirect
from project.view import FuncView
from project.fal.command import get_message
from root.modules.form import BaseForm
import settings
from fal_info import Results
from .. import settings as ps
from ..modules.backup import (Backup, FAL_BACKUPS_PATH)
from product_conf import SUDO_CMD

NAME_XPATH = "backup/job/name"
#FIELD_SUFFIX = ".field"
FIELD_SUFFIX = "-field"
JOB_LIST_KEY = "job_list"
#JOB_ID_RE = r"\.field$"
JOB_ID_RE = r"\-field$"


def _unicode(string):
    if isinstance(string, str):
        return unicode(string, "utf-8")
    return string

def root(request):
    return HttpResponseRedirect('/'.join([settings.MENU_URL, 'list']))


class Form(BaseForm):

    def __init__(self, *args, **kwargs):
        from root.modules.form.fields.checkbox import CheckboxField
        super(Form, self).__init__(*args, **kwargs)

        backup = Backup()
        for job in backup.get_job_list():
            job_id = job.get_node("id").get_text()
            self.fields[job_id + FIELD_SUFFIX] = CheckboxField(initial=False)


class ListView(FuncView):

    views = ['add', "delete_multi", "start", "stop", "enable", "disable"]
    backup = Backup()

    def __get_id_list(self):
        id_list = []
        for job in self.backup.get_job_list():
            job_id = job.get_node("id").get_text()
            id_list.append(job_id)
        return tuple(id_list)

    def __get_jobform_list(self, form):
        datalist = self.backup.get_jobinfo_list()
        for job in datalist:
            job["checkbox"] = form[job["id"] + FIELD_SUFFIX]
        return datalist

    def __get_id_queue(self, cleaned_data):
        id_list = self.__get_id_list()
        idx_list = []
        for name, value in cleaned_data.items():
            if value:
                id = re.sub(JOB_ID_RE, "", name)
                idx_list.append(id_list.index(id))
        idx_list.sort()
        del_list = []
        for idx in idx_list:
            del_list.append(id_list[idx])
        return tuple(del_list)

    def view(self, request):
        job_list = self.__get_jobform_list(Form())
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                'job_num': len(job_list),
                'job_list': job_list,
            },
        }

    def view_add(self, request):
        return {self.KEY_REDIRECT:
                "/".join([ps.MENU_URL, ps.ADD_APPLI, ps.ADD_VIEW])}

    def view_delete_multi(self, request):
        form = Form(data=request.POST)
        if form.is_valid():
            self.session[JOB_LIST_KEY] = self.__get_id_queue(form.cleaned_data)
            if self.session[JOB_LIST_KEY]:
                return {self.KEY_REDIRECT: "/".join([settings.MENU_URL,
                                                  ps.DELETE_MULTI_VIEW])}
        self.set_message(_(u'There is no job selected.'))
        #return {self.KEY_TEMPLATE: self.TEMPLATE,
        #        self.KEY_TEMPLATE_DIC: {}}

        joblist = self.__get_jobform_list(Form())
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                'job_num': len(joblist),
                'job_list': joblist,
            },
        }

    def view_start(self, request):
        from project.fal.led import LED
        from project.fal.buzzer import Buzzer
        LED().status(True, blink=True, color='green')

        form = Form(data=request.POST)
        if form.is_valid():
            from project.fal.backups import Backups as FalBackup
            backup = FalBackup()
            info = backup.load().get('backup', {})
            result = True
            jids = self.__get_id_queue(form.cleaned_data)
            for jid in jids:
                if not backup.execute(jid):
                    result = False
                    self.set_message("%s: %s" % (
                            info.get(jid, {}).get('name', "-"),
                            backup.get_message()))
            if len(jids) and result:
                self.set_message(backup.get_message())

        LED().status(True, blink=False, color='green')
        if result:
            Buzzer().ok()
            self.message_is_error = False
        else:
            Buzzer().error()
            self.message_is_error = True
        job_list = self.__get_jobform_list(Form())
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                'job_num': len(job_list),
                'job_list': job_list,
            },
        }

    def view_stop(self, request):
        from project.fal.led import LED
        from project.fal.buzzer import Buzzer
        LED().status(True, blink=True, color='green')

        form = Form(data=request.POST)
        if form.is_valid():
            from project.fal.backups import Backups as FalBackup
            backup = FalBackup()
            info = backup.load().get('backup', {})
            result = True
            jids = self.__get_id_queue(form.cleaned_data)
            for jid in jids:
                for function in ("backup", "restore"):
                    if backup.is_running(jid, function=function):
                        if not backup.stop(jid, function=function):
                            self.set_message("%s: %s" % (info.get(jid, {}).get("name", "-"), backup.get_message()))
                        break
                else:
                    result = False
                    self.set_message(_("BACKUP: Functions are not running."))
                #if not backup.is_running(jid):
                #    result = False
                #    self.set_message("%s: %s" % (
                #            info.get(jid, {}).get('name', "-"),
                #            _(u"Backup already stop.")))
                #elif not backup.stop(jid):
                #    result = False
                #    self.set_message("%s: %s" % (
                #            info.get(jid, {}).get('name', "-"),
                #            backup.get_message()))
            #if len(jids) and result:
            if len(jids):
                if result:
                    self.set_message(backup.get_message())
            else:
                self.set_message(_(u'There is no job selected.'))

        LED().status(True, blink=False, color='green')
        if result:
            Buzzer().ok()
            self.message_is_error = False
        else:
            Buzzer().error()
            self.message_is_error = True
        job_list = self.__get_jobform_list(Form())
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                'job_num': len(job_list),
                'job_list': job_list,
            },
        }

    def _view_enable_or_disable(self, request, enabled):

        with self.processing() as proc:
            proc.break_if_state_change()
            form = Form(data=request.POST)
            if form.is_valid():
                from project.fal.backups import Backups
                backup = Backups()
                loaded_data = backup.load().get('backup', {})
                #result = True
                proc.result(True)
                jids = self.__get_id_queue(form.cleaned_data)
                for jobid in jids:
                    jobname = _unicode(loaded_data.get(jobid, {}).get('name', u"-"))
                    #result = backup.schedule_enable(jobid, enabled)
                    proc.result(backup.schedule_enable(jobid, enabled))
                    #if result:
                    if proc.result():
                        message = (_(u"It was enable the schedule backup function of the job %(jobname)s.") \
                                   if enabled else \
                                   _(u"It was disable the schedule backup function of the job %(jobname)s.")) % {
                            u"jobname": jobname,
                        }
                    else:
                        message = (_(u"Failed to enable the schedule backup function of the job %(jobname)s.") \
                                   if enabled else \
                                   _(u"Failed to disable the schedule backup function of the job %(jobname)s.")) % {
                            u"jobname": jobname,
                        }
                    self.set_message(message)
            #else:
            #    self.set_message(_(u'There is no job selected.'))

        joblist = self.__get_jobform_list(Form())
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                'job_num': len(joblist),
                'job_list': joblist,
            },
        }

    def view_enable(self, request):
        return self._view_enable_or_disable(request, True)

    def view_disable(self, request):
        return self._view_enable_or_disable(request, False)


class DeleteMultiView(FuncView):
    #一括削除
    views = ['delete', 'list']
    backup = Backup()

    def __get_entrylist(self):
        entrylist = self.backup.multi_del_request_entry_list(
                    self.session.get(JOB_LIST_KEY))
        return entrylist

    def __delete(self):
        try:
            response = Request(FAL_BACKUPS_PATH, filter_cmd=SUDO_CMD
                    ).request(self.__get_entrylist())
        except:
            return False
        faillist = ""
        oklist = ""
        for entry in response.get_entrylist().get_entry_list():
            name = entry.get().get_node(NAME_XPATH).get_text(do_strip=False)
            if entry.get_result() == Results.OK:
                if oklist == "":
                    oklist += name
                else:
                    oklist += "," + name
            else:
                if faillist == "":
                    faillist += name
                else:
                    faillist += "," + name
        if faillist != "":
            self.set_message(get_message(
                'ERR_BACKUP_DELETE', arglist={'name': faillist}, is_log=True))
            return False
        if oklist != "":
            self.set_message(get_message(
                'MSG_BACKUP_DELETE', arglist={"name": oklist}, is_log=True))
        return True

    def view(self, request):
        name_list = []
        if not self.session.get(JOB_LIST_KEY):
            return {
                self.KEY_REDIRECT: "/".join([settings.MENU_URL, ps.LIST_VIEW])
            }
        for id in self.session.get(JOB_LIST_KEY):
            target = self.backup.get_target_job_node(id=id)
            name_list.append(target.get_node("name").get_text(False))
        template = os.path.join(self.TEMPLATES, "delete_multi.html")
        return {self.KEY_TEMPLATE: template,
                self.KEY_TEMPLATE_DIC: {'job_list': name_list}}

    def view_delete(self, request):
        with self.processing(constname="ERR_BACKUP_STATE_CHANGE") as p:
            form = Form(data=request.POST)
            rst = False
            p.break_if_state_change()

            if form.is_valid():
                rst = self.__delete()
                p.result(rst)

        self.session.clear()
        return {self.KEY_TEMPLATE: self.TEMPLATE,
                self.KEY_TEMPLATE_DIC: {}}

    def view_list(self, request):
        self.session.clear()
        return {self.KEY_REDIRECT: "/".join([settings.MENU_URL, ps.LIST_VIEW])}


list = ListView.as_view()
delete_multi = DeleteMultiView.as_view()
