# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django.http import HttpResponseRedirect
from project.fal.command import get_message
from project.view import FuncView
import settings
from ..modules.form import Form as BackupSettingForm
from ...modules.unc import LocalListView, NetworkListView

from .. import settings as ps


def root(request):
    return HttpResponseRedirect('/'.join([settings.MENU_URL, 'modify']))


class Form(BackupSettingForm):
    def __init__(self, id=None, *args, **kwargs):
        super(Form, self).__init__(id, *args, **kwargs)


class View(FuncView):
    from ..modules.backup import Backup
    views = ['modify', 'list']
    backup = Backup()

    def is_diff(self, x):
        try:
            from project.fal.volctrl import Volctrl
            obj = Volctrl()
            return obj.query(volume=obj.VOLUME['usb'][x])['filesystem'].lower() == 'ext4'
        except:
            return False

    def view(self, request, id):
        init_info = self.backup.modify_init_basic_info(id)
        source_list = self.backup.modify_init_sourcelist_info(id)
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                'usbx_is_diff_backup': dict([(x, self.is_diff(x)) for x in range(1, 5)]),
                'form': Form(initial=init_info),
                'source_list': source_list,
            },
        }

    def view_modify(self, request, id):
        with self.processing(constname="ERR_BACKUP_STATE_CHANGE") as p:
            form = Form(data=request.POST, id=id)
            rst = False
            p.break_if_state_change()

            if form.is_valid():
                data = form.cleaned_data
                init_info = self.backup.modify_init_basic_info(id)
                if init_info.get("name"):
                    name = init_info.get("name")
                else:
                    name = data.get("name")
                if self.backup.modify_job(data, id):
                    self.set_message(get_message('MSG_BACKUP_MODIFY',
                        arglist={"name": name}, is_log=True))
                    rst = True
                else:
                    self.set_message(get_message('ERR_BACKUP_MODIFY',
                        arglist={"name": name}, is_log=True))
                p.result(rst)
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                'usbx_is_diff_backup': dict([(x, self.is_diff(x)) for x in range(1, 5)]),
                'form': form,
            },
        }

    def view_list(self, request, id):
        return {
            self.KEY_REDIRECT:
            "/".join(["..", "..", ps.LIST_APPLI, ps.LIST_VIEW])}


modify = View.as_view()
locallist = LocalListView.as_view()
networklist = NetworkListView.as_view()
