# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
import datetime
from django.utils.translation import ugettext as _
from root.modules.form import BaseForm
from django.core.exceptions import ValidationError
from project.forms.fields.extras.backup import BackupHistoryNumberField


class Form(BaseForm):
    def __init__(self, id=None, *args, **kwargs):
        from root.modules.form.fields.validators.fal import FalValidator as fv
        from root.modules.form.fields.text import TextField
        from root.modules.form.fields.checkbox import CheckboxField
        from root.modules.form.fields.multicheckbox import MultiCheckboxField
        from root.modules.form.fields.radio import RadioField
        from root.modules.form.fields.select import SelectField
        from root.modules.form.fields.password import PasswordField
        from root.modules.form.fields.extras.selectdatetime \
            import SelectTimeField
        from root.modules.form.fields.hidden import HiddenField
        from project.forms.fields.extras.backupjobname \
            import BackupJobNameField, BackupJobNameValidator
        from project.forms.fields.extras.backupunc import BackupUncField
        from project.forms.fields.extras.backupuser import BackupUserField
        from project.forms.fields.extras.backuppass import BackupPassField
        #from project.forms.fields.extras.integer import IntegerField

        super(Form, self).__init__(*args, **kwargs)
        self.fields["sourcelist"] = HiddenField(
                label=None,
                initial='',
                required=False,
                validators=[])

        if id:
            # 変更時。自id以外のjob名との重複を禁則
            self.fields["name"] = BackupJobNameField(
                    label=_(u'Job Name'),
                    required=True,
                    validators=[BackupJobNameValidator(options={"exempt_id": id})])
        else:
            # 追加時。
            self.fields["name"] = BackupJobNameField(
                    label=_(u'Job Name'),
                    required=True,
                    validators=[BackupJobNameValidator()])

        #self.fields["times"] = IntegerField(
        #        label=_(u'Times'),
        #        max_value=31,
        #        min_value=0,
        #        initial=0,
        #        required=True)
        self.fields["times"] = BackupHistoryNumberField()
        self.fields["schedule"] = RadioField(
                label=_(u'Schedule'),
                initial=0,
                required=False,   #FIXME
                validators=[])
        self.fields["day"] = MultiCheckboxField(
                label=_(u'Day'),
                choices=((0, _(u'Sunday')), (1, _(u'Monday')),
                         (2, _(u'Tuesday')), (3, _(u'Wednesday')),
                         (4, _(u'Thursday')), (5, _(u'Friday')),
                         (6, _(u'Saturday'))),
                initial=[],
                required=False,
                validators=[])
        self.fields["time"] = SelectTimeField(
                label=_(u'Time'),
                initial=datetime.time(0, 0),
                required=False,
                validators=[])
        self.fields["shutdown"] = CheckboxField(
                label=_(u'Shutdown after backup'),
                initial=False,
                required=False,
                validators=[])
        self.fields["all_local"] = CheckboxField(
                label=_(u'All Local Folder(Excluding usb disks)'),
                initial=False,
                required=False,
                validators=[])
        self.fields["trash"] = CheckboxField(
                label=_(u'Trash Backup'),
                initial=False,
                required=False,
                validators=[])
        self.fields['dest'] = BackupUncField(
            label=_(u'Backup Destination'),
            required=True,
            validators=[])
        self.fields["destuser"] = BackupUserField(
            label=_(u'UserName'),
            required=False)
        self.fields["destpswd"] = BackupPassField(
            label=_(u'Password'),
            required=False)
        self.fields['source'] = BackupUncField(
            label=_(u'Backup Source'),
            required=False,
            validators=[])
        self.fields["sourceuser"] = BackupUserField(
            label=_(u'UserName'),
            required=False)
        self.fields["sourcepswd"] = BackupPassField(
            label=_(u'Password'),
            required=False)
        self.fields["full"] = CheckboxField(
            label=_(u'Force Full Backup'),
            initial=False,
            required=False)

    def clean(self):
        from ..modules.srclist import SourceList
        sourcelist = SourceList()
        if sourcelist.max_src_num_exceeded(self.cleaned_data.get("sourcelist"),
                self.cleaned_data.get("all_local")):
            raise ValidationError(_(u'Total number of backup sources exceeded 300.'))
        if self.cleaned_data.get("sourcelist") == "" and \
                not self.cleaned_data.get("all_local"):
            raise ValidationError(_(u'Backup source is required.'))
        else:
            if sourcelist.dest_is_in_sroucelist(self.cleaned_data.get("dest"),
                    self.cleaned_data.get("sourcelist"),
                    self.cleaned_data.get("all_local")):
                raise ValidationError(_(u'Backup source and destination can not be the same share folder.'))
        if self.cleaned_data.get("schedule") == "1":
            if len(self.cleaned_data.get("day")) <= 0:
                raise ValidationError(_(u'Backup schedule is required.'))
        return self.cleaned_data
