# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

import re
from ionas.request import (Request, Entry)
from product_conf import SUDO_CMD
from root.admin.initset.modules.initsetting import InitSetting
from root.admin.network.modules.network import Network
from fal_info import (FAL_SHARES_PATH, FAL_PORTS_PATH, ReqTypes, Results)

UNC_RE = r"^\\\\(.+)\\(.+)$"
UNC_FMT = "\\\\%s\\%s"
SHARE_XPATH = "./sharelist/share"
USB_XPATH = "./usblist/usb"
MAX_SRC_NUM = 300


class SourceList(object):

    def locallist(self):
        share_list = []
        shares_data = Request(FAL_SHARES_PATH, filter_cmd=SUDO_CMD).request(
                  Entry(ReqTypes.LOAD)).get_entrylist().get_entry()
        for node in shares_data.get().get_node_list(SHARE_XPATH):
            if node.get_node("shareservice/msshare").is_enabled():
                share_name = node.get_node("name").get_text()
                usb_match = re.match("^usb\d$", share_name)
                if not usb_match:
                    share_list.append(share_name)

        share_list += self.get_usb_share_list()
        return share_list

    def get_usb_share_list(self):
        usb_share_list = []
        port_conf = Request(FAL_PORTS_PATH, filter_cmd=SUDO_CMD).request(
                  Entry(ReqTypes.LOAD)).get_entrylist().get_entry()
        for port in port_conf.get().get_node_list(USB_XPATH):
            port_num = port.get_attr_value("port")
            mode = port.get_node("modelist/mode[@enable='true']").get_text()
            if mode == "share":
                usb_share_list.append('usb' + port_num)
        return usb_share_list

    def max_src_num_exceeded(self, unclist, all_local=False):
        src_list = []
        for info in re.split(";", unclist):
            if info == "":
                continue
            share_unc = re.split(",", info)[0]
            src_list.append(share_unc)
        if all_local:
            src_len = len(self.locallist() + src_list)
        else:
            src_len = len(src_list)
        if src_len > MAX_SRC_NUM:
            return True
        else:
            return False

    def __get_self_names(self):
        self_names = ["LOCALHOST", "127.0.0.1"]

        init = InitSetting()
        hostinfo = init.get_landisk_name()
        hostname = hostinfo.get("name")
        if hostname:
            self_names.append(hostname.upper())

        network = Network()
        lan1ipv4 = network.get_ipv4_info(netif="lan1").get("ipaddr")
        if lan1ipv4:
            self_names.append(lan1ipv4)
        lan1ipv6 = network.get_ipv6_info(netif="lan1").get("ipaddr")
        if lan1ipv6:
            self_names.append(lan1ipv6)
        try:
            lan2ipv4 = network.get_ipv4_info(netif="lan2").get("ipaddr")
            if lan2ipv4:
                self_names.append(lan2ipv4)
        except:
            pass
        try:
            lan2ipv6 = network.get_ipv6_info(netif="lan2").get("ipaddr")
            if lan2ipv6:
                self_names.append(lan2ipv6)
        except:
            pass

        return self_names

    def __unc_to_host_share(self, unc):
        match = re.match(UNC_RE, unc)
        if match:
            host = match.group(1).upper()
            share = match.group(2)
        else:
            # UNC形式でなければローカル共有名として処理
            host = "LOCALHOST"
            share = unc
        return (host, share)

    def __get_src_list(self, src_str):
        src_list = []
        for info in re.split("\b", src_str):
            if info == "":
                continue
            src = re.split("\t", info)[0]
            src = UNC_FMT % self.__unc_to_host_share(src)
            src_list.append(src)
        return src_list

    def dest_is_in_sroucelist(self, dest, sourcelist, all_local=False):
        src_list = self.__get_src_list(sourcelist)
        self_names = self.__get_self_names()
        dest_host, dest_share = self.__unc_to_host_share(dest)
        if (u"\\\\%s\\%s" % (dest_host, dest_share)).upper() \
                in [src.upper() for src in src_list]:
            return True
        if dest_host in self_names:
            if all_local:
                try:
                    from product_conf import STOR_IF_TO_VOL
                    if dest_share not in STOR_IF_TO_VOL['usb'].itervalues():
                        return True
                except:
                    return True
            else:
                for host in self_names:
                    if host == dest_host:
                        continue
                    for name in [host, host.lower()]:
                        for share in [dest_share.lower(), dest_share.upper()]:
                            unc_str = r"\\%s\%s" % (name, share)
                            if unc_str in src_list:
                                return True
            
        return False
