# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

import re
from root.admin.initset.modules.initsetting import InitSetting
from root.admin.network.modules.network import Network

UNC_RE = r"^\\\\(.+)\\(.+)$"
UNC_FMT = "\\\\%s\\%s"


class Cleaned(object):

    def __get_self_names(self):
        self_names = ["LOCALHOST", "127.0.0.1"]

        init = InitSetting()
        hostinfo = init.get_landisk_name()
        hostname = hostinfo.get("name")
        if hostname:
            self_names.append(hostname.upper())

        network = Network()
        lan1ipv4 = network.get_ipv4_info(netif="lan1").get("ipaddr")
        if lan1ipv4:
            self_names.append(lan1ipv4)
        lan1ipv6 = network.get_ipv6_info(netif="lan1").get("ipaddr")
        if lan1ipv6:
            self_names.append(lan1ipv6)
        try:
            lan2ipv4 = network.get_ipv4_info(netif="lan2").get("ipaddr")
            if lan2ipv4:
                self_names.append(lan2ipv4)
        except:
            pass
        try:
            lan2ipv6 = network.get_ipv6_info(netif="lan2").get("ipaddr")
            if lan2ipv6:
                self_names.append(lan2ipv6)
        except:
            pass

        return self_names

    def __unc_to_host_share(self, unc):
        match = re.match(UNC_RE, unc)
        if match:
            host = match.group(1).upper()
            share = match.group(2)
        else:
            # UNC形式でなければローカル共有名として処理
            host = "LOCALHOST"
            share = unc
        return (host, share)

    def dest_is_same_with_src(self, source, dest):
        self_names = self.__get_self_names()
        dest_host, dest_share = self.__unc_to_host_share(dest)
        src_unc = UNC_FMT % self.__unc_to_host_share(source)
        for host in [dest_host, dest_host.lower()]:
            for share in [dest_share.lower(), dest_share.upper()]:
                unc_str = r"\\%s\%s" % (host, share)
                if unc_str == src_unc:
                    return True
        if dest_host in self_names:
            for host in self_names:
                if host == dest_host:
                    continue
                for name in [host, host.lower()]:
                    for share in [dest_share.lower(), dest_share.upper()]:
                        unc_str = r"\\%s\%s" % (name, share)
                        if unc_str == src_unc:
                            return True
            
        return False
