# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
import datetime
import string
from iobase.xmldata import XmlNode, escape
from iobase.exception import NotFoundError
from ionas.request import (Request, Entry, EntryList)
from product_conf import SUDO_CMD
from fal_info import (FAL_BACKUPS_PATH, ReqTypes, Results)


#ADD_OR_MOD_ENTRY = '<backups><datacopy><job><name>%(name)s</name><id/><schedule/>' + \
#            '<trash enable="%(trash)s"/><shutdown enable="%(shutdown)s"/><source>' + \
#            '<unc>%(sourceunc)s</unc><user>%(sourceuser)s</user><passwd>' + \
#            '%(sourcepswd)s</passwd></source><destination><unc>%(destunc)s' + \
#            '</unc><user>%(destuser)s</user><passwd>%(destpswd)s' + \
#            '</passwd></destination></job></datacopy></backups>'
ADD_OR_MOD_ENTRY = """<backups>
    <datacopy>
        <job diff="%(diff)s" sync="%(sync)s">
            <name>%(name)s</name>
            <id/>
            <schedule/>
            <trash enable="%(trash)s"/>
            <shutdown enable="%(shutdown)s"/>
            <source>
                <unc>%(sourceunc)s</unc>
                <user>%(sourceuser)s</user>
                <passwd>%(sourcepswd)s</passwd>
            </source>
            <destination>
                <unc>%(destunc)s</unc>
                <user>%(destuser)s</user>
                <passwd>%(destpswd)s</passwd>
            </destination>
        </job>
    </datacopy>
</backups>"""
DEL_ENTRY = "<backups><datacopy><job><name/><id/></job></datacopy></backups>"
EXEC_ENTRY = "<backups><datacopy/></backups>"

JOB_XPATH = "datacopy/joblist/job"
NAME_XPATH = "datacopy/job/name"
ID_XPATH = "datacopy/job/id"
SCHEDULE_XPATH = "datacopy/job/schedule"

XPATH_JOB_BYID = 'datacopy/joblist/job[id="%s"]'
XPATH_JOB_BYNAME = 'datacopy/joblist/job[name="%s"]'
TRUE = "true"


class Datacp(object):

    def send_request(self, req_entry, bool=True):
        try:
            response = Request(FAL_BACKUPS_PATH, filter_cmd=SUDO_CMD).request(req_entry)
            res_entry = response.get_entrylist().get_entry()
            result = res_entry.get_result()
            if bool:
                if result == Results.OK:
                    return True
                else:
                    return False
            else:
                return result
        except:
            return False

    def add_job(self, data):
        entry_str = ADD_OR_MOD_ENTRY % {
                "name": data["name"],
                "trash": str(data["trash"]).lower(),
                "shutdown": str(data["shutdown"]).lower(),
                "sourceunc": escape(data["source"]),
                "sourceuser": escape(data["sourceuser"]),
                "sourcepswd": escape(data["sourcepswd"]),
                "destunc": escape(data["dest"]),
                "destuser": escape(data["destuser"]),
                "destpswd": escape(data["destpswd"]),
                "diff": str(not data["full"]).lower(),
                "sync": str(data["sync"]).lower(),
                }
        entry = Entry(ReqTypes.ADD, entry_str)
        sche_node = entry.get().get_node(SCHEDULE_XPATH)
        self.make_schedule_node(sche_node, data)
        return self.send_request(entry)

    def make_schedule_node(self, sche_node, data):
        enable_dic = {"1": "true", "0": "false"}
        sche_node.set_attrs({"enable": enable_dic[data["schedule"]]})
        if data["schedule"] == "1":
            day_dic = {"0": "Sun", "1": "Mon", "2": "Tue", "3": "Wed",
                       "4": "Thu", "5": "Fri", "6": "Sat"}
            for day in data["day"]:
                sche_node.append_child("day", day_dic[day])
            time_node = XmlNode("time")
            time_node.append_child("hour", str(data["time"].hour))
            time_node.append_child("minute", str(data["time"].minute))
            sche_node.append_child(time_node)
        return

    def delete_job(self, id):
        return self.send_request(self.delete_entry(id))

    def execute_job(self, id):
        entry = Entry(ReqTypes.EXECUTE, EXEC_ENTRY)
        target_job = self.get_target_job_node(id=id)
        if not target_job:
            return False
        entry.get().get_node("datacopy").append_child(target_job)
        return self.send_request(entry, False)

    def delete_entry(self, id):
        entry = Entry(ReqTypes.DELETE, DEL_ENTRY)
        target_job = self.get_target_job_node(id=id)
        if not target_job:
            return entry
        entry.get().get_node(ID_XPATH).set_text(id)
        name = target_job.get_node("name").get_text(False)
        entry.get().get_node(NAME_XPATH).set_text(name)
        return entry

    def multi_del_request_entry_list(self, idlist=[]):
        entry_list = EntryList()
        if not idlist:
            return entry_list
        for id in idlist:
            entry_list.append_entry(self.delete_entry(id))
        return entry_list

    def modify_job(self, data, id):
        entry_str = ADD_OR_MOD_ENTRY % {
                "name": data["name"],
                "trash": str(data["trash"]).lower(),
                "shutdown": str(data["shutdown"]).lower(),
                "sourceunc": escape(data["source"]),
                "sourceuser": escape(data["sourceuser"]),
                "sourcepswd": escape(data["sourcepswd"]),
                "destunc": escape(data["dest"]),
                "destuser": escape(data["destuser"]),
                "destpswd": escape(data["destpswd"]),
                "diff": str(not data["full"]).lower(),
                "sync": str(data["sync"]).lower(),
                }
        entry = Entry(ReqTypes.MODIFY, entry_str)
        entry.get().get_node(ID_XPATH).set_text(id)
        sche_node = entry.get().get_node(SCHEDULE_XPATH)
        self.make_schedule_node(sche_node, data)
        return self.send_request(entry)

    def get_job_list(self):
        try:
            result = Request(FAL_BACKUPS_PATH, filter_cmd=SUDO_CMD).request(
                    Entry(ReqTypes.LOAD)).get_entrylist().get_entry()
            job_list = result.get().get_node_list(JOB_XPATH)
        except NotFoundError:
            job_list = []
        return job_list

    def delete_or_execute_init_info(self, id):
        info_dic = {"id": id, "name": "", "dest": "", "source": ""}
        target_job = self.get_target_job_node(id=id)
        if not target_job:
            return None
        try:
            info_dic["name"] = target_job.get_node("name").get_text(False)
            info_dic["dest"] = target_job.get_node("destination/unc").get_text(False)
            info_dic["source"] = target_job.get_node("source/unc").get_text(False)
        except:
            pass
        return info_dic

    def get_jobinfo_list(self):
        #各ジョブの情報を取得
        job_list = []
        for job in self.get_job_list():
            info_dic = {
                "id": "",
                "name": "",
                "schedule": {},
            }
            info_dic["id"] = job.get_node("id").get_text()
            info_dic["name"] = job.get_node("name").get_text(False)
            try:
                schedule = job.get_node("schedule")
                info_dic["schedule"]["enabled"] = schedule.is_enabled()
                if schedule.get_attr_value("enable") == TRUE:
                    day_list = []
                    for node in schedule.get_node_list("day"):
                        day_list.append(node.get_text()) 
                    info_dic["schedule"]["day"] = day_list
                    hour = schedule.get_node("time").get_node("hour").get_text()
                    if len(hour) == 1:
                        hour = "0" + hour
                    minu = schedule.get_node(
                            "time").get_node("minute").get_text()
                    if len(minu) == 1:
                        minu = "0" + minu
                    info_dic["schedule"]["time"] = hour + ":" + minu
            except NotFoundError:
                pass

            job_list.append(info_dic)
        return job_list

    def get_target_job_node(self, name="", id=""):
        try:
            result = Request(FAL_BACKUPS_PATH, filter_cmd=SUDO_CMD).request(
                    Entry(ReqTypes.LOAD)).get_entrylist().get_entry()
            if id:
                target_job = result.get().get_node(XPATH_JOB_BYID % id)
            elif name:
                target_job = result.get().get_node(XPATH_JOB_BYNAME % name)
            else:
                return None
            return target_job
        except:
            return None

    def modify_init_info(self, id):
        info_dic = {
            "name": "",
            "trash": False,
            "shutdown": False,
            "schedule": 0,
            "day": [],
            "time": datetime.time(12, 0),
            "source": "",
            "sourceuser": "",
            "sourcepswd": "",
            "dest": "",
            "destuser": "",
            "destpswd": "",
            "full": False,
            "sync": False,
        }
        target_job = self.get_target_job_node(id=id)
        if not target_job:
            return info_dic
        try:
            info_dic["name"] = target_job.get_node("name").get_text(False)
        except:
            pass
        try:
            for item in ("trash", "shutdown"):
                if target_job.get_node(item).get_attr_value("enable") == "true":
                    info_dic[item] = True
        except:
            pass
        try:
            info_dic["full"] = target_job.get_attr_value("diff") == "false"
            info_dic["sync"] = target_job.get_attr_value("sync") == "true"
        except:
            pass
        try:
            sche_node = target_job.get_node("schedule")
            if sche_node.get_attr_value("enable") == "true":
                day_dic = {"Sun": 0, "Mon": 1, "Tue": 2, "Wed": 3, "Thu": 4,
                       "Fri": 5, "Sat": 6}
                info_dic["schedule"] = 1
                for node in sche_node.get_node_list("day"):
                    info_dic["day"].append(day_dic[node.get_text()])
                hour = sche_node.get_node("time").get_node("hour").get_text()
                minu = sche_node.get_node("time").get_node("minute").get_text()
                info_dic["time"] = datetime.time(string.atoi(hour),
                                             string.atoi(minu))
        except:
            pass
        try:
            source_node = target_job.get_node("source")
            info_dic["source"] = source_node.get_node("unc").get_text(False)
            info_dic["sourceuser"] = source_node.get_node("user").get_text(False)
            info_dic["sourcepswd"] = source_node.get_node("passwd").get_text(False)
        except:
            pass
        try:
            dest_node = target_job.get_node("destination")
            info_dic["dest"] = dest_node.get_node("unc").get_text(False)
            info_dic["destuser"] = dest_node.get_node("user").get_text(False)
            info_dic["destpswd"] = dest_node.get_node("passwd").get_text(False)
        except:
            pass
        return info_dic

    def id_to_jobinfo(self, id):
        for jobinfo in self.get_jobinfo_list(): 
            if jobinfo["id"] == id:
                return jobinfo
