# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
import datetime
from django.utils.translation import ugettext as _
from django.core.exceptions import ValidationError
from root.modules.form import BaseForm


class Form(BaseForm):
    def __init__(self, id=None, *args, **kwargs):
        from root.modules.form.fields.validators.fal import FalValidator as fv
        from root.modules.form.fields.text import TextField
        from root.modules.form.fields.checkbox import CheckboxField
        from root.modules.form.fields.radio import RadioField
        from root.modules.form.fields.password import PasswordField
        from root.modules.form.fields.extras.selectdatetime import \
                                                           SelectTimeField
        from root.modules.form.fields.hidden import HiddenField
        from root.modules.form.fields.multicheckbox import MultiCheckboxField
        from project.forms.fields.extras.backupjobname \
            import DatacopyJobNameField, DatacopyJobNameValidator
        from project.forms.fields.extras.backupunc import BackupUncField
        from project.forms.fields.extras.backupuser import BackupUserField
        from project.forms.fields.extras.backuppass import BackupPassField
        super(Form, self).__init__(*args, **kwargs)

        if id: 
            # 変更時。自id以外のjob名との重複を禁則
            self.fields["name"] = DatacopyJobNameField(
                    label=_(u'Job Name'),
                    required=True,
                    validators=[DatacopyJobNameValidator(options={"exempt_id": id})])
        else:
            # 追加時。
            self.fields["name"] = DatacopyJobNameField(
                    label=_(u'Job Name'),
                    required=True,
                    validators=[DatacopyJobNameValidator()])

        self.fields["schedule"] = RadioField(
                label=_(u'Schedule'),
                initial=0,
                required=False,   #FIXME
                validators=[])
        self.fields["day"] = MultiCheckboxField(
                label=_(u'Day'),
                choices=((0, _(u'Sunday')), (1, _(u'Monday')),
                         (2, _(u'Tuesday')), (3, _(u'Wednesday')),
                         (4, _(u'Thursday')), (5, _(u'Friday')),
                         (6, _(u'Saturday'))),
                initial=[],
                required=False,
                validators=[])
        self.fields["time"] = SelectTimeField(
                label=_(u'Time'),
                initial=datetime.time(0, 0),
                required=False,
                validators=[])
        self.fields["trash"] = CheckboxField(
                label=_(u'Trash Copy'),
                initial=False,
                required=False,
                validators=[])
        self.fields["shutdown"] = CheckboxField(
                label=_(u'Shutdown after datacopy'),
                initial=False,
                required=False,
                validators=[])
        self.fields['dest'] = BackupUncField(
            label=_(u'Datacopy Destination'),
            required=True)
        self.fields["destuser"] = BackupUserField(
            label=_(u'UserName'),
            required=False)
        self.fields["destpswd"] = BackupPassField(
            label=_(u'Password'),
            required=False)
        self.fields['source'] = BackupUncField(
            label=_(u'Datacopy Source'),
            required=True)
        self.fields["sourceuser"] = BackupUserField(
            label=_(u'UserName'),
            required=False)
        self.fields["sourcepswd"] = BackupPassField(
            label=_(u'Password'),
            required=False)
        self.fields["full"] = CheckboxField(
            label=_(u'Force Full Copy'),
            initial=False,
            required=False)
        self.fields["sync"] = CheckboxField(
            label=_(u'DataCopy: Sync Mode'),
            initial=False,
            required=False)

    def clean(self):
        from ..modules.cleandata import Cleaned
        cleaned = Cleaned()
        if self.cleaned_data.get("schedule") == "1":
            if len(self.cleaned_data.get("day")) <= 0:
                raise ValidationError(_(u'Datacopy schedule is required.'))
        if cleaned.dest_is_same_with_src(self.cleaned_data.get("source"),
                    self.cleaned_data.get("dest")):
            raise ValidationError(_(u'Datacopy source and destination can not be the same share folder.'))
            
        return self.cleaned_data
