# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

import time
from datetime import datetime

from django.utils.translation import ugettext as _
from django.http import HttpResponse
from django.utils import simplejson
from project.view import RootView
from project.fal.backups import Backups


RESULT_FMT = "(%s)"
TIMESTAMP_FMT = "/%s"
ENCODING = "utf-8"

STATUS_TABLE = {
    "NOT_EXEC": _(u"Not Execute"),
    "STOP": _(u"Stop"),
    "ABORT": _(u"Abort"),
    "COMPLETE": _(u"Complete"),
    "PROCESSING": _(u"Processing"),
    "QUEUE": _(u"Waiting")}
RESULT_TABLE = {
    "success": _(u"Success"),
    "failed": _(u"Failed")}


class StatusView(RootView):

    CATEGORY = ""

    def view(self, request, id):
        data = dict()
        status = self._status(request, id, data=data)
        return {self.KEY_RESPONSE: HttpResponse(
                #simplejson.dumps({'status': self._status(request, id)}),
                simplejson.dumps({
                    'status': status,
                    '_status': data.get('status', ''),
                    '_result': data.get('result', ''),
                    '_timestamp': data.get('timestamp', ''),
                }),
                mimetype="application/json; charset=UTF-8;")}

    def _node_to_str(self, job_node, data=dict()):
        status = STATUS_TABLE.get(job_node.get_value("status", ""), "")
        data['status'] = status

        result = RESULT_TABLE.get(job_node.get_value("result", ""), "")
        if result:
            data['result'] = result
            result = RESULT_FMT % result

        timestamp = job_node.get_value("timestamp", "")  # epoch
        if timestamp:
            timestamp = unicode(datetime(*time.localtime(int(timestamp))[:6]))
            data['timestamp'] = timestamp
            timestamp = TIMESTAMP_FMT % timestamp

        return status + result + timestamp

    def _status(self, request=None, id="", data=dict()):
        try:
            if not id:
                id = request.GET["id"]

            job_node = Backups().query(to_dict=False).get().get_node(
                self.CATEGORY + "/joblist/job[id='%s']" % id)

            result = self._node_to_str(job_node, data=data)

        except:
            result = ""

        return result
