# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from ionas.request import (Request, Entry)
from iobase.exception import (NotFoundError)

from django.utils.translation import ugettext as _
from root.modules.form.fields.multiselect import MultiSelectField

# FIXME fal_infoからとるべき情報
SUDO_CMD = '/usr/bin/sudo'
try:
    from fal_info import FAL_GROUPS_PATH, FAL_USERS_PATH
except:
    FAL_GROUPS_PATH = "/mnt/data/bin/fal_groups.py"
    FAL_USERS_PATH = "/mnt/data/bin/fal_users.py"
LOAD_TYPE = "load"

NAME_NAME = "name"
GROUPLIST_PATH = "grouplist/group[not(@hidden) or @hidden!='true']"
USERLIST_PATH = "userlist/user[not(@hidden) or @hidden!='true']"
DOMAIN_ATTR = "domain"


def get_target_node_list(target_fal, list_path):
    load_result = Request(target_fal, filter_cmd=SUDO_CMD).request(
        Entry(LOAD_TYPE)).get_entrylist().get_entry()
    try:
        node_list = \
            load_result.get().get_node_list(list_path)
    except NotFoundError:
        """There is nothing. initial state."""
        node_list = []
    return node_list

def get_target_tuple(target_fal, list_path, name_path):
    name_tuple_list = []
    for node in get_target_node_list(target_fal, list_path):
        try:
            name = node.get_attr_value(DOMAIN_ATTR) + "@" + node.get_node(name_path).get_text()
        except NotFoundError:
            name = node.get_node(name_path).get_text()
        name_tuple_list.append((name, name))
    return tuple(name_tuple_list)

class MultiUserSelectField(MultiSelectField):
    def __init__(self, choices=(), **kwargs):
        """Set choices dynamically."""
        MultiSelectField.__init__(
            self, choices=get_target_tuple(
            FAL_USERS_PATH, USERLIST_PATH, NAME_NAME), **kwargs)


class MultiGroupSelectField(MultiSelectField):
    def __init__(self, choices=(), **kwargs):
        """Set choices dynamically."""
        MultiSelectField.__init__(
            self, choices=get_target_tuple(
            FAL_GROUPS_PATH, GROUPLIST_PATH, NAME_NAME), **kwargs)
