# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django.utils.translation import ugettext as _
from django.core import validators
from django.core.exceptions import ValidationError
import os
from project.view import RootView
from root.modules.form import BaseForm
#from root.admin.network.proxy.views import Form as ProxyForm
from project.fal.proxy import Proxy
from project.fal.narsus import NarSuS
from project.fal.network import Network
from project.fal.sysinfo import SysInfo
from project.fal.qrcode import QRCode


class GuideView(RootView):
    NOT_LOGINED_URL = '/login'
    views = []

    def view(self, request):
        narsus = NarSuS()
        if narsus.is_registered():
            return {self.KEY_REDIRECT: '../main'}
        else:
            return {
                self.KEY_TEMPLATE: 'guidance.html',
                self.KEY_TEMPLATE_DIC: {'IS_LOGINED': True}}


class ProxyForm(BaseForm):
    from project.forms.fields.extras.proxy import ProxyModeField
    from project.forms.fields.extras.proxyhost import ProxyHostField
    from project.forms.fields.extras.portnum import RemotePortNumberField
    from project.forms.fields.extras.url import URLField
    mode = ProxyModeField()

    pacfile = URLField(label=_(u'Pacfile Script URL'), required=False)

    host = ProxyHostField(label=_(u'Address'), required=False)
    port = RemotePortNumberField(label=_(u'Port'), required=False)

    def clean_pacfile(self):
        val = self.cleaned_data['pacfile']
        if self.cleaned_data['mode'] == 'pacfile' \
                and val in validators.EMPTY_VALUES:
            raise ValidationError(_(u'This field is required.'))
        return val

    def clean_host(self):
        val = self.cleaned_data['host']
        if self.cleaned_data['mode'] == 'manual' \
                and val in validators.EMPTY_VALUES:
            raise ValidationError(_(u'This field is required.'))
        return val

    def clean_port(self):
        val = self.cleaned_data['port']
        if self.cleaned_data['mode'] == 'manual' \
                and val in validators.EMPTY_VALUES:
            raise ValidationError(_(u'This field is required.'))
        return val


class RegView(RootView):
    NOT_LOGINED_URL = '/login'
    views = ['regist']

    def _mode(self, proxy, manual, pacfile):
        if proxy:
            if manual:
                return 'manual'
            elif pacfile:
                return 'pacfile'
            else:
                # NGケースだが一応
                return 'disable'
        else:
            return 'disable'

    def _proxy_manual_pacfile(self, mode):
        if mode == 'disable':
            return False, False, False
        elif mode == 'manual':
            return True, True, False
        elif mode == 'pacfile':
            return True, False, True
        else:
            # NGケースだが一応
            return False, False, False

    def view(self, request):
        config = Proxy().load()
        initial = {
            'mode': self._mode(
                    config['enable'],
                    config['manual']['enable'],
                    config['pacfile']['enable']),
            'host': config['manual']['host'],
            'port': config['manual']['port'],
            'pacfile': config['pacfile']['url'],
        }
        form = ProxyForm(initial=initial)
        return {
            self.KEY_TEMPLATE: os.path.join(self.TEMPLATES, 'reg.html'),
            self.KEY_TEMPLATE_DIC: {'IS_LOGINED': True, 'form': form}}

    def view_regist(self, request):
        with self.processing(constname='ERR_NARSUS_STATE_CHANGE') as p:
            form = ProxyForm(data=request.POST)
            p.break_if_state_change()

            if form.is_valid():
                proxy = Proxy()
                enable, manual, pacfile = self._proxy_manual_pacfile(
                        form.cleaned_data.get('mode', 'disable'))
                proxy.save(
                    enable=enable,
                    manual=manual,
                    host=form.cleaned_data['host'],
                    port=form.cleaned_data['port'],
                    pacfile=pacfile,
                    url=form.cleaned_data['pacfile'],
                )
                if proxy.get_result():
                    self.set_message(proxy.get_message())
                narsus = NarSuS()
                result = narsus.apply(narsus=True)
                p.result(narsus.get_result(boolean=True))
                if result:
                    return {self.KEY_REDIRECT: './gonarsus'}
                else:
                    self.set_message(narsus.get_message())
                    self.set_message(_(u"In the case of the network is connected to internet, cannot modify setting."))
        return {
            self.KEY_TEMPLATE: os.path.join(self.TEMPLATES, 'reg.html'),
            self.KEY_TEMPLATE_DIC: {'IS_LOGINED': True, 'form': form}}


class GoView(RootView):
    class Form(BaseForm):
        from project.forms.fields.hidden import HiddenField
        key = HiddenField()
        macaddress = HiddenField()
        model = HiddenField()
        sendflag = HiddenField()

    NOT_LOGINED_URL = '/login'
    views = []

    def view(self, request):
        narsus = NarSuS()
        narsus.query()
        network = Network()
        network.query(to_dict=False)
        sysinfo = SysInfo()

        initial = {
            'key': narsus.get_value('key', ''),
            'macaddress': network.get_value('netiflist/netif[@name="lan1"]/mac', ''),
            'model': sysinfo.query().get('model', ''),
            'sendflag': '0',
        }

        return {
            self.KEY_TEMPLATE: 'go.html',
            self.KEY_TEMPLATE_DIC: {
                'IS_LOGINED': True,
                'is_registered': narsus.is_registered(),
                'form': self.Form(initial=initial),
                'URL_NEWREGIST': narsus.get_value('urllist/url[@type="newregist"]', default=''),
                'URL_REGIST': narsus.get_value('urllist/url[@type="regist"]', default=''),
            }
        }


class CodeForm(BaseForm):
    from project.forms.fields.text import TextField
    code1 = TextField(label='%s(1)' % _(u'Usecode'), max_length=5)
    code2 = TextField(label='%s(2)' % _(u'Usecode'), max_length=5)
    code3 = TextField(label='%s(3)' % _(u'Usecode'), max_length=5)
    code4 = TextField(label='%s(4)' % _(u'Usecode'), max_length=5)
    code5 = TextField(label='%s(5)' % _(u'Usecode'), max_length=5)


class CodeView(RootView):
    NOT_LOGINED_URL = '/login'

    URL_USERREG = NarSuS().query().get().get_value('urllist/url[@type="userregist"]')
    QRCODE = '/media/img/narsus.qr.png'
    PATH_QRCODE = os.sep.join(['/usr/local/share/www/webui', QRCODE])

    views = ['save']

    def view(self, request):
        qrobj = QRCode()
        qrobj.encode(
#                string='https://www.narsus.jp/user-reg',
                string=self.URL_USERREG,
                path=self.PATH_QRCODE)
        return {
            self.KEY_TEMPLATE: os.path.join(self.TEMPLATES, 'code.html'),
            self.KEY_TEMPLATE_DIC: {
                'IS_LOGINED': True,
#                'url': 'https://www.narsus.jp/user-reg',
                'url': self.URL_USERREG,
                'qrcode': self.QRCODE,
                'form': CodeForm(),
            }}

    def view_save(self, request):
        with self.processing(constname='ERR_NARSUS_STATE_CHANGE') as p:
            form = CodeForm(data=request.POST)
            p.break_if_state_change()

            if form.is_valid():
                usecode = ''.join(form.cleaned_data.values())
                narsus = NarSuS()
                narsus.apply(usecode=usecode, log=True)
                self.set_message(narsus.get_message())
                p.result(narsus.get_result(boolean=True))
        return {
            self.KEY_TEMPLATE_DIC: {
                'IS_LOGINED': True,
#                'url': 'https://www.narsus.jp/user-reg',
                'url': self.URL_USERREG,
                'qrcode': self.QRCODE,
                'form': form,
            }}
