# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from ionas.request import (Request, Entry)
from iobase.xmldata import XmlNode
from iobase.exception import NotFoundError
from product_conf import SUDO_CMD
from fal_info import (FAL_NETWORK_PATH, ReqTypes, Results)


QUERY_ENTRY = "<network/>"
LAN1_IP4_ENTRY = "<network><familyconfig><inet4><dns/><gateway/></inet4>" + \
        "</familyconfig><netiflist><netif name='lan1'><protocollist>" + \
        "<protocol family='inet4'><dhcp/></protocol></protocollist>" + \
        "</netif></netiflist></network>"
LAN1_IP6_ENTRY = "<network><familyconfig><inet6><dns/><gateway/></inet6>" + \
        "</familyconfig><netiflist><netif name='lan1'><protocollist>" + \
        "<protocol family='inet6'><dhcp/></protocol></protocollist>" + \
        "</netif></netiflist></network>"
LAN2_IP4_ENTRY = "<network><familyconfig><inet4><dns/><gateway/></inet4>" + \
        "</familyconfig><netiflist><netif name='lan2'><protocollist>" + \
        "<protocol family='inet4'><dhcp/></protocol></protocollist>" + \
        "</netif></netiflist></network>"
LAN2_IP6_ENTRY = "<network><familyconfig><inet6><dns/><gateway/></inet6>" + \
        "</familyconfig><netiflist><netif name='lan2'><protocollist>" + \
        "<protocol family='inet6'><dhcp/></protocol></protocollist>" + \
        "</netif></netiflist></network>"
NETIF_XPATH = "./netiflist/netif[@name='%s']"
PROTOCOL_XPATH = "./protocollist/protocol[@family='%s']"
INET_XPATH = "./familyconfig/%s"


class Network(object):
    def send_request(self, cmd, req_entry):
        try:
            response = Request(cmd,  filter_cmd=SUDO_CMD).request(req_entry)
            if response.get_entrylist().get_entry().get_result() == Results.OK:
                return True
            else:
                return False
        except:
            # FIXME  例外種類ごとに、ユーザにメッセージを渡す？
            # 検討必要
            return False

    def get_target_node(self, path):
        result = Request(FAL_NETWORK_PATH, filter_cmd=SUDO_CMD).request(
                        Entry(ReqTypes.LOAD)).get_entrylist().get_entry().get()
        return result.get_node(path)

    def query_ip_info(self):
        entry = Entry(ReqTypes.QUERY, QUERY_ENTRY)
        response = Request(FAL_NETWORK_PATH, filter_cmd=SUDO_CMD).request(
                                    entry).get_entrylist().get_entry()
        return response.get()

    def get_ipv4_info(self, netif="lan1"):
        info = {}
        query_info = None
        netif_node = self.get_target_node(NETIF_XPATH % netif)
        if netif == "lan2":
            if not netif_node.is_enabled():
                info["dhcp"] = "disable"
                return info

        protocol_node = netif_node.get_node(PROTOCOL_XPATH % "inet4")
        if netif == "lan2":
            if not protocol_node.is_enabled():
                info["dhcp"] = "disable"
                return info
        if protocol_node.get_node("dhcp").is_enabled():
            info["dhcp"] = "auto"
            query_info = self.query_ip_info()
            try:
                addr_node = query_info.get_node(NETIF_XPATH % netif).get_node(
                         PROTOCOL_XPATH % "inet4").get_node("addrlist/addr")
                info["ipaddr"] = addr_node.get_text()
                info["netmask"] = addr_node.get_attr_value("netmask")
            except NotFoundError:
                pass
        else:
            info["dhcp"] = "custom"
            info["ipaddr"] = protocol_node.get_node("addrlist/addr").get_text()
            info["netmask"] = protocol_node.get_node(
                            "addrlist/addr").get_attr_value("netmask")

        info["framesize"] = netif_node.get_attr_value("mtu")
        info["wol"] = netif_node.get_attr_value("wol")

        info.update(self.get_common_info("inet4"))
        if info.get("common") == "custom":
            inet_node = self.get_target_node(INET_XPATH % "inet4")
        else:
            if not query_info:
                query_info = self.query_ip_info()
            inet_node = query_info.get_node(INET_XPATH % "inet4")
        try:
            info["dns"] = inet_node.get_node("dns/addrlist/addr").get_text()
        except NotFoundError:
            pass
        try:
            info["gateway"] = inet_node.get_node("gateway/addr").get_text()
        except NotFoundError:
            pass

        return info

    def get_common_info(self, type):
        info = {}
        dns_node = self.get_target_node(INET_XPATH % type).get_node("dns")
        mainif = dns_node.get_attr_value("source")
        if mainif == "static":
            info["common"] = "custom"
        else:
            info["mainif"] = mainif
        return info

    def get_ipv6_info(self, netif="lan1"):
        info = {}
        query_info = None
        netif_node = self.get_target_node(NETIF_XPATH % netif)
        if netif == "lan2":
            if not netif_node.is_enabled():
                info["dhcp"] = "disable"
                return info
        protocol = netif_node.get_node(PROTOCOL_XPATH % "inet6")
        if protocol.is_enabled():
            if protocol.get_node("dhcp").is_enabled():
                info["dhcp"] = "auto"
                query_info = self.query_ip_info()
                try:
                    addr_node = query_info.get_node(
                        NETIF_XPATH % netif).get_node(
                        PROTOCOL_XPATH % "inet6").get_node("addrlist/addr")
                    info["ipaddr"] = addr_node.get_text()
                    info["prefixlen"] = addr_node.get_attr_value("prefixlen")
                except NotFoundError:
                    pass
            else:
                info["dhcp"] = "custom"
                info["ipaddr"] = protocol.get_node("addrlist/addr").get_text()
                info["prefixlen"] = protocol.get_node(
                            "addrlist/addr").get_attr_value("prefixlen")
        else:
            info["dhcp"] = "disable"
            return info

        info["framesize"] = netif_node.get_attr_value("mtu")
        info.update(self.get_common_info("inet6"))

        if info.get("common") == "custom":
            inet_node = self.get_target_node(INET_XPATH % "inet6")
        else:
            if not query_info:
                query_info = self.query_ip_info()
            inet_node = query_info.get_node(INET_XPATH % "inet6")
        try:
            info["dns"] = inet_node.get_node("dns/addrlist/addr").get_text()
        except NotFoundError:
            pass
        try:
            info["gateway"] = inet_node.get_node("gateway/addr").get_text()
        except NotFoundError:
            pass

        return info

    #def set_framesize_and_wol(self, netif, frame, wol=""):
    #    if wol == "":
    #        netif.set_attrs({"mtu": str(frame)})
    #    else:
    #        netif.set_attrs({"mtu": str(frame), "wol": wol})
    #    return
    _wol_empty_object = object()
    def set_framesize_and_wol(self, netif_node, framesize, wol=_wol_empty_object):
        if wol is self._wol_empty_object:
            netif_node.set_attrs({
                "mtu": str(self.__int(framesize, default=1500)),
            })
        else:
            netif_node.set_attrs({
                "mtu": str(self.__int(framesize, default=1500)),
                "wol": wol,
            })

    def __int(self, value, default=0):
        try:
            return int(value)
        except:
            return default

    def set_common(self, entry, data, type="inet4"):
        if data["dhcp"] == "disable":
            entry.get().get_node("familyconfig").delete_child(type)
            return

        inet = entry.get().get_node(INET_XPATH % type)
        if data["common"] == "auto":
            if data.get("mainif"):
                inet.get_node("dns").set_attrs({"source": data["mainif"]})
                inet.get_node("gateway").set_attrs({"source": data["mainif"]})
            else:
                inet.get_node("dns").set_attrs({"source": "lan1"})  # FIXME: lan1
                inet.get_node("gateway").set_attrs({"source": "lan1"})  # FIXME: lan1
        else:
            for name in ["dns", "gateway"]:
                inet.get_node(name).set_attrs({"source": "static"})

                addr = XmlNode("addr")
                addr.set_text(data[name])
                if name == "dns":
                    addrlist = XmlNode("addrlist")
                    addrlist.append_child(addr)
                    inet.get_node(name).append_child(addrlist)
                elif name == "gateway":
                    inet.get_node(name).append_child(addr)
        return

    def lan1_ipv4_set(self, data):
        entry = Entry(ReqTypes.APPLY, LAN1_IP4_ENTRY)
        netif = entry.get().get_node(NETIF_XPATH % "lan1")
        self.set_framesize_and_wol(
            netif, data["framesize"], str(data["wol"]).lower())
        self.set_common(entry, data, type="inet4")
        self.set_ipaddr(netif, data, type="inet4")
        return self.send_request(FAL_NETWORK_PATH, entry)

    def lan1_ipv6_set(self, data):
        entry = Entry(ReqTypes.APPLY, LAN1_IP6_ENTRY)
        netif = entry.get().get_node(NETIF_XPATH % "lan1")
        self.set_framesize_and_wol(netif, data["framesize"])
        self.set_common(entry, data, type="inet6")
        self.set_ipaddr(netif, data, type="inet6")
        return self.send_request(FAL_NETWORK_PATH, entry)

    def lan2_ipv4_set(self, data):
        entry = Entry(ReqTypes.APPLY, LAN2_IP4_ENTRY)
        netif = entry.get().get_node(NETIF_XPATH % "lan2")
        self.set_framesize_and_wol(
            netif, data["framesize"], str(data["wol"]).lower())
        self.set_common(entry, data, type="inet4")
        self.set_ipaddr(netif, data, type="inet4")
        return self.send_request(FAL_NETWORK_PATH, entry)

    def lan2_ipv6_set(self, data):
        entry = Entry(ReqTypes.APPLY, LAN2_IP6_ENTRY)
        netif = entry.get().get_node(NETIF_XPATH % "lan2")
        self.set_framesize_and_wol(netif, data["framesize"])
        self.set_common(entry, data, type="inet6")
        self.set_ipaddr(netif, data, type="inet6")
        return self.send_request(FAL_NETWORK_PATH, entry)

    def set_ipaddr(self, netif, data, type="inet4"):
        protocol = netif.get_node(PROTOCOL_XPATH % type)
        if data["dhcp"] == 'disable':
            protocol.set_attrs({"enable": "false"})
        elif data["dhcp"] == 'auto':
            protocol.set_attrs({"enable": "true"})
            protocol.get_node("dhcp").set_attrs({"enable": "true"})
        else:
            protocol.set_attrs({"enable": "true"})
            protocol.get_node("dhcp").set_attrs({"enable": "false"})
            addrlist = XmlNode("addrlist")
            addr = XmlNode("addr")
            addr.set_text(data["ipaddr"])
            if type == "inet6":
                addr.set_attrs({"prefixlen": str(data["prefixlen"])})
            elif type == "inet4":
                addr.set_attrs({"netmask": data["netmask"]})
            addrlist.append_child(addr)
            protocol.append_child(addrlist)
        return

    def dhcp_is_set_enabled(self, lan_name, inet):
        config = Request(FAL_NETWORK_PATH, filter_cmd=SUDO_CMD).request(
                        Entry(ReqTypes.LOAD)).get_entrylist().get_entry().get()
        lan_node = config.get_node(NETIF_XPATH % lan_name)
        if not lan_node.is_enabled():
            return False
        try:
            inet_node = lan_node.get_node(PROTOCOL_XPATH % inet)
            if inet_node.is_enabled():
                if inet_node.get_node("dhcp").is_enabled():
                    return True
        except:
            pass
        return False

    def get_msg_code(self, msg):
        try:
            from fal_msginfo import Msg
            return getattr(Msg, msg)
        except:
            return None
