# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django.utils.translation import ugettext as _
from django.http import HttpResponseRedirect
from django.core.exceptions import ValidationError
from project.view import FuncView
from project.fal.command import get_message
from root.modules.form import BaseForm
import settings

from project.forms.fields.radio import RadioField
from project.forms.fields.extras.url import URLField
from project.forms.fields.extras.proxyhost import ProxyHostField
from project.forms.fields.extras.portnum import RemotePortNumberField
from project.fal.proxy import Proxy
from .. modules.proxy import Proxy 


def root(request):
    return HttpResponseRedirect('/'.join([settings.MENU_URL, 'proxy']))


class Form(BaseForm):
    proxy_set = RadioField(
            label=_(u'Proxy'),
            choices=(('disable', _(u'Do not use')),
                     ('pacfile', _(u'Use pacfile')),
                     ('manual', _(u'Set manually'))),
            initial='disable',
            required=True)
    url = URLField(
            label=_(u'Pacfile Script URL'),
            max_length=128,
            min_length=7,
            required=False)
    host = ProxyHostField(
            label=_(u'Address'),
            required=False)
    port = RemotePortNumberField(
            label=_(u'Port'),
            required=False)

    def clean(self):
        if self.cleaned_data.get("proxy_set") == "pacfile":
            if self.cleaned_data.get("url") == "":
                raise ValidationError(_(u'"Script URL" is required.'))
        elif self.cleaned_data.get("proxy_set") == "manual":
            if self.cleaned_data.get("host") == "":
                raise ValidationError(_(u'"Address" is required.'))
            if "port" in self.cleaned_data.keys():
                if self.cleaned_data.get("port") == None:
                    raise ValidationError(_(u'"Port" is required.'))
        return self.cleaned_data

class View(FuncView):
    views = ['apply']
    proxy = Proxy()

    def view(self, request):
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                'form': Form(initial=self.proxy.get_init_config())}}

    def view_apply(self, request):
        with self.processing(constname="ERR_PROXY_STATE_CHANGE") as p:
            form = Form(extdir='fields', data=request.POST)
            rst = False
            p.break_if_state_change()

            if form.is_valid():
                if self.proxy.proxy_set(data=form.cleaned_data):
                    self.set_message(
                            get_message('MSG_PROXY_CHANGE', is_log=True))
                    rst = True
                else:
                    self.set_message(
                            get_message('ERR_PROXY_CHANGE', is_log=True))
                p.result(rst)

        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {'form': form}}

proxy = View.as_view()
