# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
import os

from ionas.request import Request
from product_conf import SUDO_CMD
from fal_info import FAL_SHARES_PATH, Results, ReqTypes

from django.http import HttpResponseRedirect
from project.view import FuncView

import settings
from .. import settings as ps
from project.fal.command import get_message as fgettext

from ..modules.utils import (
    get_request_entry, get_html_list, get_info_dic)


def root(request):
    return HttpResponseRedirect('/'.join([settings.MENU_URL, 'add']))


class AddView(FuncView):
    from ..modules.form import ShareForm as Form

    views = ['add', 'list']

    def __send_request(self, form):
        os.environ['PATH'] = os.environ['PATH'] + ":/sbin"
        response = Request(FAL_SHARES_PATH, filter_cmd=SUDO_CMD).request(
            get_request_entry(form, ReqTypes.ADD), env=os.environ)
        result = response.get_entrylist().get_entry().get_result()
        if result == Results.OK:
            msg = 'MSG_SHARE_ADD'
            return_val = True
        elif result == Results.MAX_EXCEED_ERROR:
            msg = 'ERR_SHARE_NO_MORE'
            return_val = False
        else:
            msg = 'ERR_SHARE_ADD'
            return_val = False
        self.set_message(fgettext(
                msg,
                arglist={'name': form.cleaned_data['name']},
                is_log=True))
        return return_val

    def view(self, request):
        form = self.Form(extdir="fields")
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                'case': "add",
                'form': form,
                'basic_setting_html_list':
                    get_html_list(extdir="basic_setting"),
                'info_dic':
                    get_info_dic(),
                'info_html_list':
                    get_html_list(extdir="info"),
            }
        }

    def view_add(self, request):
        result = False
        with self.processing(constname='ERR_SHARE_STATE_CHANGE') as p:
            form = self.Form(extdir="fields", data=request.POST)
            p.break_if_state_change()
            if form.is_valid():
                result = self.__send_request(form)
                p.result(result)
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                'case': "add",
                'form': form,
                'basic_setting_html_list':
                    get_html_list(extdir="basic_setting"),
                'info_dic':
                    get_info_dic(),
                'info_html_list':
                    get_html_list(extdir="info"),
                'result': result,
            }
        }

    def view_list(self, request):
        return {self.KEY_REDIRECT:
                "/".join([ps.MENU_URL, ps.LIST_APPLI, ps.LIST_VIEW])}

add = AddView.as_view()
