# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
import os
import sys

from ionas.request import (Request, Entry)
from iobase.exception import (NotFoundError)
from product_conf import SUDO_CMD
from fal_info import FAL_SHARES_PATH, Results, ReqTypes

from django.utils.translation import ugettext as _
from django.http import HttpResponseRedirect
from django.core import validators
from django.core.exceptions import ValidationError
from project.view import FuncView
from root.modules.form import BaseForm
from root.modules.utils import importer as imp, to_importable_path as to_ipath
import settings
from .. import settings as ps

from project.fal.command import get_message as fgettext
from project.fal.share import Share
from project.fal.backups import Backups
from project.forms.fields.extras.share import MultiSelectShareField


# FIXME fal_infoからとるべき情報
FAL_SHARES_REQ_BASE_XML = {ReqTypes.DELETE: "<shares><share><name/>" +
                                "</share></shares>"}

NAME_NAME = "name"
NAME_PATH = "share/name"
DESC_NAME = "desc"
PERMISSION_NAME = "permission"
D_PERM_PATH = PERMISSION_NAME + "/detailpermission"
D_RO_PATH = D_PERM_PATH + "/readonly"
D_RW_PATH = D_PERM_PATH + "/readwrite"
RO_USERLIST_PATH = D_RO_PATH + "/userlist/*"
RO_GROUPLIST_PATH = D_RO_PATH + "/grouplist/*"
RW_USERLIST_PATH = D_RW_PATH + "/userlist/*"
RW_GROUPLIST_PATH = D_RW_PATH + "/grouplist/*"
SHARESERVICE_NAME = "shareservice"
MSSHARE_PATH = SHARESERVICE_NAME + "/msshare"
LIMIT_PATH = MSSHARE_PATH + "/quota/limit"


MSSHARE_PATH = "shareservice/msshare"
LIMIT_PATH = MSSHARE_PATH + "/quota/limit"

TARGET_SHARE_NODE_FMT_STR = 'sharelist/share[name="%s"]'

BACKUP_FLAG = "B"
FLAG_START_MARK = "["
FLAG_END_MARK = "]"
FLAG_SEP_MARK = "/"
NOT_FOUND = -1


def root(request):
    return HttpResponseRedirect('/'.join([settings.MENU_URL, 'delete']))


class DeleteForm(BaseForm):
    def __init__(self, choices, *args, **kwargs):
        super(DeleteForm, self).__init__(*args, **kwargs)

        self.fields['shares_'] = MultiSelectShareField(
                choices=choices,
                required=False)
        self.fields['shares'] = MultiSelectShareField(
                label=_(u'Deletion Share'),
                choices=choices,
                required=True)


class DeleteView(FuncView):

    views = ['delete', 'list']

    def __set_flags(self, share, flag_info_list):
        for flag_info in flag_info_list:
            list = flag_info[0]
            flag = flag_info[1]
            if not share in list:
                continue
            pos = share.find(FLAG_END_MARK)
            if pos == NOT_FOUND:
                share = FLAG_START_MARK + flag + FLAG_END_MARK + share
            else:
                share = share[:pos] + FLAG_SEP_MARK + flag + share[pos:]
        return share

    def __get_bkup_flag_list(self):
        bkup_dst_list, bkup_src_list =\
            Backups().get_bkup_local_share_list("backup")
        dtcp_dst_list, dtcp_src_list =\
            Backups().get_bkup_local_share_list("datacopy")
        return (
            bkup_dst_list + bkup_src_list +\
            dtcp_dst_list + dtcp_src_list,
            BACKUP_FLAG)

    def __get_flag_list(self):
        flag_list = []
        flag_list += [self.__get_bkup_flag_list()]
        return flag_list

    def __get_choices(self, deletable=True):
        try:
            shares = Share().load()
            flag_info_list = self.__get_flag_list()
            if deletable:
                choices = [
                        (k, self.__set_flags(k, flag_info_list))
                        for k, v
                        in shares.iteritems()
                        if v.get('undeletable', False) is False]
            else:
                choices = [
                        (k, self.__set_flags(k, flag_info_list))
                        for k, v
                        in shares.iteritems()]
            return sorted(choices)
        except:
            return []

    def __is_in_backup_share(self, choices):
        for choice in choices:
            pos = choice[1].find(FLAG_END_MARK)
            if pos == NOT_FOUND:
                continue
            if choice[1][:pos].find(BACKUP_FLAG) != NOT_FOUND:
                return {"is_in_backup_share": True}
        return {}

    def __get_info_dic(self, choices):
        info_dic = {}
        info_dic.update(self.__is_in_backup_share(choices))
        return info_dic

    # FIXME BaseFormクラスに含める?__add()とほぼ一緒
    def __get_html_list(self, extdir):
        current = os.path.dirname(sys.modules[self.__module__].__file__)
        dpath = os.path.join(current, extdir)
        return [os.path.join(dpath, fname)
            for fname
            in sorted(os.listdir(dpath))]

    def __get_target_share_node(self):
        # FIXME load失敗時エラー処理
        load_result = Request(FAL_SHARES_PATH, filter_cmd=SUDO_CMD).request(
            Entry(ReqTypes.LOAD)).get_entrylist().get_entry()   # FIXME
        return load_result.get().get_node(
            TARGET_SHARE_NODE_FMT_STR % self.session[ps.NAME_INFO])

    def __get_basic_share_info_dict(self, node):
        return {
            ps.NAME_INFO: node.get_node(NAME_NAME).get_text(),
            ps.DESC_INFO: node.get_node(DESC_NAME).get_text(),
            ps.QUOTA_LIMIT_INFO: node.get_node(LIMIT_PATH).get_text()}

    def __get_detail_permission_share_info_dict(self, node):
        return {
            ps.RO_USER_INFO: ",".join([
                user_node.get_node(NAME_NAME).get_text()
                for user_node
                in node.get_node_list(RO_USERLIST_PATH)]),
            ps.RW_USER_INFO: ",".join([
                user_node.get_node(NAME_NAME).get_text()
                for user_node
                in node.get_node_list(RW_USERLIST_PATH)]),
            ps.RO_GROUP_INFO: ",".join([
                group_node.get_node(NAME_NAME).get_text()
                for group_node
                in node.get_node_list(RO_GROUPLIST_PATH)]),
            ps.RW_GROUP_INFO: ",".join([
                group_node.get_node(NAME_NAME).get_text()
                for group_node
                in node.get_node_list(RW_GROUPLIST_PATH)])}

    def __get_means(self, extdir):
        current = os.path.dirname(sys.modules[self.__module__].__file__)
        dpath = os.path.join(current, extdir)
        means = {}
        try:
            for fname in sorted(os.listdir(dpath)):
                fpath = os.path.join(dpath, fname)
                try:
                    means.update(imp(to_ipath(fpath)).means)
                except:
                    continue
        except:
            pass
        return means

    def __get_ext_share_info_dict(self, node):
        share_info_dict = {}
        means = self.__get_means(extdir="means")
        for name, ins in means.iteritems():
            share_info_dict[name] = ins.get(node)
        return share_info_dict

    def __get_share_info_dict(self, node):
        share_info_dict = {}
        share_info_dict.update(
            self.__get_basic_share_info_dict(node))
        share_info_dict.update(
            self.__get_detail_permission_share_info_dict(node))
        share_info_dict.update(
            self.__get_ext_share_info_dict(node))
        return share_info_dict

    def __get_request_entry(self, name):
        entry = Entry(ReqTypes.DELETE,
                      FAL_SHARES_REQ_BASE_XML[ReqTypes.DELETE])
        entry.get().get_node(NAME_PATH).set_text(name)
        return entry

    def __delete(self, name):
        os.environ['PATH'] = os.environ['PATH'] + ':/sbin'
        response = Request(FAL_SHARES_PATH, filter_cmd=SUDO_CMD).request(
            self.__get_request_entry(name), env=os.environ)  # FIXME
        if response.get_entrylist().get_entry().get_result() == Results.OK:
            self.set_message(fgettext(
                    'MSG_SHARE_DELETE',
                    arglist={'name': name},
                    is_log=True))
            return True
        else:
            self.set_message(fgettext(
                    'ERR_SHARE_DELETE',
                    arglist={'name': name},
                    is_log=True))
            return False

    def view(self, request, name):
        choices = self.__get_choices()
        form = DeleteForm(choices=choices, initial={'shares': [name]})
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                'form': form,
                'info_dic': self.__get_info_dic(choices),
            }
        }

    def view_delete(self, request, name):
        with self.processing(constname='ERR_SHARE_STATE_CHANGE') as p:
            form = DeleteForm(choices=self.__get_choices(), data=request.POST)
            choices = self.__get_choices()
            p.break_if_state_change()

            result = True
            if form.is_valid():
                sobj = Share()
                for name in form.cleaned_data['shares']:
                    if not self.__delete(name):
                        result = False
            p.result(result)
            choices = self.__get_choices()
            form = DeleteForm(choices=choices)
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                'form': form,
                'info_dic': self.__get_info_dic(choices),
            }
        }

    def view_list(self, request, name):
        return {self.KEY_REDIRECT:
            "/".join([ps.MENU_URL, ps.LIST_APPLI, ps.LIST_VIEW])}

delete = DeleteView.as_view()
