# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
import os
import re

from ionas.request import (Request, Entry, EntryList)
from iobase.exception import (NotFoundError)
from product_conf import SUDO_CMD
from fal_info import FAL_SHARES_PATH, Results, ReqTypes

from django.utils.translation import ugettext as _
from django.http import HttpResponseRedirect
from django.core import validators
from django.core.exceptions import ValidationError
from project.view import FuncView
from root.modules.form import BaseForm
from project.utils import importer as imp, to_importable as ipath
import settings
from .. import settings as ps

# FIXME fal_infoからとるべき情報
FAL_SHARES_REQ_BASE_XML = {ReqTypes.DELETE: "<shares><share><name/>" +
                                "</share></shares>"}

SHARELIST_PATH = "sharelist/*"
NAME_NAME = "name"
NAME_PATH = "share/name"

CHECKBOX_FIELD = "checkbox"

FIELD_SUFFIX = ".field"

SHARE_NAME_RE = FIELD_SUFFIX + "$"

SHARE_LIST_KEY = "share_list"


def root(request):
    return HttpResponseRedirect('./list')


def get_share_node_list():
    load_result = Request(FAL_SHARES_PATH, filter_cmd=SUDO_CMD).request(
        Entry(ReqTypes.LOAD)).get_entrylist().get_entry()
    try:
        share_node_list = load_result.get().get_node_list(SHARELIST_PATH)
    except NotFoundError:
        """There is nothing. initial state."""
        share_node_list = []
    return share_node_list


class Form(BaseForm):

    def __init__(self, *args, **kwargs):
        from root.modules.form.fields.checkbox import CheckboxField

        super(Form, self).__init__(*args, **kwargs)
        for node in get_share_node_list():
            share_name = node.get_node(NAME_NAME).get_text()
            self.fields[share_name + FIELD_SUFFIX] = CheckboxField(
                initial=False,
                required=False,
                validators=[])


class ListView(FuncView):

    class Icons(object):
        _installed = None

        def __init__(self):
            self._installed = self.installed()

        def installed(self):
            dict_ = {}
            try:
                dir_ = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'installed')
                for py in os.listdir(dir_):
                    if not re.search('\.py$', py):
                        continue
                    mod = imp(ipath(os.path.join(dir_, py)))
                    if 'enabled' in dir(mod) \
                        and 'name' in dir(mod) \
                        and 'icon' in dir(mod):
                        dict_[mod.name()] = mod
            except:
                pass
            return dict_

        def enabled(self, share_node):
            list_ = []
            for mod in self._installed.itervalues():
                try:
                    if mod.enabled(share_node):
                        list_.append({
                            'name': mod.name(),
                            'icon': mod.icon(),
                        })
                except:
                    pass
            return list_  # {'name':NAME, 'icon':ICON} のリスト

    views = ['add', 'delete_multi']

    def __get_share_list(self):

        def __is_enabled(node, xpath, default=False):
            try:
                return node.get_node(xpath).is_enabled()
            except:
                return default

        form = Form()
        icn = self.Icons()
        share_list = []
        for node in get_share_node_list():
            share_name = node.get_node(NAME_NAME).get_text()
            share_list.append({
                ps.NAME_INFO: share_name,
                'icons': icn.enabled(node),
                'deletable': not node.is_set_to_true('undeletable'),
                'detailpermission': __is_enabled(node, 'permission/detailpermission'),
            })
        return share_list

    def view(self, request):
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                'share_list': self.__get_share_list()}}

    def view_add(self, request):
        return {self.KEY_REDIRECT:
            "/".join([ps.MENU_URL, ps.ADD_APPLI, ps.ADD_VIEW])}


list = ListView.as_view()
