# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
import os

from ionas.request import Request
from product_conf import SUDO_CMD
from fal_info import FAL_SHARES_PATH, Results, ReqTypes

from django.http import HttpResponseRedirect
from project.view import FuncView
import settings
from .. import settings as ps
from project.fal.command import get_message as fgettext

from ..modules.utils import (
    get_request_entry, get_html_list, get_info_dic,
    get_modify_info, is_removable_share)


def root(request):
    return HttpResponseRedirect('/'.join([settings.MENU_URL, 'modify']))


class ModifyView(FuncView):
    from ..modules.form import ShareForm as Form

    views = ['modify', 'list']
    URL = '/'.join([settings.MENU_URL, 'modify'])

    def __send_request(self, form):
        os.environ['PATH'] = os.environ['PATH'] + ":/sbin"
        response = Request(FAL_SHARES_PATH, filter_cmd=SUDO_CMD).request(
            get_request_entry(form, ReqTypes.MODIFY), env=os.environ)
        if response.get_entrylist().get_entry().get_result() == Results.OK:
            self.set_message(fgettext(
                    'MSG_SHARE_INFO_CHANGE',
                    arglist={'name': form.cleaned_data['name']},
                    is_log=True))
            return True
        else:
            self.set_message(fgettext(
                    'ERR_SHARE_INFO_CHANGE',
                    arglist={'name': form.cleaned_data['name']},
                    is_log=True))
            return False

    def view(self, request, name):
        initial, is_removable = get_modify_info(name)
        form = self.Form(
            extdir="fields", modify=True, initial=initial, name=name)
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                'case': "modify",
                'is_removable': is_removable,
                'url': self.URL,
                'form': form,
                'basic_setting_html_list':
                    get_html_list(extdir="basic_setting"),
                'info_dic':
                    get_info_dic(share_name=name),
                'info_html_list':
                    get_html_list(extdir="info"),
            }
        }

    def view_modify(self, request, name):
        result = False
        with self.processing(constname='ERR_SHARE_STATE_CHANGE') as p:
            form = self.Form(
                extdir="fields", modify=True, data=request.POST, name=name)
            p.break_if_state_change()
            if form.is_valid():
                result = self.__send_request(form)
                p.result(result)
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                'case': "modify",
                'is_removable': is_removable_share(name),
                'url': self.URL,
                'form': form,
                'basic_setting_html_list':
                    get_html_list(extdir="basic_setting"),
                'info_dic':
                    get_info_dic(share_name=name),
                'info_html_list':
                    get_html_list(extdir="info"),
                'result': result
            }
        }

    def view_list(self, request, name):
        return {self.KEY_REDIRECT:
            "/".join([ps.MENU_URL, ps.LIST_APPLI, ps.LIST_VIEW])}

modify = ModifyView.as_view()
