# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django.utils.translation import ugettext as _
from django.core.exceptions import ValidationError
from root.modules.form import BaseForm

from project.forms.fields.checkbox import CheckboxField
from project.forms.fields.extras.share import ShareNameField, SelectShareField
from project.forms.fields.extras.comment import CommentField
from project.forms.fields.extras.enable import EnableField
from project.forms.fields.extras.user import MultiSelectUserField
from project.forms.fields.extras.group import MultiSelectGroupField
from project.fal.backups import Backups

from root.modules.utils import get_inss

import os
import copy


class ShareForm(BaseForm):
    name = ShareNameField(label=_(u'name'))
    desc = CommentField(label=_(u'comment'), required=False)
    readonly = CheckboxField(label=_(u'readonly'))
    noguest = CheckboxField(label=_(u'noguest'))

    detailpermission = EnableField(label=_(u'detail permission'))

    def __set_readonly_disable(self, share_name):
        if (share_name and
            Backups().is_bkup_dst(share_name)):
                self.fields['readonly'] = CheckboxField(
                    attrs={'disabled': 'disabled'},
                    label=_(u'readonly'),
                    initial=True)

    def __init__(self,
            extdir='fields',
            modify=False,
            initial=None,
            name="",
            *args, **kwargs):
        super(ShareForm, self).__init__(
                extdir=extdir,
                initial=initial,
                *args, **kwargs)
        if modify:
            self.fields['name'] = SelectShareField(label=_(u'name'))
            self.__set_readonly_disable(name)
        else:
            self.fields['detailpermission'] = EnableField(
                label=_(u'detail permission'),
                initial=True)

        self.fields['ro_user'] = MultiSelectUserField(
                label=_(u'read only user'),
                domain=True,
                selected_only=True,
                required=False)
        self.fields['rw_user'] = MultiSelectUserField(
                label=_(u'read write user'),
                domain=True,
                selected_only=True,
                required=False)
        selected_u = []
        try:
            selected_u += self['ro_user'].value() or []
            selected_u += self['rw_user'].value() or []
        except:
            pass
        self.fields['deny_user'] = MultiSelectUserField(
                label=_(u'deny user'),
                domain=True,
                exclude=selected_u,
                required=False)

        self.fields['ro_group'] = MultiSelectGroupField(
                label=_(u'read only group'),
                domain=True,
                selected_only=True,
                required=False)
        self.fields['rw_group'] = MultiSelectGroupField(
                label=_(u'read write group'),
                domain=True,
                selected_only=True,
                required=False)
        selected_g = []
        try:
            selected_g += self['ro_group'].value() or []
            selected_g += self['rw_group'].value() or []
        except:
            pass
        self.fields['deny_group'] = MultiSelectGroupField(
                label=_(u'deny group'),
                domain=True,
                exclude=selected_g,
                required=False)

    def clean(self):
        import rules as rmod
        rules = get_inss(os.path.dirname(rmod.__file__), "rules")
        for name, ins in rules.iteritems():
            ins.rule(self.cleaned_data, ValidationError, _)
        return self.cleaned_data
