# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
import os

from iobase.xmldata import (XmlNode)
from iobase.exception import (NotFoundError)
from ionas.request import (Request, Entry)
from product_conf import (SUDO_CMD)
from fal_info import (FAL_SHARES_PATH, ReqTypes)

from root.modules.utils import get_inss
from .. import settings as ps


FAL_SHARES_REQ_BASE_XML = {
    ReqTypes.MODIFY:
        "<shares><share><name/>" +
        "<desc/><path/><permission><readonly/><detailpermission>" + 
        "<readonlymember><userlist/><grouplist/></readonlymember>" + 
        "<readwritemember><userlist/><grouplist/></readwritemember>" +
        "</detailpermission></permission><shareservice>" + 
        "<msshare><trash/><hidden/><quota><limit/>" + 
        "</quota></msshare><noguest/></shareservice></share></shares>",
    ReqTypes.ADD: "<shares><share><name/>" +
        "<desc/><path/><permission><readonly/><detailpermission>" + 
        "<readonlymember><userlist/><grouplist/></readonlymember>" + 
        "<readwritemember><userlist/><grouplist/></readwritemember>" +
        "</detailpermission></permission><shareservice>" + 
        "<msshare><trash/><hidden/><quota><limit/>" + 
        "</quota></msshare><noguest/></shareservice></share></shares>"}


SHARE_NAME = "share"
NAME_NAME = "name"
DESC_NAME = "desc"
MSSHARE_PATH = "shareservice/msshare"
LIMIT_PATH = MSSHARE_PATH + "/quota/limit"
PERMISSION_PATH = "permission"
D_PERM_PATH = PERMISSION_PATH + "/detailpermission"
D_RO_PATH = D_PERM_PATH + "/readonlymember"
D_RW_PATH = D_PERM_PATH + "/readwritemember"
RO_USERLIST_PATH = D_RO_PATH + "/userlist"
RO_USERLIST_CHILDREN_PATH = D_RO_PATH + "/userlist/*"
RO_GROUPLIST_PATH = D_RO_PATH + "/grouplist"
RO_GROUPLIST_CHILDREN_PATH = D_RO_PATH + "/grouplist/*"
RW_USERLIST_PATH = D_RW_PATH + "/userlist"
RW_USERLIST_CHILDREN_PATH = D_RW_PATH + "/userlist/*"
RW_GROUPLIST_PATH = D_RW_PATH + "/grouplist"
RW_GROUPLIST_CHILDREN_PATH = D_RW_PATH + "/grouplist/*"

TARGET_SHARE_NODE_FMT_STR = 'sharelist/share[name="%s"]'
USER_NAME_XML_FMT_STR = "<user><name>%s</name></user>"
GROUP_NAME_XML_FMT_STR = "<group><name>%s</name></group>"


def __get_ext_share_info_dict(node):
    share_info_dict = {}
    import means as mmod
    means = get_inss(os.path.dirname(mmod.__file__), "means")
    for name, ins in means.iteritems():
        share_info_dict[name] = ins.get(node)
    return share_info_dict


def __get_detail_permission_share_info_dict(node):
    return {
        'detailpermission': node.get_node(D_PERM_PATH).is_set_to_true('enable'),
        ps.RW_USER_INFO: [
            user_node.get_node(NAME_NAME).get_text()
            for user_node
            in node.get_node_list(RW_USERLIST_CHILDREN_PATH)],
        ps.RO_USER_INFO: [
            user_node.get_node(NAME_NAME).get_text()
            for user_node
            in node.get_node_list(RO_USERLIST_CHILDREN_PATH)],
        ps.RW_GROUP_INFO: [
            group_node.get_node(NAME_NAME).get_text()
            for group_node
            in node.get_node_list(RW_GROUPLIST_CHILDREN_PATH)],
        ps.RO_GROUP_INFO: [
            group_node.get_node(NAME_NAME).get_text()
            for group_node
            in node.get_node_list(RO_GROUPLIST_CHILDREN_PATH)]}


def __get_basic_share_info_dict(node):
    return {
        ps.NAME_INFO: node.get_node(NAME_NAME).get_text(),
        ps.DESC_INFO: node.get_node(DESC_NAME).get_text(),
        'readonly': node.get_node(
                'permission/readonly').is_set_to_true('enable'),
        'noguest': node.get_node(
                'shareservice/noguest').is_set_to_true('enable')}


def __get_share_info_dict(node):
    share_info_dict = {}
    share_info_dict.update(
        __get_basic_share_info_dict(node))
    share_info_dict.update(
        __get_detail_permission_share_info_dict(node))
    share_info_dict.update(
        __get_ext_share_info_dict(node))
    return share_info_dict


def __get_target_share_node(name):
    load_result = Request(FAL_SHARES_PATH, filter_cmd=SUDO_CMD).request(
        Entry(ReqTypes.LOAD)).get_entrylist().get_entry()
    return load_result.get().get_node(
        TARGET_SHARE_NODE_FMT_STR % name)


def __set_ext_node(form, node):
    import means as mmod
    means = get_inss(os.path.dirname(mmod.__file__), "means")
    for name, ins in means.iteritems():
        ins.set(node, form.cleaned_data[name])


#def __set_detail_permission_data(data_list, node, fmt_str):
#    for data in data_list:
#        node.append_child(XmlNode(fmt_str % str(data)))
#
def __set_detail_permission_data(data_list, node, fmt):
    fmt = fmt.encode('utf-8') if isinstance(fmt, unicode) else fmt
    for data in data_list:
        data = data.encode('utf-8') if isinstance(data, unicode) else data
        node.append_child(XmlNode(fmt % data))


def __set_detail_permission_node(form, node):
    node.get_node(D_PERM_PATH).set_bool_attr(
            'enable',
            form.cleaned_data['detailpermission'])
    __set_detail_permission_data(
        form.cleaned_data[ps.RO_USER_INFO],
        node.get_node(RO_USERLIST_PATH),
        USER_NAME_XML_FMT_STR)
    __set_detail_permission_data(
        form.cleaned_data[ps.RW_USER_INFO],
        node.get_node(RW_USERLIST_PATH),
        USER_NAME_XML_FMT_STR)
    __set_detail_permission_data(
        form.cleaned_data[ps.RO_GROUP_INFO],
        node.get_node(RO_GROUPLIST_PATH),
        GROUP_NAME_XML_FMT_STR)
    __set_detail_permission_data(
        form.cleaned_data[ps.RW_GROUP_INFO],
        node.get_node(RW_GROUPLIST_PATH),
        GROUP_NAME_XML_FMT_STR)


def __set_basic_node(form, share_node):
    share_node.get_node(NAME_NAME).set_text(
            form.cleaned_data[ps.NAME_INFO])
    share_node.get_node(DESC_NAME).set_text(
            form.cleaned_data[ps.DESC_INFO])
    share_node.get_node('permission/readonly').set_bool_attr(
            'enable', form.cleaned_data['readonly'])
    share_node.get_node('shareservice/noguest').set_bool_attr(
            'enable', form.cleaned_data['noguest'])


def __is_removable(node):
    return node.is_set_to_true("undeletable")


def get_request_entry(form, reqtype):
    entry = Entry(reqtype, FAL_SHARES_REQ_BASE_XML[reqtype])
    node = entry.get().get_node(SHARE_NAME)
    __set_basic_node(form, node)
    __set_detail_permission_node(form, node)
    __set_ext_node(form, node)
    return entry


def get_modify_info(name):
    if name:
        node = __get_target_share_node(name)
        initial =  __get_share_info_dict(node)
        is_removable = __is_removable(node)
        return (initial, is_removable)
    return ({}, False)


def is_removable_share(name):
    if name:
        node = __get_target_share_node(name)
        is_removable = __is_removable(node)
        return is_removable
    return False


def get_html_list(extdir):
    curdir = os.path.dirname(__file__)
    dpath = os.path.join(curdir, "html", extdir)
    return [os.path.join(dpath, fname)
        for fname
        in sorted(os.listdir(dpath))]


def get_info_dic(share_name="", **kwargs):
    import info
    info_dic = {}
    get_name_list = [
        attr for attr in dir(info) if attr.startswith(info.PREFIX)]
    for get_name in get_name_list:
        get_func = getattr(info, get_name)
        info_dic.update(get_func(share_name, **kwargs))
    return info_dic


def get_quota_disable_msg_list(share_dic):
    import quota
    quota_disable_msg_list = []
    get_name_list = [
        attr for attr in dir(quota) if attr.startswith(quota.PREFIX)]
    for get_name in get_name_list:
        get_func = getattr(quota, get_name)
        quota_disable_msg_list += get_func(share_dic)
    return quota_disable_msg_list


def get_tip_node(node, fullpath):
    for path in fullpath.split("/"):
        try:
            node.get_node(path)
        except NotFoundError:
            node.append_child(path)
        finally:
            node = node.get_node(path)
    return node



class BaseMeans(object):
    PATH = ""
    IS_BOOL = True
    DEFAULT = False

    def _set_conv(self, val):
        if self.IS_BOOL:
            return {True: 'true', False: 'false'}.get(val, 'false')
        if isinstance(val, unicode):
            return val.encode("utf8")
        return str(val)

    def _get_conv(self, val):
        if val == None:
            return self.DEFAULT
        if self.IS_BOOL:
            return {'true': True, 'false': False}.get(val.lower(), False)
        return str(val)

    def _get_path(self, path=""):
        if not path:
            return self.PATH
        return path

    def _set_attr(self, node, val, path=""):
        path = self._get_path(path)
        par_path = "/".join(path.split("/")[:-1])
        attr_name = path.split("/")[-1].lstrip("@")
        tip_node = get_tip_node(node, par_path)
        tip_node.set_attrs({attr_name: self._set_conv(val)})

    def _set_text(self, node, val, path=""):
        path = self._get_path(path)
        get_tip_node(node, path).set_text(self._set_conv(val))

    def _is_attr_path(self, path=""):
        path = self._get_path(path)
        return self.PATH.split("/")[-1].startswith("@")

    def get(self, node, path=""):
        path = self._get_path(path)
        return self._get_conv(node.get_value(path, None))

    def set(self, node, val, path=""):
        path = self._get_path(path)
        if self._is_attr_path(path):
            self._set_attr(node, val, path)
        else:
            self._set_text(node, val, path)
