# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from ionas.request import (Request, Entry)
from product_conf import SUDO_CMD
from fal_info import FAL_SHARES_PATH, Results, ReqTypes

from django.utils.translation import ugettext as _
from django.http import HttpResponseRedirect
from project.view import FuncView
from root.modules.form import BaseForm

from project.fal.command import get_message as fgettext
from project.fal.share import Share
from project.fal.quota import Quota
from project.forms.fields.extras.quota import QuotaLimitField

from ..modules.utils import get_quota_disable_msg_list


def root(request):
    return HttpResponseRedirect('./quota')


class QuotaView(FuncView):
    class Form(BaseForm):
        def __init__(self, sharelist, *args, **kwargs):
            super(QuotaView.Form, self).__init__(*args, **kwargs)
            for share in sharelist:
                self.fields[share['id']] = QuotaLimitField(
                        label=share['name'],
                        required=False,
                        initial=share['quota_limit'] or 0.0)

    views = ['modify']

    def _modify(self, name, quota):
            response = Request(FAL_SHARES_PATH, filter_cmd=SUDO_CMD).request(
                    Entry(ReqTypes.MODIFY,
                    """<shares>
                        <share>
                            <name>%(name)s</name>
                            <shareservice>
                                <msshare>
                                    <quota><limit>%(quota)s</limit></quota>
                                </msshare>
                            </shareservice>
                        </share>
                    </shares>""" % {'name': name, 'quota': quota}))
            if response.get_entrylist().get_entry().get_result() == Results.OK:
                return True
            else:
                return False

    def _get_sharelist(self):
        useds = Share().query()
        sharelist = [
            share
            for share
            in Share().load().itervalues()
            if not share.get('undeletable', False)]
        for i, params in enumerate(sorted(sharelist, key=lambda x:x['name'])):
            params['id'] = str(i)
            params['used'] = useds.get(params['name'], 0.0)
            params['disable_msg_list'] = get_quota_disable_msg_list(params)
        return sharelist

    _sharelist = None

    def view(self, request):
        self._sharelist = self._get_sharelist()
        form = self.Form(self._sharelist)
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                'enable': Quota().is_enabled(),
                'form': form,
                'share_list': [{
                    'id': share['id'],
                    'name': share['name'],
                    'quota_used': share['used'],
                    'quota_field': form[share['id']],
                    'disable_msg_list': share['disable_msg_list']} \
                    for share in self._sharelist]}}

    def view_modify(self, request):
        with self.processing(constname='ERR_QUOTA_STATE_CHANGE') as p:
            self._sharelist = self._sharelist or self._get_sharelist()
            form = self.Form(self._sharelist, data=request.POST)
            p.break_if_state_change()

            if form.is_valid():
                result = True
                for share in self._sharelist:
                    quota = form.cleaned_data[share['id']]
                    if not self._modify(share['name'], float(quota or 0)):
                        result = False
                if result:
                    self.set_message(fgettext('MSG_QUOTA_CHANGE', is_log=True))
                else:
                    self.set_message(fgettext('ERR_QUOTA_CHANGE', is_log=True))
            p.result(result)
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                'enable': Quota().is_enabled(),
                'form': form,
                'share_list': [{
                    'id': share['id'],
                    'name': share['name'],
                    'quota_used': share['used'],
                    'quota_field': form[share['id']],
                    'disable_msg_list': share['disable_msg_list']} \
                    for share in self._sharelist]}}

quota = QuotaView.as_view()
