# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from ionas.request import (Request, Entry)
from product_conf import SUDO_CMD
from fal_info import (FAL_MSNETWORK_PATH, FAL_FTP_PATH,
        FAL_APPLENETWORK_PATH, ReqTypes, Results)


MS_SERVICE_ENTRY = "<msnetwork><service enable='true'/></msnetwork>"
FTP_ENTRY = "<ftp><service enable='true'/><port type='control'><number/></port><port type='transfer'><number/></port><encode/></ftp>"
APPLE_ENTRY = "<appleshare><service enable='true'/></appleshare>"


class Protocol(object):
    def msnetwork_set(self, data):
        entry = Entry(ReqTypes.APPLY, MS_SERVICE_ENTRY)
        if data["msnetwork"] == "0":
            entry.get().get_node("service").enable(False)
        else:
            entry.get().get_node("service").enable(True)
        return self.send_request(FAL_MSNETWORK_PATH, entry)

    def send_request(self, cmd, entry):
        try:
            response = Request(cmd,
                filter_cmd=SUDO_CMD).request(entry).get_entrylist().get_entry()
            if response.get_result() == Results.OK:
                return True
            else:
                return False
        except:
            return False

    def get_msnetwork_config(self):
        init = {"msnetwork": 1}
        try:
            result = Request(FAL_MSNETWORK_PATH, filter_cmd=SUDO_CMD).request(
                           Entry(ReqTypes.LOAD)).get_entrylist().get_entry()
        except:
            return init
        if not result.get().get_node("service").is_enabled():
            init["msnetwork"] = 0
        return init

    def get_ftp_config(self):
        init = {"ftp": 0, "port": "", "encode": 1}
        try:
            result = Request(FAL_FTP_PATH, filter_cmd=SUDO_CMD).request(
                           Entry(ReqTypes.LOAD)).get_entrylist().get_entry()
        except:
            return init
        if result.get().get_node("service").is_enabled():
            init["ftp"] = 1
        init["port"] = result.get().get_node(
                        "./port[@type='control']/number").get_text()
        if result.get().get_node("encode").get_text() == "shift-jis":
            init["encode"] = 0
        return init

    def ftp_set(self, data):
        encode_dic = {"0": "shift-jis", "1": "utf-8"}
        entry = Entry(ReqTypes.APPLY, FTP_ENTRY)
        if data["ftp"] == "0":
            entry.get().get_node("service").enable(False)
        else:
            entry.get().get_node("service").enable(True)
        entry.get().get_node(
            "./port[@type='control']/number").set_text(str(data["port"]))
        entry.get().get_node("encode").set_text(encode_dic[data["encode"]])
        return self.send_request(FAL_FTP_PATH, entry)

    def get_appleshare_config(self):
        info = {"appleshare": 0}
        try:
            result = Request(FAL_APPLENETWORK_PATH, filter_cmd=SUDO_CMD
                ).request(Entry(ReqTypes.LOAD)).get_entrylist().get_entry()
        except:
            return info
        if result.get().get_node("service").is_enabled():
            info["appleshare"] = 1
        return info

    def appleshare_set(self, data):
        entry = Entry(ReqTypes.APPLY, APPLE_ENTRY)
        if data["appleshare"] == "0":
            entry.get().get_node("service").enable(False)
        else:
            entry.get().get_node("service").enable(True)
        return self.send_request(FAL_APPLENETWORK_PATH, entry)
