# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django.utils.translation import ugettext as _
from django.http import HttpResponseRedirect
from django import forms
from project.view import FuncView
from root.modules.form import BaseForm
import settings
from project.forms.fields.extras.landiskpass import LanDiskPassField
from project.forms.fields.extras.email import EmailField
from project.fal.user import User

ADMIN = 'admin'

DEFAULT_TIMEOUT = 10  # min


def root(request):
    return HttpResponseRedirect('/'.join([settings.MENU_URL, 'admin']))


class View(FuncView):
    class Form(BaseForm):
        password1 = LanDiskPassField(
            label=_(u'Password'),
            min_length=4,
            required=True)
        password2 = LanDiskPassField(
            label=_(u'Password(Confirm)'),
            min_length=4,
            required=True)
        email = EmailField(label=_(u'Email'), required=False)

        timeout = forms.IntegerField(label=_(u"Label: Login Timeout Minutes"),
                                     min_value=5,
                                     max_value=60,
                                     initial=DEFAULT_TIMEOUT,
                                     required=True)

        def clean(self):
            from django import forms
            password1 = self.cleaned_data.get('password1', None)
            password2 = self.cleaned_data.get('password2', None)
            if password1 != password2:
                raise forms.ValidationError(_(u'Password unmatch.'))
            return self.cleaned_data

    views = ['apply']

    def view(self, request):
        user = User()
        info = user.load(all=True)[ADMIN]
        initial = {
            'password1': info['passwd'],
            'password2': info['passwd'],
            'email': info['mailaddr'],
            'timeout': info['timeout'] or DEFAULT_TIMEOUT,
        }
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {'form': self.Form(initial=initial)}}

    def view_apply(self, request):
        with self.processing(constname='ERR_ADMIN_STATE_CHANGE') as p:
            form = self.Form(data=request.POST)
            p.break_if_state_change()
            if form.is_valid():
                user = User()
                user.adminmod(passwd=form.cleaned_data.get('password1', ''),
                              mailaddr=form.cleaned_data.get('email', ''),
                              timeout=form.cleaned_data.get('timeout', None))
                self.set_message(user.get_message())
                p.result(user.get_result(boolean=True))
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {'form': form}}


admin = View.as_view()
