# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django.utils.translation import ugettext as _
from django.http import HttpResponseRedirect
from django.core.exceptions import ValidationError
from project.view import FuncView
from project.fal.command import get_message
from root.modules.form import BaseForm
import settings
from root.admin.initset.modules.initsetting import InitSetting


def root(request):
    return HttpResponseRedirect('/'.join([settings.MENU_URL, 'datetime']))


class View(FuncView):
    class Form(BaseForm):
        from root.modules.form.fields.radio import RadioField
        from root.modules.form.fields.checkbox import CheckboxField
        from root.modules.form.fields.select import SelectField
        from project.forms.fields.extras.datetime import SelectDateTimeField
        from project.forms.fields.extras.fqdn import FQDNField

        sync = RadioField(
                label=_(u'Datetime setting'),
                choices=((0, _(u'manual')), (1, _(u'syncronous'))),
                initial=0)

        datetime = SelectDateTimeField(
                label=_(u'Datetime'),
                required=False,
                show_seconds=False)

        ntpsrv = FQDNField(
                    label=_(u'NTP Server'),
                    max_length=128,
                    required=False)
        boot = CheckboxField(label=_(u'Boot'), required=False)
        daily = CheckboxField(label=_(u'Daily'), required=False)
        hour = SelectField(
                    choices=((h, h) for h in range(0, 24)),
                    label=_(u'hour'),
                    initial=0,
                    required=False)
        minute = SelectField(
                    choices=((h, h) for h in range(0, 60, 5)),
                    label=_(u'minute'),
                    initial=0,
                    required=False)

        def clean(self):
            if self.cleaned_data.get("sync") == "1":
                if self.cleaned_data.get("ntpsrv") == "":
                    raise ValidationError(_(u'"NTP Server" is required.'))
                if not self.cleaned_data.get("boot"):
                    if not self.cleaned_data.get("daily"):
                        raise ValidationError(_(u'"Syncronous Timing" is required.'))
            else:
                if not self.cleaned_data.get("datetime", True):
                    # NOTE: cleaned_dataにdatetimeがないのは、
                    #       datetimeのvalidateでエラーになったということ。
                    #       よって未入力エラーを出す必要はない。
                    raise ValidationError(_(u'This field is required.'))
            return self.cleaned_data

    views = ['apply']
    initsetting = InitSetting()

    def view(self, request):
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                'form': self.Form(initial=self.initsetting.get_time_info())}}

    def view_apply(self, request):
        with self.processing(constname="ERR_DATE_STATE_CHANGE") as p:
            form = self.Form(data=request.POST)
            rst = False
            p.break_if_state_change()

            if form.is_valid():
                clean_data = form.cleaned_data
                if self.initsetting.time_set(data=clean_data):
                    self.set_message(
                        get_message('MSG_DATE_CHANGE', is_log=True))
                    rst = True
                else:
                    self.set_message(
                        get_message('ERR_DATE_CHANGE', is_log=True))
                if clean_data.get("sync") == "1":
                    self.initsetting.sync_now()
                p.result(rst)

                # 変わった時刻に合わせてログイン時刻を更新する
                from project.auth import Authentication
                Authentication().update(request)

        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {'form': form}}

datetime = View.as_view()
