# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
import os
import glob
import time
from iobase.cmdexec import (CmdExec, NULL)
from django.utils.translation import ugettext as _
from django.http import HttpResponseRedirect
from project.view import FuncView
from project.fal.command import get_message
from ..modules.firmware import Firmware
from root.modules.form import BaseForm
from django.core.exceptions import ValidationError
import settings
from fal_info import Results
from project.fal.command import logging
from project.fal.buzzer import Buzzer
from project.fal.led import LED
from product_conf import (TEMP_ALT_DIR, SUDO_CMD, RM_CMD, MKDIR_CMD)
from root.modules.form.fields.radio import RadioField
from project.forms.fields.extras.file import FileField

UPDATE = "update"

# FIXME. albg_config.pyを参照
FIRMWARE_UNTAR_DIR_BASE = TEMP_ALT_DIR + "/firmware/untar"
FIRMWARE_TEMP_DIR = TEMP_ALT_DIR + "/firmware"

REBOOT_FLAG_FILE = "doreboot"
SHUTDOWN_FLAG_FILE = "doshutdown"
ANY_STR = "*"


def root(request):
    return HttpResponseRedirect('/'.join([settings.MENU_URL, 'update']))

def update_start():
    led = LED()
    led.status(True, blink=True, color='green')

def update_over(result):
    buz = Buzzer()
    led = LED()
    led.status(True, blink=False, color='green')
    if result:
        buz.ok()
    else:
        buz.error()

def get_code(msg):
    try:
        from fal_msginfo import Msg
        return getattr(Msg, msg)
    except:
        return None
    
def del_old_untar_folder():
    dirlist = glob.glob(FIRMWARE_UNTAR_DIR_BASE + ANY_STR)        
    if not dirlist:
        return
    for dir in dirlist:
        try:
            CmdExec([SUDO_CMD, RM_CMD, "-rf", dir],
                            stdout=NULL, stderr=NULL).execute()
        except:
            pass
    return 

def get_flag_name():
    i = 0
    while i < 30:
        dirlist = glob.glob(FIRMWARE_UNTAR_DIR_BASE + ANY_STR)        
        if not dirlist:
            time.sleep(2)
        else:
            break
        i += 1

    if dirlist:
        if os.path.exists(os.path.join(dirlist[0], REBOOT_FLAG_FILE)):
            return "reboot"
        if os.path.exists(os.path.join(dirlist[0], SHUTDOWN_FLAG_FILE)):
            return "shutdown"
    return ""


class View(FuncView):
    views = ['update']

    class SelectForm(BaseForm):
        method = RadioField(
            label=_(u'Update Method'),
            choices=((0, _(u'Downloading Firmware File from Server')),
                    (1, _(u'Assigning Firmware File'))),
            initial=0,
            lengthwise=True,
            required=True)

    def _get_cur_ver_str(self):
        _ver = Firmware().get_cur_ver()
        if _ver < 10 and len(str(_ver)) < 4:
            ver = "%.2f" % _ver
        else:
            ver = _ver
        return ver

    def view(self, request):
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                'cur_ver': self._get_cur_ver_str(),
                'form': self.SelectForm()}}

    def view_update(self, request):
        form = self.SelectForm(data=request.POST)
        if form.is_valid():
            if form.cleaned_data.get("method") == "1":
                dest = "/".join([settings.MENU_URL, "assign"])
            else:
                dest = "/".join([settings.MENU_URL, "auto"])

        return {self.KEY_REDIRECT: dest}


class AutoView(FuncView):
    views = ['execute', 'cancel']
    firm = Firmware()

    def view(self, request):
        infos = self.firm.get_firm_info_data()
        if infos.get("connect") != "ok":
            self.set_message(_(u'Can not connect to server. Please confirm if the network is connected to internet.'))
        return {
            self.KEY_TEMPLATE: os.path.join(self.TEMPLATES, "auto.html"),
            self.KEY_TEMPLATE_DIC: {"infos": infos}}

    def view_execute(self, request):
        update_start()
        rst = False
        try:
            del_old_untar_folder()
            result = self.firm.update_request()
            if result == Results.OK:
                rst = True
                self.set_message(get_message('MSG_FIRMWARE_UPDATE_START'))
                msg_flag = get_flag_name()
                if msg_flag == "reboot":
                    self.set_message(_(u'System will restart. Please close the browser.'))
                elif msg_flag == "shutdown":
                    self.set_message(_(u'System will shutdown. Please close the browser.'))
            elif result == Results.BUSY_ERROR:
                self.set_message(
                      get_message('ERR_FIRMWARE_STATE_CHANGE', is_log=True))
            else:
                if result == Results.INVALID_FILE:
                    logging(get_code("ERR_FIRMWARE_INVALID_FIRMWARE"))
                self.set_message(get_message('ERR_FIRMWARE_FAIL_FIRMWARE'))
        finally:
            update_over(rst)
        return {
            self.KEY_TEMPLATE: os.path.join(self.TEMPLATES, "auto.html"),
            self.KEY_TEMPLATE_DIC: {}}

    def view_cancel(self, request):
        return {self.KEY_REDIRECT: "/".join([settings.MENU_URL, "update"])}


class AssignView(FuncView):
    views = ['execute', 'cancel']

    class AssignForm(BaseForm):
        firm_file = FileField(
            label=_(u'Firmware File'),
            required=False)

        def clean(self):
            file = self.cleaned_data.get("firm_file")
            if not file:
                raise ValidationError(_(u'"Firmware File" is required.'))
            return self.cleaned_data

    def view(self, request):
        form = self.AssignForm()
        return {
            self.KEY_TEMPLATE: os.path.join(self.TEMPLATES, "assign.html"),
            self.KEY_TEMPLATE_DIC: {"form": form}}

    def __upload_file(self, firm_file, dest_path):
        CmdExec([SUDO_CMD, RM_CMD, "-rf", FIRMWARE_TEMP_DIR],
                stdout=NULL, stderr=NULL).execute()
        CmdExec([SUDO_CMD, MKDIR_CMD, "-m", "777", FIRMWARE_TEMP_DIR],
                stdout=NULL, stderr=NULL).execute()

        dest = open(dest_path, "wb")
        for chunk in firm_file.chunks():
            dest.write(chunk)
        dest.close()

    def view_execute(self, request):
        form = self.AssignForm(data=request.POST, files=request.FILES)
        firm = Firmware()
        update_start()
        rst = False
        try:
            del_old_untar_folder()
            if form.is_valid():
                firm_file = form.cleaned_data["firm_file"]
                dest_path = os.path.join(FIRMWARE_TEMP_DIR, firm_file.name)
                self.__upload_file(firm_file, dest_path)

                result = firm.update_request(file=dest_path)
                if result == Results.OK:
                    rst = True
                    self.set_message(get_message('MSG_FIRMWARE_UPDATE_START'))
                    msg_flag = get_flag_name()
                    if msg_flag == "reboot":
                        self.set_message(_(u'System will restart. Please close the browser.'))
                    elif msg_flag == "shutdown":
                        self.set_message(_(u'System will shutdown. Please close the browser.'))
                elif result == Results.BUSY_ERROR:
                    self.set_message(
                        get_message('ERR_FIRMWARE_STATE_CHANGE', is_log=True))
                else:
                    if result == Results.INVALID_FILE:
                        logging(get_code("ERR_FIRMWARE_INVALID_FIRMWARE"))
                    self.set_message(get_message('ERR_FIRMWARE_FAIL_FIRMWARE'))
        finally:
            update_over(rst)

        return {
            self.KEY_TEMPLATE: os.path.join(self.TEMPLATES, "assign.html"),
            self.KEY_TEMPLATE_DIC: {'form': form}}

    def view_cancel(self, request):
        return {self.KEY_REDIRECT: "/".join([settings.MENU_URL, "update"])}


update = View.as_view()
auto = AutoView.as_view()
assign = AssignView.as_view()
