# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from ionas.request import (Request, Entry)
from django.utils.translation import ugettext as _
from django.http import HttpResponseRedirect
from project.view import FuncView
from root.modules.form import BaseForm
from project.fal.command import (get_message, logging)
import settings
from product_conf import SUDO_CMD
from project.fal.raidctrl import Raidctrl
from project.fal.volctrl import Volctrl
from fal_info import (FAL_INITIALIZE_PATH,
                ReqTypes, Results)

INIT_REQ_XML = "<initialize><option zeroclear='false'/></initialize>"
ENABLE = "enable"
DEGRADE_STS = "degrade"
MIRR_STS = "mirroring"


def root(request):
    return HttpResponseRedirect('/'.join([settings.MENU_URL, 'init']))
    
class View(FuncView):
    class Form(BaseForm):
        from root.modules.form.fields.checkbox import CheckboxField
        enable = CheckboxField(
                label=_(u'Erase internal HDD completely.'),  # FIXME
                initial=False,
                required=True,
                validators=[])

    views = ['execute']

    def __get_request_entry(self, data):
        entry = Entry(ReqTypes.INITIALIZE, INIT_REQ_XML)
        entry.get().get_node("option").set_attrs(
                          {"zeroclear": str(data[ENABLE]).lower()})
        return entry
        
    def __send_request(self, cmd, req_entry):
        try:
            response = Request(cmd, filter_cmd=SUDO_CMD).request(req_entry)
            result = response.get_entrylist().get_entry().get_result()
            return result
        except:
            return Results.INITIALIZE_ERROR

    def __mirr_mode_enabled(self):
        volctrl = Volctrl()
        info = volctrl.query()
        if info.get("status") == MIRR_STS:
            return True
        return False

    def __all_disk_available(self):
        rdctrl = Raidctrl()
        info = rdctrl.query()
        if info.get("status") == DEGRADE_STS:
            return False
        return True

    def __initialize_enable(self):
        if self.__mirr_mode_enabled():
            return False
        if not self.__all_disk_available():
            return False
        return True

    def __get_code(self, msg):
        try:
            from fal_msginfo import Msg
            return getattr(Msg, msg)
        except:
            return None

    def __set_noavail_message(self):
        self.set_message(_(u'Can not initialize system.'))
        self.set_message(_(u'Please check if all internal disks are connected.'))
        self.set_message(_(u'And please disconnect the mirroring disk if it is enabled.'))

    def view(self, request):
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {'form': self.Form(initial={})}}

    def view_execute(self, request):
        with self.processing(constname="ERR_SYS_INIT_STATE_CHANGE") as p:
            form = self.Form(data=request.POST)
            rst = False
            p.break_if_state_change()

            if form.is_valid():
                if self.__initialize_enable():
                    result = self.__send_request(
                        FAL_INITIALIZE_PATH,
                        self.__get_request_entry(form.cleaned_data))
                    if result == Results.OK:
                        logging(self.__get_code("MSG_SYS_INIT_START"))
                        self.set_message(_(u'Starting to initialize system. System will restart. Please close the browser.'))
                        rst = True
                    elif result == Results.DEVICE_NOT_FOUND:
                        self.__set_noavail_message()
                    else:
                        self.set_message(
                                get_message('ERR_SYS_INIT_FAIL', is_log=True))
                    p.result(rst)
                else:
                    self.__set_noavail_message()

        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {'form': form}}

init = View.as_view()
