# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from iobase.exception import NotFoundError
from ionas.request import (Request, Entry)
from django.utils.translation import ugettext as _
from django.http import HttpResponseRedirect
from project.view import FuncView
from project.fal.command import get_message
from root.modules.form import BaseForm
import settings
from product_conf import SUDO_CMD
from fal_info import (FAL_LED_PATH, ReqTypes, Results)

FAL_LED_REQ_BASE_XML = "<led><brightness/></led>"
BRIGHTNESS_XPATH = "./brightness"
HIGH = "high"
MIDDLE = "middle"
LOW = "low"
BRIGHTNESS = "brightness"


def root(request):
    return HttpResponseRedirect('/'.join([settings.MENU_URL, 'led']))
    
class View(FuncView):
    class Form(BaseForm):
        from root.modules.form.fields.radio import RadioField
        brightness = RadioField(
                choices=((1, _(u'dark')), (2, _(u'medium')), (3, _(u'bright'))),
                label=_(u'LED Brightness:'),  # FIXME
                initial=2,
                required=True,
                validators=[])

    views = ['apply']

    def __get_config_init(self):
        init = {BRIGHTNESS: 2}
        try:
            result = Request(FAL_LED_PATH, filter_cmd=SUDO_CMD).request(
                    Entry(ReqTypes.LOAD)).get_entrylist().get_entry()
        except:
            return init
        try:
            config = result.get().get_node(BRIGHTNESS_XPATH).get_text()
            if config == LOW:
                init[BRIGHTNESS] = 1
            elif config == HIGH:
                init[BRIGHTNESS] = 3 
        except NotFoundError:
            pass
        return init

    def __get_request_entry(self, data):
        dic = {"1": LOW, "2": MIDDLE, "3": HIGH}
        entry = Entry(ReqTypes.APPLY, FAL_LED_REQ_BASE_XML)
        entry.get().get_node(BRIGHTNESS_XPATH).set_text(dic[data[BRIGHTNESS]])
        return entry
        
    def __send_request(self, cmd, req_entry):
        try:
            response = Request(cmd, filter_cmd=SUDO_CMD).request(req_entry)
            if response.get_entrylist().get_entry().get_result() == Results.OK:
                return True
            else:
                return False
        except:
            return False

    def view(self, request):
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                'form': self.Form(initial=self.__get_config_init())}}

    def view_apply(self, request):
        with self.processing(constname="ERR_LAMP_STATE_CHANGE") as p:
            form = self.Form(data=request.POST)
            rst = False
            p.break_if_state_change()

            if form.is_valid():
                if self.__send_request(FAL_LED_PATH,
                        self.__get_request_entry(form.cleaned_data)):
                    self.set_message(
                            get_message('MSG_LAMP_CHANGE', is_log=True))
                    rst = True
                else:
                    self.set_message(
                            get_message('ERR_LAMP_CHANGE', is_log=True))
                p.result(rst)

        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {'form': form}}

led = View.as_view()
