# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django.utils.translation import ugettext as _
from django.http import HttpResponseRedirect
from project.view import FuncView
from project.fal.command import get_message
from root.modules.form import BaseForm
import settings
from project.forms.fields.text import TextField
from project.forms.fields.extras.landiskname import LandiskNameField
from project.forms.fields.extras.comment import CommentField
from root.admin.initset.modules.initsetting import InitSetting
from project.fal.sysinfo import SysInfo


def root(request):
    return HttpResponseRedirect('./name')


class View(FuncView):
    class Form(BaseForm):
        name = LandiskNameField(label=_(u'LAN DISK Name'))
        desc = CommentField(label=_(u'LAN DISK Description'), required=False)
        location = CommentField(label=_(u'LAN DISK Location'), required=False)

    views = ['apply']
    initsetting = InitSetting()

    def view(self, request):
        sobj = SysInfo()
        sinfo = sobj.query()
        initial = {
            'name': sinfo['host']['name'],
            'desc': sinfo['host']['desc'],
            'location': sinfo['host']['location'],
        }
        form = self.Form(initial=initial)
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {'form': form}}

    def view_apply(self, request):
        with self.processing(constname="ERR_LANDISK_NAME_STATE_CHANGE") as p:
            form = self.Form(data=request.POST)
            rst = False
            p.break_if_state_change()

            if form.is_valid():
                name = form.cleaned_data['name']
                desc = form.cleaned_data['desc']
                location = form.cleaned_data['location']

                sobj = SysInfo()
                rst = sobj.apply(
                        hostname=name,
                        hostdesc=desc,
                        hostlocation=location)
                self.set_message(sobj.get_message())
                p.result(rst)

        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {'form': form}}


name = View.as_view()
