# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django.utils.translation import ugettext as _
from django.http import HttpResponseRedirect
from django.core.exceptions import ValidationError
from project.view import FuncView
from project.fal.command import get_message
from root.modules.form import BaseForm
from project.forms.fields.extras.portnum import RemotePortNumberField
from project.forms.fields.extras.fqdn import FQDNField
from project.forms.fields.extras.email import EmailField
from project.forms.fields.extras.enable import EnableField


def root(request):
    return HttpResponseRedirect('./basic')


class View(FuncView):
    from ...modules.mail import Mail

    class Form(BaseForm):
        from root.modules.form.fields.text import TextField
        from root.modules.form.fields.radio import RadioField
        from root.modules.form.fields.select import SelectField
        from root.modules.form.fields.password import PasswordField

        host = FQDNField(label=_(u'SMTP Server'), required=True)
        port = RemotePortNumberField(
                label=_(u'SMTP Port'),
                initial=25,
                required=True)
        ssl = EnableField(
                label=_(u"Use SSL"),
                initial=False)
        fromaddr = EmailField(label=_(u'Sender Address'), required=True)
        encode = RadioField(
                label=_(u'Mail Encode'),
                choices=((0, _(u'UTF-8')), (1, _(u'ISO-2022-JP'))),
                initial=1,
                required=True,
                validators=[])
        auth = SelectField(
                label=_(u'Authentication'),
                choices=((0, _(u'No Authentication')),
                         (1, _(u'POP before SMTP')),
                         (2, _(u'SMTP AUTH(PLAIN)')),
                         (3, _(u'SMTP AUTH(LOGIN)')),
                         (4, _(u'SMTP AUTH(CRAM-MD5)'))),
                initial=0,
                required=True,
                validators=[])
        popsrv = FQDNField(label=_(u'POP Server'), required=False)
        popport = RemotePortNumberField(
                label=_(u'POP Port'),
                initial=110,
                required=False)
        user = TextField(
                label=_(u'User Name'),
                max_length=155,
                required=False)
        password = PasswordField(
                label=_(u'Password'),
                max_length=20,
                required=False)

        def clean(self):
            auth = self.cleaned_data.get("auth")
            if auth == "0":
                return self.cleaned_data
            elif auth == "1":
                if self.cleaned_data.get("popsrv") == "":
                    raise ValidationError(_(u'"POP Server" is required.'))
                if "popport" in self.cleaned_data.keys():
                    if self.cleaned_data.get("popport") == None:
                        raise ValidationError(_(u'"POP Port" is required.'))
                if self.cleaned_data.get("user") == "":
                    raise ValidationError(_(u'"User Name" is required.'))
            else:
                if self.cleaned_data.get("user") == "":
                    raise ValidationError(_(u'"User Name" is required.'))
            return self.cleaned_data

    views = ['apply']
    mail = Mail()

    def view(self, request):
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                'form': self.Form(initial=self.mail.get_basic_init_config())}}

    def view_apply(self, request):
        with self.processing(constname="ERR_MAIL_STATE_CHANGE") as p:
            form = self.Form(data=request.POST)
            rst = False
            p.break_if_state_change()

            if form.is_valid():
                if self.mail.basic_set(form.cleaned_data):
                    self.set_message(
                            get_message('MSG_MAIL_CHANGE', is_log=True))
                    rst = True
                else:
                    self.set_message(
                            get_message('ERR_MAIL_CHANGE', is_log=True))
                p.result(rst)

        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {'form': form}}

basic = View.as_view()
