# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django.utils.translation import ugettext as _
from django.http import HttpResponseRedirect
from django import forms
from project.view import FuncView
from project.fal.command import get_message
from root.modules.form import BaseForm
from ...modules.mail import (Mail, MAX_MAIL_NUM)
from project.forms.fields.extras.portnum import RemotePortNumberField
from project.forms.fields.extras.fqdn import FQDNField
from project.forms.fields.extras.email import EmailField
from project.forms.fields.extras.enable import EnableField


def root(request):
    return HttpResponseRedirect('./mailnotice')


class Form(BaseForm):
    def __init__(self, *args, **kwargs):
        from root.modules.form.fields.text import TextField
        from root.modules.form.fields.checkbox import CheckboxField
        from root.modules.form.fields.radio import RadioField
        from root.modules.form.fields.select import SelectField
        from root.modules.form.fields.password import PasswordField
        super(Form, self).__init__(*args, **kwargs)

        idx = 1
        while idx <= MAX_MAIL_NUM:
            num = str(idx)
            self.fields["mail" + num] = CheckboxField(
                label='%s%s' % (_(u'Mail'), num),  
                initial=False,
                required=False,
                validators=[])
            self.fields["toaddr" + num] = EmailField(
                label="%s%s: %s" % (_(u'Mail'), num, _(u'Mail Address')),
                widget=forms.TextInput(attrs={"class": "items_mail" + num}),
                required=False)
            self.fields["detail" + num] = CheckboxField(
                label=_(u'Detail Setting'),
                widget=forms.CheckboxInput(attrs={"class": "items_mail" + num}),
                initial=False,
                required=False,
                validators=[])
            self.fields["system" + num] = CheckboxField(
                label=_(u'System Event'),
                widget=forms.CheckboxInput(attrs={"class": "items_mail" + num}),
                initial=False,
                required=False,
                validators=[])
            self.fields["backup" + num] = CheckboxField(
                label=_(u'Backup/Datacopy Result'),  #FIXME
                widget=forms.CheckboxInput(attrs={"class": "items_mail" + num}),
                initial=False,
                required=False,
                validators=[])
            self.fields["temperature" + num] = CheckboxField(
                label=_(u'Temperatrue/FAN Error'),
                widget=forms.CheckboxInput(attrs={"class": "items_mail" + num}),
                initial=False,
                required=False,
                validators=[])
            self.fields["eventlog" + num] = CheckboxField(
                label=_(u'Event Log'),
                widget=forms.CheckboxInput(attrs={"class": "items_mail" + num}),
                initial=False,
                required=False,
                validators=[])
            self.fields["accesslog" + num] = CheckboxField(
                label=_(u'Access Log'),
                widget=forms.CheckboxInput(attrs={"class": "items_mail" + num}),
                initial=False,
                required=False,
                validators=[])
            self.fields["disk" + num] = CheckboxField(
                label=_(u'Disk Information'),
                widget=forms.CheckboxInput(attrs={"class": "items_mail" + num}),
                initial=False,
                required=False,
                validators=[])
            self.fields["volume" + num] = CheckboxField(
                label=_(u'Volume Exceeded'),
                widget=forms.CheckboxInput(attrs={"class": "items_mail" + num}),
                initial=False,
                required=False,
                validators=[])
            self.fields["quota" + num] = CheckboxField(
                label=_(u'Quota Exceeded'),
                widget=forms.CheckboxInput(attrs={"class": "items_mail" + num}),
                initial=False,
                required=False,
                validators=[])
            self.fields["ups" + num] = CheckboxField(
                label=_(u'UPS Status'),
                widget=forms.CheckboxInput(attrs={"class": "items_mail" + num}),
                initial=False,
                required=False,
                validators=[])
            self.fields["report" + num] = CheckboxField(
                label=_(u'Periodic Report'),
                widget=forms.CheckboxInput(attrs={"class": "items_mail" + num}),
                initial=False,
                required=False,
                validators=[])
            self.fields["host" + num] = FQDNField(
                label="%s%s: %s" % (_(u'Mail'), num, _(u'SMTP Server')),
                widget=forms.TextInput(attrs={"class": "items_mail" + num}),
                required=False)
            self.fields["port" + num] = RemotePortNumberField(
                label="%s%s: %s" % (_(u'Mail'), num, _(u'SMTP Port')),
                widget=forms.TextInput(attrs={"class": "items_mail" + num}),
                initial=25,
                required=False)
            self.fields["ssl" + num] = EnableField(
                label=_(u"Use SSL"),
                initial=False);
            self.fields["fromaddr" + num] = EmailField(
                label="%s%s: %s" % (_(u'Mail'), num, _(u'Sender Address')),
                widget=forms.TextInput(attrs={"class": "items_mail" + num}),
                required=False)
            self.fields["encode" + num] = RadioField(
                label=_(u'Mail Encode'),
                widget=forms.RadioSelect(attrs={"class": "items_mail" + num}),
                choices=((0, _(u'UTF-8')), (1, _(u'ISO-2022-JP'))),
                initial=1,
                required=False,
                validators=[])
            self.fields["auth" + num] = SelectField(
                label=_(u'Authentication'),
                widget=forms.Select(attrs={"class": "items_mail" + num}),
                choices=((0, _(u'No Authentication')),
                         (1, _(u'POP before SMTP')),
                         (2, _(u'SMTP AUTH(PLAIN)')),
                         (3, _(u'SMTP AUTH(LOGIN)')),
                         (4, _(u'SMTP AUTH(CRAM-MD5)'))),
                initial=0,
                required=False,
                validators=[])
            self.fields["popsrv" + num] = FQDNField(
                label="%s%s:%s" % (_(u'Mail'), num, _(u'POP Server')),
                widget=forms.TextInput(attrs={"class": "items_mail" + num}),
                required=False)
            self.fields["popport" + num] = RemotePortNumberField(
                label="%s%s:%s" % (_(u'Mail'), num, _(u'POP Port')),
                widget=forms.TextInput(attrs={"class": "items_mail" + num}),
                initial=110,
                required=False)
            self.fields["user" + num] = TextField(
                label="%s%s:%s" % (_(u'Mail'), num, _(u'User Name')),
                widget=forms.TextInput(attrs={"class": "items_mail" + num}),
                max_length=155,
                required=False)
            self.fields["password" + num] = PasswordField(
                label=_(u'Password'),
                widget=forms.PasswordInput(render_value=True,
                        attrs={"class": "items_mail" + num}),
                max_length=20,
                required=False)
            idx += 1

    def __mail_basic_is_set(self):
        mailobj = Mail()
        basic_conf = mailobj.get_basic_init_config()
        if basic_conf.get("host"):
            return True
        else:
            return False

    def clean(self):
        mail_list = []
        idx = 1
        while idx <= MAX_MAIL_NUM:
            err_list = []
            if self.cleaned_data.get("mail" + str(idx)):
                mail_addr = self.cleaned_data.get("toaddr" + str(idx))
                if mail_addr == "":
                    err_list.append(
                        "%s: %s" % (
                        _(u'Mail Address'),
                        _(u'This field is required.')))
                else:
                    if mail_addr in mail_list:
                        err_list.append(
                            "%s: %s" % (
                            _(u'Mail Address'),
                            _(u'This mail address already exists.')))
                    else:
                        if mail_addr:
                            mail_list.append(mail_addr)

                if not self.cleaned_data.get("detail" + str(idx)):
                    if not self.__mail_basic_is_set():
                        err_list.append(
                            "%s: %s" % (
                            _(u'Detail Setting'),
                            _(u'This field is required.')))
                else:
                    if not self.__mail_basic_is_set():
                        if self.cleaned_data.get("host" + str(idx)) == "":
                            err_list.append(
                                "%s: %s" % (
                                _(u'SMTP Server'),
                                _(u'This field is required.')))
                        if "port" + str(idx) in self.cleaned_data.keys():
                            if self.cleaned_data.get("port" + str(idx)) == None:
                                err_list.append(
                                    "%s: %s" % (
                                    _(u'SMTP Port'),
                                    _(u'This field is required.')))
                        if self.cleaned_data.get("fromaddr" + str(idx)) == "":
                            err_list.append(
                                "%s: %s" % (
                                _(u'Sender Address'),
                                _(u'This field is required.')))

                    auth = self.cleaned_data.get("auth" + str(idx))
                    if auth == "0":
                        pass
                    elif auth == "1":
                        if self.cleaned_data.get("popsrv" + str(idx)) == "":
                            err_list.append(
                                "%s: %s" % (
                                _(u'POP Server'),
                                _(u'This field is required.')))
                        if "popport" + str(idx) in self.cleaned_data.keys():
                            if not self.cleaned_data.get("popport" + str(idx)):
                                err_list.append(
                                    "%s: %s" % (
                                    _(u'POP Port'),
                                    _(u'This field is required.')))
                        if self.cleaned_data.get("user" + str(idx)) == "":
                            err_list.append(
                                "%s: %s" % (
                                _(u'User Name'),
                                _(u'This field is required.')))
                    else:
                        if self.cleaned_data.get("user" + str(idx)) == "":
                            err_list.append(
                                "%s: %s" % (
                                _(u'User Name'),
                                _(u'This field is required.')))

                if err_list:
                    self._errors["mail" + str(idx)] = self.error_class(err_list)
            idx += 1
        return self.cleaned_data

class View(FuncView):
    views = ['apply']
    mailobj = Mail()

    def __get_mail_list(self):
        maillist = []
        form = Form(initial=self.mailobj.get_mailnotice_init_config())
        idx = 1
        while idx <= MAX_MAIL_NUM:
            num = str(idx)
            mail_dic = {"num": num}
            for item in [
                    "mail",
                    "toaddr",
                    "detail",
                    "system",
                    "backup",
                    "temperature",
                    "report",
                    "ups",
                    "disk",
                    "volume",
                    "quota",
                    "host",
                    "port",
                    "ssl",
                    "auth",
                    "fromaddr",
                    "encode",
                    "popsrv",
                    "popport",
                    "user",
                    "password",
                    "eventlog",
                    "accesslog"]:
                mail_dic[item] = form[item + num]
            maillist.append(mail_dic)
            idx += 1
        return maillist

    def __is_basic_not_set(self):
        conf = self.mailobj.get_basic_init_config()
        if conf.get("host") and conf.get("port") and conf.get("fromaddr"):
            return {}
        return {"is_basic_not_set": True}

    def __get_info_dic(self):
        info_dic = {}
        info_dic.update(self.__is_basic_not_set())
        return info_dic

    def view(self, request):
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                'mails': self.__get_mail_list(),
                'info_dic': self.__get_info_dic()}}

    def view_apply(self, request):
        with self.processing(constname="ERR_MAIL_STATE_CHANGE") as p:
            form = Form(data=request.POST)
            rst = False
            p.break_if_state_change()

            if form.is_valid():
                if self.mailobj.mailnotice_set(form.cleaned_data):
                    self.set_message(
                            get_message('MSG_MAIL_CHANGE', is_log=True))
                    rst = True
                else:
                    self.set_message(
                            get_message('ERR_MAIL_CHANGE', is_log=True))
                p.result(rst)

        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {'form': form}}

mailnotice = View.as_view()
