# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
import datetime
import string
from ionas.request import (Request, Entry)
from iobase.exception import NotFoundError
from iobase.xmldata import (XmlNode, XmlConfNode, escape)
from product_conf import SUDO_CMD
from fal_info import (FAL_NOTICE_PATH, FAL_USERS_PATH, ReqTypes, Results)

#MAIL_BASIC_ENTRY = "<notice><mailbasic><mailconf><host type='smtp'/>" + \
#        "<port type='smtp'/><fromaddr/><auth/><encode/></mailconf>" + \
#        "</mailbasic></notice>"
MAIL_BASIC_ENTRY = """<notice>
    <mailbasic>
        <mailconf>
            <usessl enable="false"/>
            <host type='smtp'/>
            <port type='smtp'/>
            <fromaddr/>
            <auth/>
            <encode/>
        </mailconf>
    </mailbasic>
</notice>"""
REPORT_ENTRY = "<notice><report><eventtimelist/></report></notice>"
REGULAR_NOTICE_ENTRY = "<notice><regularnotice/></notice>"
MAIL_NOTICE_ENTRY = "<notice><mailnotice><mailconflist/></mailnotice></notice>"
#MAIL_NOTICE_ENTRY_DATA = '<mailconf detail="%(detail)s" enable="true">' + \
#    '<host type="smtp">%(host)s</host><port type="smtp">%(port)s</port>' + \
#    '<fromaddr>%(fromaddr)s</fromaddr><toaddrlist><toaddr>%(toaddr)s' + \
#    '</toaddr></toaddrlist><auth type="%(auth)s"><pop3><host>%(popsrv)s' + \
#    '</host><port>%(popport)s</port></pop3><user>%(user)s' + \
#    '</user><password>%(password)s</password></auth><encode>%(encode)s' + \
#    '</encode><notifylist><notify name="system" enable="%(system)s"/>' + \
#    '<notify name="backup" enable="%(backup)s"/><notify name="eventlog" ' + \
#    'enable="%(eventlog)s"/><notify name="accesslog" ' + \
#    'enable="%(accesslog)s"/><notify name="temperature" ' + \
#    'enable="%(temperature)s"/><notify ' + \
#    'name="disk" enable="%(disk)s"/><notify name="volume" enable=' + \
#    '"%(volume)s"/><notify name="quota" enable' + \
#    '="%(quota)s"/><notify name="ups" enable' + \
#    '="%(ups)s"/><notify name="report" enable="%(report)s"/>' + \
#    '</notifylist></mailconf>'
MAIL_NOTICE_ENTRY_DATA = """<mailconf detail="%(detail)s" enable="true">
    <usessl enable="%(usessl)s"/>
    <host type="smtp">%(host)s</host>
    <port type="smtp">%(port)s</port>
    <fromaddr>%(fromaddr)s</fromaddr>
    <toaddrlist>
        <toaddr>%(toaddr)s</toaddr>
    </toaddrlist>
    <auth type="%(auth)s">
        <pop3>
            <host>%(popsrv)s</host>
            <port>%(popport)s</port>
        </pop3>
        <user>%(user)s</user>
        <password>%(password)s</password>
    </auth>
    <encode>%(encode)s</encode>
    <notifylist>
        <notify name="system" enable="%(system)s"/>
        <notify name="backup" enable="%(backup)s"/>
        <notify name="eventlog" enable="%(eventlog)s"/>
        <notify name="accesslog" enable="%(accesslog)s"/>
        <notify name="temperature" enable="%(temperature)s"/>
        <notify name="disk" enable="%(disk)s"/>
        <notify name="volume" enable="%(volume)s"/>
        <notify name="quota" enable="%(quota)s"/>
        <notify name="ups" enable="%(ups)s"/>
        <notify name="report" enable="%(report)s"/>
    </notifylist>
</mailconf>"""
HOST_XPATH = "./mailbasic/mailconf/host"
PORT_XPATH = "./mailbasic/mailconf/port"
FROMADDR_XPATH = "./mailbasic/mailconf/fromaddr"
AUTH_XPATH = "./mailbasic/mailconf/auth"
ENCODE_XPATH = "./mailbasic/mailconf/encode"
MAILCONF_XPATH = "./mailnotice/mailconflist/mailconf"
MAILCONFLIST_XPATH = "./mailnotice/mailconflist"
REPORT_XPATH = "./report"
EVENTTIME_XPATH = "./report/eventtimelist/eventtime"
REGULARNOTICE_XPATH = "./regularnotice"

MAX_MAIL_NUM = 5
AUTH_SET_DIC = {"0": "no", "1": "pop", "2": "plain",
                "3": "login", "4": "crammd5"}
ENCODE_SET_DIC = {"0": "utf-8", "1": "iso-2022-jp"}
AUTH_INFO_DIC = {"no": 0, "pop": 1, "plain": 2, "login": 3, "crammd5": 4}
TRUE = "true"
FALSE = "false"


class Mail(object):

    def send_request(self, req_entry):
        try:
            response = Request(FAL_NOTICE_PATH, filter_cmd=SUDO_CMD).request(req_entry)
            if response.get_entrylist().get_entry().get_result() == Results.OK:
                return True
            else:
                return False
        except:
            # FIXME  例外種類ごとに、ユーザにメッセージを渡す？
            # 検討必要
            return False

    def basic_set(self, data):
        entry = Entry(ReqTypes.APPLY, MAIL_BASIC_ENTRY)
        body = entry.get()
        body.get_node('mailbasic/mailconf/usessl').enable(data.get("ssl", False));
        body.get_node(HOST_XPATH).set_text(data["host"])
        body.get_node(PORT_XPATH).set_text(str(data["port"]))
        body.get_node(FROMADDR_XPATH).set_text(data["fromaddr"])
        body.get_node(ENCODE_XPATH).set_text(ENCODE_SET_DIC[data["encode"]])
        auth_node = body.get_node(AUTH_XPATH)
        auth_node.set_attrs({"type": AUTH_SET_DIC[data["auth"]]})
        if data["auth"] != "0":
            auth_node.append_child("user", data["user"])
            auth_node.append_child("password", data["password"])
            if data["auth"] == "1":
                auth_node.append_child("pop3")
                pop3_node = auth_node.get_node("pop3")
                pop3_node.append_child("host", data["popsrv"])
                pop3_node.append_child("port", str(data["popport"]))
        return self.send_request(entry)

    def report_set(self, data):
        entry = Entry(ReqTypes.APPLY, REPORT_ENTRY)
        report = entry.get().get_node(REPORT_XPATH)
        if data["report"] == "1":
            report.set_attrs({"enable": TRUE})
            day_dic = {"0": "Sun", "1": "Mon", "2": "Tue", "3": "Wed",
                       "4": "Thu", "5": "Fri", "6": "Sat"}
            for day in data["day"]:
                eventtime_node = XmlNode("eventtime")
                eventtime_node.append_child("day", day_dic[day])
                eventtime_node.append_child("hour", str(data["time"].hour))
                eventtime_node.append_child("minute", str(data["time"].minute))
                report.get_node("eventtimelist").append_child(eventtime_node)
        else:
            report.set_attrs({"enable": FALSE})
        return self.send_request(entry)

    def regularnotice_set(self, data):
        entry = Entry(ReqTypes.APPLY, REGULAR_NOTICE_ENTRY)
        regular = entry.get().get_node(REGULARNOTICE_XPATH)
        if data["notice"] == "1":
            regular.set_attrs({"enable": TRUE})
            regular.append_child("diskerror",
                    attrs={"enable": str(data["diskerror"]).lower()})
            regular.append_child("ups",
                    attrs={"enable": str(data["ups"]).lower()})
            regular.append_child("volume", text=data["volumelimit"],
                    attrs={"enable": str(data["volume"]).lower()})
            regular.append_child("quota", text=data["quotalimit"],
                    attrs={"enable": str(data["quota"]).lower()})
        else:
            regular.set_attrs({"enable": FALSE})
        return self.send_request(entry)

    def mailnotice_set(self, data):
        num_list = []   # 有効にしたメールの番号リスト
        entry = Entry(ReqTypes.APPLY, MAIL_NOTICE_ENTRY)
        num = 1
        while num <= MAX_MAIL_NUM:
            if data["mail" + str(num)] == True:
                num_list.append(str(num))
            num += 1
        for num in num_list:
            if not data["port" + num]:
                data["port" + num] = "25"
            if not data["popport" + num]:
                data["popport" + num] = "110"
            entry_data = XmlConfNode(MAIL_NOTICE_ENTRY_DATA % {
                "detail": str(data["detail" + num]).lower(),
                "usessl": str(data.get("ssl" + num, False)).lower(),
                "host": data["host" + num],
                "port": data["port" + num],
                "fromaddr": data["fromaddr" + num],
                "toaddr": data["toaddr" + num],
                "auth": AUTH_SET_DIC[str(data["auth" + num])],
                "popsrv": data["popsrv" + num],
                "popport": data["popport" + num],
                "user": escape(data["user" + num]),
                "password": escape(data["password" + num]),
                "encode": ENCODE_SET_DIC[str(data["encode" + num])],
                "system": str(data["system" + num]).lower(),
                "backup": str(data["backup" + num]).lower(),
                "eventlog": str(data["eventlog" + num]).lower(),
                "accesslog": str(data["accesslog" + num]).lower(),
                "temperature": str(data["temperature" + num]).lower(),
                "disk": str(data["disk" + num]).lower(),
                "volume": str(data["volume" + num]).lower(),
                "quota": str(data["quota" + num]).lower(),
                "ups": str(data["ups" + num]).lower(),
                "report": str(data["report" + num]).lower()})
            entry.get().get_node(MAILCONFLIST_XPATH).append_child(entry_data)
        return self.send_request(entry)

    def get_basic_init_config(self):
        info_dic = {
            "ssl": False,
            "host": "",
            "port": "",
            "fromaddr": "",
            "encode": 0,
            "auth": 0,
        }
        try:
            result = Request(FAL_NOTICE_PATH, filter_cmd=SUDO_CMD).request(
                     Entry(ReqTypes.LOAD)).get_entrylist().get_entry().get()
        except:
            return info_dic
        try:
            info_dic["ssl"] = result.get_node('mailbasic/mailconf/usessl').is_enabled()
        except:
            pass
        info_dic["host"] = result.get_node(HOST_XPATH).get_text()
        info_dic["port"] = result.get_node(PORT_XPATH).get_text()
        info_dic["fromaddr"] = result.get_node(FROMADDR_XPATH).get_text()
        if result.get_node(ENCODE_XPATH).get_text() != "utf-8":
            info_dic["encode"] = 1
        auth_node = result.get_node(AUTH_XPATH)
        if auth_node.get_attr_value("type") in AUTH_INFO_DIC.keys():
            info_dic["auth"] = AUTH_INFO_DIC[auth_node.get_attr_value("type")]
        try:
            info_dic["user"] = auth_node.get_node(
                "user").get_text(do_strip=False)
            info_dic["password"] = auth_node.get_node(
                "password").get_text(do_strip=False)
            info_dic["popsrv"] = auth_node.get_node("pop3/host").get_text()
            info_dic["popport"] = auth_node.get_node("pop3/port").get_text()
        except NotFoundError:
            pass
        return info_dic

    def get_admin_mail(self):
        config = Request(FAL_USERS_PATH, filter_cmd=SUDO_CMD).request(
                     Entry(ReqTypes.LOAD)).get_entrylist().get_entry().get()
        adminuser = config.get_node("userlist/user[@admin='true']")
        return  adminuser.get_node("mailaddr").get_text()

    def get_mailnotice_init_config(self):
        mails_dic = {}
        enable_dic = {"true": True, "false": False}
        try:
            result = Request(FAL_NOTICE_PATH, filter_cmd=SUDO_CMD).request(
                    Entry(ReqTypes.LOAD)).get_entrylist().get_entry().get()
        except:
            return mails_dic   # FIXME
        admin_mail = self.get_admin_mail()
        admin_exist = 0    #管理者メールアドレスが設定されているかを判別
        idx = 0
        mail_list = result.get_node_list(MAILCONF_XPATH)
        while idx < len(mail_list):
            num = str(idx + 1)
            mails_dic["mail" + num] = True
            address = mail_list[idx].get_node("toaddrlist/toaddr").get_text()
            mails_dic["toaddr" + num] = address 
            if address == admin_mail:
                admin_exist = 1   #管理者メールアドレスが設定されている
            if mail_list[idx].get_attr_value("detail") == TRUE:
                mails_dic["detail" + num] = True
                try:
                    mails_dic["ssl" + num] = mail_list[idx].get_node("usessl").is_enabled()
                except:
                    pass
                mails_dic["host" + num] = \
                    mail_list[idx].get_node("host").get_text()
                mails_dic["port" + num] = \
                    mail_list[idx].get_node("port").get_text()
                mails_dic["fromaddr" + num] = \
                    mail_list[idx].get_node("fromaddr").get_text()
                try:
                    auth = mail_list[idx].get_node(
                                "auth").get_attr_value("type")
                    if auth in AUTH_INFO_DIC.keys():
                        mails_dic["auth" + num] = AUTH_INFO_DIC[auth]
                    else:
                        mails_dic["auth" + num] = 0
                except NotFoundError:
                    mails_dic["auth" + num] = 0

                try:
                    mails_dic["popsrv" + num] = \
                        mail_list[idx].get_node("auth/pop3/host").get_text()
                    mails_dic["popport" + num] = \
                        mail_list[idx].get_node("auth/pop3/port").get_text()
                except NotFoundError:
                    pass
                try:
                    mails_dic["user" + num] = mail_list[idx].get_node(
                        "auth/user").get_text(do_strip=False)
                    mails_dic["password" + num] = mail_list[idx].get_node(
                        "auth/password").get_text(do_strip=False)
                except NotFoundError:
                    pass
                if mail_list[idx].get_node("encode").get_text() == "utf-8":
                    mails_dic["encode" + num] = 0
                else:
                    mails_dic["encode" + num] = 1
            for item in ["system", "backup", "eventlog", "temperature", "quota",
                         "disk", "volume", "ups", "report", "accesslog"]:
                try:
                    item_node = mail_list[idx].get_node(
                                    "notifylist/notify[@name='%s']" % item)
                    mails_dic[item + num] = enable_dic[
                            item_node.get_attr_value("enable")]
                except NotFoundError:
                    pass
            idx += 1

        if (idx < MAX_MAIL_NUM) and (admin_exist == 0):
            # n(n<MAX_MAIL_NUM)個以下のメールが設定されてる場合、
            # n+1 のメール欄に管理者のメールアドレスを表示させる。
            mails_dic["toaddr" + str(idx + 1)] = admin_mail

        return mails_dic

    def get_report_init_config(self):
        info_dic = {"report": 0, "day": [], "time": datetime.time(12, 0)}
        try:
            result = Request(FAL_NOTICE_PATH, filter_cmd=SUDO_CMD).request(
                     Entry(ReqTypes.LOAD)).get_entrylist().get_entry().get()
        except:
            return info_dic   #FIXME
        report_node = result.get_node(REPORT_XPATH)
        try:
            if report_node.get_attr_value("enable") == TRUE:
                info_dic["report"] = 1
                day_dic = {"Sun": 0, "Mon": 1, "Tue": 2, "Wed": 3, "Thu": 4,
                           "Fri": 5, "Sat": 6}
                for node in report_node.get_node_list(
                            "eventtimelist/eventtime"):
                    info_dic["day"].append(
                            day_dic[node.get_node("day").get_text()])
                    hour = node.get_node("hour").get_text()
                    min = node.get_node("minute").get_text()
                info_dic["time"] = datetime.time(string.atoi(hour),
                                              string.atoi(min))
        except NotFoundError:
            pass
        return info_dic

    def get_notice_init_config(self):
        info_dic = {"notice": 0,
                    "diskerror": False,
                    "volume": False,
                    "quota": False,
                    "ups": False,
                    "volumelimit": "90",
                    "quotalimit": "90"}
        try:
            result = Request(FAL_NOTICE_PATH, filter_cmd=SUDO_CMD).request(
                     Entry(ReqTypes.LOAD)).get_entrylist().get_entry().get()
        except:
            return info_dic   #FIXME
        notice_node = result.get_node(REGULARNOTICE_XPATH)
        try:
            if notice_node.is_enabled():
                info_dic["notice"] = 1
                enable_dic = {"true": True, "false": False}
                info_dic["diskerror"] = enable_dic[
                    notice_node.get_node("diskerror").get_attr_value("enable")]
                info_dic["volume"] = enable_dic[
                    notice_node.get_node("volume").get_attr_value("enable")]
                info_dic["quota"] = enable_dic[
                    notice_node.get_node("quota").get_attr_value("enable")]
                info_dic["ups"] = enable_dic[
                    notice_node.get_node("ups").get_attr_value("enable")]
                info_dic["volumelimit"] = \
                    notice_node.get_node("volume").get_text()
                info_dic["quotalimit"] = \
                    notice_node.get_node("quota").get_text()
        except NotFoundError:
            pass
        return info_dic
