# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from ionas.request import (Request, Entry)
from django.utils.translation import ugettext as _
from django.http import HttpResponseRedirect
from project.view import FuncView
from project.fal.command import get_message
from root.modules.form import BaseForm
from product_conf import SUDO_CMD
from fal_info import (FAL_NARSUS_PATH, ReqTypes, Results)
from project.fal.proxy import Proxy
from project.fal.narsus import NarSuS
from project.fal.network import Network
from project.fal.sysinfo import SysInfo
import settings


SERVICE_XPATH = "./service"
NARSUS_ENTRY = "<narsus><service enable='false'/></narsus>"


def root(request):
    return HttpResponseRedirect('./narsus')

class View(FuncView):
    class Form(BaseForm):
        from root.modules.form.fields.radio import RadioField
        narsus = RadioField(
                label=_(u'NarSuS'),
                initial=0,
                required=True,
                validators=[])

    class HiddenForm(BaseForm):
        from project.forms.fields.hidden import HiddenField
        key = HiddenField()
        macaddress = HiddenField()
        model = HiddenField()
        sendflag = HiddenField()

    views = ['apply']

    def __get_narsus_config(self):
        conf = Request(FAL_NARSUS_PATH, filter_cmd=SUDO_CMD).request(
                Entry(ReqTypes.QUERY)).get_entrylist().get_entry().get()
        return conf

    def __get_init_config(self):
        config = {"narsus": 0}
        try:
            conf = self.__get_narsus_config()
            if conf.get_node(SERVICE_XPATH).is_enabled():
                config["narsus"] = 1
        except:
            pass
        return config

    def __send_request(self, data):
        try:
            entry = Entry(ReqTypes.APPLY, NARSUS_ENTRY)
            if data["narsus"] == "1":
                entry.get().get_node(SERVICE_XPATH).enable()
            else:
                entry.get().get_node(SERVICE_XPATH).enable(False)
            response = Request(FAL_NARSUS_PATH, filter_cmd=SUDO_CMD).request(
                    entry).get_entrylist().get_entry()
            if response.get_result() == Results.OK:
                return True
            else:
                return False
        except:
            return False

    def view(self, request):
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                    'form': self.Form(initial=self.__get_init_config())}}

    def view_apply(self, request):
        with self.processing(constname="ERR_NARSUS_STATE_CHANGE") as p:
            form = self.Form(data=request.POST)
            p.break_if_state_change()

            if form.is_valid():
                if self.__send_request(form.cleaned_data):
                    self.set_message(
                            get_message('MSG_NARSUS_CHANGE', is_log=True))
                    p.result(True)

                    narsus = NarSuS()
                    if bool(int(form.cleaned_data['narsus'])) and not narsus.is_registered():
                        template = 'narsus.html'
                        narsus.query()
                        network = Network()
                        network.query(to_dict=False)
                        sysinfo = SysInfo()

                        initial = {
                            'key': narsus.get_value('key', ''),
                            'macaddress': network.get_value('netiflist/netif[@name="lan1"]/mac', ''),
                            'model': sysinfo.query().get('model', ''),
                            'sendflag': '0',
                        }

                        return {
                            self.KEY_TEMPLATE: 'narsus.html',
                            self.KEY_TEMPLATE_DIC: {
                                'IS_LOGINED': True,
                                'form': self.HiddenForm(initial=initial),
                                'URL_NEWREGIST': narsus.get_value('urllist/url[@type="newregist"]', default=''),
                                'URL_REGIST': narsus.get_value('urllist/url[@type="regist"]', default=''),
                            }
                        }
                else:
                    self.set_message(
                            get_message('ERR_NARSUS_CHANGE', is_log=True))

        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {'form': form}}


narsus = View.as_view()
