# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django.utils.translation import ugettext as _
from django.http import HttpResponseRedirect
from django.core.exceptions import ValidationError
from project.view import FuncView
from project.fal.command import get_message
from root.modules.form import BaseForm


def root(request):
    return HttpResponseRedirect('./periodic')


class View(FuncView):
    from ..modules.mail import Mail

    class Form(BaseForm):
        from root.modules.form.fields.checkbox import CheckboxField
        from root.modules.form.fields.radio import RadioField
        from project.forms.fields.extras.usedvol import UsedVolField

        notice = RadioField(
                label=_(u'Periodic Notice'),
                initial=0,
                required=False,
                validators=[])
        diskerror = CheckboxField(
                label=_(u'Disk Error'),
                initial=False,
                required=False,
                validators=[])
        volume = CheckboxField(
                label=_(u'Volume Exceeded'),
                initial=False,
                required=False,
                validators=[])
        quota = CheckboxField(
                label=_(u'Quota Exceeded'),
                initial=False,
                required=False,
                validators=[])
        ups = CheckboxField(
                label=_(u'UPS Alert'),
                initial=False,
                required=False,
                validators=[])
        volumelimit = UsedVolField(
                label=_(u'Volume Limit'),
                initial="90",
                required=False)
        quotalimit = UsedVolField(
                label=_(u'Quota Limit'),
                initial="90",
                required=False)

        def clean(self):
            if self.cleaned_data.get("notice") == "1":
                if not self.cleaned_data.get("diskerror"):
                    if not self.cleaned_data.get("volume"):
                        if not self.cleaned_data.get("quota"):
                            if not self.cleaned_data.get("ups"):
                                raise ValidationError(_(u'Please select notice items.'))
                if self.cleaned_data.get("volume"):
                    if self.cleaned_data.get("volumelimit") == "":
                        raise ValidationError(_(u'"Volume limit" is required.'))
                if self.cleaned_data.get("quota"):
                    if self.cleaned_data.get("quotalimit") == "":
                        raise ValidationError(_(u'"Quota limit" is required.'))
            return self.cleaned_data

    views = ['apply']
    mail = Mail()

    def view(self, request):
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                'form': self.Form(initial=self.mail.get_notice_init_config())}}

    def view_apply(self, request):
        with self.processing(constname="ERR_PERIODIC_STATE_CHANGE") as p:
            form = self.Form(data=request.POST)
            rst = False
            p.break_if_state_change()

            if form.is_valid():
                if self.mail.regularnotice_set(form.cleaned_data):
                    self.set_message(
                            get_message('MSG_PERIODIC_CHANGE', is_log=True))
                    rst = True
                else:
                    self.set_message(
                            get_message('ERR_PERIODIC_CHANGE', is_log=True))
                p.result(rst)

        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {'form': form}}


periodic = View.as_view()
