# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
import datetime
from django.utils.translation import ugettext as _
from django.http import HttpResponseRedirect
from project.view import FuncView
from django.core.exceptions import ValidationError
from project.fal.command import get_message
from root.modules.form import BaseForm


def root(request):
    return HttpResponseRedirect('./report')


class View(FuncView):
    from ..modules.mail import Mail

    class Form(BaseForm):
        from root.modules.form.fields.multicheckbox import MultiCheckboxField
        from root.modules.form.fields.radio import RadioField
        from root.modules.form.fields.extras.selectdatetime \
                                                  import SelectTimeField
        report = RadioField(
                label=_(u'Periodic Report'),
                initial=0,
                required=True,
                validators=[])
        day = MultiCheckboxField(
                label=_(u'Day'),
                choices=((0, _(u'Sunday')), (1, _(u'Monday')),
                         (2, _(u'Tuesday')), (3, _(u'Wednesday')),
                         (4, _(u'Thursday')), (5, _(u'Friday')),
                         (6, _(u'Saturday'))),
                initial=[],
                required=False,
                validators=[])
        time = SelectTimeField(
                label=_(u'Time'),
                initial=datetime.time(0,0),
                required=False,
                validators=[])

        def clean(self):
            if self.cleaned_data.get("report") == "1":
                if len(self.cleaned_data.get("day")) <= 0:
                    raise ValidationError(_(u'Schedule day is required.'))
            return self.cleaned_data

    views = ['apply']
    mail = Mail()

    def view(self, request):
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                'form': self.Form(initial=self.mail.get_report_init_config())}}

    def view_apply(self, request):
        with self.processing(constname="ERR_REPORT_STATE_CHANGE") as p:
            form = self.Form(data=request.POST)
            rst = False
            p.break_if_state_change()

            if form.is_valid():
                if self.mail.report_set(form.cleaned_data):
                    self.set_message(
                            get_message('MSG_REPORT_CHANGE', is_log=True))
                    rst = True
                else:
                    self.set_message(
                            get_message('ERR_REPORT_CHANGE', is_log=True))
                p.result(rst)

        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {'form': form}}


report = View.as_view()
