# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from ionas.request import (Request, Entry)
from django.utils.translation import ugettext as _
from django.http import HttpResponseRedirect
from django.core import validators
from django.core.exceptions import ValidationError
from project.view import FuncView
from project.fal.command import get_message
from project.forms.fields.extras.portnum import RemotePortNumberField
from project.forms.fields.extras.fqdn import FQDNField
from root.modules.form import BaseForm
from product_conf import SUDO_CMD
from fal_info import (FAL_SNMP_PATH, ReqTypes, Results)

SNMP_ENTRY = "<snmp><trap enable='false'><port/><host/></trap></snmp>"


def root(request):
    return HttpResponseRedirect('./snmp')


class View(FuncView):
    class Form(BaseForm):
        from root.modules.form.fields.radio import RadioField

        snmp = RadioField(
                label=_(u'SNMP Trap'),
                initial=0,   #FIXME
                required=True,
                validators=[])
        port = RemotePortNumberField(label=_(u'Port Number'), required=False)
        host = FQDNField(label=_(u'Destination Host'), required=False)

        def clean_port(self):
            val = self.cleaned_data.get('port')
            snmp = self.cleaned_data.get('snmp')
            if int(snmp) == 1 and val in validators.EMPTY_VALUES:
                raise ValidationError(_(u'This field is required.'))
            return val

        def clean_host(self):
            val = self.cleaned_data.get('host')
            snmp = self.cleaned_data.get('snmp')
            if int(snmp) == 1 and val in validators.EMPTY_VALUES:
                raise ValidationError(_(u'This field is required.'))
            return val

    views = ['apply']

    def __get_init_config(self):
        config = {"snmp": 0, "port": "162", "host": ""}
        try:
            result = Request(FAL_SNMP_PATH, filter_cmd=SUDO_CMD).request(
                    Entry(ReqTypes.LOAD)).get_entrylist().get_entry().get()
            trap_node = result.get_node("trap")
            if trap_node.is_enabled():
                config["snmp"] = 1
                config["port"] = trap_node.get_node("port").get_text()
                config["host"] = trap_node.get_node("host").get_text()
        except:
            pass
        return config

    def __send_request(self, data):
        try:
            entry = Entry(ReqTypes.SAVE, SNMP_ENTRY)
            if data["snmp"] == "1":
                entry.get().get_node("trap").set_attrs({"enable": "true"})
                entry.get().get_node("trap/port").set_text(
                        '' if data["port"] is None else str(data['port']))
                entry.get().get_node("trap/host").set_text(data["host"])
            response = Request(FAL_SNMP_PATH, filter_cmd=SUDO_CMD).request(
                                entry).get_entrylist().get_entry()
            if response.get_result() == Results.OK:
                return True
            else:
                return False
        except:    #FIXME
            return False

    def view(self, request):
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                'form': self.Form(initial=self.__get_init_config())}}

    def view_apply(self, request):
        with self.processing(constname="ERR_SNMP_STATE_CHANGE") as p:
            form = self.Form(data=request.POST)
            rst = False
            p.break_if_state_change()

            if form.is_valid():
                if self.__send_request(form.cleaned_data):
                    self.set_message(
                            get_message('MSG_SNMP_CHANGE', is_log=True))
                    rst = True
                else:
                    self.set_message(
                            get_message('ERR_SNMP_CHANGE', is_log=True))
                p.result(rst)

        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {'form': form}}


snmp = View.as_view()
