# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

from __future__ import with_statement

import re
from django.utils.translation import ugettext as _
from django.http import HttpResponseRedirect
from project.view import FuncView
from root.modules.form import BaseForm
from project.fal.command import (get_message, logging)
from ..modules.package import Package
from ..modules.util import get_append_err_msg
from django.core.exceptions import ValidationError
from .. import settings as ps


CHECK_STR = "check"
UPDATE_STATE = "updating"


def get_code(msg):
    try:
        from fal_msginfo import Msg
        return getattr(Msg, msg)
    except:
        return None

def root(request):
    return HttpResponseRedirect('./add')

class Form(BaseForm):

    def __init__(self, *args, **kwargs):
        from root.modules.form.fields.checkbox import CheckboxField
        super(Form, self).__init__(*args, **kwargs)

        package = Package()
        (download, datalist) = package.get_package_release_list()
        for pack in datalist:
            id = pack["id"]
            self.fields[id + CHECK_STR] = CheckboxField(
                    initial=False,
                    required=True,
                    validators=[])

    def clean(self):
        num = 0
        for key, val in self.cleaned_data.items():
            if val:
                num += 1
        if num == 0:
            package = Package()
            (download, datalist) = package.get_package_release_list()
            if not download:
                logging(get_code("ERR_PKG_CONNECT"))
                raise ValidationError(_(u'Can not connect to server. Please confirm if the network is connected to internet.'))
            else:
                raise ValidationError(_(u'There is no package selected.'))
        return self.cleaned_data

class View(FuncView):
    views = ['add_multi', 'list']
    package = Package()

    def __get_datalist(self, form, datalist):
        for item in datalist:
            if item.get('available', True):
                item["check"] = form[item["id"] + "check"]
        return datalist

    def __set_append_err_msg(self):
        msg = get_append_err_msg()
        if msg:
            self.set_message(msg)

    def view(self, request):
        (download, datalist) = self.package.get_package_release_list()

        if not download:
            self.set_message(_(u'Can not connect to server. Please confirm if the network is connected to internet.'))
            logging(get_code("ERR_PKG_CONNECT"))
        form = Form()

        return {self.KEY_TEMPLATE: self.TEMPLATE,
                self.KEY_TEMPLATE_DIC: {
                    'pkg_num': len(datalist),
                    'list': self.__get_datalist(form, datalist)}}

    def view_add_multi(self, request):
        rst = False
        with self.processing(state=UPDATE_STATE,
                constname="ERR_PKG_STATE_CHANGE") as p:
            form = Form(data=request.POST)
            p.break_if_state_change()

            if form.is_valid():
                checklist = []
                for key, value in form.cleaned_data.items():
                    if value:
                        checklist.append(re.sub(CHECK_STR, "", key))
                if self.package.install_package(checklist):
                    self.set_message(
                            get_message('MSG_PKG_INSTALL', is_log=False))
                    rst = True
                else:
                    self.set_message(
                            get_message('ERR_PKG_INSTALL', is_log=False))
                    self.__set_append_err_msg()
                p.result(rst)
        if rst:
            logging('MSG_PKG_INSTALL')
        else:
            logging('ERR_PKG_INSTALL')

        return {self.KEY_TEMPLATE: self.TEMPLATE,
                self.KEY_TEMPLATE_DIC: {"form": form}}

    def view_list(self, request):
        return {self.KEY_REDIRECT:
                "/".join([ps.MENU_URL, ps.LIST_APPLI, ps.LIST_VIEW])}


add = View.as_view()
