# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
import re
import os
from django.utils.translation import ugettext as _
from django.http import HttpResponseRedirect
from project.view import FuncView
from root.modules.form import BaseForm
from project.fal.command import get_message, logging
from django.core.exceptions import ValidationError
import settings
from .. import settings as ps
from ..modules.package import Package


DEL_LIST = "del_list"
UP_LIST = "up_list"
CHECK_STR = "check"
UPDATE_STATE = "updating"

package = Package()
#pkglist = package.get_package_list()


def root(request):
    return HttpResponseRedirect('./list')


class Form(BaseForm):
    def __init__(self, *args, **kwargs):
        from root.modules.form.fields.checkbox import CheckboxField
        super(Form, self).__init__(*args, **kwargs)

        pkglist = package.get_package_list()
        for item in pkglist:
            id = item["id"]
            self.fields[id + CHECK_STR] = CheckboxField(
                    initial=False,
                    required=True,
                    validators=[])

    def clean(self):
        num = 0
        for key, val in self.cleaned_data.items():
            if val:
                num += 1
        if num == 0:
            raise ValidationError(_(u'There is no package selected.'))
        return self.cleaned_data


class View(FuncView):
    views = ['add', 'delete_multi', 'update_multi']

    def __get_datalist(self, form):
        release_list = []
        for item in package.get_package_release_list()[1]:
            if item.get('available', True) is False:
                continue
            release_list.append(item["id"])
        pkglist = package.get_package_list()
        for item in pkglist:
            item["check"] = form[item["id"] + "check"]
            if item["id"] in release_list:
                item["new"] = "yes"    # 新しいリリースがあり
            else:
                item["new"] = "no"     # 新しいリリースがない
        return pkglist

    def view(self, request):
        form = Form()
        pkglist = package.get_package_list()
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                    'pkg_num': len(pkglist),
                    'list': self.__get_datalist(form)}}

    def view_add(self, request):
        return {self.KEY_REDIRECT:
                "/".join([ps.MENU_URL, ps.ADD_APPLI, ps.ADD_VIEW])}

    def view_update_multi(self, request):
        form = Form(data=request.POST)
        if form.is_valid():
            self.session[UP_LIST] = []
            for key, value in form.cleaned_data.items():
                if value:
                    self.session[UP_LIST].append(re.sub(CHECK_STR, "", key))
            if self.session[UP_LIST]:
                return {self.KEY_REDIRECT: "/".join([settings.MENU_URL,
                        ps.UPDATE_MULTI_VIEW])}

        return {self.KEY_TEMPLATE: self.TEMPLATE,
                self.KEY_TEMPLATE_DIC: {"form": form}}

    def view_delete_multi(self, request):
        form = Form(data=request.POST)
        if form.is_valid():
            self.session[DEL_LIST] = []
            for key, value in form.cleaned_data.items():
                if value:
                    self.session[DEL_LIST].append(
                            re.sub(CHECK_STR, "", key))
            if self.session[DEL_LIST]:
                return {self.KEY_REDIRECT: "/".join([settings.MENU_URL,
                        ps.DELETE_MULTI_VIEW])}

        return {self.KEY_TEMPLATE: self.TEMPLATE,
                self.KEY_TEMPLATE_DIC: {"form": form}}


class DeleteMultiView(FuncView):
    #一括削除
    views = ['delete', 'list']

    def view(self, request):
        namelist = []
        if self.session.get(DEL_LIST):
            for id in self.session[DEL_LIST]:
                target = package.get_package_info_by_id(id)
                namelist.append(target["name"])

        if not namelist:
            return {self.KEY_REDIRECT: "/".join(
                    [settings.MENU_URL, ps.LIST_VIEW])}

        template = os.path.join(self.TEMPLATES, "delete_multi.html")
        return {self.KEY_TEMPLATE: template,
                self.KEY_TEMPLATE_DIC: {
                        'package_list': namelist}}

    def view_delete(self, request):
        with self.processing(constname="ERR_PKG_STATE_CHANGE") as p:
            rst = False
            p.break_if_state_change()

            if package.uninstall_package(self.session[DEL_LIST]):
                self.set_message(get_message('MSG_PKG_UNINSTALL', is_log=True))
                rst = True
            else:
                self.set_message(get_message('ERR_PKG_UNINSTALL', is_log=True))
            p.result(rst)

        self.session.clear()
        return {self.KEY_TEMPLATE: self.TEMPLATE,
                self.KEY_TEMPLATE_DIC: {}}

    def view_list(self, request):
        return {self.KEY_REDIRECT: "/".join([settings.MENU_URL, ps.LIST_VIEW])}


class UpdateMultiView(FuncView):
    #一括更新
    views = ['update', 'list']

    def view(self, request):
        oklist = []
        ok_id_list = []
        nglist = []
        if self.session.get(UP_LIST):
            for pkg in package.get_package_release_list()[1]:
                id = pkg.get("id")
                if id in self.session.get(UP_LIST):
                    oklist.append(pkg.get("name"))
                    ok_id_list.append(id)
            for id in self.session.get(UP_LIST):
                if not id in ok_id_list:
                    target = package.get_package_info_by_id(id)
                    nglist.append(target.get("name"))

        if not (oklist + nglist):
            return {self.KEY_REDIRECT: "/".join(
                    [settings.MENU_URL, ps.LIST_VIEW])}

        self.session[UP_LIST] = ok_id_list

        template = os.path.join(self.TEMPLATES, "update_multi.html")
        return {self.KEY_TEMPLATE: template,
                self.KEY_TEMPLATE_DIC: {
                        'ok_list': oklist,
                        'ng_list': nglist,
                        'ok_num': len(oklist),
                        'ng_num': len(nglist)}}

    def view_update(self, request):
        rst = False
        with self.processing(state=UPDATE_STATE,
                constname="ERR_PKG_STATE_CHANGE") as p:
            p.break_if_state_change()

            if package.install_package(self.session[UP_LIST]):
                self.set_message(get_message('MSG_PKG_UPDATE', is_log=False))
                rst = True
            else:
                self.set_message(get_message('ERR_PKG_UPDATE', is_log=False))
            p.result(rst)

        if rst:
            logging('MSG_PKG_UPDATE')
        else:
            logging('ERR_PKG_UPDATE')

        self.session.clear()
        return {self.KEY_TEMPLATE: self.TEMPLATE,
                self.KEY_TEMPLATE_DIC: {}}

    def view_list(self, request):
        return {self.KEY_REDIRECT: "/".join([settings.MENU_URL, ps.LIST_VIEW])}


list = View.as_view()
delete_multi = DeleteMultiView.as_view()
update_multi = UpdateMultiView.as_view()
