# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django.http import HttpResponseRedirect
from project.fal.command import get_message, logging
from project.view import FuncView
from .. import settings as ps
from ..modules.util import get_append_err_msg

UPDATE_STATE = "updating"


def root(request):
    return HttpResponseRedirect('./releasedetail')


class View(FuncView):
    from ..modules.package import Package
    views = ["addpkg", "releaselist"]
    package = Package()

    def __set_append_err_msg(self):
        msg = get_append_err_msg()
        if msg:
            self.set_message(msg)

    def view(self, request, id, ver=None):
        infos = self.package.get_package_info_by_id(
            id, version=ver, type="update")
        if infos == {}:
            return {self.KEY_REDIRECT:
                    "/".join([ps.MENU_URL, ps.ADD_APPLI, ps.ADD_VIEW])}
        return {self.KEY_TEMPLATE: self.TEMPLATE,
                self.KEY_TEMPLATE_DIC: {'infos': infos}}

    def view_addpkg(self, request, id, ver=None):
        rst = False
        with self.processing(state=UPDATE_STATE,
                constname="ERR_PKG_STATE_CHANGE") as p:
            p.break_if_state_change()

            if self.package.install_package([id]):
                self.set_message(
                    get_message('MSG_PKG_INSTALL', is_log=False))
                rst = True
            else:
                self.set_message(
                    get_message('ERR_PKG_INSTALL', is_log=False))
                self.__set_append_err_msg()
            p.result(rst)

        if rst:
            logging('MSG_PKG_INSTALL')
        else:
            logging('ERR_PKG_INSTALL')

        return {self.KEY_TEMPLATE: self.TEMPLATE,
               self.KEY_TEMPLATE_DIC: {}}

    def view_releaselist(self, request, id):
        return {self.KEY_REDIRECT:
            "/".join([ps.MENU_URL, ps.ADD_APPLI, ps.ADD_VIEW])}


releasedetail = View.as_view()
