# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django.http import HttpResponseRedirect
from project.fal.command import get_message, logging
from project.view import FuncView
from .. import settings as ps
from ..modules.util import get_append_err_msg

UPDATE_STATE = "updating"


def root(request):
    return HttpResponseRedirect('./update')


class View(FuncView):
    from ..modules.package import Package
    views = ["update", "list"]
    package = Package()

    def view(self, request, id):
        infos = self.package.get_package_info_by_id(id, type="update")
        return {self.KEY_TEMPLATE: self.TEMPLATE,
                self.KEY_TEMPLATE_DIC: {'infos': infos}}

    def __set_append_err_msg(self):
        msg = get_append_err_msg()
        if msg:
             self.set_message(msg)

    def view_update(self, request, id):
        rst = False
        with self.processing(state=UPDATE_STATE,
                constname="ERR_PKG_STATE_CHANGE") as p:
            p.break_if_state_change()

            if self.package.install_package([id]):
                self.set_message(get_message('MSG_PKG_UPDATE', is_log=False))
                rst = True
            else:
                self.set_message(get_message('ERR_PKG_UPDATE', is_log=False))
                self.__set_append_err_msg()
            p.result(rst)

        if rst:
            logging('MSG_PKG_UPDATE')
        else:
            logging('ERR_PKG_UPDATE')

        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {}}

    def view_list(self, request, id):
        return {self.KEY_REDIRECT:
                "/".join([ps.MENU_URL, ps.LIST_APPLI, ps.LIST_VIEW])}


update = View.as_view()
