# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from iobase.xmldata import XmlNode
from ionas.request import (Request, Entry)
from iobase.exception import NotFoundError
from django.utils.translation import ugettext as _
from django.http import HttpResponseRedirect
from project.view import FuncView
from project.fal.command import get_message
from root.modules.form import BaseForm
import settings
from product_conf import SUDO_CMD
from product_conf import VOL_LIST
from fal_info import (FAL_ECO_PATH, ReqTypes, Results)

ECO_ENTRY = "<eco><time><minute/></time><volumelist/></eco>"
TIME_XPATH = "./time/minute"
VOLUME_XPATH = "./volumelist/volume"
ENABLE = "enable"
TIME = "time"
TRUE = "true"


def root(request):
    return HttpResponseRedirect('/'.join([settings.MENU_URL, 'eco']))
    
def get_disk_list():
    disk_list = []
    for disk in VOL_LIST:
        disk_list.append(disk)
    return disk_list

def vol_humanize(vol):
    return {
        "sataraid1": _(u"eco: sataraid1"),
        "usb1": _(u"eco: usb1"),
        "usb2": _(u"eco: usb2"),
        "usb3": _(u"eco: usb3"),
        "usb4": _(u"eco: usb4"),
    }.get(vol, vol)


class Form(BaseForm):
    def __init__(self, *args, **kwargs):
        from root.modules.form.fields.validators.fal import FalValidator as fv
        from root.modules.form.fields.radio import RadioField
#        from root.modules.form.fields.select import SelectField
        from project.forms.fields.extras.spindown import SpindownTimeField
        super(Form, self).__init__(*args, **kwargs)

#        self.fields["time"] = SelectField(
#                label=_(u'Time Setting'),   #FIXME
#                initial='no',
#                required=True,
#                choices=(('10', _(u'10 Minutes')), ('15', _(u'15 Minutes')),
#                         ('20', _(u'20 Minutes')), ('30', _(u'30 Minutes')),
#                         ('60', _(u'1 Hour')), ('no', _(u'NO'))),
#                help_text='time',
#                validators=[])
        self.fields["time"] = SpindownTimeField(
                label=_(u'Time Setting'),
                required=True)
        for name in get_disk_list():
            self.fields[name] = RadioField(
                #label=name,  # FIXME
                label=vol_humanize(name),
                initial=0,
                required=False,
                validators=[])

class EcoView(FuncView):
    views = ['apply']

    def __get_config_init(self):
#        config = {TIME: "no"}
        config = {TIME: "0"}
        result = Request(FAL_ECO_PATH, filter_cmd=SUDO_CMD).request(
                Entry(ReqTypes.LOAD)).get_entrylist().get_entry()
        try:
            config[TIME] = result.get().get_node(TIME_XPATH).get_text()
#            time = result.get().get_node(TIME_XPATH).get_text()
#            if time in ["10", "15", "20", "30", "60"]:
#                config[TIME] = time
#            else:
#                config[TIME] = "no"
        except NotFoundError:
            pass
        for node in result.get().get_node_list(VOLUME_XPATH):
            if node.get_attr_value("enable") == TRUE:
                config[node.get_node("name").get_text()] = 1
            else:
                config[node.get_node("name").get_text()] = 0
        return config

    def __get_request_entry(self, data):
        enable_dic = {"1": "true", "0": "false"}
        entry = Entry(ReqTypes.APPLY, ECO_ENTRY)
        body = entry.get()
#        if data[TIME] == "no":
        if data[TIME] == "0":
            body.get_node(TIME_XPATH).set_text("0")
        else:
            body.get_node(TIME_XPATH).set_text(data[TIME])
            for name in get_disk_list():
                volume = XmlNode("volume")
                volume.append_child("name", name)
                volume.set_attrs({"enable": enable_dic[data[name]]})
                body.get_node("volumelist").append_child(volume)
        return entry

    def __send_request(self, cmd, req_entry):
        try:
            response = Request(cmd, filter_cmd=SUDO_CMD).request(req_entry)
            if response.get_entrylist().get_entry().get_result() == Results.OK:
                return True
            else:
                return False
        except:
            return False

    def view(self, request):
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                'form': Form(initial=self.__get_config_init())}}

    def view_apply(self, request):
        with self.processing(constname="ERR_ECO_STATE_CHANGE") as p:
            form = Form(data=request.POST)
            rst = False
            p.break_if_state_change()

            if form.is_valid():
                if self.__send_request(FAL_ECO_PATH,
                        self.__get_request_entry(form.cleaned_data)):
                    self.set_message(
                            get_message('MSG_ECO_CHANGE', is_log=True))
                    rst = True
                else:
                    self.set_message(
                            get_message('ERR_ECO_CHANGE', is_log=True))
                p.result(rst)

        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {'form': form}}

eco = EcoView.as_view()
