# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django.utils.translation import ugettext as _
from django.http import HttpResponseRedirect
#from root.modules.view import FuncView
from project.view import FuncView
from root.modules.form import BaseForm
import settings
from project.fal.power import Power


def root(request):
    return HttpResponseRedirect('/'.join([settings.MENU_URL, 'schedule']))


from root.modules.form.fields.checkbox import CheckboxField
from root.modules.form.fields.extras.selectdatetime import SelectTimeField


class EnableField(CheckboxField):
    def __init__(self, day=''):
        super(EnableField, self).__init__(
                initial=False,
                required=True)


class TimeField(SelectTimeField):
    def __init__(self, day=''):
        super(TimeField, self).__init__(
                initial=False,  # FIXME
                required=True)


class View(FuncView):
    WEEKDAYS = ('sun', 'mon', 'tue', 'wed', 'thu', 'fri', 'sat')
    FMT_TIME = "%02d:%02d"

    class Form(BaseForm):
        FMT_ON = "%(weekday)s_on"
        FMT_ONTIME = "%(weekday)s_on_time"
        FMT_OFF = "%(weekday)s_off"
        FMT_OFFTIME = "%(weekday)s_off_time"

        delay = CheckboxField(
                label=_(u"Shutdown delay on accessing."),
                required=False)

        def __init__(self, **kwargs):
            super(View.Form, self).__init__(**kwargs)
            for day in View.WEEKDAYS:
                self.fields[self.FMT_ON % {'weekday': day}] = EnableField()
                self.fields[self.FMT_ONTIME % {'weekday': day}] = TimeField()
                self.fields[self.FMT_OFF % {'weekday': day}] = EnableField()
                self.fields[self.FMT_OFFTIME % {'weekday': day}] = TimeField()

    views = ['apply']

    def view(self, request):
        power = Power()
        power.load()
        initial = {}
        initial['delay'] = power.is_true(power.get_value("powerschedule/@delay"))
        for day in self.WEEKDAYS:
            xpath_dayschedule = 'powerschedule/dayschedule[@day="%s"]' \
                    % day.capitalize()
            initial[self.Form.FMT_ON % {'weekday': day}] \
                    = power.is_true(power.get_value(
                            "%s/ontime/@enable" % xpath_dayschedule, False))
            initial[self.Form.FMT_ONTIME % {'weekday': day}] = \
                    self.FMT_TIME % (
                            int(power.get_value("%s/ontime/hour" \
                                    % xpath_dayschedule, '00')),
                            int(power.get_value("%s/ontime/minute" \
                                    % xpath_dayschedule, '00')))
            initial[self.Form.FMT_OFF % {'weekday': day}] = \
                    power.is_true(power.get_value(
                            "%s/offtime/@enable" % xpath_dayschedule, False))
            initial[self.Form.FMT_OFFTIME % {'weekday': day}] = \
                    self.FMT_TIME % (
                            int(power.get_value("%s/offtime/hour" % \
                                    xpath_dayschedule, '00')),
                            int(power.get_value("%s/offtime/minute" % \
                                    xpath_dayschedule, '00')))
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {'form': self.Form(initial=initial)}}

    def view_apply(self, request):
        form = self.Form(data=request.POST)
        with self.processing() as p:
            p.break_if_state_change()
            if form.is_valid():
#                schedule = {}
#                for day in self.WEEKDAYS:
#                    if form.cleaned_data["%s_on" % day] \
#                            or form.cleaned_data["%s_off" % day]:
#                        schedule[day] = {}
#                        for onoff in ('on', 'off'):
#                            if form.cleaned_data["%s_%s" % (day, onoff)]:
#                                schedule[day]["%stime" % onoff] = {
#                                    'hour': form.cleaned_data["%s_%s_time" % (
#                                            day, onoff)].hour,
#                                    'min': form.cleaned_data["%s_%s_time" % (
#                                            day, onoff)].minute}
#                power = Power()
#                power.apply2(schedule=schedule)

                daylist = {}
                for day in self.WEEKDAYS:
                    daylist[day] = {}
                    for onoff in ('on', 'off'):
                        daylist[day][onoff] = {}
                        daylist[day][onoff]['enable'] = form.cleaned_data["%s_%s" % (day, onoff)]
                        daylist[day][onoff]['time'] = form.cleaned_data["%s_%s_time" % (day, onoff)]
                config = {
                    'schedule': {
                        'delay': form.cleaned_data['delay'],
                        'daylist': daylist,
                    },
                }
                power = Power()
                power.apply2(config=config)

                self.set_message(power.get_message())
                p.result(power.get_result(boolean=True))
        return self.view(request)
#        return {
#            self.KEY_TEMPLATE: self.TEMPLATE,
#            self.KEY_TEMPLATE_DIC: {'form': form}}


schedule = View.as_view()
