# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from ionas.request import (Request, Entry)
from iobase.xmldata import XmlConfNode
from django.utils.translation import ugettext as _
from django.core import validators
from django.core.exceptions import ValidationError
from project.view import FuncView
from root.modules.form import BaseForm
from product_conf import SUDO_CMD
from project.fal.command import get_message
from project.fal.led import LED
from fal_info import (FAL_UPS_PATH, ReqTypes, Results)

from root.modules.form.fields.radio import RadioField
from project.forms.fields.extras.ups import UPSShutdownDelayField
from project.forms.fields.checkbox import CheckboxField
from project.forms.fields.extras.fqdn import FQDNField
from project.forms.fields.extras.landiskpass import LanDiskPassField

TARGET_XPATH = 'netshutdown/targetlandisklist/targetlandisk[@number="%s"]'
STATUS_XPATH = './status'
APPLY_ENTRY = "<ups><netshutdown><targetlandisklist/></netshutdown>" + \
    "<warning/><time><minute/></time></ups>"
TARGET_XML = "<targetlandisk number='%(num)s' enable='%(enable)s'>" + \
    "<passwd>%(passwd)s</passwd><host>%(host)s</host></targetlandisk>"

WARN_STS_LIST = ["UNKNOWN", "COMMLOST", "DISCONNECT"]


class View(FuncView):
    class Form(BaseForm):
        warning = RadioField(
                label=_(u'UPS Warning'),
                choices=(('true', _(u'Enable')),
                         ('false', _(u'Disable'))),
                initial='false')
        delay = UPSShutdownDelayField(
                label=_(u'UPS Shutdown'),
                initial=5)

        def __init__(self, **kwargs):
            super(View.Form, self).__init__(**kwargs)
            for num in [str(n) for n in range(1, 6)]:
                self.fields['enable%s' % num] = CheckboxField(
                        initial=False)
                self.fields['landisk%s' % num] = FQDNField(
                        label=("%(landisk)s (%(number)s)" % {
                            'landisk': _(u'Shutdown LAN DISK'),
                            'number': num}),
                        required=False)
                self.fields['password%s' % num] = LanDiskPassField()

#                def _setattr(num):
#                    setattr(
#                            self,
#                            'clean_landisk%s' % num,
#                            lambda: self._clean_landisk(num))
#                _setattr(num)

#        def _clean_landisk(self, num):
#            val = self.cleaned_data.get('landisk%s' % num, '')
#            enable = self.cleaned_data.get('enable%s' % num, False)
#            if enable and val == "":
#                raise ValidationError(_(u'This field is required.'))
#            return val

        def clean(self):
            for i in range(1, 6):
                #if self.cleaned_data["enable%s" % i]:
                if self.cleaned_data["enable%s" % i] or self.cleaned_data["password%s" % i]:
                    if self.cleaned_data["landisk%s" % i] == "":
                        raise ValidationError("%s: %s" % (
                            "%s (%s)" % (_(u'Shutdown LAN DISK'), i),
                            _(u'This field is required.')))
            return self.cleaned_data

    views = ['apply']

    def get_config(self):
        config = Request(FAL_UPS_PATH, filter_cmd=SUDO_CMD).request(
                 Entry(ReqTypes.LOAD)).get_entrylist().get_entry().get()
        return config

    def ups_is_connected(self):
        try:
            result = Request(FAL_UPS_PATH, filter_cmd=SUDO_CMD).request(
                Entry(ReqTypes.QUERY, "<ups/>")).get_entrylist().get_entry().get()
            status = result.get_node(STATUS_XPATH).get_text()
        except:
            return False

        if status in WARN_STS_LIST:
            return False

        return True

    def view(self, request):
        config = self.get_config()
        initial = {
            'warning': config.get_node("warning").get_attr_value("enable"),
            'delay': int(config.get_node("time/minute").get_text())}
        for i in range(1, 6):
            target = config.get_node(TARGET_XPATH % str(i))
            if target.get_attr_value("enable") == "true":
                initial['enable%s' % i] = True
            initial['landisk%s' % i] = target.get_node("host").get_text()
            initial['password%s' % i] = target.get_node("passwd").get_text(do_strip=False)
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {'form': self.Form(initial=initial)}}

    def send_request(self, data):
        entry = Entry(ReqTypes.APPLY, APPLY_ENTRY)
        entry.get().get_node("warning").set_attrs({"enable": data["warning"]})
        entry.get().get_node("time/minute").set_text(data["delay"])
        list_node = entry.get().get_node("netshutdown/targetlandisklist")
        for i in range(1, 6):
            target = XmlConfNode(str(TARGET_XML % {
                    "num": str(i),
                    "enable": str(data["enable%s" % str(i)]).lower(),
                    "host": data["landisk%s" % str(i)],
                    "passwd": data["password%s" % str(i)]}))
            list_node.append_child(target)
        response = Request(FAL_UPS_PATH, filter_cmd=SUDO_CMD).request(entry)
        if response.get_entrylist().get_entry().get_result() == Results.OK:
            return True
        else:
            return False

    def view_apply(self, request):
        with self.processing(constname="ERR_UPS_STATE_CHANGE") as p:
            form = self.Form(data=request.POST)
            rst = False
            p.break_if_state_change()

            if form.is_valid():
                if self.send_request(form.cleaned_data):
                    self.set_message(get_message('MSG_UPS_CHANGE', is_log=True))
                    rst = True
                else:
                    self.set_message(get_message('ERR_UPS_CHANGE', is_log=True))
                p.result(rst)

                if form.cleaned_data.get("warning") == "true":
                    if not self.ups_is_connected():
                        LED().status(True, blink=True, color="red")

        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {'form': form}}


ups = View.as_view()
