# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
import os
from ionas.request import (Request, Entry)
from django.utils.translation import ugettext as _
from django.http import HttpResponseRedirect
from project.view import FuncView
from product_conf import (SUDO_CMD, DEV_DIR, devinfo)
from project.fal.command import get_message
from fal_info import (FAL_PRINTER_PATH, ReqTypes, Results)


def root(request):
    return HttpResponseRedirect('./printer')


class View(FuncView):

    views = ['execute']

    def __printer_is_connected(self):
        for dev in devinfo.USB_LP_DEV_LIST:
            dev_path = os.path.join(DEV_DIR, dev)
            if os.path.exists(dev_path):
                return "true"
        return "false"

    def __send_request(self):
        try:
            response = Request(FAL_PRINTER_PATH,
                filter_cmd=SUDO_CMD).request(Entry(ReqTypes.CLEAR))
            if response.get_entrylist().get_entry().get_result() == Results.OK:
                return True
            else:
                return False
        except:
            return False

    def view(self, request):
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {"connected": self.__printer_is_connected()}}

    def view_execute(self, request):
        with self.processing(constname="ERR_PRINTER_STATE_CHANGE") as p:
            rst = False
            p.break_if_state_change()

            if self.__send_request():
                self.set_message(
                    get_message('MSG_PRINTER_BUFFER_CLEAR', is_log=True))
                rst = True
            else:
                self.set_message(
                    get_message('ERR_PRINTER_BUFFER_CLEAR', is_log=True))
            p.result(rst)

        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {}}


printer = View.as_view()
