# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

import time
from datetime import datetime

from django.utils.translation import ugettext as _
from django.http import HttpResponse
from django.utils import simplejson
from project.view import RootView
from project.fal.selfcheck import SelfCheck


ENCODING = "utf-8"

RESULT_AND_TIMESTAMP_FMT = "(%s: %s)\n%s"

STATUS_TABLE = {
    "NOT_EXEC": _(u"Not Execute"),
    "ABORT": _(u"Abort"),
    "PROCESSING": _(u"Processing")}


class StatusView(RootView):

    CATEGORY = ""

    def view(self, request, name):
        return {self.KEY_RESPONSE: HttpResponse(
                simplejson.dumps({'status': self._status(request, name)}),
                mimetype="application/json; charset=UTF-8;")}

    def _node_to_str(self, chk_node):
        status = chk_node.get_value("status", "")
        result = chk_node.get_value("result", "")
        timestamp = chk_node.get_value("timestamp", "")  # epoch

        if status == "COMPLETE" and timestamp:
            result_str = RESULT_AND_TIMESTAMP_FMT % (
                _(u"Last Update"),
                unicode(datetime(*time.localtime(int(timestamp))[:6])),
                result.decode(ENCODING))
        else:
            result_str = _(STATUS_TABLE.get(status))

        return result_str

    def _status(self, request=None, name=""):
        try:
            if not name:
                name = request.GET["name"]

            chk_node = SelfCheck().query().get().get_node(
                "checklist/check[@name='%s']" % name)

            result = self._node_to_str(chk_node)

        except:
            result = _(u"unknown")

        return result.encode(ENCODING)
