# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

from django.utils.translation import ugettext as _
from django.http import HttpResponseRedirect
#from root.modules.view import FuncView
from project.view import FuncView
#from root.modules.form import BaseForm
#import settings
from project.utils import importer, to_importable, is_django_app
from modules.utils import get_html_list
from modules.status_view import StatusView
import os



class View(FuncView):
    views = ['exe']

    def _get_result(selfi, status, result):
        _STATUS_TABLE = {
            'NOT_EXEC': _(u'Not Execute'),
            'PROCESSING': _(u'Processing'),
            'COMPLETE': _(u'Complete'),
            'ABORT': _(u'Abort'),
        }
        if status == 'COMPLETE':
            return result
        else:
            return _STATUS_TABLE.get(status, _(u'Unknown'))  # FIXME NOT_EXEC?

    def _get_checks(self):
        checks = {}
        try:
            checks_dir = os.path.join(self.CURRENT, 'checks')
            for name in os.listdir(checks_dir):
                check_dir = os.path.join(checks_dir, name)
                if is_django_app(check_dir):
                    try:
                        views = importer(to_importable(
                                os.path.join(check_dir, 'views')))
                    except:
                        continue
                    name = getattr(views, 'NAME', name)
                    num = getattr(views, 'NUM', 1000)
                    form = getattr(views, 'Form', None)
                    check = getattr(views, 'Check', None)
#                    result = check().result()
                    chkobj = check()
                    result = self._get_result(chkobj.status(), chkobj.result())
                    checks.update(
                        {chkobj.NAME:
                            {'name': name,
                             'num': num,
                             'form': form,
                             'check': check,
                             'result': result}})
        except:
            pass
        return checks

    def _sort_html(self, html_list, checks):
        sorted_list = []
        for html in html_list:
            html_name = os.path.splitext(os.path.basename(html))[0]
            try:
               html_info = (html, checks[html_name]["num"])
            except KeyError:
               continue
            sorted_list.append(html_info)
        sorted_list = sorted(sorted_list, key=lambda html_info: html_info[1])
        return [
            html_info[0]
            for html_info
            in sorted_list]
        
    def view(self, request):
        self._checks = self._get_checks()
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                'checks': self._checks,
                'js_list': get_html_list(
                    extdir='js'),
                'html_list': self._sort_html(
                    get_html_list(extdir='html'), self._checks),
                }}

    def view_exe(self, request):
        with self.processing() as p:
            p.break_if_state_change()

            run = False
            for name, check in self._checks.iteritems():
                form = check['form'](data=request.POST)
                if form.is_valid():
                    if form.cleaned_data['ENABLE']:
                        run = True
                    try:
                        chk = check['check']()
                        chk.run(form.cleaned_data)
                    except:
                        continue
                else:
                    if '__all__' in form.errors:
                        for error in form.errors['__all__']:
                            self.set_message("%(name)s: %(error)s" % {
                                'name': check['name'],
                                'error': error})
                    for field in form:
                        for error in field.errors:
                            self.set_message('%(name)s: %(field)s: %(error)s' % {
                                'name': check['name'],
                                'field': field.label,
                                'error': error})
            if run:
                self.set_message(_(u'Selfcheck started.'))
            else:
                self.set_message(_(u'Selfcheck not started.'))
            p.result(True)
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                'checks': self._checks,
                'js_list': get_html_list(
                    extdir='js'),
                'html_list': self._sort_html(
                    get_html_list(extdir='html'), self._checks),
                }}


name = View.as_view()
status = StatusView.as_view()
