# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django.conf import settings
from django.utils.translation import ugettext as _
from ionas.request import (Request, Entry)
from product_conf import SUDO_CMD
from fal_info import (FAL_NARSUS_PATH, FAL_FIRMWARE_PATH, FAL_PACKAGES_PATH,
        FAL_UPS_PATH, FAL_SNMP_PATH, FAL_SYSTEMLOG_PATH, FAL_MSNETWORK_PATH,
        FAL_QUOTA_PATH, ReqTypes, Results)

ACCESSLOG_ENTRY = "<systemlog><accesslog/></systemlog>"
NARSUS_ENTRY = "<narsus><service enable='false'/></narsus>"
FIRM_ENTRY = "<firmware><updatenotice enable='false'/><autoupdate enable='false'/></firmware>"
PKG_ENTRY = "<packages><updatenotice enable='false'/><autoupdate enable='false'/></packages>"
MSNETWORK_ENTRY = "<msnetwork><service enable='true'/></msnetwork>"

QUOTA_PATH = "volumelist/volume[text()='/mnt/sataraid1']"


class Services(object):

    def send_request(self, cmd, req_entry):
        try:
            response = Request(cmd, filter_cmd=SUDO_CMD).request(req_entry)
            if response.get_entrylist().get_entry().get_result() == Results.OK:
                return True
            else:
                return False
        except:
            return False

    def load_request(self, cmd):
        result = Request(cmd, filter_cmd=SUDO_CMD).request(
                Entry(ReqTypes.LOAD)).get_entrylist().get_entry().get()
        return result

    def get_accesslog(self):
        config = self.load_request(FAL_SYSTEMLOG_PATH)
        return config.get_node("accesslog").get_attr_value("enable")

    def set_accesslog(self, enable):
        if self.get_accesslog() == enable:
            return True
        entry = Entry(ReqTypes.APPLY, ACCESSLOG_ENTRY)
        entry.get().get_node("accesslog").set_attrs({"enable": enable})
        return self.send_request(FAL_SYSTEMLOG_PATH, entry)

    def get_msnetwork(self):
        config = self.load_request(FAL_MSNETWORK_PATH)
        return config.get_node("service").get_attr_value("enable")

    def set_msnetwork(self, enable):
        if self.get_msnetwork() == enable:
            return True
        entry = Entry(ReqTypes.APPLY, MSNETWORK_ENTRY)
        entry.get().get_node("service").set_attrs({"enable": enable})
        return self.send_request(FAL_MSNETWORK_PATH, entry)

    def get_ups(self):
        config = self.load_request(FAL_UPS_PATH)
        return config.get_node("warning").get_attr_value("enable")

    def set_ups(self, enable):
        if self.get_ups() == enable:
            return True
        config = self.load_request(FAL_UPS_PATH)
        config.get_node("warning").set_attrs({"enable": enable})
        entry = Entry(ReqTypes.APPLY, config)
        return self.send_request(FAL_UPS_PATH, entry)

    def get_narsus(self):
        config = Request(FAL_NARSUS_PATH, filter_cmd=SUDO_CMD).request(
            Entry(ReqTypes.QUERY)).get_entrylist().get_entry().get()
        return config.get_node("service").get_attr_value("enable")

    def set_narsus(self, enable):
        if self.get_narsus() == enable:
            return True
        entry = Entry(ReqTypes.APPLY, NARSUS_ENTRY)
        entry.get().get_node("service").set_attrs({"enable": enable})
        return self.send_request(FAL_NARSUS_PATH, entry)

    def get_snmp(self):
        config = self.load_request(FAL_SNMP_PATH)
        return config.get_node("trap").get_attr_value("enable")

    def get_info_snmp(self):
        config = self.load_request(FAL_SNMP_PATH)
        if config.get_value("trap/host"): 
            return True
        return False

    def get_info_narsus(self):
        config = self.load_request(FAL_NARSUS_PATH)
        if config.get_value("usecode/@enable") == "true": 
            return True
        return False

    def set_snmp(self, enable):
        if self.get_snmp() == enable:
            return True
        config = self.load_request(FAL_SNMP_PATH)
        config.get_node("trap").set_attrs({"enable": enable})
        entry = Entry(ReqTypes.SAVE, config)
        return self.send_request(FAL_SNMP_PATH, entry)

    def get_firmware(self):
        info = {"firm_notice": "false", "firm_auto": "false"}
        config = self.load_request(FAL_FIRMWARE_PATH)
        info["firm_notice"] = config.get_node(
                        "updatenotice").get_attr_value("enable")
        info["firm_auto"] = config.get_node(
                        "autoupdate").get_attr_value("enable")
        return info

    def set_firmware(self, notice, auto):
        conf = self.get_firmware()
        if notice == conf.get("firm_notice") and auto == conf.get("firm_auto"):
            return True
        entry = Entry(ReqTypes.APPLY, FIRM_ENTRY)
        entry.get().get_node("updatenotice").set_attrs({"enable": notice})
        entry.get().get_node("autoupdate").set_attrs({"enable": auto})
        return self.send_request(FAL_FIRMWARE_PATH, entry)

    def get_package(self):
        info = {"pkg_notice": "false", "pkg_auto": "false"}
        config = self.load_request(FAL_PACKAGES_PATH)
        info["pkg_notice"] = config.get_node(
                        "updatenotice").get_attr_value("enable")
        info["pkg_auto"] = config.get_node(
                        "autoupdate").get_attr_value("enable")
        return info

    def set_package(self, notice, auto):
        conf = self.get_package()
        if notice == conf.get("pkg_notice") and \
                auto == conf.get("pkg_auto"):
            return True
        entry = Entry(ReqTypes.APPLY, PKG_ENTRY)
        entry.get().get_node("updatenotice").set_attrs({"enable": notice})
        entry.get().get_node("autoupdate").set_attrs({"enable": auto})
        return self.send_request(FAL_PACKAGES_PATH, entry)

    def get_quota(self):
        config = self.load_request(FAL_QUOTA_PATH)
        return config.get_node(QUOTA_PATH).get_attr_value("enable")

    def set_quota(self, enable):
        if self.get_quota() == enable:
            return True
        config = self.load_request(FAL_QUOTA_PATH)
        config.get_node(QUOTA_PATH).set_attrs({"enable": enable})
        entry = Entry(ReqTypes.APPLY, config)
        return self.send_request(FAL_QUOTA_PATH, entry)

    def get_fix_service_init_config(self):
        config = {}
        config["quota"] = self.get_quota()
        config["ups"] = self.get_ups()
        config["snmp"] = self.get_snmp()
        if settings.NARSUS_IS_SUPPORTED:
            config["narsus"] = self.get_narsus()
        config["accesslog"] = self.get_accesslog()
        config.update(self.get_firmware())
        config.update(self.get_package())
        config["msnetwork"] = self.get_msnetwork()
        return config

    def set_fix_services(self, data):
        fail_setting = []
        if not self.set_quota(data["quota"]):
            fail_setting.append(_(u"quota"))
        if not self.set_ups(data["ups"]):
            fail_setting.append(_(u"UPS Warning"))
        if not self.set_snmp(data["snmp"]):
            fail_setting.append(_(u"SNMP Trap"))
        if settings.NARSUS_IS_SUPPORTED:
            if not self.set_narsus(data["narsus"]):
                fail_setting.append(_(u"NarSuS"))
        if not self.set_accesslog(data["accesslog"]):
            fail_setting.append(_(u"Access Log"))
        if not self.set_firmware(data["firm_notice"],
                                        data["firm_auto"]):
            fail_setting.append(_(u"Firmware Update Notice"))
            fail_setting.append(_(u"Firmware Auto Updating"))
        if not self.set_package(data["pkg_notice"], data["pkg_auto"]):
            fail_setting.append(_(u"Packages Update Notice"))
            fail_setting.append(_(u"Packages Auto Updating"))
        if not self.set_msnetwork(data["msnetwork"]):
            fail_setting.append(_(u"MS Network"))
        return fail_setting

    def get_fix_info_dict(self):
        info_dict = {}
        info_dict["snmp"] = self.get_info_snmp()
        if settings.NARSUS_IS_SUPPORTED:
            info_dict["narsus"] = self.get_info_narsus()
        return info_dict
