# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django.conf import settings
import os
import sys
from django.utils.translation import ugettext as _
from django.http import HttpResponseRedirect
from project.fal.command import (get_message, logging)
from project.view import FuncView
from root.modules.form import BaseForm
from root.modules.form.fields.radio import RadioField
from modules.services import Services


IMPORT_PREFIX = "from methods."
IMPORT_SUFFIX = " import Method"


def root(request):
    return HttpResponseRedirect('./list')


class View(FuncView):
    class Form(BaseForm):
        quota = RadioField(
            label=_(u'quota'),
            choices=(('true', _(u'enable')), ('false', _(u'disable'))),
            initial='false',
            required=True,
            validators=[])
        ups = RadioField(
            label=_(u'UPS Warning'),
            choices=(('true', _(u'enable')), ('false', _(u'disable'))),
            initial='false',
            required=True,
            validators=[])
        firm_auto = RadioField(
            label=_(u'Firmware Auto Updating'),
            choices=(('true', _(u'enable')), ('false', _(u'disable'))),
            initial='false',
            required=True,
            validators=[])
        firm_notice = RadioField(
            label=_(u'Firmware Update Notice'),
            choices=(('true', _(u'enable')), ('false', _(u'disable'))),
            initial='false',
            required=True,
            validators=[])
        pkg_auto = RadioField(
            label=_(u'Packages Auto Updating'),
            choices=(('true', _(u'enable')), ('false', _(u'disable'))),
            initial='false',
            required=True,
            validators=[])
        pkg_notice = RadioField(
            label=_(u'Packages Update Notice'),
            choices=(('true', _(u'enable')), ('false', _(u'disable'))),
            initial='false',
            required=True,
            validators=[])
        snmp = RadioField(
            label=_(u'SNMP Trap'),
            choices=(('true', _(u'enable')), ('false', _(u'disable'))),
            initial='false',
            required=True,
            validators=[])
        if settings.NARSUS_IS_SUPPORTED:
            narsus = RadioField(
                label=_(u'NarSuS'),
                choices=(('true', _(u'enable')), ('false', _(u'disable'))),
                initial='false',
                required=True,
                validators=[])
        accesslog = RadioField(
            label=_(u'Access Log'),
            choices=(('true', _(u'enable')), ('false', _(u'disable'))),
            initial='false',
            required=True,
            validators=[])
        msnetwork = RadioField(
            label=_(u'MS Network'),
            choices=(('true', _(u'enable')), ('false', _(u'disable'))),
            initial='false',
            required=True,
            validators=[])


    views = ['apply']
    service = Services()

    def __get_ext_service_list(self):
        ext_services = []
        current = os.path.dirname(sys.modules[self.__module__].__file__)
        dpath = os.path.join(current, "fields")
        for fname in sorted(os.listdir(dpath)):
            if (fname != "__init__.py" and
                fname.endswith(".py")):
                ext_services.append(fname.replace(".py", ""))
        return ext_services

    def __get_ext_init_config(self):
        config = {}
        for name in self.__get_ext_service_list():
            exec(IMPORT_PREFIX + name + IMPORT_SUFFIX)
            method = Method()
            config[name] = method.get()
        return config

    def __get_ext_info_dict(self):
        info_dict = {}
        for name in self.__get_ext_service_list():
            exec(IMPORT_PREFIX + name + IMPORT_SUFFIX)
            method = Method()
            info_dict[name] = method.get_info()
        return info_dict

    def __set_ext_services(self, data):
        fail_list = []
        for name in self.__get_ext_service_list():
            exec(IMPORT_PREFIX + name + IMPORT_SUFFIX)
            method = Method()
            if method.get() == data[name]:
                continue
            if not method.set(data[name]):
                fail_list.append(method.NAME)
        return fail_list

    def __get_html_list(self, extdir):
        current = os.path.dirname(sys.modules[self.__module__].__file__)
        dpath = os.path.join(current, extdir)
        return [os.path.join(dpath, fname)
                for fname
                in sorted(os.listdir(dpath))]

    def __get_init_config(self):
        config = {}
        config.update(self.__get_ext_init_config())
        config.update(self.service.get_fix_service_init_config())
        return config

    def __set_services(self, data):
        fix_fail_list = self.service.set_fix_services(data)
        ext_fail_list = self.__set_ext_services(data)
        return fix_fail_list + ext_fail_list

    def __get_info_dict(self):
        info_dict = {}
        info_dict.update(self.service.get_fix_info_dict())
        info_dict.update(self.__get_ext_info_dict())
        return info_dict
        
    def __get_code(self, msg):
        try:
            from fal_msginfo import Msg
            return getattr(Msg, msg)
        except:
            return None

    def view(self, request):
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                "NARSUS_IS_SUPPORTED": settings.NARSUS_IS_SUPPORTED,
                'form': self.Form(extdir="fields",
                        initial=self.__get_init_config()),
                'html_list': self.__get_html_list(extdir="html"),
                'info_dict': self.__get_info_dict(),
            },
        }

    def view_apply(self, request):
        with self.processing(constname="ERR_SERVICE_STATE_CHANGE") as p:
            form = self.Form(extdir="fields", data=request.POST)
            rst = False
            p.break_if_state_change()

            if form.is_valid():
                fail_list = self.__set_services(form.cleaned_data)
                if len(fail_list) <= 0:
                    self.set_message(
                        get_message('MSG_SERVICE_CHANGE', is_log=True))
                    rst = True
                else:
                    logging(self.__get_code("ERR_SERVICE_CHANGE"))
                    self.set_message(_(u'Fail to set following services:'))
                    self.set_message(", ".join(fail_list))
                p.result(rst)

        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                "NARSUS_IS_SUPPORTED": settings.NARSUS_IS_SUPPORTED,
                'form': form,
                'html_list': self.__get_html_list(extdir="html"),
                'info_dict': self.__get_info_dict(),
            },
        }

list = View.as_view()
