# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from ionas.request import (Request, Entry, EntryList)
from iobase.exception import NotFoundError
from iobase.xmldata import XmlNode
from product_conf import SUDO_CMD
from fal_info import (FAL_GROUPS_PATH, ReqTypes, Results)


FAL_GROUP_REQ_BASE_XML = {
        "add": "<groups><group><name/><id/><quota><limit/>" + \
               "</quota><userlist/></group></groups>",
        "delete": "<groups><group><name/></group></groups>",
        "modify": "<groups/>"}
LOCAL_GROUP_XPATH = "grouplist/group" + \
    "[(not(@hidden) or @hidden!='true') and not(@domain)]"
LIMIT_XPATH = "group/quota/limit"
USERLIST_XPATH = "group/userlist"
NAME_XPATH = "group/name"
XPATH_GROUP_BYNAME = "grouplist/group" + \
        "[(not(@hidden) or @hidden!='true') and not(@domain) and name='%s']"
TXT_MARK = "_text"
CBOX_MARK = "_checkbox"


class Group(object):

    def send_request(self, req_entry):
        try:
            response = Request(FAL_GROUPS_PATH, filter_cmd=SUDO_CMD).request(req_entry)
            if response.get_entrylist().get_entry().get_result() == Results.OK:
                return True
            else:
                return False
        except:
            # FIXME  例外種類ごとに、ユーザにメッセージを渡す？
            # 検討必要
            return False

    def add_group(self, data):
        entry = Entry(ReqTypes.ADD, FAL_GROUP_REQ_BASE_XML["add"])
        body = entry.get()
        body.get_node(NAME_XPATH).set_text(data["name"])
        body.get_node(LIMIT_XPATH).set_text(data["limit"])
        user_list = data["groupusers"]
        for user in user_list:
            user_node = XmlNode("user")
            user_node.append_child("name", user)
            body.get_node("group").get_node("userlist").append_child(user_node)
        return self.send_request(entry)

    def delete_entry(self, name):
        entry = Entry(ReqTypes.DELETE, FAL_GROUP_REQ_BASE_XML["delete"])
        entry.get().get_node(NAME_XPATH).set_text(name)
        return entry

    def delete_group(self, name):
        return self.send_request(self.delete_entry(name))

    def multi_del_request_entry_list(self, namelist=[]):
        entry_list = EntryList()
        for name in namelist:
            entry_list.append_entry(self.delete_entry(name))
        return entry_list

    def get_target_group_node(self, name):
        try:
            result = Request(FAL_GROUPS_PATH, filter_cmd=SUDO_CMD).request(
                    Entry(ReqTypes.LOAD)).get_entrylist().get_entry()
            target_group = result.get().get_node(
                    XPATH_GROUP_BYNAME % name)
            return target_group
        except NotFoundError:
            return      #FIXME

    def modify_group(self, name, data):
        entry = Entry(ReqTypes.MODIFY, FAL_GROUP_REQ_BASE_XML["modify"])
        entry.get().append_child(self.get_target_group_node(name))
        entry.get().get_node(LIMIT_XPATH).set_text(data["limit"])
        entry.get().get_node(USERLIST_XPATH).clear()
        user_list = data["groupusers"]
        for user in user_list:
            user_node = XmlNode("user")
            user_node.append_child("name", user)
            entry.get().get_node(USERLIST_XPATH).append_child(user_node)
        return self.send_request(entry)

    def get_group_node_list(self):
        try:
            load_result = Request(FAL_GROUPS_PATH, filter_cmd=SUDO_CMD).request(
                         Entry(ReqTypes.LOAD)).get_entrylist().get_entry()
            group_list = load_result.get().get_node_list(LOCAL_GROUP_XPATH)
        except NotFoundError:
            group_list = []
        return group_list
    
    def get_group_quota_limit_list(self):
        group_limit_dic = {}
        for group in self.get_group_node_list():
            group_name = group.get_node("name").get_text()
            group_limit = group.get_node("quota").get_node("limit").get_text()
            group_limit_dic[group_name] = group_limit
        return group_limit_dic

    def get_group_quota_dic(self):
        group_quota_dic = {}
        try:
            load_result = Request(FAL_GROUPS_PATH, filter_cmd=SUDO_CMD).request(
                      Entry(ReqTypes.QUERY)).get_entrylist().get_entry()
        except:
            return group_quota_dic    #FIXME
        try:
            for node in load_result.get().get_node_list(LOCAL_GROUP_XPATH):
                group_quota_dic[node.get_node("name").get_text()] = \
                        node.get_node("quota/use").get_text()
        except NotFoundError:
            pass
        return group_quota_dic

    def quota_request_entry_list(self, data):
        entry_list = EntryList()
        for node in self.get_group_node_list():
            name = node.get_node("name").get_text()
            entry = Entry(ReqTypes.MODIFY, FAL_GROUP_REQ_BASE_XML["modify"])
            entry.get().append_child(node)
            entry.get().get_node(LIMIT_XPATH).set_text(data[name + TXT_MARK])
            entry_list.append_entry(entry)
        return entry_list

    def get_target_group_info(self, name):
        info = {"groupusers": [], "limit": ""}
        try:
            result = Request(FAL_GROUPS_PATH, filter_cmd=SUDO_CMD).request(
                   Entry(ReqTypes.LOAD)).get_entrylist().get_entry()
            target_group = result.get().get_node(
                             XPATH_GROUP_BYNAME % name)
            for node in target_group.get_node_list("userlist/user"):
                info["groupusers"].append(node.get_node("name").get_text())
            info["limit"] = target_group.get_node("quota/limit").get_text()
            return info
        except NotFoundError:
            return info     #FIXME

