# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django.utils.translation import ugettext as _
from django.http import HttpResponseRedirect
from project.view import FuncView
from root.modules.form import BaseForm
from project.fal.command import get_message as fgettext
from project.fal.group import Group
from project.fal.quota import Quota

from project.forms.fields.extras.quota import QuotaLimitField


def root(request):
    return HttpResponseRedirect('./quota')


class QuotaView(FuncView):
    views = ['modify']

    class Form(BaseForm):
        def __init__(self, groups, *args, **kwargs):
            super(QuotaView.Form, self).__init__(*args, **kwargs)
            for name, param in groups.iteritems():
                gid = param.get('gid', None)
                if gid is None:
                    continue
                #self.fields[name] = QuotaLimitField(
                #        label=name,
                #        attrs={'id': 'id_%s' % gid},
                #        required=False,
                #        initial=param.get('quota') or 0.0)
                self.fields["gid_%s" % gid] = QuotaLimitField(label=name,
                                                              attrs={'id': 'id_%s' % gid},
                                                              required=False,
                                                              initial=param.get('quota') or 0.0)

    def _get_domain_groups(self):
        domain_groups = Group().domain_list()
        grouplist = Quota().query()['grouplist']
        gid = 20000  # dummy id. smb.conf winbind uid = 20000-30000
        for name, param in sorted(domain_groups.iteritems()):
            param['gid'] = str(gid)
            gid = gid + 1
            try:
                name = name.encode("utf-8")
                param['quota'] = grouplist[name]['limit']
                param['used'] = grouplist[name]['used']
            except:
                continue
        return domain_groups

    def view(self, request):
        gobj = Group()
        useds = gobj.query()
        groups = gobj.load()
        domain_groups = self._get_domain_groups()
        form = self.Form(dict(groups, **domain_groups))
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                'enable': Quota().is_enabled(),
                'form': form,
                #'group_list': [{
                #    'name': name,
                #    'quota_used': useds.get(name, 0),
                #    'quota_field': form[name],
                #} for name in groups.keys()],
                'group_list': [{
                    'name': name,
                    'quota_used': useds.get(name, 0),
                    'quota_field': form["gid_%s" % groupinfo["gid"]],
                } for name, groupinfo in groups.iteritems() if groupinfo.get("gid")],
                'domain_group_list': [{
                    'name': domain_groups[name].get('upn'),
                    'quota_used': domain_groups[name].get('used', 0.0),
                    'quota_field': form["gid_%s" % groupinfo["gid"]],
                } for name, groupinfo in domain_groups.iteritems() if groupinfo.get("gid")],
            }
        }

    def view_modify(self, request):
        with self.processing(constname="ERR_QUOTA_STATE_CHANGE") as p:
            gobj = Group()
            groups = gobj.load()
            domain_groups = self._get_domain_groups()
            useds = gobj.query()
            form = self.Form(dict(groups, **domain_groups), data=request.POST)
            p.break_if_state_change()

            if form.is_valid():
                result = True

                # ローカルグループ制限値設定
                #for name in groups.keys():
                #    quota = form.cleaned_data[name]
                #    gobj.modify(name, quota=quota, log=False)
                #    if gobj.get_result():
                #        result = False
                for groupname, groupinfo in groups.iteritems():
                    if groupinfo.get("gid", None) is None:
                        continue
                    quota = form.cleaned_data.get("gid_%s" % groupinfo["gid"], None)
                    if quota is None:
                        continue
                    gobj.modify(groupname, quota=quota, log=False)
                    if gobj.get_result():
                        result = False

                # ドメイングループ制限値設定
                qdict = {'grouplist': {}}
                #for name in domain_groups.keys():
                #    quota = form.cleaned_data.get(name, None)
                #    if quota is None:
                #        continue
                #    qdict['grouplist'][name] = {'limit': quota}
                for groupname, groupinfo in domain_groups.iteritems():
                    if groupinfo.get("gid", None) is None:
                        continue
                    quota = form.cleaned_data.get("gid_%s" % groupinfo["gid"], None)
                    if quota is None:
                        continue
                    qdict['grouplist'][groupname] = {'limit': quota}
                qobj = Quota()
                qobj.apply(qdict, log=False)
                if not qobj.get_result(boolean=True):
                    result = False

                if result:
                    self.set_message(fgettext('MSG_QUOTA_CHANGE', is_log=True))
                else:
                    self.set_message(fgettext('ERR_QUOTA_CHANGE', is_log=True))
                p.result(result)
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                'enable': Quota().is_enabled(),
                'form': form,
                #'group_list': [{
                #    'name': name,
                #    'quota_used': useds.get(name, 0),
                #    'quota_field': form[name]} for name in groups.keys()],
                'group_list': [{
                    'name': name,
                    'quota_used': useds.get(name, 0),
                    'quota_field': form["gid_%s" % groupinfo["gid"]],
                } for name, groupinfo in groups.iteritems() if groupinfo.get("gid")],
                'domain_group_list': [{
                    'name': domain_groups[name].get('upn'),
                    'quota_used': domain_groups[name].get('used', 0.0),
                    'quota_field': form["gid_%s" % groupinfo["gid"]],
                } for name, groupinfo in domain_groups.iteritems() if groupinfo.get("gid")],
            },
        }


quota = QuotaView.as_view()
