# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from ionas.request import (Request, Entry)
from product_conf import SUDO_CMD
from fal_info import (FAL_MSNETWORK_PATH, ReqTypes, Results)


MS_ENTRY = "<msnetwork><assign><wg><workgroup/></wg><nt><domain/><dcname/>" + \
           "</nt><ad><netbios/><fulldns/><dcname/><adminname/>" + \
           "<adminpasswd/></ad></assign></msnetwork>"
WG_XPATH = "./assign/wg/workgroup"
NT_XPATH = "./assign/nt/domain"
NT_DC_XPATH = "./assign/nt/dcname"
NETBIOS_XPATH = "./assign/ad/netbios"
FULLDNS_XPATH = "./assign/ad/fulldns"
AD_DC_XPATH = "./assign/ad/dcname"
ADMIN_XPATH = "./assign/ad/adminname"
PASSWD_XPATH = "./assign/ad/adminpasswd"


class MSnetwork(object):
    def msnetwork_set(self, data):
        entry = Entry(ReqTypes.APPLY, MS_ENTRY)
        if data["method"] == "1":
            entry.get().get_node("assign").set_attrs({"method": "wg"})
            entry.get().get_node(WG_XPATH).set_text(data["wg_name"])
        elif data["method"] == "2":
            entry.get().get_node("assign").set_attrs({"method": "nt"})
            entry.get().get_node(NT_XPATH).set_text(data["nt_name"])
            entry.get().get_node(NT_DC_XPATH).set_text(data["nt_dcname"])
        elif data["method"] == "3":
            entry.get().get_node("assign").set_attrs({"method": "ad"})
            entry.get().get_node(NETBIOS_XPATH).set_text(data["netbios"])
            entry.get().get_node(FULLDNS_XPATH).set_text(data["fulldns"])
            entry.get().get_node(AD_DC_XPATH).set_text(data["ad_dcname"])
            entry.get().get_node(ADMIN_XPATH).set_text(data["adminname"])
            entry.get().get_node(PASSWD_XPATH).set_text(data["passwd"])
        entry.get().append_child("winssrv", data["winssrv"])
        return self.send_request(FAL_MSNETWORK_PATH, entry)

    def send_request(self, cmd, entry):
        try:
            response = Request(cmd,
                filter_cmd=SUDO_CMD).request(entry).get_entrylist().get_entry()
            if response.get_result() == Results.OK:
                return True
            else:
                return False
        except:
            return False

    def get_msnetwork_config(self):
        init = {}

        try:
            result = Request(FAL_MSNETWORK_PATH, filter_cmd=SUDO_CMD).request(
                           Entry(ReqTypes.LOAD)).get_entrylist().get_entry()
        except:
            return init     # FIXME

        method = result.get().get_node("assign").get_attr_value("method")

        if method == "wg":
            init["method"] = 1
        elif method == "nt":
            init["method"] = 2
        elif method == "ad":
            init["method"] = 3

        init["wg_name"] = result.get().get_value(WG_XPATH)
        init["nt_name"] = result.get().get_value(NT_XPATH)
        init["nt_dcname"] = result.get().get_value(NT_DC_XPATH)
        init["netbios"] = result.get().get_value(NETBIOS_XPATH)
        init["fulldns"] = result.get().get_value(FULLDNS_XPATH)
        init["ad_dcname"] = result.get().get_value(AD_DC_XPATH)
        init["adminname"] = result.get().get_value(ADMIN_XPATH)
        init["passwd"] = result.get().get_value(PASSWD_XPATH, do_strip=False)
        init["winssrv"] = result.get().get_value("winssrv")

        return init

