# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django.utils.translation import ugettext as _
from django.http import HttpResponseRedirect
from django.core.exceptions import ValidationError
from project.view import FuncView
from project.fal.command import get_message
from root.modules.form import BaseForm
from modules.msnetwork import MSnetwork
from project.forms.fields.extras.microsoftnetwork import MicrosoftNetworkMethodField


def root(request):
    return HttpResponseRedirect('./msusergroup')


class Form(BaseForm):
    def __init__(self, *args, **kwargs):
        from root.modules.form.fields.radio import RadioField
        from project.forms.fields.extras.domain import (DomainNameField,
                AdControllerField, NetBIOSDomainField, WorkGroupField,
                NtControllerField, AdAdminPassField, WinsSrvField, AdUserField)
        super(Form, self).__init__(*args, **kwargs)

        #self.fields["method"] = RadioField(
        #        choices=(1, _(u'WorkGroup')), (2, _(u'NT Domain')),
        #                 (3, _(u'ActiveDirectory'))),
        #        label=_(u'Method'),
        #        initial=1,
        #        required=True,
        #        validators=[])
        self.fields["method"] = MicrosoftNetworkMethodField(label=_(u"Method"))
        self.fields["wg_name"] = WorkGroupField(
                label=_(u'WorkGroup Name'),
                max_length=15,
                required=False)
        self.fields["nt_name"] = NetBIOSDomainField(
                label=_(u'Domain Name'),
                max_length=15,
                required=False)
        self.fields["nt_dcname"] = NtControllerField(
                label=_(u'Domain Controller Name'),
                max_length=15,
                required=False)
        self.fields["ad_dcname"] = AdControllerField(
                label=_(u'Domain Controller Name'),
                max_length=155,
                required=False)
        self.fields["netbios"] = NetBIOSDomainField(
                label=_(u'Domain Name(NetBIOS Name)'),
                max_length=15,
                required=False)
        self.fields["fulldns"] = DomainNameField(
                label=_(u'Domain Name(Full DNS Name)'),
                max_length=155,
                required=False)
        self.fields["adminname"] = AdUserField(
                label=_(u'Administrator Name'),
                max_length=20,
                required=False)
        self.fields["passwd"] = AdAdminPassField(
                label=_(u'Administrator Password'),
                max_length=20,
                required=False)
        self.fields["winssrv"] = WinsSrvField(
                label=_(u'WINS Server'),
                required=False)

    def clean(self):
        if self.cleaned_data.get("method") == "wg":
            if self.cleaned_data.get("wg_name") == "":
                raise ValidationError(_(u'"WorkGroup Name" is required.'))
        elif self.cleaned_data.get("method") == "nt":
            if self.cleaned_data.get("nt_name") == "":
                raise ValidationError(_(u'"Domain Name" is required.'))
            if self.cleaned_data.get("nt_dcname") == "":
                raise ValidationError(_(u'"Domain Controller Name" is required.'))
        elif self.cleaned_data.get("method") == "ad":
            if self.cleaned_data.get("netbios") == "":
                raise ValidationError(_(u'"Domain Name(NetBIOS Name)" is required.'))
            if self.cleaned_data.get("fulldns") == "":
                raise ValidationError(_(u'"Domain Name(Full DNS Name)" is required.'))
            if self.cleaned_data.get("ad_dcname") == "":
                raise ValidationError(_(u'"Domain Controller Name" is required.'))
            if self.cleaned_data.get("adminname") == "":
                raise ValidationError(_(u'"Administrator Name" is required.'))
            if self.cleaned_data.get("passwd") == "":
                raise ValidationError(_(u'"Administrator Password" is required.'))
        return self.cleaned_data


class View(FuncView):
    views = ['apply']
    msnetwork = MSnetwork()

    def __initial(self):
        initial = self.msnetwork.get_msnetwork_config()
        initial["method"] = {
            1: "wg",
            2: "nt",
            3: "ad",
        }.get(initial["method"], "wg")
        return initial

    def __data(self, data):
        data["method"] = {
            "wg": "1",
            "nt": "2",
            "ad": "3",
        }.get(data["method"], "1")
        return data

    def view(self, request):
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                #'form': Form(initial=self.msnetwork.get_msnetwork_config()),
                'form': Form(initial=self.__initial()),
            },
        }

    def view_apply(self, request):
        with self.processing(constname="ERR_MS_NET_STATE_CHANGE") as p:
            form = Form(data=request.POST)
            rst = False
            p.break_if_state_change()

            if form.is_valid():
                #if self.msnetwork.msnetwork_set(data=form.cleaned_data):
                if self.msnetwork.msnetwork_set(data=self.__data(form.cleaned_data)):
                    self.set_message(
                            get_message('MSG_MS_NET_CHANGE', is_log=True))
                    rst = True
                else:
                    self.set_message(
                            get_message('ERR_MS_NET_CHANGE', is_log=True))
                p.result(rst)

        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {'form': form}}


msusergroup = View.as_view()
