# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django.utils.translation import ugettext as _
from django.http import HttpResponseRedirect
from project.view import FuncView
from root.modules.form import BaseForm
import settings
from ..modules.form import Form as UserSettingForm

from project.fal.user import User
from project.fal.quota import Quota


def root(request):
    return HttpResponseRedirect('./modify')


class Form(UserSettingForm):
    def __init__(self, *args, **kwargs):
        super(Form, self).__init__(modify=True, *args, **kwargs)


class ModifyView(FuncView):
    views = ['modify', 'list']

    URL = '/'.join([settings.MENU_URL, 'modify'])

    def view(self, request, name):
        uobj = User()
        try:
            uinfo = uobj.load()[name]
        except:
            uinfo = {}
        initial = {
            'name': uinfo.get('name', ''),
            'passwd1': uinfo.get('passwd', ''),
            'passwd2': uinfo.get('passwd', ''),
            'fullname': uinfo.get('fullname', ''),
            'desc': uinfo.get('desc', ''),
            'primary_group': uinfo.get('gid', None),
            'quota': uinfo.get('quota', None),
        }
        form = Form(initial=initial)
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                'url': self.URL,
                'form':form,
                'quota_enable': Quota().is_enabled()}}

    def view_modify(self, request, name):
        with self.processing(constname="ERR_USER_STATE_CHANGE") as p:
            form = Form(data=request.POST)
            p.break_if_state_change()

            if form.is_valid():
                name = form.cleaned_data['name']
                fullname = form.cleaned_data['fullname']
                desc = form.cleaned_data['desc']
                passwd = form.cleaned_data['passwd1']
                quota = form.cleaned_data['quota']
                gid = form.cleaned_data['primary_group']

                uobj = User()
                uobj.modify(name,
                        fullname=fullname,
                        desc=desc,
                        passwd=passwd,
                        gid=int(gid) if gid else '',
                        quota=quota)
                self.set_message(uobj.get_message())
                p.result(uobj.get_result(boolean=True))
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                'url': self.URL,
                'form':form,
                'quota_enable': Quota().is_enabled()}}

#    def view_list(self, request, name):
#        return {self.KEY_REDIRECT: '../../list'}

modify = ModifyView.as_view()
