# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from ionas.request import (Request, Entry)
from iobase.exception import (NotFoundError)

from django.utils.translation import ugettext as _
from root.modules.form.fields.select import SelectField

# FIXME fal_infoからとるべき情報
try:
    from fal_info import FAL_GROUPS_PATH
except:
    FAL_GROUPS_PATH = "/mnt/data/bin/fal_groups.py"
LOAD_TYPE = "load"

NAME_NAME = "name"
ID_NAME = "id"
GROUPLIST_PATH = "grouplist/*"

NOT_SELECTED_TUPLE = (("", _(u"not selected")),)

class GroupSelectField(SelectField):
    def __init__(self, choices=(), **kwargs):
        """Set choices dynamically."""
        SelectField.__init__(
            self, choices=self.__get_group_tuple(), **kwargs)

    def __get_group_node_list(self):
        load_result = Request(FAL_GROUPS_PATH).request(
            Entry(LOAD_TYPE)).get_entrylist().get_entry()
        try:
            group_node_list = \
                load_result.get().get_node_list(GROUPLIST_PATH)
        except NotFoundError:
            """There is nothing. initial state."""
            group_node_list = []
        return group_node_list

    def __get_group_tuple(self):
        group_tuple = NOT_SELECTED_TUPLE
        group_tuple += tuple(
            [(group_node.get_node(ID_NAME).get_text(),
            group_node.get_node(NAME_NAME).get_text())
            for group_node
            in self.__get_group_node_list()])
        return group_tuple
