# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django.utils.translation import ugettext as _
from django.http import HttpResponseRedirect
from project.view import FuncView
from root.modules.form import BaseForm
from project.fal.command import get_message as fgettext
from project.fal.user import User
from project.fal.quota import Quota

from project.forms.fields.extras.quota import QuotaField, QuotaLimitField


def root(request):
    return HttpResponseRedirect('./quota')


class QuotaView(FuncView):
    views = ['modify']

    class Form(BaseForm):
        def __init__(self, users, *args, **kwargs):
            super(QuotaView.Form, self).__init__(*args, **kwargs)
            for name, param in users.iteritems():
                uid = param.get('uid', None)
                if uid is None:
                    continue
                #self.fields[name] = QuotaLimitField(
                #        label=name,
                #        attrs={'id': 'id_%s' % uid},
                #        required=False,
                #        initial=param.get('quota') or 0.0)
                self.fields["uid_%s" % uid] = QuotaLimitField(label=name,
                                                              attrs={'id': 'id_%s' % uid},
                                                              required=False,
                                                              initial=param.get('quota') or 0.0)

    def _get_domain_users(self):
        domain_users = User().domain_list()
        userlist = Quota().query()['userlist']
        uid = 20000  # dummy id. smb.conf winbind uid = 20000-30000
        for name, param in sorted(domain_users.iteritems()):
            param['uid'] = str(uid)
            uid = uid + 1
            try:
                param['quota'] = userlist[name]['limit']
                param['used'] = userlist[name]['used']
            except:
                continue
        return domain_users

    def _domain_users_info(self):
        domain_users = User().domain_list()
        userlist = Quota().query()['userlist']
        uid = 20000  # dummy id. smb.conf winbind uid = 20000-30000
        for name, param in sorted(domain_users.iteritems()):
            try:
                name = name.encode("utf-8")
                qlimit = userlist.get(name, {}).get('limit', 0.0)
                used = userlist.get(name, {}).get('used', 0.0)

                param['name'] = name
                param['quota'] = qlimit
                param['used'] = used
                param['uid'] = uid
                uid += 1
            except:
                pass
        return domain_users


    def view(self, request):
        uobj = User()
        useds = uobj.query()
        users = uobj.load()
#        domain_users = self._get_domain_users()
        domain_users = self._domain_users_info()
        form = self.Form(dict(users, **domain_users))
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                'enable': Quota().is_enabled(),
                'form': form,
                #'user_list': [{
                #    'name': name,
                #    'quota_used': useds[name],
                #    'quota_field': form[name],
                #} for name in users.keys()],
                'user_list': [{
                    'name': name,
                    'quota_used': useds[name],
                    'quota_field': form["uid_%s" % userinfo["uid"]],
                } for name, userinfo in users.iteritems() if userinfo.get("uid")],
                'domain_user_list': [{
                    'name': domain_users[name].get('name'),
                    'quota_used': domain_users[name].get('used', '0.0'),
                    'quota_field': form["uid_%s" % userinfo["uid"]],
                } for name, userinfo in domain_users.iteritems() if userinfo.get("uid")],
            },
        }

    def view_modify(self, request):
        with self.processing(constname="ERR_QUOTA_STATE_CHANGE") as p:
            uobj = User()
            users = uobj.load()
#            domain_users = self._get_domain_users()
            domain_users = self._domain_users_info()
            useds = uobj.query()
            form = self.Form(dict(users, **domain_users), data=request.POST)
            p.break_if_state_change()

            if form.is_valid():
                result = True

                # ローカルユーザー制限値設定
                #for name in users.keys():
                #    quota = form.cleaned_data[name]
                #    uobj.modify(name, quota=quota, log=False)
                #    if uobj.get_result():
                #        result = False
                for username, userinfo in users.iteritems():
                    if userinfo.get("uid", None) is None:
                        continue
                    quota = form.cleaned_data.get("uid_%s" % userinfo["uid"], None)
                    if quota is None:
                        continue
                    uobj.modify(username, quota=quota, log=False)
                    if uobj.get_result():
                        result = False

                # ドメインユーザー制限値設定
                qdict = {'userlist': {}}
                #for name in domain_users.keys():
                #    quota = form.cleaned_data.get(name, None)
                #    if quota is None:
                #        continue
                #    qdict['userlist'][name] = {'limit': quota}
                for username, userinfo in domain_users.iteritems():
                    if userinfo.get("uid", None) is None:
                        continue
                    quota = form.cleaned_data.get("uid_%s" % userinfo["uid"], None)
                    if quota is None:
                        continue
                    qdict['userlist'][username] = {'limit': quota}
                qobj = Quota()
                qobj.apply(qdict, log=False)
                if not qobj.get_result(boolean=True):
                    result = False

                if result:
                    self.set_message(fgettext('MSG_QUOTA_CHANGE', is_log=True))
                else:
                    self.set_message(fgettext('ERR_QUOTA_CHANGE', is_log=True))
                p.result(result)
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                'enable': Quota().is_enabled(),
                'form': form,
                #'user_list': [{
                #    'name': name,
                #    'quota_used': useds[name],
                #    'quota_field': form[name],
                #    'used': None} for name in users.keys()],
                'user_list': [{
                    'name': name,
                    'quota_used': useds[name],
                    'quota_field': form["uid_%s" % userinfo["uid"]],
                } for name, userinfo in users.iteritems() if userinfo.get("uid")],
                'domain_user_list': [{
                    'name': domain_users[name].get('name'),
                    'quota_used': domain_users[name].get('used', '0.0'),
                    'quota_field': form["uid_%s" % userinfo["uid"]],
                } for name, userinfo in domain_users.iteritems() if userinfo.get("uid")],
            },
        }

quota = QuotaView.as_view()
