# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django.conf import settings
from django.utils.translation import ugettext as _
from django.http import HttpResponseRedirect, HttpResponse
from django.utils import simplejson
from project.view import RootView, FuncView
from project.view import MenuView
from project.utils import importer, to_importable
from project.fal.buzzer import Buzzer
import os
import re


def root(request):
    return HttpResponseRedirect('./main')


class MainView(RootView):

    def view(self, request):
        return {self.KEY_TEMPLATE: 'main.html', self.KEY_TEMPLATE_DIC: {
            "INTERNAL_VOLUME_RAID_IS_SUPPORTED": settings.INTERNAL_VOLUME_RAID_IS_SUPPORTED,
            'FRAME_SRC': request.session.get('FRAME_SRC', None) or './menu',
            'SLIDES': self._get_slides(),
            'IS_LOGINED': True,
        }}

    def _get_slides(self):
        slides = []
        slides_dir = os.path.join(self.CURRENT, 'slides')
        if os.path.isdir(slides_dir):
            for name in sorted(os.listdir(slides_dir)):
                if not re.match('.*\.py$', name):
                    continue
                file = os.path.join(slides_dir, name)
                if os.path.isfile(file):
                    try:
                        slide = importer(to_importable(file))
                        reload(slide)
                        slides.append({
                            'template': os.path.join(slides_dir,
                                                     slide.template),
                            'template_dic': slide.template_dic,
                        })
                    except:
                        continue
        return slides


class StatusView(RootView):
    def view(self, request):
        return {self.KEY_RESPONSE: HttpResponse(
                simplejson.dumps(self._get_status()),
                mimetype="application/json; charset=UTF-8;")}

    def _get_status(self):
        status = {}
        try:
            from project.fal.sysinfo import SysInfo
            sobj = SysInfo()
            sysinfo = sobj.query()
            status['name'] = sysinfo['host']['name']
            status['version'] = sysinfo['fwversion']
            nowarranty = False
            try:
                with open('/var/lib/landisk/nowarranty', 'r') as fh:
                    nowarranty = bool(int(fh.read()))
            except:
                pass
            if nowarranty:
                status['version'] = _(u'not supported')
#            from project.fal.eeprom import EEPROM
#            if EEPROM().gplv3():
#                status['version'] = _(u'not supported')

            from project.fal.volctrl import Volctrl
            vobj = Volctrl()
            volinfo = vobj.query()
            status['volspaceused'] = vobj.gb(volinfo['size']['use'])
            status['volspacetotal'] = vobj.gb(volinfo['size']['total'])

            from project.fal.statemgr import Statemgr
            status['raid'] = Statemgr().raid_status()
#            from project.fal.volinfo import Volinfo
#            status['raid'] = Volinfo().status()

            from project.fal.network import Network
            nwinfo = Network().query()
            status['ip'] = nwinfo['lan1']['ipv4']['addr'] or '-'
            status['mac'] = nwinfo['lan1']['macaddr'] or '-'
        except:
            status = {
                'name': _(u'unknown'),
                'version': _(u'unknown'),
                'ip': _(u'unknown'),
                'mac': _(u'unknown'),
                'volspaceused': _(u'unknown'),
                'volspacetotal': _(u'unknown'),
                'raid': _(u'unknown'),
                'notice': _(u'unknown'),
            }
        return status


class MelodyStartView(RootView):

    views = []

    def view(self, request):
        buz = Buzzer()
        buz.melody()
        return {
            self.KEY_RESPONSE: HttpResponse(simplejson.dumps({}),
                                            mimetype="application/json; charset=UTF-8;"),
        }


class MelodyStopView(RootView):

    views = []

    def view(self, request):
        buz = Buzzer()
        buz.stop()
        return {
            self.KEY_RESPONSE: HttpResponse(simplejson.dumps({}),
                                            mimetype="application/json; charset=UTF-8;"),
        }


class HelpOnView(RootView):

    views = []

    def view(self, request):
        json = {
            "result": False
        }
        try:
            from project.fal.user import User
            json = {
                "result": User().adminmod(help=True, log=False),
            }
        except:
            pass
        return {
            self.KEY_RESPONSE: HttpResponse(simplejson.dumps(json),
                                            mimetype="application/json;charset=UTF-8;"),
        }


class HelpOffView(RootView):

    views = []

    def view(self, request):
        json = {
            "result": False,
        }
        try:
            from project.fal.user import User
            json = {
                "result": User().adminmod(help=False, log=False),
            }
        except:
            pass
        return {
            self.KEY_RESPONSE: HttpResponse(simplejson.dumps(json),
                                            mimetype="application/json;charset=UTF-8;"),
        }


class MenuView(MenuView):

    pass


main = MainView.as_view()
menu = MenuView.as_view()
