# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
import os
import datetime

from django.utils.translation import ugettext as _
from django.http import HttpResponseRedirect
from django.core.exceptions import ValidationError
from project.view import FuncView
from project.view import StatusView
from root.modules.form import BaseForm

from project.fal.raidctrl import Raidctrl
from project.forms.fields.extras.enable import EnableField
from project.forms.fields.radio import RadioField
from project.forms.fields.extras.datetime import SelectTimeField


def root(request):
    return HttpResponseRedirect('./activerepair')


class View(FuncView):
    class Form(BaseForm):
        schedule_enable = EnableField(
                label=_(u'schedule function'))
        schedule_day = RadioField(
                choices=(
                    ('Sun', _(u'Sun')),
                    ('Mon', _(u'Mon')),
                    ('Tue', _(u'Tue')),
                    ('Wed', _(u'Wed')),
                    ('Thu', _(u'Thu')),
                    ('Fri', _(u'Fri')),
                    ('Sat', _(u'Sat'))),
                initial='Sat',
                label='%s(%s)' % (
                        _(u'days of the week'),
                        _(u'of the every month beginning')),
                required=False)
        schedule_time = SelectTimeField(
                label=_(u'time'),
                show_seconds=False,
                required=False)

        def clean_schedule_day(self):
            val = self.cleaned_data['schedule_day']
            enable = self.cleaned_data['schedule_enable']
            if enable and not val:
                raise ValidationError(_(u'This field is required.'))
            return val

        def clean_schedule_time(self):
            val = self.cleaned_data['schedule_time']
            enable = self.cleaned_data['schedule_enable']
            if enable and not isinstance(val, datetime.time):
                raise ValidationError(_(u'This field is required.'))
            return val

    views = ['start', 'stop', 'apply']

    def view(self, request):
        info = Raidctrl().load()
        initial = {
            'schedule_enable': info.get('enable', True),
            'schedule_day': info.get('day', 'Sun') or 'Sun',
            'schedule_time': info.get('time', None),
        }
        form = self.Form(initial=initial)
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                'form': form,
                'status': ActiverepairStatusView()._status(),
                'running': ActiverepairStatusView().is_running(),
            },
        }

    def view_start(self, request):
        with self.processing(constname='ERR_VOLUME_STATE_CHANGE') as p:
            form = self.Form(data=request.POST)
            p.break_if_state_change()

            robj = Raidctrl()
            robj.activerepair(
                    volume=robj.VOLUME['internal'][1],
                    operation="start",
                    log=False)
            self.set_message(robj.get_message())
            p.result(robj.get_result(boolean=True))

            if p.result():
                import time
                for i in range(0, 100):
                    if ActiverepairStatusView().is_running():
                        break
                    time.sleep(3)
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                'form': form,
                'status': ActiverepairStatusView()._status(),
                'running': ActiverepairStatusView().is_running(),
            },
        }

    def view_stop(self, request):
        with self.processing(constname='ERR_VOLUME_STATE_CHANGE') as p:
            form = self.Form(data=request.POST)
            p.break_if_state_change()

            robj = Raidctrl()
            robj.activerepair(
                    volume=robj.VOLUME['internal'][1],
                    operation="stop",
                    log=False)
            self.set_message(robj.get_message())
            p.result(robj.get_result(boolean=True))

            if p.result():
                import time
                for i in range(0, 100):
                    if not ActiverepairStatusView().is_running():
                        break
                    time.sleep(3)
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                'form': form,
                'status': ActiverepairStatusView()._status(),
                'running': ActiverepairStatusView().is_running(),
            },
        }

    def view_apply(self, request):
        with self.processing(constname='ERR_ACTIVE_REPAIR_STATE_CHANGE') as p:
            form = self.Form(data=request.POST)
            p.break_if_state_change()
        
            if form.is_valid():
                self.logger.debug(form.cleaned_data)
                enable = form.cleaned_data['schedule_enable']
                day = form.cleaned_data['schedule_day']
                time = form.cleaned_data['schedule_time']

                robj = Raidctrl()
                robj.activerepair(
                        operation='schedule',
                        schedule=enable,
                        day=day,
                        time=time)
                self.set_message(robj.get_message())
                p.result(robj.get_result(boolean=True))
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                'form': form,
                'status': ActiverepairStatusView()._status(),
                'running': ActiverepairStatusView().is_running(),
            },
        }


class ActiverepairStatusView(StatusView):
    def is_running(self):
        try:
            from project.fal.statemgr import Statemgr
            return Statemgr().query().get('activerepair', 'nooperation') == 'running'
        except:
            return False

    def _status(self):
        try:
            status = ''
            _dict = Raidctrl().activerepair_getstatus()
            status = {
                'NOT_EXEC': _(u"ActiveRepair is not exec"),
                'PROCESSING': _(u"ActiveRepair is running"),
                'COMPLETE': _(u"ActiveRepair is complete"),
                'STOP': _(u"ActiveRepair is suspended"),
                'CANCEL': _(u"ActiveRepair is canceled."),
            }.get(_dict['status'], '')
            if _dict['status'] == 'COMPLETE':
                status = "%s(%s)" % (
                    status,
                    {
                        'success': _(u"success"),
                        'failed': _(u"failed"),
                    }.get(_dict['result']))
            return status
        except:
            return ''


activerepair = View.as_view()
status = ActiverepairStatusView.as_view()
