# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
import os
from django.utils.translation import ugettext as _
from django import forms
from project.view import FuncView
from project.forms.fields.extras.enable import EnableField
import settings
from project.fal.raidctrl import Raidctrl


class ExtendForm(forms.Form):
    autoextend = EnableField(label=_(u"Auto Extend"))


class ExtendView(FuncView):
    views = ['apply', 'extend_6']

    def view(self, request):
        form = ExtendForm(initial=self.__get_initial())

        template = os.path.join(self.TEMPLATES, 'template.html')
        directory = {
            'form': form,
            'disklist': self.__get_disklist(),
            'changeable': Raidctrl().check(operation='changemode', mode='vfsext_6'),
        }
        return {
            self.KEY_TEMPLATE: template,
            self.KEY_TEMPLATE_DIC: directory,
        }

    def view_apply(self, request):
        form = ExtendForm(data=request.POST)

        with self.processing(constname='ERR_VOLUME_STATE_CHANGE') as p:
            if form.is_valid():
                rc = Raidctrl()
                p.result(rc.set_autoextend(form.cleaned_data['autoextend']))
                self.set_message(rc.get_message())

        template = os.path.join(self.TEMPLATES, 'template.html')
        directory = {
            'form': form,
            'disklist': self.__get_disklist(),
            'changeable': Raidctrl().check(operation='changemode', mode='vfsext_6'),
        }
        return {
            self.KEY_TEMPLATE: template,
            self.KEY_TEMPLATE_DIC: directory,
        }

    def view_extend_6(self, request):
        form = ExtendForm(data=request.POST)

        with self.processing(constname='ERR_VOLUME_STATE_CHANGE') as p:
            if form.is_valid():
                rc = Raidctrl()
                p.result(rc.extend('vfsext_6'))
                self.set_message(rc.get_message())

        template = os.path.join(self.TEMPLATES, 'template.html')
        directory = {
            'form': form,
            'disklist': self.__get_disklist(),
            'changeable': Raidctrl().check(operation='changemode', mode='vfsext_6'),
        }
        return {
            self.KEY_TEMPLATE: template,
            self.KEY_TEMPLATE_DIC: directory,
        }

    def __get_initial(self):
        return {
            'autoextend': Raidctrl().get_autoextend(),
        }

    def __is_incorporated(self, no):
        pass


    def __get_disklist(self):
        disklist = []
        try:
            from product_conf.devinfo import SATA_PORT_NO_LIST
            for no in SATA_PORT_NO_LIST:
                dict_ = {'no': no}
                dict_.update(self.__slot(no))
                disklist.append(dict_)
                #disklist.append({
                #    'no': no,
                #    'status': self.__status(no),
                #    'hdd': self.__hdd(no),
                #})
        except:
            pass
        return disklist

    def __slot(self, no, humanize=True):
        try:
            from project.fal.diskctrl import Diskctrl

            info = Diskctrl().query(disk=int(no))
            status = info['status']
            hdd = ""
            if status:
                model = info['model']
                size = float(info['size'] or 0) * 512 / 1000000000
                hdd = "%s   %.2f GB" % (model, size)

            elementlist = Raidctrl().query()['elementlist']
            if ("satad%sp6" % no) in elementlist:
                status = 'incorporated'

            if humanize:
                status = {
                    #'normal': _(u'Normal'),
                    #'fail': _(u'Fail'),
                    #'failpending': _(u'FailPending'),
                    #'smarterror': _(u'SmartError'),
                    #'capshortage': _(u'CapShortage'),
                    #'smarterrorpending': _(u'SmartErrorPending'),
                    #'unconnected': _(u'Unconnected'),
                    'normal':            _(u'DISK Status: Normal'),
                    'fail':              _(u'DISK Status: Fail'),
                    'failpending':       _(u'DISK Status: FailPending'),
                    'smarterror':        _(u'DISK Status: SmartError'),
                    'capshortage':       _(u'DISK Status: CapShortage'),
                    'smarterrorpending': _(u'DISK Status: SmartErrorPending'),
                    'incorporated':      _(u"DISK Status: Incorporated"),
                }.get(status,            _(u"DISK Status: Disconnected"))
            
            return {
                'status': status,
                'hdd': hdd,
            }
        except:
            return {}

    def __get_chaneablelist(self, modelist):
        changeable = {}
        for mode in modelist:
            changeable[mode] = Raidctrl().check(operation='changemode', mode=mode)
        return changeable
