# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
import os
from django.utils.translation import ugettext as _
from django.http import HttpResponseRedirect
from project.view import FuncView
from project.fal.command import get_message
from fal_info import Results
from ..modules.lockkey import LockKey


def root(request):
    return HttpResponseRedirect('./copykey')


class View(FuncView):
    views = ['copy', 'execute']
    key_str = ""
    mac_str = ""

    def view(self, request):
        return {self.KEY_TEMPLATE: self.TEMPLATE,
                self.KEY_TEMPLATE_DIC: {}}

    def view_copy(self, request):
        (result, mac, key) = LockKey().query_key()

        template = self.TEMPLATE
        if result == Results.OK:
            self.key_str = key
            self.mac_str = mac
            template = os.path.join(self.TEMPLATES, "writein.html")
            self.set_message(_(u'Obtained key successfully. Please change the USB memory.'))
        elif result == Results.DEVICE_NOT_FOUND:
            self.set_message(
                    get_message('ERR_LOCKKEY_DISK_NOT_FOUND', is_log=True))
        else:
            self.set_message(
                    get_message('ERR_LOCKKEY_NO_KEY_FILE', is_log=True))
        return {self.KEY_TEMPLATE: template,
                self.KEY_TEMPLATE_DIC: {}}

    def __write_key(self, key, mac):
        result = LockKey().write_key(key, mac)
        if result == Results.OK:
            self.set_message(get_message('MSG_LOCKKEY_COPY', is_log=True))
            return True
        else:
            if result == Results.CAP_SHORTAGE:
                self.set_message(
                    get_message('ERR_LOCKKEY_DISK_CAP_SHORTAGE', is_log=True))
            elif result == Results.DEVICE_NOT_FOUND:
                self.set_message(
                    get_message('ERR_LOCKKEY_DISK_NOT_FOUND', is_log=True))
            elif result == Results.NO_LOCKKEY:
                self.set_message(
                    get_message('ERR_LOCKKEY_NO_KEY_FILE', is_log=True))
            self.set_message(get_message('ERR_LOCKKEY_COPY', is_log=True))
            return False

    def view_execute(self, request):
        if self.mac_str and self.key_str:
            with self.processing(constname="ERR_LOCKKEY_STATE_CHANGE") as p:
                rst = False
                p.break_if_state_change()
                rst = self.__write_key(self.key_str, self.mac_str)
                p.result(rst)
        else:
            self.set_message(get_message('ERR_LOCKKEY_COPY', is_log=True))

        template = os.path.join(self.TEMPLATES, "writein.html")
        return {self.KEY_TEMPLATE: template,
                    self.KEY_TEMPLATE_DIC: {}}

copykey = View.as_view()
