# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django.utils.translation import ugettext as _
from django import forms
from django.forms.formsets import formset_factory
from project.view import FuncView
import os
import re
from project.forms.fields.checkbox import CheckboxField
from project.forms.fields.extras.fqdn import FQDNField
from project.forms.fields.extras.portnum import RemotePortNumberField
from project.fal.lockkey import Lockkey


KEYSERVER_NUM = 3
DEFAULT_PORT = 51056


class KeyserverPasswordField(forms.CharField):

    widget = forms.widgets.PasswordInput

    def __init__(self,
                 #max_length=20,  # FIXME
                 #min_length=4,  # FIXME
                 *args, **kwargs):
        #super(KeyserverPasswordField, self).__init__(max_length=max_length,
        #                                             min_length=min_length,
        #                                             *args, **kwargs)
        super(KeyserverPasswordField, self).__init__(*args, **kwargs)

    def clean(self, value):
        return value
        #value = super(KeyserverPasswordField, self).clean(value)
        #if re.match(r"[!-~]*", value):
        #    return value
        #else:
        #    raise forms.ValidationError(_(u"Enter a valid value."))


class KeyserverForm(forms.Form):

    no = forms.IntegerField(label=_(u"KeyServerManagement: No"),
                            widget=forms.HiddenInput,
                            required=True)
    enabled = CheckboxField(label=_(u"Keyserver Function"),
                            required=False)
    host = FQDNField(label=_(u"Host"),
                     required=False)
    port = RemotePortNumberField(label=_(u"Port"),
                                 initial=DEFAULT_PORT,
                                 required=False)
    #password = forms.CharField(label=_(u"Password"),
    password = KeyserverPasswordField(label=_(u"Password"),
                                      widget=forms.widgets.PasswordInput,
                                      initial="",
                                      required=False)

    def clean_host(self):
        if self.cleaned_data.get("enabled", None):
            if not self.cleaned_data["host"]:
                raise forms.ValidationError(_(u"This field is required."))
        return self.cleaned_data["host"]

    def clean_port(self):
        if self.cleaned_data.get("enabled", None):
            if not self.cleaned_data["port"]:
                raise forms.ValidationError(_(u"This field is required."))
        return self.cleaned_data["port"]


KeyserverFormset = formset_factory(KeyserverForm,
                                   extra=0,
                                   max_num=KEYSERVER_NUM)


class TargetForm(forms.Form):

    no = forms.IntegerField(widget=forms.HiddenInput,
                            required=True)


class KeysrvView(FuncView):

    views = ['apply', 'put', 'get']

    def view(self, request):
        kslist, formset = self.__get_keyserverlist_and_formset()
        form = TargetForm()

        template = os.path.join(self.TEMPLATES, 'template.html')
        dictionary = {
            'keyserverlist': kslist,
            'formset': formset,
            'form': form,
        }
        return {
            self.KEY_TEMPLATE: template,
            self.KEY_TEMPLATE_DIC: dictionary,
        }

    def view_apply(self, request):
        kslist, formset = self.__get_keyserverlist_and_formset(data=request.POST)
        form = TargetForm()

        with self.processing(constname="ERR_LOCKKEY_STATE_CHANGE") as p:
            keyserverlist = []
            if formset.is_valid():
                for form in formset:
                    if form.is_valid():
                        if form.cleaned_data == {}:
                            continue
                        if form.cleaned_data.get("DELETE"):
                            continue
                        try:
                            keyserverlist.append({
                                "registno": form.cleaned_data["no"],
                                "enabled": form.cleaned_data["enabled"],
                                "hostname": form.cleaned_data["host"],
                                "port": form.cleaned_data["port"] or DEFAULT_PORT,
                            })
                        except:
                            pass
                lockkey = Lockkey()
                p.result(lockkey.save(keyserverlist=keyserverlist))
                self.set_message(lockkey.get_message())

                if p.result():
                    kslist, formset = self.__get_keyserverlist_and_formset()
                    form = TargetForm()

        template = os.path.join(self.TEMPLATES, 'template.html')
        dictionary = {
            'keyserverlist': kslist,
            'formset': formset,
            'form': form,
        }
        return {
            self.KEY_TEMPLATE: template,
            self.KEY_TEMPLATE_DIC: dictionary,
        }

    def view_put(self, request):
        kslist, formset = self.__get_keyserverlist_and_formset(data=request.POST)
        form = TargetForm(data=request.POST)

        with self.processing(constname="ERR_LOCKKEY_STATE_CHANGE") as p:
            if form.is_valid():
                no = form.cleaned_data.get("no", None)
                if no is not None:
                    for i, _form in enumerate(formset):
                        if no == self.__get_cleaned_value(_form, "no", default=None):
                            if _form.is_valid():
                                data = _form.cleaned_data
                                lockkey = Lockkey()
                                #if lockkey.modify({no: {
                                #    "enabled": data.get("enabled"),
                                #    "hostname": data.get("host"),
                                #    "port": data.get("port")}}):
                                #    pass  # FIXME: register
                                #    self.set_message(_(u"No,%(no)s :SUCCESS") % {"no": no})  # FIXME
                                #    p.result(True)  # FIXME
                                #else:
                                #    self.set_message(lockkey.get_message())
                                p.result(lockkey.register(no, data.get("password", "")))
                                self.set_message(lockkey.get_message())
                                if p.result():
                                    kslist, formset = self.__get_keyserverlist_and_formset()
                                    #form = TargetForm()
                        else:
                            for k in _form.errors.keys():
                                _form.errors[k] = _form.error_class()
                else:
                    self.set_message(_(u"ERROR"))  # FIXME
        #kslist, formset = self.__get_keyserverlist_and_formset()
        form = TargetForm()

        template = os.path.join(self.TEMPLATES, 'template.html')
        dictionary = {
            'keyserverlist': kslist,
            'formset': formset,
            'form': form,
        }
        return {
            self.KEY_TEMPLATE: template,
            self.KEY_TEMPLATE_DIC: dictionary,
        }

    def view_get(self, request):
        kslist, formset = self.__get_keyserverlist_and_formset(data=request.POST)
        form = TargetForm(data=request.POST)

        with self.processing(constname="ERR_LOCKKEY_STATE_CHANGE") as p:
            if form.is_valid():
                no = form.cleaned_data.get("no", None)
                if no is not None:
                    for i, _form in enumerate(formset):
                        if no == self.__get_cleaned_value(_form, "no", default=None):
                            if _form.is_valid():
                                data = _form.cleaned_data
                                lockkey = Lockkey()
                                p.result(lockkey.set_lockkey("download", keyserverno=no, password=data.get("password", "")))
                                self.set_message(lockkey.get_message())
                                if p.result():
                                    kslist, formset = self.__get_keyserverlist_and_formset()
                                    #form = TargetForm()
                        else:
                            for k in _form.errors.keys():
                                _form.errors[k] = _form.error_class()
                else:
                    self.set_message(_(u"ERROR"))  # FIXME
        #kslist, formset = self.__get_keyserverlist_and_formset()
        form = TargetForm()

        template = os.path.join(self.TEMPLATES, 'template.html')
        dictionary = {
            'keyserverlist': kslist,
            'formset': formset,
            'form': form,
        }
        return {
            self.KEY_TEMPLATE: template,
            self.KEY_TEMPLATE_DIC: dictionary,
        }

    def __get_cleaned_value(self, form, name, default=None):
        try:
            field = form[name].field
            value = field.widget.value_from_datadict(form.data, form.files, form.add_prefix(name))
            return field.clean(value)
        except:
            return default

    def __get_keyserverlist_and_formset(self, data=None):
        formset = self.__get_formset(data=data)
        keyserverlist = []
        try:
            kss = Lockkey().list()
            for form in formset:
                no = int(form["no"].value())
                ks = kss.get(no, {})
                enabled = ks.get("enabled")
                registered = ks.get("registered")
                keyserverlist.append({
                    #"no": form["no"].value(),
                    "no": no,
                    "form": form,
                    "enabled": enabled,
                    "registered": registered,

                    "status": self.__get_status(enabled, registered),

                    "put_ok": enabled and (registered in [True, False]),
                    "get_ok": enabled and (registered in [True]),
                })
        except:
            pass
        return keyserverlist, formset

    def __get_status(self, enabled, registered):
        if enabled:
            return {
                True: _(u"KeyServerStatus: Registered"),
                False: _(u"KeyServerStatus: Unregistered"),
                None: _(u"KeyServerStatus: Error"),
            }.get(registered, _(u"KeyServerStatus: Error"))
        else:
            return u"-"

    def __get_formset(self,
                      data=None,
                      cls=KeyserverFormset,
                      prefix="keyserver"):
        try:
            return cls(prefix=prefix,
                       data=data,
                       initial=None if data else self.__get_formset_initial())
        except:
            return cls(prefix=prefix,
                       initial=[{"no": i} for i in range(1, 1 + KEYSERVER_NUM)])

    def __get_formset_initial(self):
        initial = []
        nolist = [i + 1 for i in range(0, KEYSERVER_NUM)]
        try:
            #for no, param in Lockkey().load()["keyserverlist"].iteritems():
            for no, param in Lockkey().list().iteritems():
                nolist.remove(no)
                initial.append({
                    "no": no,
                    "enabled": param.get("enabled", False),
                    "host": param.get("hostname", ""),
                    "port": param.get("port", DEFAULT_PORT) or DEFAULT_PORT,
                })
        except:
            pass
        return (initial + [{"no": no} for no in nolist])[:KEYSERVER_NUM]
        #return initial
