# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from iobase.xmldata import XmlNode
from ionas.request import (Request, Entry)
from product_conf import SUDO_CMD
from fal_info import (FAL_LOCKKEY_PATH, ReqTypes, Results)
from iobase.exception import NotFoundError


SAVE_ENTRY = "<lockkey><keyserverlist/></lockkey>"
WRITE_ENTRY = "<lockkey><option operation='write'/><mac/><key/></lockkey>"
DELETE_ENTRY = "<lockkey><option operation='delete'/></lockkey>"
SERVER_XPATH = "keyserverlist/keyserver"
KEY_XPATH = "./key"
MAC_XPATH = "./mac"

SRV_PORT_STR = "srv%s_port"
DEFAULT_PORT = 51056
SRV_NUM = 2


class LockKey(object):
    def query_key(self):
        try:
            entry = Entry(ReqTypes.QUERY)
            response = Request(FAL_LOCKKEY_PATH, filter_cmd=SUDO_CMD).request(
                        entry).get_entrylist().get_entry()
        except:
            return (Results.QUERY_ERROR, "", "")

        result = response.get_result()
        if result == Results.OK:
            mac = response.get().get_node(MAC_XPATH).get_text()
            key = response.get().get_node(KEY_XPATH).get_text()
            return (result, mac, key)

        return (result, "", "")

    def send_request(self, entry):
        try:
            response = Request(FAL_LOCKKEY_PATH,
                        filter_cmd=SUDO_CMD).request(entry)
        except:
            return Results.SET_ERROR
        return response.get_entrylist().get_entry().get_result()

    def write_key(self, key, mac):
        entry = Entry(ReqTypes.SET_LOCKKEY, WRITE_ENTRY)
        entry.get().get_node(KEY_XPATH).set_text(key)
        entry.get().get_node(MAC_XPATH).set_text(mac)
        return self.send_request(entry)
 
    def delete_key(self):
        entry = Entry(ReqTypes.SET_LOCKKEY, DELETE_ENTRY)
        return self.send_request(entry)
        
    def set_server(self, data):
        entry = Entry(ReqTypes.SAVE, SAVE_ENTRY)

        num = 1
        while num <= SRV_NUM:
            srv_node = XmlNode("keyserver")
            srv_node.append_child("registno", str(num))
            if data["keysrv"] == "1":
                srv_node.set_attrs({"enable": "false"})
                host = data["srv%s_host" % num]
                port = data["srv%s_port" % num]
                if host != "":
                    srv_node.set_attrs({"enable": "true"})
                    srv_node.append_child("hostname", host)
                    if not port:
                        port = DEFAULT_PORT
                    srv_node.append_child("port", str(port))
            else:
                srv_node.set_attrs({"enable": "false"})
            entry.get().get_node("keyserverlist").append_child(srv_node)
            num += 1
        if self.send_request(entry) != Results.OK:
            return False
        return True

    def get_server_info(self):
        info_dic = {"keysrv": 0}
        try:
            config = Request(FAL_LOCKKEY_PATH, filter_cmd=SUDO_CMD).request(
                    Entry(ReqTypes.LOAD)).get_entrylist().get_entry()
        except:
            return info_dic    #FIXME
        try:
            enable_srv_list = config.get().get_node_list(
                    "./keyserverlist/keyserver[@enable='true']")
            if len(enable_srv_list) <= 0:
                info_dic["keysrv"] = 0
            else:
                info_dic["keysrv"] =1 
        except NotFoundError:
            info_dic["keysrv"] = 0

        num = 1
        while num <= SRV_NUM:
            try:
                node = config.get().get_node(
                        "./keyserverlist/keyserver[registno='%s']" % num)
                info_dic["srv%s_host" % num] = \
                        node.get_node("hostname").get_text()
                port = node.get_node("port").get_text()
                if port == "":
                    info_dic[SRV_PORT_STR % num] = str(DEFAULT_PORT)
                else:
                    info_dic[SRV_PORT_STR % num] = port
            except:
                pass
            num += 1
        return info_dic
