# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from project.fal.statemgr import Statemgr
from project.fal.raidctrl import Raidctrl
from project.fal.diskctrl import Diskctrl


def remove(
        dev,  # ex) 'satad1'
        user=True,
        debug=False):
    result, msgcode = False, None
    try:
        statemgr = Statemgr()
        raidctrl = Raidctrl()
        diskctrl = Diskctrl()

        # devからmsgcodeに埋め込む用のデバイス名(SATA1やUSB3等)を生成
        _DEV = dict(
                [('satad%s' % n, 'SATA%s' % n) for n in range(1, 7)] +
                [('usbd%s' % n, 'USB%s' % n) for n in range(1, 5)]).get(dev)
        if _DEV is None:
            raise
        msgcode = 'ERR_DISK_%s_REMOVE' % _DEV

        # Step1. RAID状態確認
        status = statemgr.query()['raid']
        if status in ['running']:
            if statemgr.query()['raidvol'] == 'rebuilding':
                raise
        else:
            msgcode = {
                'crash': 'ERR_DISK_%s_REMOVE_RAID_CRASH' % _DEV,
                'degrade': 'ERR_DISK_%s_REMOVE_RAID_DEGRADE' % _DEV,
                'fail': 'ERR_DISK_%s_REMOVE_RAID_FAIL' % _DEV,
                }.get(status, 'ERR_DISK_%s_REMOVE_RAID_FAIL' % _DEV)
            raise

        # Step2. RAIDモード確認(vfsextのみ)
        if raidctrl.query().get('level', '').split("_")[0] == 'vfsext':
            pass
        else:
            msgcode = 'ERR_DISK_%s_REMOVE' % _DEV
            raise

        # Step3. RAIDからディスクを取り外す
        if raidctrl.remove(dev, user=True):
            pass
        else:
            msgcode = 'ERR_DISK_%s_REMOVE' % _DEV
            raise

        # Step4. システムからディスクを取り外す
        if diskctrl.delete(dev):
            msgcode = 'MSG_DISK_%s_REMOVE' % _DEV
            result = True
        else:
            msgcode = 'ERR_DISK_%s_REMOVE' % _DEV
            raise
    except:
        if debug:
            import traceback
            traceback.print_exc()
    finally:
        return result, msgcode


if __name__ == '__main__':
    # Test
    print remove('satad2', debug=True)
