# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
import os

from django.utils.translation import ugettext as _
from django.http import HttpResponseRedirect
from project.view import FuncView
from project.view import StatusView
from root.modules.form import BaseForm
import settings

from project.fal.command import get_message as fgettext
from project.fal.volctrl import Volctrl
from project.fal.devctrl import Devctrl


def root(request):
    return HttpResponseRedirect('./checkdisk')


def is_connected(humanize=False):
    connected = Devctrl().is_connected(device="usbd%s" % settings.PORT)
    if humanize:
        connected = {
            True: _(u'USB device is connected.'),
            False: _(u'USB device is not connected.'),
        }.get(connected, connected)
    return connected


def is_checking():
    try:
        vobj = Volctrl()
        return vobj.query(volume=vobj.VOLUME['usb'][settings.PORT])['status'] == 'checking'
    except:
        return False

def is_vfsext_mode():
    try:
        from project.fal.port import Port
        return Port().load()["usblist"][int(settings.PORT)]["mode"] == "vfsext"
    except:
        return False


class View(FuncView):
    views = ['execute']

    def view(self, request):
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                'checkdisk_status': CheckdiskStatusView()._status()}}

    def view_execute(self, request):
        with self.processing(constname='ERR_VOLUME_STATE_CHANGE') as p:
            p.break_if_state_change()
            if is_checking():
                self.set_message(_(u'Now checkdisking.'))
            elif is_vfsext_mode():
                self.set_message(_(u'Can not execute this operation'))
            elif is_connected():
                vobj = Volctrl()
                vol = vobj.VOLUME['usb'][settings.PORT]
                vobj.mount(volume=vol, log=False)
                vinfo = vobj.query(volume=vol)
#                vobj.unmount(volume=vol, log=False)
                fs = vinfo.get('filesystem', None)
                if fs is None:
                    if vinfo.get('status', None) == 'lockkey':
                        self.set_message(_(u'Cannot execute checkdisk. (USB volume is lockkey.)'))
                    elif vinfo.get('status', None) == 'keywaiting':
                        self.set_message(_(u'Cannot execute checkdisk. (USB volume is keywaiting.)'))
                    else:
                        self.set_message(_(u'Can not execute this operation'))
                else:
                    vobj.fsck(volume=vol, fs=fs)
                    self.set_message(vobj.get_message())
                    p.result(vobj.get_result(boolean=True))
            else:
                self.set_message(_(u'USB device is not connected.'))
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {}}


class CheckdiskStatusView(StatusView):
    STATUS_FILE_PATH = "/mnt/data/conf/volume/status/usb%s/fsck" % settings.PORT

#    def _progress(self):
#        try:
#            with open(self.STATUS_FILE_PATH, "r") as f:
#                sts = f.read().rstrip().split(":")
#            if sts[0] == "PROCESSING":
#                try:
#                    progress = sts[1].decode('utf-8')
#                except:
#                    progress = _(u'unknown')
#                return "%s(%s)" % (_(u"PROCESSING"), progress)
#            else:
#                return None
#        except:
#            return None
#
#    def _status(self):
#        return self._progress() or is_connected(humanize=True)


checkdisk = View.as_view()
status = CheckdiskStatusView.as_view()
