# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
import os

from django.utils.translation import ugettext as _
from django.http import HttpResponseRedirect
from django.core import validators
from django.core.exceptions import ValidationError
from project.view import FuncView, StatusView
from root.modules.form import BaseForm
import settings

from project.fal.volctrl import Volctrl
from project.fal.devctrl import Devctrl
from project.fal.lockkey import Lockkey
from project.forms.fields.radio import RadioField
from project.forms.fields.extras.enable import EnableField


def root(request):
    return HttpResponseRedirect('./format')

def is_connected(humanize=False):
    connected = Devctrl().is_connected(device="usbd%s" % settings.PORT)
    if humanize:
        connected = {
            True: _(u'USB device is connected.'),
            False: _(u'USB device is not connected.'),
        }.get(connected, connected)
    return connected


class View(FuncView):
    class Form(BaseForm):
        fs = RadioField(
            label=_(u'File System'),
            choices=(('ext4', _(u'exclusive format')), ('fat', 'FAT32')),
            initial='ext4',
            required=True)

        if settings.PORT != 1:
            encrypt = EnableField(label=_(u'Encrypt'), required=False)

    views = ['execute']

    def _usbinfo(self):
        try:
            vobj = Volctrl()
            vinfo = vobj.query(volume=vobj.VOLUME['usb'][settings.PORT])
            return {
                'fs': {
                    'ext4': 'ext4',
                    'vfat': 'fat'}.get(vinfo['filesystem'], 'ext4'),
                'encrypt': vinfo['encrypt'],
            }
        except:
            return {}

    def view(self, request):
        form = self.Form(initial=self._usbinfo())
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                'format_status': FormatStatusView()._status(),
                'form': form}}

    def view_execute(self, request):
        with self.processing(constname="ERR_VOLUME_STATE_CHANGE") as p:
            form = self.Form(data=request.POST)
            p.break_if_state_change()

            if is_connected():
                if Volctrl().is_lockkey(settings.PORT):
                    self.set_message(_(u'Can not format lockkey.'))
                elif form.is_valid():
                    fs = form.cleaned_data['fs']
                    encrypt = form.cleaned_data.get('encrypt', False)

                    if fs != 'ext4' and encrypt == True:
                        self.set_message(_(u'encrypt is ext4 only'))
                    else:
                        try:
                            if encrypt:
#                                if 'usbd1' not in Devctrl().list().keys():
#                                    self.set_message(
#                                            _(u'Please connect lockkey drive.'))
#                                    raise
                                lobj = Lockkey()
                                if not lobj.query()['key'] and not lobj.create(log=True):
                                    self.set_message(lobj.get_message())
                                    raise

                            vobj = Volctrl()
                            vobj.mkfs(
                                    volume=settings.USB,
                                    fs=fs,
                                    encrypt=encrypt)
                            from fal_info import Results
                            if vobj.get_result() == int(Results.NOT_SUPPORTED):
                                self.set_message(_(u'Can not execute this operation'))
                            else:
                                self.set_message(vobj.get_message())
                            p.result(vobj.get_result(boolean=True))
                        except:
                            pass
            else:
                self.set_message(_(u'USB device is not connected.'))
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                'format_status': FormatStatusView()._status(),
                'form': form}}


class FormatStatusView(StatusView):
    STATUS_FILE_PATH = "/mnt/data/conf/volume/status/usb%s/mkfs" % settings.PORT

#    def _progress(self):
#        try:
#            with open(self.STATUS_FILE_PATH, "r") as f:
#                sts = f.read().rstrip().split(":")
#            if sts[0] == "PROCESSING":
#                try:
#                    progress = sts[1].decode('utf-8')
#                except:
#                    progress = _(u'unknown')
#                return "%s(%s)" % (_(u"PROCESSING"), progress)
#            else:
#                return None
#        except:
#            return None
#
#    def _status(self):
#        return self._progress() or is_connected(humanize=True)


format = View.as_view()
status = FormatStatusView.as_view()
