# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django.utils.translation import ugettext as _
from django.http import HttpResponseRedirect
from django.core.exceptions import ValidationError
from project.view import FuncView
from project.view import StatusView
from root.modules.form import BaseForm
import settings

from project.fal.port import Port
from project.forms.fields.extras.usb import USBModeField
from project.forms.fields.extras.share import SelectShareField
from project.fal.volctrl import Volctrl
from project.fal.raidctrl import Raidctrl
from fal_info import Results


def root(request):
    return HttpResponseRedirect('./mode')


def is_connected():
    try:
        from project.fal.devctrl import Devctrl
        return Devctrl().is_connected(device='usbd%s' % settings.PORT)
    except:
        return False


def is_mirroring():
    try:
        vobj = Volctrl()
        if vobj.query(volume=vobj.VOLUME['usb'][settings.PORT]).get('status') == 'mirroring':
            return True
        return False
    except:
        return False


def is_mirrordisk():
    try:
        if Port().load()['usblist'][settings.PORT]['mode'] == 'mirroring' and is_connected():
            return True
        return False
    except:
        return False


def is_backup_dest():
    try:
        from project.fal.backups import Backups
        if Backups().is_bkup_dst("usb%s" % settings.PORT, fs_list=[]):
            return True
        return False
    except:
        return False


def cannot_select_mirroring(form):
    try:
        if int(settings.PORT) == 3 and \
                Raidctrl().query().get('level', None) == 'vfsext':
            return True
        return False
    except:
        return False


def cannot_select_vfsext(form):
    try:
        from product_conf import PORT_TO_MODE_LIST
        if ('vfsext' in PORT_TO_MODE_LIST['usb'][int(settings.PORT)]) and \
                ('vfsext' not in [c[0] for c in form.fields['mode'].choices]):
            return True
        return False
    except:
        return False


class View(FuncView):
    class Form(BaseForm):
        def __init__(self, *args, **kwargs):
            super(View.Form, self).__init__(*args, **kwargs)
            self.fields["mode"] = USBModeField(label=_(u"USB: Mode"),
                                               port=settings.PORT)
            if 'quickcopy' in [
                    choice[0] for choice in self.fields['mode'].choices]:
                self.fields['quickcopy_share'] = SelectShareField(label=_(u'Quick Copy Share'),
                                                                  exclude=['usb1'],
                                                                  required=False)

        def clean_quickcopy_share(self):
            val = self.cleaned_data['quickcopy_share']
            mode = self.cleaned_data['mode']
            if mode == 'quickcopy' and not val:
                raise ValidationError(_(u'This field is required.'))
            return val

    views = ['apply']

    def view(self, request):
        pobj = Port()
        pinfo = pobj.load()
        initial = {
            'mode': pinfo['usblist'][settings.PORT]['mode'],
            'quickcopy_share': pinfo['usblist'][settings.PORT]['share'],
        }
        form = self.Form(initial=initial)
        if is_backup_dest():
            self.set_message(_(u'This volume is backup destination.'))
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                'status': StatusView()._status(),
                'form': form,
                'cannot_select_mirroring': cannot_select_mirroring(form),
                'cannot_select_vfsext': cannot_select_vfsext(form),
            },
        }

    def view_apply(self, request):
        with self.processing(constname='ERR_VOLUME_STATE_CHANGE') as p:
            form = self.Form(data=request.POST)
            p.break_if_state_change()

            if form.is_valid():
                mode = form.cleaned_data['mode']
                qcopy_dest = form.cleaned_data.get('quickcopy_share', None)

                if mode != 'mirroring' and is_mirroring() and is_mirrordisk():
                    self.set_message(_(u'Cannot mode change. Please try again after remove mirrordisk.'))
                    p.result(False)
                else:
                    kwargs = {'mode': mode}
                    if qcopy_dest:
                        kwargs['share'] = qcopy_dest
                    pobj = Port()
                    pobj.apply(devtype="usb", portnum=settings.PORT, **kwargs)
                    self.set_message(pobj.get_message())
                    p.result(pobj.get_result(boolean=True))
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                'status': StatusView()._status(),
                'form': form,
                'cannot_select_mirroring': cannot_select_mirroring(form),
                'cannot_select_vfsext': cannot_select_vfsext(form),
            },
        }


class StatusView(StatusView):
    def _status(self):
        try:
            if Port().load()['usblist'][settings.PORT]['mode'] != 'mirroring':
                return ''
        except:
            return ''

        vobj = Volctrl()
        vsts = vobj.query(volume=vobj.VOLUME['usb'][settings.PORT]).get('status', None)
        if vobj.get_result() == int(Results.DEVICE_NOT_FOUND):
            return _(u'Unconnected')
        else:
            return {
                'waiting': _(u'VolumeStatusWaiting'),
                'capshortage': _(u'VolumeStatusCapshortage'),
                'fail': _(u'VolumeStatusFail'),
                'mirroring': Raidctrl().status(depth=1, humanize=True),
                'othermirror': _(u"Mirror Storage (unrecoverable)"),
                'recoverablemirror': _(u"Mirror Storage (recoverable)"),
                'vfsext': _(u'VolumeStatusVFSEXT'),
            }.get(vsts, _(u'processing (usb mode change)'))


mode = View.as_view()
