# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django.utils.translation import ugettext as _
from django.http import HttpResponseRedirect
from django.core import validators
from django.core.exceptions import ValidationError
from project.view import FuncView
from project.view import StatusView
from root.modules.form import BaseForm
import settings

from project.fal.volctrl import Volctrl
from project.fal.devctrl import Devctrl
from project.fal.power import Power
from project.fal.command import get_message as fgettext
from root.admin.volume.modules.utils import remove as vfsremove


def root(request):
    return HttpResponseRedirect('./remove')


def is_connected(humanize=False):
    connected = True
    try:
        from fal_info import Results
        volctrl = Volctrl()
        volume = volctrl.VOLUME['usb'][int(settings.PORT)]
        entry = volctrl.query(volume=volume, to_dict=False)
        if entry.get_result() == Results.DEVICE_NOT_FOUND:
            connected = False
        else:
            if volctrl.query(volume=volume, to_dict=True)['status'] in [
                    'mounted', 'checking', 'formatting', 'mirroring', 'vfsext']:
                connected = True
            else:
                connected = False
    except:
        connected = True
    finally:
        if humanize:
            connected = {
                False: _(u'USB device is remove OK'),
                True: _(u'USB device is remove NG'),
            }.get(connected, _(u'USB device is remove NG'))
        return connected


def is_mirrordisk():
    try:
        from project.fal.port import Port
        if Port().load()['usblist'][settings.PORT]['mode'] == 'mirroring' and is_connected():
            return True
        return False
    except:
        return False


def is_vfs():
    try:
        from project.fal.port import Port
        if Port().load()['usblist'][settings.PORT]['mode'] == 'vfsext' and is_connected():
            return True
        return False
    except:
        return False


class View(FuncView):
    views = ['execute']

    def view(self, request):
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                'mirrordisk': is_mirrordisk(),
                'vfsmode': is_vfs(),
                'status': ConnectStatusView()._status()}}

    def view_execute(self, request):
        disabled = False
        with self.processing(constname='ERR_VOLUME_STATE_CHANGE') as p:
            p.break_if_state_change()

            if is_connected():
                if is_mirrordisk():
                    Power().shutdown(force=True)
                    self.set_message(fgettext('MSG_VOLUME_MIRROR_REMOVE_START', is_log=True))
                    disabled = p.result(True)
                elif is_vfs():
                    result, msgcode = vfsremove('usbd%s' % settings.PORT)
                    self.set_message(fgettext(msgcode, is_log=True))
                    p.result(result)
                else:
                    vobj = Volctrl()
                    vobj.unmount(volume=vobj.VOLUME['usb'][int(settings.PORT)])
                    self.set_message(vobj.get_message())
                    p.result(vobj.get_result(boolean=True))
            else:
                self.set_message(_(u'USB device is not connected.'))
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                'disabled': disabled,
                'mirrordisk': is_mirrordisk(),
                'vfsmode': is_vfs(),
                'status': ConnectStatusView()._status()}}


class ConnectStatusView(StatusView):
    def _status(self):
        return is_connected(humanize=True)

remove = View.as_view()
status = ConnectStatusView.as_view()
