# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django.utils.translation import ugettext as _
from django import forms
from project.view import RootView
from project.forms.fields.extras.agree import AgreeField
from project.fal.power import Power
from project.fal.eeprom import EEPROM
import re


class View(RootView):
    REQUIRED_LOGIN = True
    NOT_LOGINED_URL = '/login/gate/_gplv3/_gplv3.cgi'

    class Form(forms.Form):
        agree = AgreeField(label=_(u'agree'))

    views = ['apply']

    def view(self, request):
        self.logger.debug("GPLv3: %s" % EEPROM().gplv3())
        form = self.Form()
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                u"method": self.__get_method(),
                u"agreed": bool(int(EEPROM().gplv3())),
                u"form": form,
            },
        }

    def view_apply(self, request):
        with self.processing() as p:
            form = self.Form(data=request.POST)
            p.break_if_state_change()
            if form.is_valid():
                p.result(bool(EEPROM().gplv3(True)))
                if p.result():
                    Power().reboot(force=True)
                    self.set_message(_(u'Permited connect for GPLv3. Reboot.'))
                else:
                    self.set_message(_(u'Failed to setting.'))
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                u"method": self.__get_method(),
                u"agreed": bool(int(EEPROM().gplv3())),
                u"form": form,
            },
        }

    def __get_method(self):
        try:
            from project.fal.sysinfo import SysInfo
            series = SysInfo().query()[u"series"]
            if series in [u"HDL-AA", u"HDL-AAW", u"HDL2-AA", u"HDL2-AAW"]:
                return u"ssh"
            else:
                return u"getty"
        except:
            return u"getty"  # FIXME
