# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django.conf import settings
from project.utils import importer as imp, to_importable as imppath
from project.view import RootView
from project.fal.sysinfo import SysInfo
from project.fal.network import Network
from project.fal.devctrl import Devctrl
from project.fal.eeprom import EEPROM
from datetime import datetime
from subprocess import call, Popen, PIPE
import re
import os
import sys


DO_NOT_CHECK = '__DO_NOT_CHECK__'

IDEMA = {
    8: 15649200,
    10: 19556208,
    20: 39091248,
    32: 62533296,
    40: 78161328,
    64: 125045424,
    80: 156301488,
    120: 234441648,
    128: 250069680,
    160: 312581808,
    250: 488397168,
    256: 500118192,
    300: 586072368,
    320: 625142448,
    400: 781422768,
    450: 879097968,
    500: 976773168,
    512: 1000215216,
    600: 1172123568,
    640: 1250263728,
    750: 1465149168,
    1000: 1953525168,
    1500: 2930277168,
    2000: 3907029168,
    3000: 5860533168,
    4000: 7814037168,
    5000: 9767541168,
    6000: 11721045168,
    8000: 15628053168,
    10000: 19535061168,
    12000: 23442069168,
    16000: 31256085168,
}

def _calc_idema(size):
    return 97696368 + (1953504 * (int(size) - 50))


class View(RootView):
    REQUIRED_LOGIN = False

    def _memsize(self):
        from subprocess import Popen, PIPE
        proc = Popen(['cat', '/proc/meminfo'], stdout=PIPE)
#        if proc.returncode != 0:
#            return None
        for l in proc.stdout:
            m = re.match('MemTotal:\s*([0-9]+).*', str(l))
            if m:
                try:
                    return int(m.group(1))
                except:
                    pass
        return None

    def _autoboot(self):
        try:
            from project.fal.power import Power
            pobj = Power()
            entry = pobj.load()
            return int(entry.get().get_node('autoresume').is_enabled())
        except:
            return None

    #def _fw(self):
    #    val = SysInfo().query().get('fwversion', '-')
    #    return {
    #        'val': val,
    #        'valid': val in getattr(self._settings, 'FIRMWARE_VERSIONS', []),
    #    }

    def _fw(self):
        allows = getattr(self._settings, 'FIRMWARE_VERSIONS', [])

        value = SysInfo().query().get('fwversion', '-')
        valid = (True) if (allows == DO_NOT_CHECK) else (value in allows)
        return {
            u"value": value,
            u"valid": valid,
        }

    def _bios(self):
        GET_BIOS_VERSION = "/usr/local/bin/get_bios_version"
        if os.path.exists(GET_BIOS_VERSION):
            proc = Popen([GET_BIOS_VERSION], stdout=PIPE, stderr=PIPE)
            (stdout, stderr) = proc.communicate()
            version = stdout.strip()
            return {
                "version": {
                    "val": version,
                    "valid": version in getattr(self._settings, "BIOS_VERSIONS", []),
                },
            }
        else:
            return None

    def _uboot(self):
        val = None
        try:
            # /bin/busybox strings /dev/mtdblock0 | /bin/egrep '^U-Boot .* \(.*\)'
            proc1 = Popen(["sudo", "/bin/busybox", "strings", "/dev/mtdblock0"],
                          stdout=PIPE,
                          stderr=PIPE)
            (stdout1, stderr1) = proc1.communicate()
            proc2 = Popen(["sudo", "/bin/egrep", "^U-Boot .* \(.*\)"],
                          stdin=PIPE,
                          stdout=PIPE,
                          stderr=PIPE)
            (stdout2, stderr2) = proc2.communicate(input=stdout1)
            val = stdout2.strip()
        except:
            pass
        return {
            "val": val,
            "valid": val in getattr(self._settings, "UBOOT_VERSIONS", []),
        }

    def _lan(self, num):
        macaddr = Network().query().get('lan%s' % num, {}).get('macaddr', '-')
        macaddr_valid = bool(re.match(getattr(self._settings,
                                              'RE_MAC_ADDRESS', '^$'),
                                      macaddr.lower()))
        apt = self._apt_is_on(num)
        apt_valid = apt is False
        return {
            u"macaddr": {
                u"val": macaddr,
                u"valid": macaddr_valid,
            },
            u"apt": {
                u"val": apt,
                u"valid": apt_valid,
            },
        }

    def _apt_is_on(self, number):
        try:
            proc = Popen([u"sudo", u"/usr/local/bin/set_e1000e_atp", u"--netif=lan%s" % number],
                         stdout=PIPE,
                         stderr=PIPE)
            (stdout, stderr) = proc.communicate()
            return stdout.strip().upper() == u"ON"
        except:
            return False

    #def _model(self):
    #    model = SysInfo().query().get('model', '-')
    #    return {
    #        'val': model,
    #        'valid': model in getattr(self._settings, 'MODELS', []),
    #    }

    def _model(self):
        model_allows = getattr(self._settings, 'MODELS', [])
        model = SysInfo().query().get('model', None)
        return {
            'value': model,
            'valid': (True) if (model_allows == DO_NOT_CHECK) else (model in model_allows),
        }


    def _memory(self):
        size = self._memsize()
        ranges = getattr(self._settings, 'MEMSIZE_RENGES', [])
        valid = False
        if isinstance(ranges, (list, tuple)):
            for r in ranges:
                rmin, rmax = r[0], r[1]
                if rmin < size and size < rmax:
                    valid = True
                    break
        return {
            'size': {
                'val': size,
                'valid': valid,
            },
        }

    def _satalist(self):
        def _portnolist():
            try:
                from product_conf.devinfo import SATA_PORT_NO_LIST
                return [int(no) for no in SATA_PORT_NO_LIST]
            except:
                return range(1, 7)
        hddnolist = getattr(self._settings, 'HDD_NUMBERS', range(1, 7))
        portnolist = _portnolist()
        return dict([(n, self._sata(n, n in hddnolist)) for n in portnolist])

    def _sata(self, num, required):
        satadisk = Devctrl().list().get('satad%s' % num, {}).get('disk', {})
        model = satadisk.get('model', None)
        fw = satadisk.get('rev', None)
        size = int(satadisk.get('size', None))

        size_allows = getattr(self._settings, 'HDD_SIZES', [])
        size_valid = (True) if (size_allows == DO_NOT_CHECK) else (size in [_calc_idema(s) for s in size_allows])
        return {
            'model': {
                'val': model,
                'valid': bool(model) if required else not bool(model),
            },
            'fw': {
                'val': fw,
                'valid': bool(fw) if required else not bool(fw),
            },
            'size': {
                'val': size,
                'valid': size_valid if required else not bool(size),
            },
        }

    def _atp(self):
        data = dict()
        for LANx in ['lan1', 'lan2']:
            try:
                proc = Popen(
                    ['sudo', '/usr/local/bin/set_e1000e_atp', '--netif=%s' % LANx],
                    stdout=PIPE,
                    stderr=PIPE)
                (stdout, stderr) = proc.communicate()
                data[LANx] = {'ON': True, 'OFF': False}.get(stdout.strip())
            except:
                data[LANx] = None
        return {
            'lan1': {
                'val': data['lan1'],
                'valid': data['lan1'] is False,
            },
            'lan2': {
                'val': data['lan2'],
                'valid': data['lan2'] is False,
            },
        }

    def _tpm(self):
        value = int(os.path.exists('/dev/tpm0'))
        valid = bool(value)
        return {
            'value': value,
            'valid': valid,
        }

    def _eeprom(self):
        eeprom = EEPROM()

        uptime = eeprom.uptime()
        uptime_valid = uptime < 1440

        fantime = eeprom.fantime()
        fantime_valid = fantime < 1440

        powertime = eeprom.powertime()
        powertime_valid = powertime < 1440

        powercycle = eeprom.powercycle()
        powercycle_valid = powercycle < 10

        fanfail = eeprom.fanfail()
        fanfail_valid = not bool(fanfail)

        tempfail = eeprom.tempfail()
        tempfail_valid = not bool(tempfail)

        gplv3 = eeprom.gplv3()
        gplv3_valid = not bool(gplv3)

        autoboot = self._autoboot()
        autoboot_valid = not bool(autoboot)
        return {
            'uptime': {'val': uptime, 'valid': uptime_valid},
            'fantime': {'val': fantime, 'valid': fantime_valid},
            'powertime': {'val': powertime, 'valid': powertime_valid},
            'powercycle': {'val': powercycle, 'valid': powercycle_valid},
            'fanfail': {'val': fanfail, 'valid': fanfail_valid},
            'tempfail': {'val': tempfail, 'valid': tempfail_valid},
            'gplv3': {'val': gplv3, 'valid': gplv3_valid},
            'autoboot': {'val': autoboot, 'valid': autoboot_valid},
        }

    def load(self):
        from product_conf.devinfo import VOL_INFO
        self._settings = None
        for cdir in sorted([os.path.join(info['mnt_point'], 'kensa')
                            for usbx, info in VOL_INFO.iteritems()]):
            if not os.path.isdir(cdir):
                continue
            for cfile in sorted([xxx
                                 for xxx in os.listdir(cdir) if re.match('^for_', xxx)]):
                try:
                    sys.path.insert(0, cdir)
                    try:
                        self._settings = imp(imppath(os.path.join(cdir, cfile)))
                        reload(self._settings)
                        return self._settings
                    except:
                        pass
                    finally:
                        sys.path.pop(0)
                except:
                    pass
        return self._settings

    def has_settings(self):
        if self._settings is None:
            return False
        return True

    def view(self, request):
        call(['touch', '/tmp/infocgi_run'])  # ログ削除用のフラグ
        self.load()
        lanportlist = [netif["port"] for netif in settings.FEATURES(["deviceinfo", "netiflist"], default=[]) if netif.get("port")]

        template = self.TEMPLATE
        dictionary = {
            'has_settings': self.has_settings(),
            'firmware': self._fw(),
            'bios': self._bios(),
            'datetime': datetime.now(),
            # 'lan1': self._lan(1),
            # 'lan2': self._lan(2),
            'lanlist': dict([(p, self._lan(p)) for p in lanportlist]),
            'model': self._model(),
            'memory': self._memory(),
            'satalist': self._satalist(),
            #'atp': self._atp(),
            'eeprom': self._eeprom(),
        }
        series = SysInfo().query()["series"]
        if series in ["HDL-AA", "HDL-AAW", "HDL2-AA", "HDL2-AAW"]:
            dictionary["uboot"] = self._uboot()
        if series in ["HDL2-X", "HDL4-X"]:
            dictionary["tpm"] = self._tpm()
        return {
            self.KEY_TEMPLATE: template,
            self.KEY_TEMPLATE_DIC: dictionary,
        }
