#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
import os
import stat
import shutil
import time
from subprocess import Popen, PIPE


def snmpget(oid):
    proc = Popen(
            ['snmpget', '-v', '2c', '-Oqv', '-c', 'public', 'localhost', oid],
            stdout=PIPE,
            stderr=PIPE)
    (stdout, stderr) = proc.communicate()
    return stdout

def get_oid_int_value(oid):
    try:
        return int(snmpget(oid))
    except:
        return 0

def rxcnt():
    return get_oid_int_value('.1.3.6.1.2.1.31.1.1.1.6.4') + \
            get_oid_int_value('.1.3.6.1.2.1.31.1.1.1.6.3')

def txcnt():
    return get_oid_int_value('.1.3.6.1.2.1.31.1.1.1.10.4') + \
            get_oid_int_value('.1.3.6.1.2.1.31.1.1.1.10.3')

def run(interval=15, max=100, path='/var/tmp/www/packet'):
    __rxcnt = rxcnt()
    __txcnt = txcnt()

    if not os.path.exists(os.path.dirname(path)):
        os.makedirs(os.path.dirname(path))
    os.chmod(os.path.dirname(path), stat.S_IRWXU | stat.S_IRWXG | stat.S_IRWXO)

    with open(path, 'w') as f:
        f.write("%s %s\n" % (0, 0))
    while True:
        rxdata = list()
        txdata = list()
        if os.path.exists(path):
            with open(path, 'r') as f:
                for l in f.readlines():
                    (rxd, txd) = l.split()[:2]
                    rxdata.append(int(rxd))
                    txdata.append(int(txd))
        _rxcnt = rxcnt()
        _txcnt = txcnt()
        rxdata.append(_rxcnt - __rxcnt)
        txdata.append(_txcnt - __txcnt)
        __rxcnt = _rxcnt
        __txcnt = _txcnt
        rxdata = rxdata[-max:]
        txdata = txdata[-max:]
        _path = "%s.temp" % path
        with open(_path, 'w') as f:
            for rxd, txd in zip(rxdata, txdata):
                f.write("%s %s\n" % (rxd, txd))
        shutil.move(_path, path)

        time.sleep(interval)

if __name__ == "__main__":
    try:
        run()
    except:
        import traceback
        traceback.print_exc()
