# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django.http import HttpResponse
from django.utils import simplejson
from project.utils import importer as imp, to_importable as imppath
from project.view import RootView
#from django.http import HttpResponse
from project.fal.sysinfo import SysInfo
from project.fal.network import Network
from project.fal.devctrl import Devctrl
from project.fal.eeprom import EEPROM
from datetime import datetime
from utils import meminfo
from subprocess import call, Popen, PIPE
import re
import os
import sys
import glob
import time


def snmpget(oid):
    proc = Popen(
            ['snmpget', '-v', '2c', '-Oqv', '-c', 'public', 'localhost', oid],
            stdout=PIPE,
            stderr=PIPE)
    (stdout, stderr) = proc.communicate()
    return stdout


class View(RootView):
    REQUIRED_LOGIN = True
    NOT_LOGINED_URL = '/login/gate/resource'

    def _fanrpm(self):
        def _fannolist():
            try:
                from product_conf.devinfo import FAN_NO_LIST
                return [int(no) for no in FAN_NO_LIST]
            except:
                return [1]
        proc = Popen(['/usr/bin/sensors'], stdout=PIPE, stderr=PIPE)
        (stdout, stderr) = proc.communicate()
        rpm = dict([(no, None) for no in _fannolist()])
        for num in rpm.keys():
            match = re.search("fan%s:\s+([0-9]+) RPM" % num, stdout)
            if match:
                rpm[num] = int(match.group(1))
        return rpm

    def _temp(self):
        temp = {
            1: None,  # dimm
            2: None,  # mb
            3: None,  # cpu
        }
        proc = Popen(['/usr/bin/sensors'], stdout=PIPE, stderr=PIPE)
        (stdout, stderr) = proc.communicate()
        for num in temp.keys():
            match = re.search("temp%s:\s+\+([0-9]+\.[0-9])" % num, stdout)
            if match:
                temp[num] = int(float(match.group(1)))
        return temp

#    def _spindown(self):
#        try:
#            from product_conf.devinfo import SATA_PORT_NO_LIST, SATA_HDD_DEV_LIST
#            from product_conf import DEV_DIR
#
#            spindown = dict([(no, None) for no in SATA_PORT_NO_LIST])
#            sdxlist = [os.path.basename(p) \
#                    for p in glob.glob(os.path.join('/var/tmp/spindownd', '*'))]
#            for no in spindown.keys():
#                dpath = os.path.join(DEV_DIR, "satad%s" % no)
#                if os.path.exists(dpath):
#                    sdx = os.readlink(dpath)
#                    spindown[no] = sdx in sdxlist
#            return spindown
#        except:
#            return dict()
    def _spindown(self):
        try:
            from product_conf.devinfo import SATA_HDD_DEV_LIST
            from product_conf import DEV_DIR
            if len(
                    set([os.readlink(os.path.join(DEV_DIR, satadx)) for satadx in SATA_HDD_DEV_LIST]) \
                    and \
                    set([os.path.basename(p) for p in glob.glob(os.path.join('/var/tmp/spindownd', '*'))])):
                return True
            return False
        except:
            return False

    def _cpu(self):
        try:
            return 100 - int(snmpget('.1.3.6.1.4.1.2021.11.11.0'))
        except:
            return 0

    def _cpu1_10(self):
        data = list()
        try:
            with open('/tmp/cpulog', 'r') as f:
                for l in f.readlines():
                    data.append(int(l))
        except:
            pass
        return data[-10:]

    def _rxcount(self):
        try:
            return int(snmpget('.1.3.6.1.2.1.31.1.1.1.6.4')) + \
                    int(snmpget('.1.3.6.1.2.1.31.1.1.1.6.3'))
        except:
            return 0

    def _txcount(self):
        try:
            return int(snmpget('.1.3.6.1.2.1.31.1.1.1.10.4')) + \
                    int(snmpget('.1.3.6.1.2.1.31.1.1.1.10.3'))
        except:
            return 0

    def _mem(self):
        return meminfo()

    def _data(self):
        data = dict()
        data["mem"] = self._mem()
        data['fanrpm'] = self._fanrpm()
        data['temp'] = self._temp()
        data['spindown'] = self._spindown()
        data['cpu'] = self._cpu()
        #data['cpu'] = self._cpu1_10()
        data['disk'] = snmpget('1.3.6.1.4.1.2021.9.1.9.1')
        data['rxcount'] = self._rxcount()
        data['txcount'] = self._txcount()
        return data

    def view(self, request):
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                'data': self._data(),
            },
        }


class CPUView(RootView):

    def view(self, request):
        #data = list()
        #try:
        #    with open('/tmp/cpulog', 'r') as f:
        #        for l in f.readlines():
        #            data.append(int(l))
        #except:
        #    pass
        #data = data[-10:]
        try:
            cpu = 100 - int(snmpget('.1.3.6.1.4.1.2021.11.11.0'))
        except:
            cpu = 0
        return {self.KEY_RESPONSE: HttpResponse(
                #simplejson.dumps({'cpu':cpu}),
                simplejson.dumps([[""], [cpu]]),
                #simplejson.dumps([[""],data]),
                mimetype="application/json;charset=UTF-8;"),
        }


class PacketView(RootView):

    def view(self, request):
        rxcnt = list()
        txcnt = list()
        try:
            with open('/var/tmp/www/packet', 'r') as f:
                for l in f.readlines():
                    (rxd, txd) = l.split()
                    rxcnt.append(int(rxd))
                    txcnt.append(int(txd))
        except:
            pass
        rxcnt = rxcnt[-1:]
        txcnt = txcnt[-1:]
        return {self.KEY_RESPONSE: HttpResponse(
                simplejson.dumps([[""], rxcnt, txcnt]),
                mimetype="application/json;charset=UTF-8;"),
        }


class MemView(RootView):

    def view(self, request):
        mem = list()
        swap = list()
        total = list()

        data = meminfo()
        mem.append(data['Mem'])
        swap.append(data['Swap'])
        total.append(data['Total'])
        mem = mem[-1:]
        swap = swap[-1:]
        total = total[-1:]

        return {self.KEY_RESPONSE: HttpResponse(
                #simplejson.dumps([[""],mem,swap,total]),
                simplejson.dumps([[""], mem, swap, total, [data['Cache']]]),
                mimetype="application/json;charset=UTF-8;"),
        }
