# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django.utils.translation import ugettext as _
from django import forms
from ..field import Field
#from django.forms.extras.widgets import SelectDateWidget
from widgets.selectdate import SelectDateWidget
from widgets.selecttime import SelectTimeWidget
from widgets.selectdatetime import SelectDateTimeWidget


class SelectDateField(forms.DateField, Field):
    def __init__(self,
            years=range(2012, 2038),
            validators=[],
            widget=None,  # 無視
            label=None,
            initial=None,
            required=True,
            help_text=None):
        forms.DateField.__init__(self,
                widget=SelectDateWidget(years=years),
                label=label,
                initial=initial,
                required=False,
                help_text=help_text)
        Field.init(self, validators=list(validators), required=required)

    def clean(self, value):
        return Field.clean(self, value)


class SelectTimeField(forms.TimeField, Field):
    def __init__(self,
            show_seconds=False,
            validators=[],
            widget=None,  # 無視
            label=None,
            initial=None,
            required=True,
            help_text=None):
        forms.TimeField.__init__(self,
                widget=SelectTimeWidget(
                        show_seconds=show_seconds,
                        minute_step=5,
                        second_step=5),
                label=label,
                initial=initial,
                required=False,
                help_text=help_text)
        Field.init(self, validators=list(validators), required=required)

    def clean(self, value):
        return Field.clean(self, value)


class SelectDateTimeField(forms.DateTimeField, Field):
    def __init__(self,
            years=range(2012, 2038),
            show_seconds=False,
            validators=[],
            widget=None,  # 無視
            label=None,
            initial=None,
            required=True,
            help_text=None):
        forms.DateTimeField.__init__(self,
                widget=SelectDateTimeWidget(
                        years=years,
                        show_seconds=show_seconds),
                label=label,
                initial=initial,
                required=False,
                help_text=help_text)
        Field.init(self, validators=list(validators), required=required)

    def clean(self, value):
        return Field.clean(self, value)
