# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django.utils.translation import ugettext as _
from django import forms
from django.utils.safestring import mark_safe
from django.utils.encoding import force_unicode
from field import Field
from validators import inchoices


class RadioField(forms.ChoiceField, Field):

    class RendererWidthwise(forms.widgets.RadioFieldRenderer):
        def render(self):
            return mark_safe(u'%s'
                    % u'&nbsp;'.join([force_unicode(w) for w in self]))

    class RendererLengthwise(forms.widgets.RadioFieldRenderer):
        def render(self):
            return mark_safe(u'%s'
                    % u'<br />'.join([force_unicode(w) for w in self]))

    def __init__(self,
            attrs={},
            choices=((1, _(u'enable')), (0, _(u'disable'))),
            lengthwise=False,
            validators=[],
            widget=None,
            label=None,
            initial=0,
            required=True,
            help_text=None):
        if lengthwise:
            widget = forms.widgets.RadioSelect(renderer=self.RendererLengthwise, attrs=attrs)
        else:
            widget = forms.widgets.RadioSelect(renderer=self.RendererWidthwise, attrs=attrs)
        #choices = ((1, _(u'enable')), (0, _(u'disable')))
        choices = tuple(choices)
        super(RadioField, self).__init__(
                widget=widget,
                label=label,
                choices=choices,
                initial=initial)

        validators = [
                inchoices.InchoicesValidator(choices=choices)]
        Field.init(self, validators=validators, required=required)

    def clean(self, value):
        return Field.clean(self, value)
