# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django.contrib.sessions.backends import file
from django.contrib.sessions.backends.base import CreateError


class SessionStore(file.SessionStore):
    def __init__(self, session_key=None):
        super(SessionStore, self).__init__(session_key=session_key)
        self._changed_list = []

    def __setitem__(self, key, value):
        super(SessionStore, self).__setitem__(key, value)
        self._changed_list.append(key)

    def __delitem__(self, key):
        super(SessionStore, self).__delitem__(key)
        self._changed_list.append(key)

    def pop(self, key, *args):
        super(SessionStore, self).pop(key, *args)
        self.modified = self.modified or key in self._session
        self._changed_list.append(key)

    def setdefault(self, key, value):
        super(SessionStore, self).setdefault(key, value)
        if key in self._session:
            pass
        else:
            self._changed_list.append(key)

    def update(self, dict_):
        super(SessionStore, self).update(dict_)
        for key in self.iterkeys():
            self._changed_list.append(key)

    def clear(self):
        super(SessionStore, self).clear()
        for key in self.iterkeys():
            self._changed_list.append(key)

    BLOCK = False
    def save(self, must_create=False):
        # ベースクラスのメソッド load() を使うが、
        # その実装はバックエンドによって異なり、
        # load() 中で save() が呼ばれる可能性を否定できない。
        # そのため、flag によって無限ループを防ぐ。
        if self.BLOCK is False:
            self.BLOCK = True
            # FIXME lock 開始処理入れる
            loaded_session = super(SessionStore, self).load()
            for k, v in loaded_session.iteritems():
                if k in self._changed_list:
                    continue
                #self[k] = v
                super(SessionStore, self).__setitem__(k, v)
            self._changed_list = []
            try:
                super(SessionStore, self).save(must_create=must_create)
            except CreateError:
                super(SessionStore, self).save(must_create=False)
            # FIXME lock 終了処理入れる
            self.BLOCK = False
        else:
            super(SessionStore, self).save(must_create=must_create)

    def cleanup(self):
        from glob import glob
        import os
        limit_num = 3  # 総セッションファイル数 FIXME 数は検討 > 0
        limit_size = 2000  # 総セッションファイルサイズ  FIXME サイズは検討
        files = []
        for fpath in glob(os.path.join(self.storage_path, self.file_prefix + '*')):
            files.append({
                'path': fpath,
                'update': os.stat(fpath).st_mtime,
                'size': os.path.getsize(fpath)})
        files.sort(key=lambda file_: file_['update'], reverse=True)
        while len(files) > limit_num:
            os.unlink(files.pop()['path'])
        # 総サイズが一定以上で削除 FIXME 1個のファイルで越えたらどうする?
        while sum([f['size'] for f in files]) > limit_size:
            os.unlink(files.pop()['path'])
