# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.utils import simplejson
from django.http import HttpResponse, HttpResponseRedirect
import sys
import os
import re
import logging
from root.modules.utils import importer as imp, to_importable_path as to_ipath, is_django_app
from project.auth import Authentication


class View(object):
    KEY_RESPONSE = 'response'
    KEY_REDIRECT = 'redirect'
    KEY_TEMPLATE = 'template'
    KEY_TEMPLATE_DIC = 'template_dic'

    views = []  # ex) ['save', 'add', 'delete']

    logger = logging.getLogger("custom")

    def __init__(self):
        self.CURRENT = os.path.dirname(sys.modules[self.__module__].__file__)
        self.TEMPLATES = os.path.join(self.CURRENT, 'templates')

    @classmethod
    def as_view(cls):
        self = cls()
        return self.__view

    def __view(self, request, *args, **kwargs):
        view = getattr(self,
                'view_' + self.__get_requested_view(request),
                # FIXME self.views の順に優先されるとは限らない
                # FIXME request.POSTは常に存在するか?
                #'view_' + tuple(set(self.views) & set(request.POST.keys()))[0],
                self.view)
        try:
            params = self._view(view, request, *args, **kwargs)
        except:
            import traceback
            self.logger.debug(traceback.format_exc())
            raise

        if self.KEY_RESPONSE in params and params[self.KEY_RESPONSE]:
            return params[self.KEY_RESPONSE]
        else:
            if self.KEY_REDIRECT in params and params[self.KEY_REDIRECT]:
                return HttpResponseRedirect(params[self.KEY_REDIRECT])
            else:
                return render_to_response(
                        params[self.KEY_TEMPLATE],
                        params[self.KEY_TEMPLATE_DIC],
                        context_instance=RequestContext(request))

    def __get_requested_view(self, request):
        if request.method == 'POST':
            for v in self.views:
                if v in request.POST:
                    return v
        return ''

    def _view(self, view, request, *args, **kwargs):
        params = {
            self.KEY_RESPONSE: None,
            self.KEY_REDIRECT: None,
            self.KEY_TEMPLATE: None,
            self.KEY_TEMPLATE_DIC: {}}
        params.update(view(request, *args, **kwargs))
        return params


class RootView(View):
    REQUIRED_LOGIN = True
    NOT_LOGINED_URL = '/'  # FIXME 名前 非ログイン時のリダイレクト先

    AJAXFORM_HTML = 'ajaxform.html'  # FIXME

    def __init__(self):
        super(RootView, self).__init__()
        self.TEMPLATE = os.path.join(self.TEMPLATES, 'page.html')

    def _view(self, view, request, *args, **kwargs):
        self.session = self.Session(
                request.session,
                # FIXME [6:]
                map(lambda k: k.upper(), self.CURRENT.split(os.sep)[6:]))
        self.messages = []

        auth = Authentication()
        if self.REQUIRED_LOGIN and not auth.is_logined(request):
            # ログインしてない
            params = super(RootView, self)._view(
                    lambda r: {},  # ビュー関数のダミー
                    request, *args, **kwargs)
            params.update({
                self.KEY_RESPONSE: None,
                self.KEY_REDIRECT: self.NOT_LOGINED_URL})
        else:
            params = super(RootView, self)._view(view, request, *args, **kwargs)
            params[self.KEY_TEMPLATE_DIC].update({
                'AJAXFORM_HTML': self.AJAXFORM_HTML,
                'MESSAGES': self.messages,
                'PRODUCT': 'HDL6-H'})  # FIXME 決め打ちNG
            if request.is_ajax() and not params[self.KEY_RESPONSE]:
                params[self.KEY_RESPONSE] = self._to_ajaxform(params)
        return params

    def _to_ajaxform(self, params):
        json = {
            'redirect': '',
            'uplist': {}}  # {id1: [p1, p2, p3], id2: [pa, pb, pc]} な形式
        if params[self.KEY_REDIRECT]:
            json['redirect'] = params[self.KEY_REDIRECT]
        else:
            msgs = []
            for msg in self.messages:
                msgs.append(msg)
            if self.KEY_TEMPLATE_DIC in params \
                    and 'form' in params[self.KEY_TEMPLATE_DIC]:
                form = params[self.KEY_TEMPLATE_DIC]['form']
                NON_FIELD_ERRORS = '__all__'  # FIXME django.forms.NON_FIELD_ERRORS
                if NON_FIELD_ERRORS in form.errors:
                    for error in form.errors[NON_FIELD_ERRORS]:
                        msgs.append(error)
                for field in form:
                    for error in field.errors:
                        msgs.append('%s: %s' % (field.label, error))
            json['uplist']['ajaxform_modified'] = msgs

        return HttpResponse(
                simplejson.dumps(json),
                mimetype="application/json; charset=UTF-8;")

    def set_message(self, msg):  # FIXME 微妙 いらないかも
        self.messages.append(msg)

    class Session(object):
        def __init__(self, session, keys):
            self._prefix = '-'.join(keys)
            self._root_session = session
            if not self._prefix in self._root_session:
                self._root_session[self._prefix] = {}
            self._session = self._root_session[self._prefix]

        def __contains__(self, key):
            return key in self._session

        def __getitem__(self, key):
            return self._session[key]

        def __setitem__(self, key, value):
            self._session[key] = value
            self._root_session[self._prefix] = self._root_session[self._prefix]

        def __delitem__(self, key):
            del self._session[key]
            self._root_session[self._prefix] = self._root_session[self._prefix]

        def get(self, key, default=None):
            return self._session.get(key, default)

        def pop(self, key, *args):
            self._root_session[self._prefix] = self._root_session[self._prefix]
            return self._session.pop(key, *args)

        def update(self, dict_):
            self._session.update(dict_)
            self._root_session[self._prefix] = self._root_session[self._prefix]

        def has_key(self, key):
            return key in self._session

        def keys(self):
            return self._session.keys()

        def values(self):
            return self._session.values()

        def items(self):
            return self._session.items()

        def iterkeys(self):
            return self._session.iterkeys()

        def itervalues(self):
            return self._session.itervalues()

        def iteritems(self):
            return self._session.iteritems()

        def clear(self):
            self._session.clear()
            self._root_session[self._prefix] = self._root_session[self._prefix]

        def save(self):
            self._root_session.save()

        def load(self):
            return self._root_session.load()[self._prefix]


class AdminView(RootView):
    ADMIN_BASE_HTML = os.path.join('admin', 'templates', 'base.html')
    ADMIN_STATUS_HTML = os.path.join('admin', 'templates', 'status.html')
    ADMIN_MENU_LIST_HTML = os.path.join('admin', 'templates', 'mlist.html')

    def _view(self, view, request, *args, **kwargs):
        params = super(AdminView, self)._view(view, request, *args, **kwargs)

        from root import admin
        admin_dir = os.path.abspath(os.path.dirname(admin.__file__))

        params[self.KEY_TEMPLATE_DIC].update({
            'INIT_URL': request.session['INIT_URL'] if 'INIT_URL' in request.session else '',
            'SLIDES': self._get_slides(admin_dir),
            'STATUS': self._get_status(),
            'URL_LOGOUT': '/logout',
            'ADMIN_BASE_HTML': self.ADMIN_BASE_HTML,
            'ADMIN_STATUS_HTML': self.ADMIN_STATUS_HTML,
            'ADMIN_MENU_LIST_HTML': self.ADMIN_MENU_LIST_HTML,
            'MENU_LIST': self._get_menu(admin_dir)})
        return params

    def _get_status(self):
        return {
            'landiskname': 'landisk-abcdef',
            'version': '1.00',
            'available_capacity': '9,876 GB',
            'total_capacity': '12,345 GB',
            'notice': 'critical error'}

    def _get_slides(self, admin_dir):
        slides = []
        slides_dir = os.path.join(admin_dir, 'slides')
        if os.path.isdir(slides_dir):
            for name in sorted(os.listdir(slides_dir)):
                file = os.path.join(slides_dir, name)
                if os.path.isfile(file):
                    try:
                        s = imp(to_ipath(file))
                        slides.append({
                            'template': os.path.join(slides_dir, s.template),
                            'template_dic': s.template_dic})
                    except:
                        continue
        return slides

    def _get_menu(self, directory):
        """
        ex) return [
            {~~~, child: None},
            {~~~, child: None},
            {~~~, child: [
                {~~~, child: [
                    {~~~, child: None}
                ]},
                {~~~, child: None}
            ]},
            ~~~
        ]
        """
        menu = []
        try:
            for name in os.listdir(directory):
                path = os.path.join(directory, name)
                if is_django_app(path):
                    try:
                        settings = imp(to_ipath(os.path.join(path, 'settings.py')))
                        if not settings.IS_MENU_ITEM:
                            continue
                        settings.NAME_FORDISPLAY
                        settings.MENU_ORDER
                        settings.ID
                        settings.MENU_URL
                        settings.MENU_ICON
                    except:
                        continue

                    try:
                        if settings.IS_MENU_TAIL:
                            func = lambda d: []
                        else:
                            func = self._get_menu
                    except:
                        func = self._get_menu

                    menu.append({
                        'id': settings.ID,
                        'name': settings.NAME_FORDISPLAY,
                        'url': settings.MENU_URL,
                        'order': settings.MENU_ORDER,
                        'icon': 'img/icon/' + settings.MENU_ICON,
                        'child': func(path)})
        except:
            pass
        return menu


class FuncView(RootView):
    FUNC_BASE_HTML = os.path.join('admin', 'templates', 'func', 'base.old.html')
    NOT_LOGINED_URL = '/admin/timeout/timeout'  # FIXME

    TITLE = None

    def _view(self, view, request, *args, **kwargs):
        params = super(FuncView, self)._view(view, request, *args, **kwargs)
        if self.TITLE:
            pass
        else:
            try:
                settings = imp(
                        to_ipath(os.path.join(self.CURRENT, 'settings.py')))
                self.TITLE = settings.NAME_FORDISPLAY
                if settings.IS_MENU_ITEM:
                    request.session['INIT_URL'] = settings.MENU_URL
            except:
                self.TITLE = ''

        params[self.KEY_TEMPLATE_DIC].update({
            'FUNC_BASE_HTML': self.FUNC_BASE_HTML,
            'TITLE': self.TITLE})
        return params


class StatusView(RootView):
    """
    状態取得用ベースビュークラス
    任意の処理の状態を取得し、それをJSON形式で返すビュー
    STATUS_GETTERに登録されたメソッドがfalへ問い合わせる。
    データ形式は辞書であること以外にルールはなく
    STATUS_GETTERに登録されたメソッドの中にて辞書の内容を決定する
    
    使い方
        1. views.py にこのクラスを継承して状態取得ビューを作る
        2. urls.py にそのビューへのマッピングを書く
        3. html で status.js を読み込むように書く
        4. html で 取得した情報を反映したい要素に任意の一意のIDをつける
        5. html で new Status(url, {opts}) でインスタンス作成
            url は手順2でマッピングしたURL
            opts は種々のオプションを指定する連想配列
                timeout: 60000
                    タイムアウト
                    デフォルトでは60000
                func_success: function(that, data, dataType){...}
                    通信成功時のコールバック関数
                    デフォルトでは何もしない
                func_error: function(that, XMLHttpRequest, textStatus, errorThrown){...}
                    通信失敗時のコールバック関数
                    デフォルトでは何もしない
                func_complete: function(that, XMLHttpRequest, textStatus){...}
                    通信完了時のコールバック関数(成功、失敗のさらに後に実行される)
                    デフォルトでは何もしない
                上記コールバック関数の中で手順4で指定した要素を操作する

                etc: {result_prefix: "{% trans 'Result:' %}"}
                    その他の、クライアント側に渡したいデータ。
                    上記の例では、結果を示すprefixをサーバ側で翻訳し渡している。

        6. そのインスタンス.get() することでajaxリクエスト送信する
    """
    # FIXME この通信モデルは所謂10KC問題を孕むのでは?
    # まぁ製品的にそれほどのクライアントからのアクセスはないはずだが...
    # でもこのビューのページをリロードしまくったりしたら発生するかも
    STATUS_GETTER = lambda self: {}
    TIMEOUT = 100

    def view(self, request):
        return {self.KEY_RESPONSE: HttpResponse(
                simplejson.dumps(self._get_status()),
                mimetype="application/json; charset=UTF-8;")}

    def _get_status(self):
        try:
            import time

            status = self.STATUS_GETTER()
            for i in range(self.TIMEOUT):
                time.sleep(1)
                new = self.STATUS_GETTER()
                if status != new:
                    break
            return new
        except:
            return {"status_get_error": True}
