# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django import template
from django.conf import settings
from django.utils.encoding import iri_to_uri
import re
import os
from urlparse import urljoin


register = template.Library()

@register.filter()
def with_version(value):
    try:
        if re.match(settings.STATIC_URL, value):
            url = settings.STATIC_URL
            root = settings.STATIC_ROOT
        elif re.match(settings.MEDIA_URL, value):
            url = settings.MEDIA_URL
            root = settings.MEDIA_ROOT
        else:
            return value
        fpath = os.path.join(root, *re.sub("^%s" % url, '', value).split('/'))
        epoch = int(os.stat(fpath).st_mtime)
        if re.search('\?', value):
            return "%s,version=%s" % (value, epoch)
        return "%s?version=%s" % (value, epoch)
    except:
        return value

@register.simple_tag
def static(path):  # path from STATIC_URL
    _STATIC_URL = getattr(settings, "STATIC_URL", "")
    _STATIC_ROOT = getattr(settings, "STATIC_ROOT", "")

    url = urljoin(iri_to_uri(_STATIC_URL), path)
    version = 0
    try:
        version = int(os.stat(os.path.join(_STATIC_ROOT, path)).st_mtime)
    except OSError:
        pass  # ファイルが存在しない等
    return "%(url)s?v=%(version)s" % {
        "url": url,
        "version": version,
    }

@register.filter
def index(l, i):
    return l[int(i)]

