# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django.utils.translation import ugettext as _
from django import forms
import os
from project.view import RootView
from project.forms.fields.password import PasswordField
from project.forms.fields.extras.landiskpass import LanDiskPassField
from project.forms.fields.text import TextField
from project.fal.user import User
from project.fal.command import get_message as fgettext
from project.utils import is_crash, is_keywait


class View(RootView):
    REQUIRED_LOGIN = False

    class Form(forms.Form):
        username = TextField(label=_(u'User Name'))
        old_password = PasswordField(label=_(u'Old Password'))
        new_password1 = LanDiskPassField(label=_(u'New Password'))
        new_password2 = LanDiskPassField(label=_(u'New Password (confirm)'))

        def clean(self):
            password1 = self.cleaned_data.get('new_password1', None)
            password2 = self.cleaned_data.get('new_password2', None)
            if password1 != password2:
                raise forms.ValidationError(
                        _(u'%(passwd2)s did not match %(passwd1)s.') % {
                            'passwd1': _(u'New Password'),
                            'passwd2': _(u'New Password (confirm)'),
                        })
            return self.cleaned_data

    views = ['apply']

    def view(self, request, username=''):
        disabled = False
        if is_crash() or is_keywait():
            self.set_message(_(u'Cannot be used now.'))
            disabled = True

        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                'form': self.Form(initial={'username': username}),
                'disabled': disabled,
            }
        }

    def view_apply(self, request, username=''):
        disabled = False
        if is_crash() or is_keywait():
            self.set_message(_(u'Cannot be used now.'))
            disabled = True

        form = self.Form(data=request.POST)
        with self.processing(constname='ERR_USER_STATE_CHANGE') as p:
            if form.is_valid():
                username = form.cleaned_data['username']
                passwd_old = form.cleaned_data.get('old_password', '')
                passwd_new = form.cleaned_data['new_password1']

                uobj = User()
                if username in uobj.load().keys() \
                        and uobj.auth(user=username, password=passwd_old):
                    uobj.modify(name=username, passwd=passwd_new, log=False)
                    if uobj.get_result(boolean=True):
                        self.set_message(fgettext(
                                'MSG_USER_PASSWORD_CHANGE',
                                arglist={'name': username},
                                is_log=True))
                        p.result(True)
                    else:
                        self.set_message(fgettext(
                                'ERR_USER_PASSWORD_CHANGE',
                                arglist={'name': username},
                                is_log=True))
                        p.result(False)
                else:
                    self.set_message(
                        _(u'[%(name)s] or [%(passwd)s] is Error.') % {
                        'name': _(u'User Name'),
                        'passwd': _(u'Old Password')})
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                'form': form,
                'disabled': disabled,
            }
        }


userpassword = View.as_view()
