(function($) {

/*
 * autocomplete
 *
 * source: [{label: "abc", value: ""}]
 */
$.widget('ui.autocomplete', $.ui.autocomplete, {
    _normalize: function( items ) {
        // assume all items have the right format when the first item is complete
        if ( items.length && items[ 0 ].label && items[ 0 ].value ) {
            return items;
        }
        return $.map( items, function( item ) {
            if ( typeof item === "string" ) {
                return {
                    label: item,
                    value: item
                };
            }
            return $.extend( {}, item, {
                // NOTE: orig
                //label: item.label || item.value,
                //value: item.value || item.label
                label: (item.label === undefined) ? item.value : item.lavel,
                value: (item.value === undefined) ? item.label : item.value
            });
        });
    },
})

/* 
 * timespinner
 */
$.widget('ui.timespinner', $.ui.spinner, {
	options: {
		step: 1,
		page: 1
	},
	round: function() {
		var step = this.options.step;
		this.element.val(this._format(Math.floor(this.value() / step) * step));
	},
	_parse: function(value) {
		if (typeof value === 'string') {
			if (value == '') {
				return null;
			}
			if (Number(value) == value) {
				return Number(value);
			}
			var t = value.split(':', 2);
			var n = (Number(t[0]) * 60 + Number(t[1]));
			return Number(n);
		}
		return value;
	},
	_format: function(value) {
		if (value == null) {
			return '';
		}
		var v = Number(value);
		while (v < 0) {
			v += (24 * 60);
		}
		v = (v % (24 * 60));  // ~23:59
		//var mm = '00' + (v % 60);
		//mm = mm.substring(mm.length - 2);
		var mm = v % 60;
		mm = '00' + (mm !== mm ? '00' : mm);
		mm = mm.substring(mm.length - 2);
		//var hh = '00' + (Math.floor(v / 60) % 24);
		//hh = hh.substring(hh.length - 2);
		var hh = Math.floor(v / 60) % 24;
		hh = '00' + (hh !== hh ? '00' : hh);
		hh = hh.substring(hh.length - 2);
		return hh + ':' + mm;
	}
});

/* 
 * mcautocomplete
 *
 * showHeader: false
 * columns: [{name: "Title", width: "200px"}, {name: "Price", width: "100px"}]
 * source: [{label: ["Guri and Gura", "$190"], value: "Guri and Gura"},
 *          {label: ["Hansel and Gretel", "$130"], value: "Hansel and Gretel"}]
 */
$.widget('ui.mcautocomplete', $.ui.autocomplete, {
	_create: function() {
		this._super();
		this.widget().menu('option', 'items', '> :not(.ui-widget-header)');
	},
	_renderMenu: function(ul, items) {
		if (this.options.showHeader) {
			var table = $('<div class="ui-widget-header" style="width:100%"></div>');
			$.each(this.options.columns, function(idx, item) {
				table.append('<span style="padding:0 4px;float:left;width:' + item.width + ';word-break:break-all;">' + (item.name ? item.name : '&nbsp;') + '</span>');
			});
			table.append('<div style="clear:both"></div>');
			ul.append(table);
		}
		this._super(ul, items);
	},
	_renderItem: function(ul, item) {
		if (this.options.columns) {
			var t = '', result = '';
			$.each(this.options.columns, function (idx, column) {
				t += '<span style="padding:0 4px;float:left;width:' + column.width + ';word-break:break-all;">' + (idx ? ':' : '') +(item.label[idx] ? item.label[idx] : '') + '</span>';
			});
			result = $('<li></li>')
				.append('<div>' + t + '<div style="clear:both;"></div>')
				.appendTo(ul)
				.data('ui-autocomplete-item', item)
			return result;
		} else {
			return this._super(ul, item);
		}
	}
});

/* 
 * descautocomplete
 *
 * source: [{label: ["Guri and Gura", "$190"], value: "Guri and Gura"},
 *          {label: ["Hansel and Gretel", "$130"], value: "Hansel and Gretel"}]
 */
$.widget('ui.descautocomplete', $.ui.autocomplete, {
	_create: function() {
		this._super();
		this.widget().menu('option', 'items', '> :not(.ui-widget-header)');
	},
	_renderItem: function(ul, item) {
		var width = this.options.width ? this.options.width : '300px';
		var t = '', result = '';
		t += '<div style="width:' + width + ';word-break:break-all;padding:0 4px;">' + (item.label[0] ? item.label[0] : '') + '</div>';
		t += '<div style="width:' + width + ';word-break:break-all;padding:0 4px 0 2em;font-size:88%;">' + (item.label[1] ? item.label[1] : '') + '</div>';
		result = $('<li></li>')
			.append('<div>' + t + '<div style="clear:both;width:300px;"></div>')
			.appendTo(ul)
			.data('ui-autocomplete-item', item)
		return result;
		//return this._super(ul, item);
	}
});

/* 
 * mcbselect
 *
 * source: [{label: ["Guri and Gura", "$190"], value: "Guri and Gura"},
 *          {label: ["Hansel and Gretel", "$130"], value: "Hansel and Gretel"}]
 */
$.widget('ui.mcbselect', {
	version: '1.0.0',
	options: {
		width: '20em',
		height: '10em',
		__dummy__: 1
	},
	_create: function() {
		var $element = this.element;
		var options = this.options;

		$element.addClass('c-ui-mcbselect');
		$element.css({
			width: options.width,
			height: options.height});
		$element.find('input:checkbox').each(function() {
			var $cb = $(this);
			$cb
			.on('change', function() {
				if ($cb.prop('checked')) {
					$cb.parent().addClass('c-ui-mcbselect-on');
				} else {
					$cb.parent().removeClass('c-ui-mcbselect-on');
				}
			})
			.trigger('change');
		});
	},
	__dummy__: 1
});

})(jQuery);
