# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
# Django settings for webui project.
import os
from project.utils import get_series
try:
    from product_conf import TEMP_DIR
except ImportError:
    TEMP_DIR = '/var/tmp'


DEBUG = False
TEMPLATE_DEBUG = DEBUG

PROJECT_ROOT = os.path.abspath(os.path.join(os.path.dirname(__file__), os.pardir))

TMP = os.path.join(TEMP_DIR, 'www')
not os.path.exists(TMP) and os.mkdir(TMP)
LOGIN_TIMEOUT = 1200 if os.path.exists('/etc/landisk_beta') else 600

ADMINS = (
    # ('Your Name', 'your_email@example.com'),
)

MANAGERS = ADMINS

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.',  # Add 'postgresql_psycopg2', 'mysql', 'sqlite3' or 'oracle'.
        'NAME': '',                       # Or path to database file if using sqlite3.
        'USER': '',                       # Not used with sqlite3.
        'PASSWORD': '',                   # Not used with sqlite3.
        'HOST': '',                       # Set to empty string for localhost. Not used with sqlite3.
        'PORT': '',                       # Set to empty string for default. Not used with sqlite3.
    }
}
if DEBUG:
    DATABASES['default']['ENGINE'] = 'django.db.backends.sqlite3'

# Local time zone for this installation. Choices can be found here:
# http://en.wikipedia.org/wiki/List_of_tz_zones_by_name
# although not all choices may be available on all operating systems.
# On Unix systems, a value of None will cause Django to use the same
# timezone as the operating system.
# If running in a Windows environment this must be set to the same as your
# system time zone.
# TIME_ZONE = 'America/Chicago'
TIME_ZONE = 'Asia/Tokyo'

# Language code for this installation. All choices can be found here:
# http://www.i18nguy.com/unicode/language-identifiers.html
# LANGUAGE_CODE = 'en-us'
LANGUAGE_CODE = 'ja'

LOCALE_PATHS = (
    '/usr/local/share/www/webui/locale',
    os.path.join(PROJECT_ROOT, "root", "admin", "mo", "helptext", get_series(default="HDL-AA"), "locale"),
)

SITE_ID = 1

# If you set this to False, Django will make some optimizations so as not
# to load the internationalization machinery.
USE_I18N = True

# If you set this to False, Django will not format dates, numbers and
# calendars according to the current locale.
USE_L10N = True

# If you set this to False, Django will not use timezone-aware datetimes.
USE_TZ = True

# Absolute filesystem path to the directory that will hold user-uploaded files.
# Example: "/home/media/media.lawrence.com/media/"
# MEDIA_ROOT = ''
MEDIA_ROOT = os.path.join(PROJECT_ROOT, 'media')

# URL that handles the media served from MEDIA_ROOT. Make sure to use a
# trailing slash.
# Examples: "http://media.lawrence.com/media/", "http://example.com/media/"
MEDIA_URL = '/media/'

# Absolute path to the directory static files should be collected to.
# Don't put anything in this directory yourself; store your static files
# in apps' "static/" subdirectories and in STATICFILES_DIRS.
# Example: "/home/media/media.lawrence.com/static/"
# STATIC_ROOT = ''
STATIC_ROOT = os.path.join(PROJECT_ROOT, 'static')

# URL prefix for static files.
# Example: "http://media.lawrence.com/static/"
STATIC_URL = '/static/'

# Additional locations of static files
STATICFILES_DIRS = (
    # Put strings here, like "/home/html/static" or "C:/www/django/static".
    # Always use forward slashes, even on Windows.
    # Don't forget to use absolute paths, not relative paths.
)

# List of finder classes that know how to find static files in
# various locations.
STATICFILES_FINDERS = (
    'django.contrib.staticfiles.finders.FileSystemFinder',
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
    # 'django.contrib.staticfiles.finders.DefaultStorageFinder',
)


# Make this unique, and don't share it with anybody.
def create_secret_key():
    from django.utils.crypto import get_random_string
    from django.utils.html import escape
    chars = 'abcdefghijklmnopqrstuvwxyz0123456789!@#$%^&*(-_=+)'
    return escape(get_random_string(50, chars))

SECRET_KEY = '5=njq(nl@kii$490k47i=xm-ne&amp;0j-3(8aik7-@t0m3rmbpfn#'
SECRET_KEY = create_secret_key()

# List of callables that know how to import templates from various sources.
TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.Loader',
    'django.template.loaders.app_directories.Loader',
    # 'django.template.loaders.eggs.Loader',
)

MIDDLEWARE_CLASSES = (
    'django.middleware.common.CommonMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    # Uncomment the next line for simple clickjacking protection:
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'root.modules.middlewares.common.Common',
)

SESSION_EXPIRE_AT_BROWSER_CLOSE = False
# SESSION_ENGINE = 'django.contrib.sessions.backends.file'
SESSION_ENGINE = 'root.modules.session'
SESSION_FILE_PATH = TMP
SESSION_COOKIE_AGE = 60 * 60 * 24 * 365

ROOT_URLCONF = 'webui.urls'

# Python dotted path to the WSGI application used by Django's runserver.
WSGI_APPLICATION = 'webui.wsgi.application'

TEMPLATE_DIRS = (
    # Put strings here, like "/home/html/django_templates" or "C:/www/django/templates".
    # Always use forward slashes, even on Windows.
    # Don't forget to use absolute paths, not relative paths.
    '/usr/local/share/www/webui/root',
)


def __apps(root_app_name):
    def __is_app(dir):
        if not os.path.isdir(dir):
            return False
        for name in os.listdir(dir):
            if name == 'models.py' or name == 'models.pyc':
                return True
        return False

    def __get_child_apps(papp_dir):
        apps = []
        if __is_app(papp_dir):
            papp_name = os.path.basename(papp_dir)
            apps.append(papp_name)
            for capp_name in sorted(os.listdir(papp_dir)):
                capp_dir = os.path.join(papp_dir, capp_name)
                apps = apps + ['.'.join([papp_name, capp_name]) for capp_name in __get_child_apps(capp_dir)]
        return apps
    root_app_dir = os.path.sep.join(os.path.abspath(__file__).split(os.path.sep)[:-2] + [root_app_name])
    apps = tuple(__get_child_apps(root_app_dir))
    return apps
INSTALLED_APPS = __apps('root') + (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    # Uncomment the next line to enable the admin:
    # 'django.contrib.admin',
    # Uncomment the next line to enable admin documentation:
    # 'django.contrib.admindocs',
    # 'root',
)

# A sample logging configuration. The only tangible logging
# performed by this configuration is to send an email to
# the site admins on every HTTP 500 error when DEBUG=False.
# See http://docs.djangoproject.com/en/dev/topics/logging for
# more details on how to customize your logging configuration.
LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'filters': {
        'require_debug_false': {
            '()': 'django.utils.log.RequireDebugFalse'
        }
    },
    'formatters': {
        'verbose': {
            'format': '%(asctime)s [%(levelname)s] %(message)s'
        }
    },
    'handlers': {
        'file': {
            'level': 'DEBUG',
            'class': 'logging.handlers.RotatingFileHandler',
            'formatter': 'verbose',
            'filename': os.path.join(TMP, 'django'),
            'maxBytes': 1024 * 1024,
            'backupCount': 3,
        }
    },
    'loggers': {
        'custom': {
            'handlers': ['file'],
            'level': 'DEBUG',
        }
    }
}

FILE_UPLOAD_TEMP_DIR = TMP  # FIXME


# Features

class Dict(dict):

    def __call__(self, path=[], default=None):
        try:
            return self.__call(self, path)
        except:
            return default

    def __call(self, dictionary, path):
        key, path = path[0], path[1:]
        if path:
            return self.__call(dictionary[key], path)
        else:
            return dictionary[key]


from project.fal.features import Features
FEATURES = Dict(Features().query())


def __get_raidmodelist(volumename):
    for raidvolume in FEATURES(["raidvolumelist"], default=[]):
        if raidvolume.get("name") == volumename:
            return raidvolume.get("modelist", [])
    return []

INTERNAL_VOLUME_NAME = "sataraid1"

INTERNAL_VOLUME_RAID_IS_SUPPORTED = bool(len(__get_raidmodelist(INTERNAL_VOLUME_NAME)))
VOLUME_ENCRYPTION_IS_SUPPORTED = "volume_encryption" in FEATURES(["functionlist"], default=[])
NARSUS_IS_SUPPORTED = "narsus" in FEATURES(["functionlist"], default=[])
LAN1_IS_SUPPORTED = bool(len([netif for netif in FEATURES(["deviceinfo", "netiflist"]) if str(netif["port"]) == "1"]))
LAN2_IS_SUPPORTED = bool(len([netif for netif in FEATURES(["deviceinfo", "netiflist"]) if str(netif["port"]) == "2"]))
NETWORK_INTERFACES = FEATURES(["deviceinfo", "netiflist"], default=[])
